@@ -1,5 +1,5 @@
 --
---  $Id: sioc_ab.sql,v 1.33.2.3 2010/01/29 20:21:28 source Exp $
+--  $Id: sioc_ab.sql,v 1.33.2.15 2011/08/01 14:15:23 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -117,6 +117,27 @@ create procedure fill_ods_addressbook_sioc2 (
   declare graph_iri, addressbook_iri, socialnetwork_iri, contact_iri, creator_iri, role_iri, iri varchar;
 
   {
+    fill_ods_addressbook_services ();
+
+    for (select WAI_ID,
+                WAI_TYPE_NAME,
+                WAI_NAME,
+                WAI_ACL
+           from DB.DBA.WA_INSTANCE
+          where ((_wai_name is null) or (WAI_NAME = _wai_name))
+            and WAI_TYPE_NAME = 'AddressBook') do
+    {
+      graph_iri := SIOC..acl_graph (WAI_TYPE_NAME, WAI_NAME);
+      exec (sprintf ('sparql clear graph <%s>', graph_iri));
+      SIOC..wa_instance_acl_insert (WAI_TYPE_NAME, WAI_NAME, WAI_ACL);
+      for (select P_DOMAIN_ID, P_ID, P_ACL
+             from AB.WA.PERSONS
+            where P_DOMAIN_ID = WAI_ID and P_ACL is not null) do
+      {
+        contact_acl_insert (P_DOMAIN_ID, P_ID, P_ACL);
+      }
+    }
+
     id := -1;
     deadl := 3;
     cnt := 0;
@@ -128,8 +149,8 @@ create procedure fill_ods_addressbook_sioc2 (
       deadl := deadl - 1;
       goto l0;
     };
-  l0:
 
+  l0:
     for (select WAI_ID,
                 WAI_IS_PUBLIC,
                 WAI_NAME,
@@ -181,7 +202,9 @@ create procedure fill_ods_addressbook_sioc2 (
                 P_UPDATED,
 								P_TAGS,
 	              P_FOAF,
-		      P_IRI
+                P_IRI,
+                P_ACL,
+                P_CERTIFICATE
            from DB.DBA.WA_INSTANCE,
                 DB.DBA.WA_MEMBER,
                 AB.WA.PERSONS
@@ -251,7 +274,8 @@ create procedure fill_ods_addressbook_sioc2 (
                     P_UPDATED,
 											P_TAGS,
 		                  P_FOAF,
-				  P_IRI);
+                      P_IRI,
+                      P_CERTIFICATE);
 
       cnt := cnt + 1;
 		   if (mod (cnt, 500) = 0)
@@ -261,44 +285,37 @@ create procedure fill_ods_addressbook_sioc2 (
       }
     }
     commit work;
-
-		id := -1;
-		deadl := 3;
-		cnt := 0;
-    declare exit handler for sqlstate '40001'
-    {
-			if (deadl <= 0)
-				resignal;
-			rollback work;
-			deadl := deadl - 1;
-			goto l1;
-		};
-	l1:
-		for (select WAI_ID,
-                WAI_IS_PUBLIC,
-								WAI_NAME
-					 from DB.DBA.WA_INSTANCE
-					where ((WAI_IS_PUBLIC = 1 and _wai_name is null) or WAI_NAME = _wai_name)
-					  and WAI_TYPE_NAME = 'AddressBook'
-					order by WAI_ID) do
-		{
-      addressbook_iri := SIOC..addressbook_iri (WAI_NAME);
-      graph_iri := SIOC..get_graph_new (null, coalesce (_access_mode, WAI_IS_PUBLIC), addressbook_iri);
-      iri := sprintf ('http://%s%s/services/addressbook', get_cname(), get_base_path ());
-      ods_sioc_service (graph_iri, iri, addressbook_iri, null, 'text/xml', iri||'/services.wsdl', iri, 'SOAP');
-
-			cnt := cnt + 1;
-	    if (mod (cnt, 500) = 0)
-	    {
-				commit work;
-				id := WAI_ID;
-			}
-    }
-		commit work;
   }
 }
 ;
 
+-------------------------------------------------------------------------------
+--
+create procedure fill_ods_addressbook_services ()
+{
+  declare graph_iri, services_iri, service_iri, service_url varchar;
+  declare svc_functions any;
+
+  graph_iri := get_graph ();
+
+  -- instance
+  svc_functions := vector ('addressbook.search', 'addressbook.new', 'addressbook.import', 'addressbook.export', 'addressbook.publication.new', 'addressbook.subscription.new', 'addressbook.options.set',  'addressbook.options.get');
+  ods_object_services (graph_iri, 'addressbook', 'ODS AddressBook instance services', svc_functions);
+
+  -- contact
+  svc_functions := vector ('addressbook.get', 'addressbook.edit', 'addressbook.delete', 'addressbook.comment.new', 'addressbook.relationship.new', 'addressbook.relationship.delete', 'addressbook.annotation.new');
+  ods_object_services (graph_iri, 'addressbook/contact', 'ODS AddressBook contact services', svc_functions);
+
+  -- contact comment
+  svc_functions := vector ('addressbook.comment.get', 'addressbook.comment.delete');
+  ods_object_services (graph_iri, 'addressbook/contact/comment', 'ODS AddressBook comment services', svc_functions);
+
+  -- contact annotation
+  svc_functions := vector ('addressbook.annotation.get', 'addressbook.annotation.claim', 'addressbook.annotation.delete');
+  ods_object_services (graph_iri, 'addressbook/contact/annotation', 'ODS AddressBook annotation services', svc_functions);
+}
+;
+
 -------------------------------------------------------------------------------
 --
 create procedure clean_ods_addressbook_sioc2 (
@@ -413,10 +430,12 @@ create procedure contact_insert (
   inout updated datetime,
 	inout tags varchar,
 	inout foaf varchar,
-	inout ext_iri varchar)
+  inout ext_iri varchar,
+  inout certificate varchar)
 {
   declare iri, iri2, temp_iri varchar;
 	declare person_iri varchar;
+  declare info, modulus, exponent, certificate_iri any;
 
 	declare exit handler for sqlstate '*'
 	{
@@ -485,11 +504,13 @@ create procedure contact_insert (
     }
     if (not DB.DBA.is_empty_or_null (relationships))
     {
-      for (select relationship from DB.DBA.WA_USER_INTERESTS (txt) (relationship varchar) P where txt = relationships) do
+      for (select fld1, fld2 from DB.DBA.WA_USER_INTERESTS (txt) (fld1 varchar, fld2 varchar) P where txt = relationships) do
   	  {
-  	    if (length (relationship))
+  	    if (length (fld1))
   	    {
-  	      DB.DBA.ODS_QUAD_URI (graph_iri, iri, relationship, person_iri);
+          if (DB.DBA.is_empty_or_null (fld2))
+            fld2 := person_iri;
+  	      DB.DBA.ODS_QUAD_URI (graph_iri, iri, ODS.ODS_API."ontology.denormalize"(fld1), fld2);
   	    }
   	  }
     }
@@ -645,6 +666,24 @@ create procedure contact_insert (
         DB.DBA.ODS_QUAD_URI_L (graph_iri, temp_iri, vcard_iri ('Country'), bCountry);
     }
   }
+
+  -- certificate
+  info := get_certificate_info (9, cast (certificate as varchar), 0);
+  if (info is not null and isarray (info) and cast (info[0] as varchar) = 'RSAPublicKey')
+  {
+    modulus := info[2];
+    exponent := info[1];
+    certificate_iri := iri || '#cert';
+    DB.DBA.ODS_QUAD_URI (graph_iri, certificate_iri, cert_iri ('identity'), iri);
+    DB.DBA.ODS_QUAD_URI (graph_iri, certificate_iri, rdf_iri ('type'), rsa_iri ('RSAPublicKey'));
+
+    DB.DBA.ODS_QUAD_URI_L_TYPED (graph_iri, certificate_iri, rsa_iri ('modulus'), bin2hex (modulus), cert_iri ('hex'), null);
+    DB.DBA.ODS_QUAD_URI_L_TYPED (graph_iri, certificate_iri, rsa_iri ('public_exponent'), cast (exponent as varchar), cert_iri ('int'), null);
+  }
+
+  -- contact services
+  SIOC..ods_object_services_attach (graph_iri, iri2, 'addressbook/contact');
+
   SIOC..contact_comments_insert (graph_iri, addressbook_iri, domain_id, contact_id);
   SIOC..contact_annotations_insert (graph_iri, domain_id, contact_id);
 }
@@ -676,6 +715,8 @@ create procedure contact_delete (
 	scot_tags_delete (domain_id, iri, tags);
 	delete_quad_s_or_o (graph_iri, iri, iri);
 
+  SIOC..ods_object_services_dettach (graph_iri, iri, 'addressbook/contact');
+
   -- Social Data
   iri := SIOC..socialnetwork_contact_iri (domain_id, contact_id);
 	scot_tags_delete (domain_id, iri, tags);
@@ -742,7 +783,8 @@ create trigger PERSONS_SIOC_I after insert on AB.WA.PERSONS referencing new as N
                   N.P_UPDATED,
 									N.P_TAGS,
 									N.P_FOAF,
-									N.P_IRI);
+                  N.P_IRI,
+                  N.P_CERTIFICATE);
 }
 ;
 
@@ -806,7 +848,8 @@ create trigger PERSONS_SIOC_U after update on AB.WA.PERSONS referencing old as O
                   N.P_UPDATED,
 									N.P_TAGS,
 									N.P_FOAF,
-									N.P_IRI);
+                  N.P_IRI,
+                  N.P_CERTIFICATE);
 }
 ;
 
@@ -821,6 +864,96 @@ create trigger PERSONS_SIOC_D before delete on AB.WA.PERSONS referencing old as
 }
 ;
 
+-------------------------------------------------------------------------------
+--
+create procedure contact_acl_insert (
+  inout domain_id integer,
+  inout contact_id integer,
+  inout acl any)
+{
+  declare graph_iri, iri varchar;
+  declare exit handler for sqlstate '*'
+  {
+    sioc_log_message (__SQL_MESSAGE);
+    return;
+  };
+  iri := SIOC..addressbook_contact_iri (domain_id, contact_id);
+  graph_iri := AB.WA.acl_graph (domain_id);
+
+  SIOC..acl_insert (graph_iri, iri, acl);
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create procedure contact_acl_delete (
+  inout domain_id integer,
+  inout contact_id integer,
+  inout acl any)
+{
+  declare graph_iri, iri varchar;
+  declare exit handler for sqlstate '*'
+  {
+    sioc_log_message (__SQL_MESSAGE);
+    return;
+  };
+  iri := SIOC..addressbook_contact_iri (domain_id, contact_id);
+  graph_iri := AB.WA.acl_graph (domain_id);
+
+  SIOC..acl_delete (graph_iri, iri, acl);
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create trigger PERSONS_SIOC_ACL_I after insert on AB.WA.PERSONS order 100 referencing new as N
+{
+  if (coalesce (N.P_ACL, '') <> '')
+  {
+    contact_acl_insert (N.P_DOMAIN_ID,
+                        N.P_ID,
+                        N.P_ACL);
+
+    SIOC..acl_ping (N.P_DOMAIN_ID,
+                    SIOC..addressbook_contact_iri (N.P_DOMAIN_ID, N.P_ID),
+                    null,
+                    N.P_ACL);
+  }
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create trigger PERSONS_SIOC_ACL_U after update (P_ACL) on AB.WA.PERSONS order 100 referencing old as O, new as N
+{
+  if (coalesce (O.P_ACL, '') <> '')
+    contact_acl_delete (O.P_DOMAIN_ID,
+                        O.P_ID,
+                        O.P_ACL);
+
+  if (coalesce (N.P_ACL, '') <> '')
+    contact_acl_insert (N.P_DOMAIN_ID,
+                        N.P_ID,
+                        N.P_ACL);
+
+  SIOC..acl_ping (N.P_DOMAIN_ID,
+                  SIOC..addressbook_contact_iri (N.P_DOMAIN_ID, N.P_ID),
+                  O.P_ACL,
+                  N.P_ACL);
+}
+;
+
+-------------------------------------------------------------------------------
+--
+create trigger PERSONS_SIOC_ACL_D before delete on AB.WA.PERSONS order 100 referencing old as O
+{
+  if (coalesce (O.P_ACL, '') <> '')
+    contact_acl_delete (O.P_DOMAIN_ID,
+                        O.P_ID,
+                        O.P_ACL);
+}
+;
+
 -------------------------------------------------------------------------------
 --
 create procedure contact_comments_insert (
@@ -910,23 +1043,24 @@ create procedure contact_comment_insert (
   master_id := cast (master_id as integer);
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
 	if (isnull (graph_iri))
-		{
     graph_iri := get_graph_new (domain_id, null, master_iri);
+
     if (isnull (graph_iri))
       return;
-		}
+
   if (isnull (forum_iri))
-	{
     forum_iri := AB.WA.forum_iri (domain_id);
+
     if (isnull (forum_iri))
       return;
-  }
+
 		comment_iri := addressbook_comment_iri (domain_id, master_id, comment_id);
   if (isnull (comment_iri))
     return;
 
       foaf_maker (graph_iri, u_url, u_name, u_mail);
-      ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_sioc_post (graph_iri, comment_iri, forum_iri, null, title, last_update, last_update, null, comment, null, null, u_url);
+  SIOC..ods_object_services_attach (graph_iri, comment_iri, 'addressbook/contact/comment');
   DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, sioc_iri ('has_reply'), comment_iri);
   DB.DBA.ODS_QUAD_URI (graph_iri, comment_iri, sioc_iri ('reply_of'), master_iri);
     }
@@ -949,13 +1083,15 @@ create procedure contact_comment_delete (
   master_id := cast (master_id as integer);
   master_iri := SIOC..addressbook_contact_iri (domain_id, master_id);
   if (isnull (graph_iri))
-  {
     graph_iri := SIOC..get_graph_new (domain_id, null, master_iri);
+
     if (isnull (graph_iri))
       return;
-  }
+
   comment_iri := SIOC..addressbook_comment_iri (domain_id, master_id, comment_id);
   delete_quad_s_or_o (graph_iri, comment_iri, comment_iri);
+  -- comment services
+  SIOC..ods_object_services_dettach (graph_iri, comment_iri, 'addressbook/contact/comment');
 }
 ;
 
@@ -1084,8 +1220,7 @@ create procedure contact_annotation_insert (
 	inout created datetime,
 	inout updated datetime)
 {
-	declare master_iri, annotattion_iri varchar;
-
+  declare master_iri, annotation_iri varchar;
 	declare exit handler for sqlstate '*'
 	{
 		sioc_log_message (__SQL_MESSAGE);
@@ -1100,15 +1235,16 @@ create procedure contact_annotation_insert (
     if (isnull (graph_iri))
       return;
 		}
-  annotattion_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
-  DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, an_iri ('annotates'), master_iri);
-  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotattion_iri);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('author'), author);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('body'), body);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('created'), created);
-  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotattion_iri, an_iri ('modified'), updated);
+  annotation_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
+  DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, an_iri ('annotates'), master_iri);
+  DB.DBA.ODS_QUAD_URI (graph_iri, master_iri, an_iri ('hasAnnotation'), annotation_iri);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('author'), author);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('body'), body);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('created'), created);
+  DB.DBA.ODS_QUAD_URI_L (graph_iri, annotation_iri, an_iri ('modified'), updated);
 
-	  addressbook_claims_insert (graph_iri, annotattion_iri, claims);
+  addressbook_claims_insert (graph_iri, annotation_iri, claims);
+  SIOC..ods_object_services_attach (graph_iri, annotation_iri, 'addressbook/contact/annotation');
 	}
 ;
 
@@ -1121,7 +1257,7 @@ create procedure contact_annotation_delete (
   inout annotation_id integer,
   inout claims any)
 {
-  declare master_iri, annotattion_iri varchar;
+  declare master_iri, annotation_iri varchar;
   declare exit handler for sqlstate '*'
   {
 		sioc_log_message (__SQL_MESSAGE);
@@ -1136,10 +1272,9 @@ create procedure contact_annotation_delete (
     if (isnull (graph_iri))
       return;
   }
-  annotattion_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
-  SIOC..delete_quad_s_or_o (graph_iri, annotattion_iri, annotattion_iri);
-
-	addressbook_claims_delete (graph_iri, annotattion_iri, claims);
+  annotation_iri := addressbook_annotation_iri (domain_id, master_id, annotation_id);
+  SIOC..delete_quad_s_or_o (graph_iri, annotation_iri, annotation_iri);
+  SIOC..ods_object_services_dettach (graph_iri, annotation_iri, 'addressbook/contact/annotation');
 }
 ;
 
@@ -1147,7 +1282,7 @@ create procedure contact_annotation_delete (
 --
 create procedure addressbook_claims_insert (
   in graph_iri varchar,
-  in annotattion_iri varchar,
+  in annotation_iri varchar,
   in claims any)
 {
   declare N integer;
@@ -1159,21 +1294,13 @@ create procedure addressbook_claims_insert (
     cPedicate := V[N][1];
     cValue := V[N][2];
     if (0 = length (cPedicate))
+    {
       cPedicate := rdfs_iri ('seeAlso');
-
-    DB.DBA.ODS_QUAD_URI (graph_iri, annotattion_iri, cPedicate, cValue);
+    } else {
+      cPedicate := ODS.ODS_API."ontology.denormalize" (cPedicate);
   }
+    DB.DBA.ODS_QUAD_URI (graph_iri, annotation_iri, cPedicate, cValue);
 }
-;
-
--------------------------------------------------------------------------------
---
-create procedure addressbook_claims_delete (
-  in graph_iri varchar,
-  in annotattion_iri varchar,
-  in claims any)
-{
-  SIOC..delete_quad_so (graph_iri, annotattion_iri, annotattion_iri);
 }
 ;
 
@@ -1245,9 +1372,24 @@ create procedure ods_addressbook_sioc_init ()
 }
 ;
 
---AB.WA.exec_no_error('ods_addressbook_sioc_init ()');
+-------------------------------------------------------------------------------
+--
+create procedure AB.WA.tmp_update ()
+{
+  if (registry_get ('ab_services_update') = '1')
+    return;
 
+  SIOC..fill_ods_addressbook_services();
+  registry_set ('ab_services_update', '1');
+}
+;
+
+AB.WA.tmp_update ();
+
+-------------------------------------------------------------------------------
+--
 use DB;
+
 -------------------------------------------------------------------------------
 --
 wa_exec_no_error ('drop view ODS_ADDRESSBOOK_CONTACTS');
