@@ -1,5 +1,5 @@
 /*
- *  $Id: bookmark.js,v 1.27.2.3 2010/01/29 20:21:28 source Exp $
+ *  $Id: bookmark.js,v 1.27.2.15 2011/06/09 17:43:16 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -20,6 +20,15 @@
  *  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
+function setFooter() {
+  if ($('pane_main')) {
+    var wDims = OAT.Dom.getViewport()
+    var hDims = OAT.Dom.getWH('FT')
+    var cPos = OAT.Dom.position('pane_main')
+    $('pane_main').style.height = (wDims[1] - hDims[1] - cPos[1] - 20) + 'px';
+  }
+}
+
 function myPost(frm_name, fld_name, fld_value)
 {
   createHidden(frm_name, fld_name, fld_value);
@@ -49,6 +58,19 @@ function toolbarPost(value)
   doPost ('F1', 'toolbar');
 }
 
+function odsPost(obj, fields, button) {
+  var form = getParent (obj, 'form');
+  var formName = form.name;
+  for (var i = 0; i < fields.length; i += 2)
+    createHidden(formName, fields[i], fields[i+1]);
+
+  if (button) {
+    doPost(formName, button);
+  } else {
+    form.submit();
+  }
+}
+
 function dateFormat(date, format) {
 	function long(d) {
 		return ((d < 10) ? "0" : "") + d;
@@ -226,20 +248,6 @@ function enableElement (id, id_gray, flag, doc)
     o.style.display = mode;
 }
 
-function showCell (cell)
-{
-  var c = getObject (cell);
-  if ((c) && (c.style.display == "none"))
-    c.style.display = "";
-}
-
-function hideCell(cell)
-{
-  var c = getObject(cell);
-  if ((c) && (c.style.display != "none"))
-    c.style.display = "none";
-}
-
 function selectAllCheckboxes (obj, prefix) {
   var objForm = obj.form;
   for (var i = 0; i < objForm.elements.length; i++) {
@@ -298,16 +306,14 @@ function updateGrants(objName)
 }
 
 function coloriseTable(id) {
-  if (document.getElementsByTagName) {
-    var table = document.getElementById(id);
-    if (table != null) {
+  var table = $(id);
+  if (table) {
       var rows = table.getElementsByTagName("tr");
       for (i = 0; i < rows.length; i++) {
         rows[i].className = rows[i].className + " tr_" + (i % 2);;
       }
     }
   }
-}
 
 function coloriseRow(obj, checked) {
   obj.className = (obj.className).replace('tr_select', '');
@@ -339,72 +345,11 @@ function clickNode2(obj) {
   }
 }
 
-function addOption (form, text_name, box_name) {
-  var box = form.elements[box_name];
-  if (box) {
-    var text = form.elements[text_name];
-    if (text) {
-      text.value = BMK.trim(text.value);
-      if (text.value == '')
-        return;
-    	for (var i=0; i<box.options.length; i++)
-		    if (text.value == box.options[i].value)
-		      return;
-	    box.options[box.options.length] = new Option(text.value, text.value, false, true);
-	    sortSelect(box);
-	    text.value = '';
-	  }
-	}
-}
-
-function deleteOption (form, box_name) {
-  var box = form.elements[box_name];
-  if (box)
-	  box.options[box.selectedIndex] = null;
-}
-
-function composeOptions (form, box_name, text_name) {
-  var box = form.elements[box_name];
-  if (box) {
-    var text = form.elements[text_name];
-    if (text) {
-		  text.value = '';
-    	for (var i=0; i<box.options.length; i++)
-    	  if (text.value == '')
-		      text.value = box.options[i].value;
-		    else
-		      text.value = text.value + '\n' + box.options[i].value;
-	  }
-	}
-}
-
 function showTag(tag) {
   createHidden2(parent.document, 'F1', 'tag', tag);
   parent.document.forms['F1'].submit();
 }
 
-// sortSelect(select_object)
-//   Pass this function a SELECT object and the options will be sorted
-//   by their text (display) values
-function sortSelect(box) {
-	var o = new Array();
-	for (var i=0; i<box.options.length; i++)
-		o[o.length] = new Option( box.options[i].text, box.options[i].value, box.options[i].defaultSelected, box.options[i].selected) ;
-
-	if (o.length==0)
-	  return;
-
-	o = o.sort(function(a,b) {
-                      			if ((a.text+"") < (b.text+"")) { return -1; }
-                      			if ((a.text+"") > (b.text+"")) { return 1; }
-                      			return 0;
-			                     }
-		        );
-
-	for (var i=0; i<o.length; i++)
-		box.options[i] = new Option(o[i].text, o[i].value, o[i].defaultSelected, o[i].selected);
-}
-
 function showTab(tabs, tabsCount, tabNo)
 {
   if ($(tabs))
@@ -431,99 +376,155 @@ function showTab(tabs, tabsCount, tabNo)
   }
 }
 
-function windowShow(sPage, width, height)
-{
-  if (width == null)
-    width = 500;
-  if (height == null)
-    height = 420;
-  sPage = sPage + '&sid=' + document.forms[0].elements['sid'].value + '&realm=' + document.forms[0].elements['realm'].value;
-  win = window.open(sPage, null, "width="+width+",height="+height+", top=100, left=100, scrollbars=yes, resize=yes, menubar=no");
+function windowShow(sPage, sPageName, width, height) {
+	if (width == null)
+    width = 700;
+	if (height == null)
+		height = 500;
+  if (sPage.indexOf('form=') == -1)
+    sPage += '&form=F1';
+  if (sPage.indexOf('sid=') == -1)
+    sPage += urlParam('sid');
+  if (sPage.indexOf('realm=') == -1)
+    sPage += urlParam('realm');
+  win = window.open(sPage, sPageName, "width="+width+",height="+height+",top=100,left=100,status=yes,toolbar=no,menubar=no,scrollbars=yes,resizable=yes");
   win.window.focus();
 }
 
-function rowSelect(obj)
+function rowSelected(tr)
 {
-  var submitMode = false;
-  if (window.document.F1.elements['src'])
-    if (window.document.F1.elements['src'].value.indexOf('s') != -1)
-      submitMode = true;
-  if (submitMode)
-    if (window.opener.document.F1)
-      if (window.opener.document.F1.elements['submitting'])
-        return false;
-  var closeMode = true;
-  if (window.document.F1.elements['dst'])
-    if (window.document.F1.elements['dst'].value.indexOf('c') == -1)
-      closeMode = false;
-  var singleMode = true;
-  if (window.document.F1.elements['dst'])
-    if (window.document.F1.elements['dst'].value.indexOf('s') == -1)
-      singleMode = false;
+  var cbInput;
+  var s1Input;
+  var s2Input;
+  var inputs = tr.getElementsByTagName('input');
+  for (var i = 0; i < inputs.length; i++) {
+    if (inputs[i].name == 'cb_item')
+      cbInput = inputs[i];
+    if (inputs[i].name == 's1_item')
+      s1Input = inputs[i];
+    if (inputs[i].name == 's2_item')
+      s2Input = inputs[i];
+  }
+  if (cbInput) {
+    cbInput.checked = !cbInput.checked;
+    updateChecked(cbInput, cbInput.name);
+  }
+  else if (s1Input)
+{
+    commitChecked(s1Input.value);
+  }
+}
 
-  var s2 = (obj.name).replace('b1', 's2');
-  var s1 = (obj.name).replace('b1', 's1');
+function commitChecked(s1Value, s2Value)
+{
+  var srcForm = window.document.F1;
+  var dstForm = window.opener.document.F1;
+
+  var submitMode = false;
+  if (srcForm.elements['src'] && (srcForm.elements['src'].value.indexOf('s') != -1)) {
+      submitMode = true;
+    if (dstForm && dstForm.elements['submitting'])
+        return false;
+  }
+  var closeMode = true;
+  var singleMode = true;
+  if (srcForm.elements['dst']) {
+    if (srcForm.elements['dst'].value.indexOf('c') == -1)
+      closeMode = false;
+    if (srcForm.elements['dst'].value.indexOf('s') == -1)
+      singleMode = false;
+  }
 
   var myRe = /^(\w+):(\w+);(.*)?/;
-  var params = window.document.forms['F1'].elements['params'].value;
+  var params = srcForm.elements['params'].value;
   var myArray;
+  if (dstForm) {
   while(true) {
     myArray = myRe.exec(params);
     if (myArray == undefined)
       break;
-    if (myArray.length > 2)
-      if (window.opener.document.F1)
-        if (window.opener.document.F1.elements[myArray[1]]) {
+      if (myArray.length > 2) {
+        var fld = dstForm.elements[myArray[1]];
+        if (fld) {
           if (myArray[2] == 's1')
-            if (window.opener.document.F1.elements[myArray[1]])
-              rowSelectValue(window.opener.document.F1.elements[myArray[1]], window.document.F1.elements[s1], singleMode, submitMode);
+            rowSelectValue(fld, s1Value, singleMode);
           if (myArray[2] == 's2')
-            if (window.opener.document.F1.elements[myArray[1]])
-              rowSelectValue(window.opener.document.F1.elements[myArray[1]], window.document.F1.elements[s2], singleMode, submitMode);
+            rowSelectValue(fld, s2Value, singleMode);
+        }
         }
     if (myArray.length < 4)
       break;
     params = '' + myArray[3];
   }
+  }
   if (submitMode) {
     window.opener.createHidden('F1', 'submitting', 'yes');
-    window.opener.document.F1.submit();
+    dstForm.submit();
   }
   if (closeMode)
     window.close();
 }
 
-function rowSelectValue(dstField, srcField, singleMode)
+function updateChecked (obj, objName, event)
 {
-  if (singleMode)
+  if (event)
+	  event.cancelBubble = true;
+  var objForm = obj.form;
+  coloriseRow(getParent(obj, 'tr'), obj.checked);
+
+  var s1Value = objForm.s1.value;
+  s1Value = BMK.trim(s1Value);
+  s1Value = BMK.trim(s1Value, ',');
+  s1Value = BMK.trim(s1Value);
+  s1Value = s1Value + ',';
+  for (var i = 0; i < objForm.elements.length; i = i + 1) {
+    var obj = objForm.elements[i];
+    if (obj != null && obj.type == "checkbox" && obj.name == objName) {
+      if (obj.checked) {
+        if (s1Value.indexOf(obj.value+',') == -1)
+          s1Value = s1Value + obj.value + ',';
+      } else {
+        s1Value = (s1Value).replace(obj.value+',', '');
+      }
+    }
+  }
+  objForm.s1.value = BMK.trim(s1Value, ',');
+}
+
+function addChecked(srcForm, checkboxName, selectionMsq)
   {
-    dstField.value = srcField.value;
+  if (!anySelected (srcForm, checkboxName, selectionMsq, 'confirm'))
+    return false;
+
+  return commitChecked(srcForm.s1.value, srcForm.s2.value);
+}
+
+function rowSelectValue(dstField, srcValue, singleMode) {
+	if (singleMode) {
+		dstField.value = srcValue;
   } else {
     dstField.value = BMK.trim(dstField.value);
     dstField.value = BMK.trim(dstField.value, ',');
     dstField.value = BMK.trim(dstField.value);
-    if (dstField.value.indexOf(srcField.value) == -1)
-    {
-      if (dstField.value == '')
-      {
-        dstField.value = srcField.value;
+		if (dstField.value.indexOf(srcValue) == -1) {
+			if (dstField.value == '') {
+				dstField.value = srcValue;
       } else {
-        dstField.value = dstField.value + ', ' + srcField.value;
+				dstField.value = dstField.value + ',' + srcValue;
       }
     }
   }
 }
 
 // Hiddens functions
-function createHidden(frm_name, fld_name, fld_value) {
-  var hidden;
-
-  createHidden2(document, frm_name, fld_name, fld_value);
+function createHidden(frm_name, fld_name, fld_value)
+{
+  return createHidden2(document, frm_name, fld_name, fld_value);
 }
 
-function createHidden2(doc, frm_name, fld_name, fld_value) {
+function createHidden2(doc, frm_name, fld_name, fld_value)
+{
   var hidden;
-
   if (doc.forms[frm_name]) {
     hidden = doc.forms[frm_name].elements[fld_name];
     if (hidden == null) {
@@ -535,6 +536,7 @@ function createHidden2(doc, frm_name, fld_name, fld_value) {
     }
     hidden.value = fld_value;
   }
+  return hidden;
 }
 
 function changeExportName(fld_name, from, to) {
@@ -543,78 +545,6 @@ function changeExportName(fld_name, from, to) {
     obj.value = (obj.value).replace(from, to);
 }
 
-function updateChecked (obj, objName)
-{
-  var objForm = obj.form;
-  coloriseRow(getParent(obj, 'tr'), obj.checked);
-  objForm.s1.value = BMK.trim(objForm.s1.value);
-  objForm.s1.value = BMK.trim(objForm.s1.value, ',');
-  objForm.s1.value = BMK.trim(objForm.s1.value);
-  objForm.s1.value = objForm.s1.value + ',';
-  for (var i = 0; i < objForm.elements.length; i = i + 1)
-  {
-    var obj = objForm.elements[i];
-    if (obj != null && obj.type == "checkbox" && obj.name == objName)
-    {
-      if (obj.checked)
-      {
-        if (objForm.s1.value.indexOf(obj.value+',') == -1)
-          objForm.s1.value = objForm.s1.value + obj.value+',';
-      } else {
-        objForm.s1.value = (objForm.s1.value).replace(obj.value+',', '');
-      }
-    }
-  }
-  objForm.s1.value = BMK.trim(objForm.s1.value, ',');
-}
-
-function addChecked (form, txt, selectionMsq)
-{
-  if (!anySelected (form, txt, selectionMsq, 'confirm'))
-    return;
-
-  var submitMode = false;
-  if (window.document.F1.elements['src'])
-    if (window.document.F1.elements['src'].value.indexOf('s') != -1)
-      submitMode = true;
-  if (submitMode)
-    if (window.opener.document.F1)
-      if (window.opener.document.F1.elements['submitting'])
-        return false;
-  var singleMode = true;
-  if (window.document.F1.elements['dst'])
-    if (window.document.F1.elements['dst'].value.indexOf('s') == -1)
-      singleMode = false;
-
-  var s1 = 's1';
-  var s2 = 's2';
-
-  var myRe = /^(\w+):(\w+);(.*)?/;
-  var params = window.document.forms['F1'].elements['params'].value;
-  var myArray;
-  while(true) {
-    myArray = myRe.exec(params);
-    if (myArray == undefined)
-      break;
-    if (myArray.length > 2)
-      if (window.opener.document.F1)
-        if (window.opener.document.F1.elements[myArray[1]]) {
-          if (myArray[2] == 's1')
-            if (window.opener.document.F1.elements[myArray[1]])
-              rowSelectValue(window.opener.document.F1.elements[myArray[1]], window.document.F1.elements[s1], singleMode, submitMode);
-          if (myArray[2] == 's2')
-            if (window.opener.document.F1.elements[myArray[1]])
-              rowSelectValue(window.opener.document.F1.elements[myArray[1]], window.document.F1.elements[s2], singleMode, submitMode);
-        }
-    if (myArray.length < 4)
-      break;
-    params = '' + myArray[3];
-  }
-  if (submitMode)
-    window.opener.document.F1.submit();
-  window.close();
-}
-
 function addTag(tag, objName)
 {
   var obj = document.F1.elements[objName];
@@ -690,133 +620,69 @@ function openIFrame (id, accountID, uri)
 
 function urlParam (fldName)
 {
-  var S = '';
   var O = document.forms[0].elements[fldName];
-  if (O)
-    S += '&' + fldName + '=' + encodeURIComponent(O.value);
-  return S;
+  if (O && O.value != '')
+    return '&' + fldName + '=' + encodeURIComponent(O.value);
+  return '';
 }
 
-function showObject(id)
-{
-  var o = document.getElementById(id);
-  if (o)
-  {
-    o.style.display="";
-    o.visible = true;
+function myA(obj) {
+  if (obj.href) {
+    document.location = obj.href + '?' + urlParam('sid') + urlParam('realm');
+    return false;
   }
 }
 
-function hideObject(id)
-{
-  var o = document.getElementById(id);
-  if (o != null)
-  {
-    o.style.display="none";
-    o.visible = false;
-  }
-}
-
-function initRequest ()
-{
-  var xmlhttp = null;
-  try {
-    xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
-  } catch (e) { }
-
-  if (xmlhttp == null) {
-    try {
-      xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
-    } catch (e) { }
-  }
-
-  // Gecko / Mozilla / Firefox
-  if (xmlhttp == null)
-    xmlhttp = new XMLHttpRequest();
-
-  return xmlhttp;
-}
-
-var timer = null;
+var progressTimer = null;
 var progressID = null;
 var progressMax = null;
-var progressStop = null;
-
-function resetState()
-{
-  var xmlhttp = initRequest();
-  var URL = 'ajax.vsp?a=import&sa=reset';
-  xmlhttp.open("POST", URL + urlParam("sid") + urlParam("realm"), false);
-  xmlhttp.setRequestHeader("Pragma", "no-cache");
-  xmlhttp.send(null);
-  try {
-    progressID = xmlhttp.responseXML.getElementsByTagName("id")[0].firstChild.nodeValue;
-  } catch (e) { }
-}
+var progressSize = 40;
+var progressInc = 100 / progressSize;
 
 function stopState()
 {
-  timer = null;
-
-  var xmlhttp = initRequest();
-  var URL = 'ajax.vsp?a=import&sa=stop';
-  xmlhttp.open("POST", URL+"&id="+progressID+urlParam("sid")+urlParam("realm"), false);
-  xmlhttp.setRequestHeader("Pragma", "no-cache");
-  xmlhttp.send(null);
+  progressTimer = null;
+  OAT.AJAX.POST('ajax.vsp', "a=load&sa=stop&id="+progressID+urlParam("sid")+urlParam("realm"), null, {async: false});
 }
 
 function initState ()
 {
-  // reset state first
-  resetState();
-
-  // init state
-  var xmlhttp = initRequest();
-  var URL = 'ajax.vsp';
-  xmlhttp.open("POST", URL, false);
-  xmlhttp.setRequestHeader("Pragma", "no-cache");
-  xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
-  xmlhttp.send("a=import&sa=init&id="+progressID+urlParam("sid")+urlParam("realm")+urlParam("folder_id")+urlParam("folder_name")+urlParam("tags"));
+  progressTimer = null;
+  var x = function (data) {
+    try {
+      var xml = OAT.Xml.createXmlDoc(data);
+      progressID = OAT.Xml.textValue(xml.getElementsByTagName('id')[0]);
+    } catch (e) {}
 
   createProgressBar();
-  timer = setTimeout("checkState()", 1000);
+    progressTimer = setTimeout("checkState()", 500);
 
   document.forms['F1'].action = 'bookmarks.vspx';
 }
+  OAT.AJAX.POST('ajax.vsp', "a=load&sa=init"+urlParam("sid")+urlParam("realm")+urlParam("folder_id")+urlParam("folder_name")+urlParam("tags"), x, {async: false});
+}
 
 function checkState()
 {
-  var xmlhttp = initRequest();
-  var URL = 'ajax.vsp?a=import&sa=state';
-  xmlhttp.open("POST", URL+"&id="+progressID+urlParam("sid")+urlParam("realm"), true);
-  xmlhttp.onreadystatechange = function() {
-    if (xmlhttp.readyState == 4) {
+  var x = function (data) {
       var progressIndex;
-
-      // progressIndex
       try {
-        progressIndex = xmlhttp.responseXML.getElementsByTagName("index")[0].firstChild.nodeValue;
+      var xml = OAT.Xml.createXmlDoc(data);
+      progressIndex = OAT.Xml.textValue(xml.getElementsByTagName('index')[0]);
       } catch (e) { }
 
-      if (timer != null)
         showProgress (progressIndex);
-      if ((progressIndex != null) && (progressIndex != progressMax) && (timer != null))
-      {
+
+    if ((progressIndex != null) && (progressIndex != progressMax)) {
         setTimeout("checkState()", 500);
       } else {
-        timer = null;
+      progressTimer = null;
         $('btn_Stop').click();
       }
     }
-  }
-  xmlhttp.setRequestHeader("Pragma", "no-cache");
-  xmlhttp.send("");
+  OAT.AJAX.POST('ajax.vsp', "a=load&sa=state&id="+progressID+urlParam("sid")+urlParam("realm"), x);
 }
 
-var size = 40;
-var increment = 100 / size;
-
-// create the progress bar
 function createProgressBar()
 {
   progressMax = getObject('progressMax').innerHTML;
@@ -824,12 +690,12 @@ function createProgressBar()
   var centerCellName;
   var tableText = "";
   var tdText = "";
-  for (x = 0; x < size; x++) {
+  for (x = 0; x < progressSize; x++) {
     if (progressMax != null) {
-	    if (x == (size/2))
+      if (x == (progressSize/2))
 	      centerCellName = "progress_" + x;
 	  }
-    tableText += "<td id=\"progress_" + x + "\" width=\"" + increment + "%\" height=\"20\" bgcolor=\"blue\" />";
+    tableText += "<td id=\"progress_" + x + "\" width=\"" + progressInc + "%\" height=\"20\" bgcolor=\"blue\" />";
   }
   var idiv = window.document.getElementById("progressText");
   if (idiv)
@@ -840,7 +706,6 @@ function createProgressBar()
   centerCell = window.document.getElementById(centerCellName);
 }
 
-// show the current percentage
 function showProgress (progressIndex)
 {
   if (!progressMax)
@@ -863,10 +728,10 @@ function showProgress (progressIndex)
     percentageText = percentage;
   }
   centerCell.innerHTML = "<font color=\"white\">" + percentageText + "%</font>";
-  for (x = 0; x < size; x++)
+  for (x = 0; x < progressSize; x++)
   {
     var cell = window.document.getElementById("progress_" + x);
-    if ((cell) && (percentage/x < increment))
+    if ((cell) && (percentage/x < progressInc))
     {
       cell.style.backgroundColor = "blue";
     } else {
@@ -883,14 +748,15 @@ function readBookmark (id)
   var realm = '';
   if (document.forms[0].elements['realm'])
     realm = document.forms[0].elements['realm'].value;
-  OAT.AJAX.POST ("ajax.vsp", "sid="+sid+"&realm="+realm+"&id="+id+"&a=visited", function(){}, {onstart:function(){}, onerror:function(){}});
+  OAT.AJAX.POST ('ajax.vsp', "sid="+sid+"&realm="+realm+"&id="+id+"&a=visited", function(){}, {onstart:function(){}, onerror:function(){}});
 }
 
 function davBrowse (fld)
 {
-  var options = { mode: 'browser',
-                  onConfirmClick: function(path, fname) {$(fld).value = path + fname;}
-                };
+  var options = {
+    mode: 'browser',
+    onConfirmClick: function(path, fname) {$(fld).value = '/DAV' + path + fname;}
+  };
   OAT.WebDav.open(options);
 }
 
@@ -930,12 +796,9 @@ function destinationChange(obj, actions)
 }
 
 var BMK = new Object();
-
 BMK.trim = function (sString, sChar)
 {
-
-  if (sString)
-  {
+  if (sString) {
     if (sChar == null)
     {
       sChar = ' ';
@@ -985,10 +848,8 @@ BMK.readField = function (field, doc)
   {
     v = doc.forms[0].elements[field];
     if (v)
-    {
       v = v.value;
     }
-  }
   return v;
 }
 
@@ -1011,8 +872,12 @@ BMK.initState = function (state)
   if (!state)
     var state = new Object();
 
-  state.sid = BMK.readField('sid');
-  state.realm = BMK.readField('realm');
+  var v = BMK.readField('sid');
+  if (v)
+    state.sid = v;
+  var v = BMK.readField('realm');
+  if (v)
+    state.sid = v;
   if (!state.tab)
     state.tab = 'tree';
 
@@ -1062,7 +927,7 @@ BMK.initLeftPane = function ()
   BMK.forms = new Object();
   BMK.forms['import'] = {height: '400', postActions:['BMK.loadTree(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
   BMK.forms['export'] = {height: '160'};
-  BMK.forms['bookmark'] = {height: '380', postActions:['BMK.loadTags(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
+  BMK.forms['bookmark'] = {height: '400', postActions:['BMK.loadTags(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
   BMK.forms['folder'] = {height: '180', postActions:['BMK.loadTree(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
   BMK.forms['smart folder'] = {height: '320', postActions:['BMK.loadTree(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
   BMK.forms['move'] = {params: {items: true}, height: '100', postActions:['BMK.loadTree(true)', 'BMK.reloadItems()', 'BMK.resetToolbars()']};
@@ -1250,7 +1115,7 @@ BMK.loadPath = function (w, wIndex)
     node.toggleSelect({ctrlKey:false});
   }
   BMK.selectedPath();
-  BMK.nodeAction();
+  BMK.execNodeAction();
 }
 
 BMK.findPath = function (path)
@@ -1308,7 +1173,7 @@ BMK.loadTreeData = function(nodePath, node, nodeFunction)
   OAT.AJAX.GET(S, '', x);
 }
 
-BMK.nodeAction = function()
+BMK.execNodeAction = function()
 {
   var a = $('nodeAction');
   if (a && (a.value != ''))
@@ -1430,8 +1295,7 @@ BMK.loadItems = function(nodeID, nodePath)
   pane.innerHTML = '';
   var URL = 'forms.vspx?sa=browse&node='+encodeURIComponent(nodeID)+'&path='+encodeURIComponent(nodePath)+BMK.sessionParams();
   var v = $('nodeItem');
-  if (v && (v.value != ''))
-  {
+  if (v && (v.value != '')) {
     URL += '&item=' + v.value;
     v.value = '';
   }
@@ -1459,20 +1323,16 @@ BMK.formParams = function (doc)
   if (!doc) {doc = document;}
   var S = '';
   var o = doc.forms[0].elements;
-  for (var i = 0; i < o.length; i++)
-  {
-    if (o[i])
-    {
-      if ((o[i].type == "checkbox" && o[i].checked) || (o[i].type != "checkbox"))
-      {
+  for (var i = 0; i < o.length; i++) {
+    if (!o[i] || !o[i].name)
+      continue;
+
+    if ((o[i].type == "checkbox" && o[i].checked) || (o[i].type != "checkbox")) {
         var n = o[i].name;
         if ((n != '') && (n.indexOf('page_') != 0) && (n.indexOf('__') != 0))
-        {
           S += '&' + n + '=' + encodeURIComponent(o[i].value);
         }
       }
-    }
-  }
   return S;
 }
 
@@ -1563,67 +1423,17 @@ BMK.formPostAfter = function (action)
     formDialog.hide();
 }
 
-BMK.updateClaim = function (claimNo)
-{
-  if (claimNo == 'xxx')
-  {
-    if (($v('c_iri_xxx') == '') || ($v('c_relation_xxx') == '') || ($v('c_value_xxx') == ''))
-    {
-      alert ('The IRI, relation and value fileld can not be empty|');
-    }
-    else
-    {
-      var tr = $('c_tr_xxx');
-      if (tr)
-      {
-        var seqNo = parseInt($v('c_seqNo'));
-
-        var tr_add = OAT.Dom.create('tr');
-        tr_add.id = 'c_tr_'+seqNo;
-
-        var S = tr.innerHTML;
-        S = S.replace(/xxx/g, ''+seqNo);
-        S = S.replace(/add_16/g, 'del_16');
-
-        var tr_parent = $('c_tr').parentNode;
-        tr_parent.insertBefore(tr_add, $('c_tr'));
-        tr_add.innerHTML = S;
-
-        var cl = new OAT.Combolist([], 'rdfs:seeAlso');
-        cl.input.name = 'c_relation_'+seqNo;
-        cl.input.id = 'c_relation_'+seqNo;
-        cl.input.style.width = "80%";
-        var td = $('c_td_'+seqNo);
-        td.innerHTML = '';
-        td.appendChild(cl.div);
-        cl.addOption('rdfs:seeAlso');
-        cl.addOption('foaf:made');
-        cl.addOption('foaf:maker');
-
-        $('c_iri_'+seqNo).value = $v('c_iri_xxx');
-        $('c_relation_'+seqNo).value = $v('c_relation_xxx');
-        $('c_value_'+seqNo).value = $v('c_value_xxx');
-
-        $('c_seqNo').value = seqNo + 1;
-        $('c_iri_xxx').value = '';
-        $('c_relation_xxx').value = '';
-        $('c_value_xxx').value = '';
-      }
-    }
-  }
-  else
-  {
-    OAT.Dom.unlink('c_tr_'+claimNo);
-  }
-}
-
 BMK.aboutDialog = function ()
 {
   var aboutDiv = $('aboutDiv');
   if (aboutDiv) {OAT.Dom.unlink(aboutDiv);}
-  aboutDiv = OAT.Dom.create('div', {width:'430px', height:'150px'});
+  aboutDiv = OAT.Dom.create('div', {
+    width:'430px',
+    height: '170px',
+    overflow: 'hidden'
+  });
   aboutDiv.id = 'aboutDiv';
-  aboutDialog = new OAT.Dialog('About ODS Booomarks', aboutDiv, {width:430, buttons: 0, resize:0, modal:1});
+  aboutDialog = new OAT.Dialog('About ODS Booomarks', aboutDiv, {width:445, buttons: 0, resize:0, modal:1});
 	aboutDialog.cancel = aboutDialog.hide;
 
   var x = function (txt) {
@@ -1639,380 +1449,3 @@ BMK.aboutDialog = function ()
   }
   OAT.AJAX.POST("ajax.vsp", "a=about", x, {type:OAT.AJAX.TYPE_TEXT, onstart:function(){}, onerror:function(){}});
 }
-
-BMK.updateRow = function (prefix, No, optionObject)
-{
-  if (No != null)
-  {
-    OAT.Dom.unlink(prefix+'_tr_'+No);
-    var No = parseInt($(prefix+'_no').value);
-    for (var N = 0; N < No; N++)
-    {
-      if ($(prefix+'_tr_' + N))
-        return;
-    }
-    OAT.Dom.show (prefix+'_tr_no');
-  }
-  else
-  {
-    var No = parseInt($v(prefix+'_no'));
-    var tbl = $(prefix+'_tbl');
-    if (tbl)
-    {
-      options = {};
-      for (var p in optionObject) {options[p] = optionObject[p]; }
-
-      OAT.Dom.hide (prefix+'_tr_no');
-
-      var tr = OAT.Dom.create('tr');
-      tr.id = prefix+'_tr_' + No;
-      tbl.appendChild(tr);
-
-      var fldOptions = options.fld1;
-      if (fldOptions)
-      {
-        var td = OAT.Dom.create('td');
-        tr.appendChild(td);
-        BMK.updateCell (td, prefix, '_fld_1_', No, fldOptions)
-      }
-      var fldOptions = options.fld2;
-      if (fldOptions)
-      {
-        var td = OAT.Dom.create('td');
-        tr.appendChild(td);
-        BMK.updateCell (td, prefix, '_fld_2_', No, fldOptions)
-      }
-      var fldOptions = options.fld3;
-      if (fldOptions)
-      {
-        var td = OAT.Dom.create('td');
-        tr.appendChild(td);
-        BMK.updateCell (td, prefix, '_fld_3_', No, fldOptions)
-      }
-      var td = OAT.Dom.create('td');
-      tr.appendChild(td);
- 		  var fld = OAT.Dom.create("input");
- 		  fld.type = 'button';
- 		  fld.value = 'Remove';
- 		  fld.className = 'button';
-      fld.onclick = function (){BMK.updateRow(prefix, No);};
-      td.appendChild(fld);
-
-      $(prefix+'_no').value = No + 1;
-    }
-  }
-}
-
-BMK.updateCell = function (td, prefix, fldName, No, optionObject)
-{
-  fldName = prefix + fldName + No;
-  if (optionObject.mode == 1)
-  {
-	  BMK.updateRowCombo(td, fldName, optionObject);
-	}
-  else if (optionObject.mode == 2)
-  {
-	  BMK.updateRowCombo2(td, fldName, optionObject);
-  }
-  else if (optionObject.mode == 3)
-  {
-	  BMK.updateRowCombo3(td, fldName, optionObject);
-  }
-  else
-  {
-	  BMK.updateInput(td, fldName, optionObject);
-  }
-}
-
-BMK.updateInput = function (elm, fldName, fldOptions)
-{
-  var fld = OAT.Dom.create("input");
-  fld.type = 'text';
-  fld.id = fldName;
-  fld.name = fld.id;
-  if (fldOptions.value)
-    fld.value = fldOptions.value;
-  fld.className = fldOptions.className;
-  fld.onblur = fldOptions.onBlur;
-  fld.style.width = '95%';
-  var elm = $(elm);
-  elm.appendChild(fld);
-}
-
-BMK.updateRowCombo = function (elm, fldName, fldOptions)
-{
-  var cc = new OAT.Combolist([], fldOptions.value, {name: fldName});
-
-  cc.input.name = fldName;
-  cc.input.id = fldName;
-  cc.input.style.width = "90%";
-
-  var elm = $(elm);
-  elm.appendChild(cc.div);
-}
-
-BMK.updateRowComboOption = function (cc, optionName)
-{
-  cc.addOption(optionName, optionName);
-}
-
-BMK.updateRowComboOption2 = function (elm, elmValue, optionName, optionValue)
-{
-	var o = OAT.Dom.option(optionName, optionValue, elm);
-	if (elmValue == optionValue)
-	  o.selected = true;
-}
-
-BMK.updateRowCombo2 = function (elm, fldName, fldOptions)
-{
-	var cc = OAT.Dom.create("select");
-  cc.name = fldName;
-  cc.id = fldName;
-	BMK.updateRowComboOption2(cc, fldOptions.value, "Person URI", "URI");
-  BMK.updateRowComboOption2(cc, fldOptions.value, "Relationship Property", "Property");
-	// BMK.updateRowComboOption2(cc, fldOptions.value, "SPARQL", "SPARQL
-	// Expression");
-
-  var elm = $(elm);
-  elm.appendChild(cc);
-}
-
-BMK.updateRowCombo3 = function (elm, fldName, fldOptions)
-{
-	var cc = OAT.Dom.create("select");
-  cc.name = fldName;
-  cc.id = fldName;
-	BMK.updateRowComboOption2 (cc, fldOptions.value, "Grant", "G");
-	BMK.updateRowComboOption2 (cc, fldOptions.value, "Revoke", "R");
-
-  var elm = $(elm);
-  elm.appendChild(cc);
-}
-
-BMK.validateError = function (fld, msg)
-{
-  alert(msg);
-  setTimeout(function(){fld.focus();}, 1);
-  return false;
-}
-
-BMK.validateMail = function (fld)
-{
-  if ((fld.value.length == 0) || (fld.value.length > 40))
-    return BMK.validateError(fld, 'E-mail address cannot be empty or longer then 40 chars');
-
-  var regex = /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;
-  if (!regex.test(fld.value))
-    return BMK.validateError(fld, 'Invalid E-mail address');
-
-  return true;
-}
-
-BMK.validateURL = function (fld)
-{
-  var regex = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/
-  if (!regex.test(fld.value))
-    return BMK.validateError(fld, 'Invalid URL address');
-
-  return true;
-}
-
-BMK.validateField = function (fld)
-{
-  if ((fld.value.length == 0) && OAT.Dom.isClass(fld, '_canEmpty_'))
-    return true;
-  if (OAT.Dom.isClass(fld, '_mail_'))
-    return BMK.validateMail(fld);
-  if (OAT.Dom.isClass(fld, '_url_'))
-    return BMK.validateURL(fld);
-  return true;
-}
-
-BMK.validateInputs = function (fld)
-{
-  var retValue = true;
-  var form = fld.form;
-  for (i = 0; i < form.elements.length; i++)
-  {
-    var fld = form.elements[i];
-    if (OAT.Dom.isClass(fld, '_validate_'))
-    {
-      retValue = BMK.validateField(fld);
-      if (!retValue)
-        return retValue;
-    }
-  }
-  return retValue;
-}
-
-var TBL = new Object();
-TBL.createRow = function (prefix, No, optionObject)
-{
-  if (No != null)
-  {
-    OAT.Dom.unlink(prefix+'_tr_'+No);
-    var No = parseInt($(prefix+'_no').value);
-    for (var N = 0; N < No; N++)
-    {
-      if ($(prefix+'_tr_'+N))
-        return;
-    }
-    OAT.Dom.show (prefix+'_tr_no');
-  }
-  else
-  {
-    var tbl = $(prefix+'_tbl');
-    if (tbl)
-    {
-      options = {btn_1: {mode: 1}};
-      for (var p in optionObject) {options[p] = optionObject[p]; }
-
-      No = optionObject.No;
-      if (!No) {
-        if (!$(prefix+'_no')) {
-        	var fld = OAT.Dom.create("input");
-          fld.type = 'hidden';
-          fld.name = prefix+'_no';
-          fld.id = fld.name;
-          fld.value = '0';
-          tbl.appendChild(fld);
-        }
-        No = $v(prefix+'_no');
-      }
-      No = parseInt(No)
-
-      OAT.Dom.hide (prefix+'_tr_no');
-
-      var tr = OAT.Dom.create('tr');
-      tr.id = prefix+'_tr_' + No;
-      tbl.appendChild(tr);
-
-      // fields
-      for (var fld in options)
-      {
-        if (fld.indexOf('fld') == 0)
-        {
-          var fldOptions = options[fld];
-          var td = OAT.Dom.create('td');
-          td.id = prefix+'_td_'+ No+'_'+fld.replace(/fld_/, '');
-          tr.appendChild(td);
-          TBL.createCell (td, prefix, fld, No, fldOptions)
-        }
-      }
-
-      // actions
-      var td = OAT.Dom.create('td');
-      td.id = prefix+'_td_'+ No+'_btn';
-      td.style.whiteSpace = 'nowrap';
-      tr.appendChild(td);
-      for (var btn in options)
-      {
-        if (btn.indexOf('btn') == 0)
-        {
-          var btnOptions = options[btn];
-          TBL.createButton(td, prefix, btn, No, btnOptions)
-        }
-      }
-      $(prefix+'_no').value = No + 1;
-    }
-  }
-}
-
-TBL.createCell = function (td, prefix, fldName, No, fldOptions)
-{
-  fldName = prefix + '_' + fldName + '_' + No;
-  if (fldOptions.tdCssText)
-    td.style.cssText = fldOptions.tdCssText;
-  var fn = TBL["createCell"+fldOptions.mode];
-  if (fn)
-	  fn(td, prefix, fldName, No, fldOptions);
-}
-
-TBL.createCell0 = function (td, prefix, fldName, No, fldOptions)
-{
-  var fld = OAT.Dom.create('input');
-  fld.type = (fldOptions.type)? (fldOptions.type): 'text';
-  fld.id = fldName;
-  fld.name = fld.id;
-  if (fldOptions.value)
-  {
-    fld.value = fldOptions.value;
-    fld.defaultValue = fld.value;
-  }
-  if (fldOptions.className)
-    fld.className = fldOptions.className;
-  if (fldOptions.onblur)
-    fld.onblur = fldOptions.onblur;
-  fld.style.width = '95%';
-  if (fldOptions.cssText)
-    fld.style.cssText = fldOptions.cssText;
-
-  td.appendChild(fld);
-  return fld;
-}
-
-TBL.createCell1 = function (td, prefix, fldName, No, fldOptions)
-{
-  var fld = TBL.createCell0 (td, prefix, fldName, No, fldOptions)
-  td.appendChild(OAT.Dom.text(' '));
-  var img = OAT.Dom.image('image/select.gif');
-  img.className = "pointer";
-  img.onclick = function (){windowShow('users_select.vspx?dst=m&params='+fldName+':s1;',520)};
-
-  td.appendChild(img);
-  return fld;
-}
-
-TBL.createCell2 = function (td, prefix, fldName, No, fldOptions)
-{
-  var fld = new OAT.Combolist([], fldOptions.value, {name: fldName});
-  fld.input.name = fldName;
-  fld.input.id = fldName;
-  fld.input.style.width = "80%";
-  fld.addOption('rdfs:seeAlso');
-  fld.addOption('foaf:made');
-  fld.addOption('foaf:maker');
-
-  td.appendChild(fld.div);
-  return fld;
-}
-
-TBL.createButton = function (td, prefix, fldName, No, fldOptions)
-{
-  fldName = prefix + '_' + fldName + '_' + No;
-  if (fldOptions.tdCssText)
-    td.style.cssText = fldOptions.tdCssText;
-  var fn = TBL["createButton"+fldOptions.mode];
-  if (fn)
-  {
-	  var btn = fn(td, prefix, fldName, No, fldOptions);
-    if (btn)
-    { if (fldOptions.cssText)
-        btn.style.cssText = fldOptions.cssText;
-      if (fldOptions.className)
-        btn.className = fldOptions.className;
-    }
-  }
-}
-
-TBL.createButton1 = function (td, prefix, fldName, No, fldOptions)
-{
-  var fld = OAT.Dom.create("input");
-  fld.id = fldName;
-  fld.type = 'button';
-  fld.value = 'Remove';
-  fld.onclick = function(){TBL.createRow(prefix, No);};
-
-  td.appendChild(fld);
-  return fld;
-}
-
-TBL.createButton2 = function (td, prefix, fldName, No, fldOptions)
-{
-  var fld = OAT.Dom.image('image/del_16.png');
-  fld.onclick = function(){TBL.createRow(prefix, No);};
-  OAT.Dom.addClass(fld, 'pointer');
-
-  td.appendChild(fld);
-  return fld;
-}
