@@ -1,5 +1,5 @@
 --
---  $Id: calendar-a-api.sql,v 1.10.2.3 2010/09/20 10:15:02 source Exp $
+--  $Id: calendar-a-api.sql,v 1.10.2.6 2011/06/09 17:43:16 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -94,6 +94,9 @@ create procedure ODS.ODS_API."calendar.get" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   ods_describe_iri (SIOC..calendar_event_iri (inst_id, event_id));
   return '';
 }
@@ -134,6 +137,9 @@ create procedure ODS.ODS_API."calendar.event.new" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   cTimezone := CAL.WA.settings_usedTimeZone (inst_id);
   eventStart := CAL.WA.event_user2gmt (eventStart, cTimezone);
   eventEnd := CAL.WA.event_user2gmt (eventEnd, cTimezone);
@@ -200,6 +206,9 @@ create procedure ODS.ODS_API."calendar.event.edit" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   cTimezone := CAL.WA.settings_usedTimeZone (inst_id);
   eventStart := CAL.WA.event_user2gmt (eventStart, cTimezone);
   eventEnd := CAL.WA.event_user2gmt (eventEnd, cTimezone);
@@ -260,6 +269,9 @@ create procedure ODS.ODS_API."calendar.task.new" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   cTimezone := CAL.WA.settings_usedTimeZone (inst_id);
   eventStart := CAL.WA.event_user2gmt (CAL.WA.dt_join (CAL.WA.dt_dateClear (eventStart), CAL.WA.dt_timeEncode (12, 0)), cTimezone);
   eventEnd := CAL.WA.event_user2gmt (CAL.WA.dt_join (CAL.WA.dt_dateClear (eventEnd), CAL.WA.dt_timeEncode (12, 0)), cTimezone);
@@ -318,6 +330,9 @@ create procedure ODS.ODS_API."calendar.task.edit" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   cTimezone := CAL.WA.settings_usedTimeZone (inst_id);
   eventStart := CAL.WA.event_user2gmt (CAL.WA.dt_join (CAL.WA.dt_dateClear (eventStart), CAL.WA.dt_timeEncode (12, 0)), cTimezone);
   eventEnd := CAL.WA.event_user2gmt (CAL.WA.dt_join (CAL.WA.dt_dateClear (eventEnd), CAL.WA.dt_timeEncode (12, 0)), cTimezone);
@@ -395,6 +410,9 @@ create procedure ODS.ODS_API."calendar.import" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   if (isnull (userName))
   {
     userName := uname;
@@ -457,6 +475,9 @@ create procedure ODS.ODS_API."calendar.export" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   http (CAL.WA.export_vcal (inst_id, null, vector ('events', events, 'tasks', tasks, 'periodFrom', periodFrom, 'periodTo', periodTo, 'tagsInclude', tagsInclude, 'tagsExclude', tagsExclude)));
 
   return '';
@@ -712,12 +733,29 @@ create procedure ODS.ODS_API."calendar.publication.new" (
 
   declare rc integer;
   declare uname varchar;
-  declare _type, options any;
+  declare _type, _name, _permissions, options any;
 
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   _type := ODS.ODS_API.calendar_type_check (destinationType, destination);
+  _name := trim (destination);
+  if (_type = 1)
+  {
+    _name := '/' || _name;
+    _name := replace (_name, '//', '/');
+  }
+  _name := ODS..dav_path_normalize(_name);
+  if (_type = 1)
+  {
+    _name := CAL.WA.dav_parent (_name);
+    _permissions := '11_';
+    if (not CAL.WA.dav_check_authenticate (_name, userName, userPassword, _permissions))
+      signal ('TEST', 'The user has no rights for this folder.<>');
+  }
   options := vector ('type', _type, 'name', destination, 'user', userName, 'password', userPassword, 'events', events, 'tasks', tasks);
   insert into CAL.WA.EXCHANGE (EX_DOMAIN_ID, EX_TYPE, EX_NAME, EX_UPDATE_TYPE, EX_UPDATE_PERIOD, EX_UPDATE_FREQ, EX_OPTIONS)
     values (inst_id, 0, name, updateType, updatePeriod, updateFreq, serialize (options));
@@ -810,7 +848,7 @@ create procedure ODS.ODS_API."calendar.publication.edit" (
   declare rc integer;
   declare uname varchar;
   declare inst_id integer;
-  declare _type, options any;
+  declare _type, _name, _permissions, options any;
 
   inst_id := (select EX_DOMAIN_ID from CAL.WA.EXCHANGE where EX_ID = publication_id);
   if (not ods_check_auth (uname, inst_id, 'author'))
@@ -820,6 +858,20 @@ create procedure ODS.ODS_API."calendar.publication.edit" (
     return ods_serialize_sql_error ('37000', 'The item is not found');
 
   _type := ODS.ODS_API.calendar_type_check (destinationType, destination);
+  _name := trim (destination);
+  if (_type = 1)
+  {
+    _name := '/' || _name;
+    _name := replace (_name, '//', '/');
+  }
+  _name := ODS..dav_path_normalize(_name);
+  if (_type = 1)
+  {
+    _name := CAL.WA.dav_parent (_name);
+    _permissions := '11_';
+    if (not CAL.WA.dav_check_authenticate (_name, userName, userPassword, _permissions))
+      signal ('TEST', 'The user has no rights for this folder.<>');
+  }
   options := vector ('type', _type, 'name', destination, 'user', userName, 'password', userPassword, 'events', events, 'tasks', tasks);
   update CAL.WA.EXCHANGE
      set EX_NAME = name,
@@ -915,12 +967,29 @@ create procedure ODS.ODS_API."calendar.subscription.new" (
 
   declare rc integer;
   declare uname varchar;
-  declare _type, options any;
+  declare _type, _name, _permissions, options any;
 
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   _type := ODS.ODS_API.calendar_type_check (sourceType, source);
+  _name := trim (source);
+  if (_type = 1)
+  {
+    _name := '/' || _name;
+    _name := replace (_name, '//', '/');
+  }
+  _name := ODS..dav_path_normalize(_name);
+  if (_type = 1)
+  {
+    _name := CAL.WA.dav_parent (_name);
+    _permissions := '1__';
+    if (not CAL.WA.dav_check_authenticate (_name, userName, userPassword, _permissions))
+      signal ('TEST', 'The user has no rights for this folder.<>');
+  }
   options := vector ('type', _type, 'name', source, 'user', userName, 'password', userPassword, 'events', events, 'tasks', tasks);
   insert into CAL.WA.EXCHANGE (EX_DOMAIN_ID, EX_TYPE, EX_NAME, EX_UPDATE_TYPE, EX_UPDATE_PERIOD, EX_UPDATE_FREQ, EX_OPTIONS)
     values (inst_id, 1, name, updateType, updatePeriod, updateFreq, serialize (options));
@@ -1013,7 +1082,7 @@ create procedure ODS.ODS_API."calendar.subscription.edit" (
   declare rc integer;
   declare uname varchar;
   declare inst_id integer;
-  declare _type, options any;
+  declare _type, _name, _permissions, options any;
 
   inst_id := (select EX_DOMAIN_ID from CAL.WA.EXCHANGE where EX_ID = subscription_id);
   if (not ods_check_auth (uname, inst_id, 'author'))
@@ -1023,6 +1092,20 @@ create procedure ODS.ODS_API."calendar.subscription.edit" (
     return ods_serialize_sql_error ('37000', 'The item is not found');
 
   _type := ODS.ODS_API.calendar_type_check (sourceType, source);
+  _name := trim (source);
+  if (_type = 1)
+  {
+    _name := '/' || _name;
+    _name := replace (_name, '//', '/');
+  }
+  _name := ODS..dav_path_normalize(_name);
+  if (_type = 1)
+  {
+    _name := CAL.WA.dav_parent (_name);
+    _permissions := '1__';
+    if (not CAL.WA.dav_check_authenticate (_name, userName, userPassword, _permissions))
+      signal ('TEST', 'The user has no rights for this folder.<>');
+  }
   options := vector ('type', _type, 'name', source, 'user', userName, 'password', userPassword, 'events', events, 'tasks', tasks);
   update CAL.WA.EXCHANGE
      set EX_NAME = name,
@@ -1119,6 +1202,9 @@ create procedure ODS.ODS_API."calendar.upstream.new" (
   if (not ods_check_auth (uname, inst_id, 'author'))
     return ods_auth_failed ();
 
+  if (not exists (select 1 from DB.DBA.WA_INSTANCE where WAI_ID = inst_id and WAI_TYPE_NAME = 'Calendar'))
+    return ods_serialize_sql_error ('37000', 'The instance is not found');
+
   CAL.WA.test (name, vector('name', 'Upstream Name', 'class', 'varchar', 'minLength', 1, 'maxLength', 255));
   CAL.WA.test (source, vector('name', 'Upstream URI', 'class', 'varchar', 'minLength', 1, 'maxLength', 255));
   CAL.WA.test (userName, vector('name', 'Upstream User', 'class', 'varchar', 'minLength', 1, 'maxLength', 255));
@@ -1263,6 +1349,7 @@ create procedure ODS.ODS_API."calendar.options.set" (
   ODS.ODS_API.calendar_setting_set (settings, optionsParams, 'dateFormat');
   ODS.ODS_API.calendar_setting_set (settings, optionsParams, 'timeZone');
   ODS.ODS_API.calendar_setting_set (settings, optionsParams, 'showTasks');
+  ODS.ODS_API.calendar_setting_set (settings, optionsParams, 'daylichtEnable');
 	if (CAL.WA.discussion_check ())
 	{
   ODS.ODS_API.calendar_setting_set (settings, optionsParams, 'conv');
@@ -1333,6 +1420,7 @@ create procedure ODS.ODS_API."calendar.options.get" (
   http (ODS.ODS_API.calendar_setting_xml (settings, 'dateFormat'));
   http (ODS.ODS_API.calendar_setting_xml (settings, 'timeZone'));
   http (ODS.ODS_API.calendar_setting_xml (settings, 'showTasks'));
+  http (ODS.ODS_API.calendar_setting_xml (settings, 'daylichEnable'));
 
   http (ODS.ODS_API.calendar_setting_xml (settings, 'conv'));
   http (ODS.ODS_API.calendar_setting_xml (settings, 'conv_init'));
