@@ -1,5 +1,5 @@
 /*
- *  $Id: calendar.js,v 1.1.2.1 2010/01/14 16:04:14 source Exp $
+ *  $Id: calendar.js,v 1.1.2.3 2010/09/20 10:15:38 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -31,7 +31,7 @@ var GCallendar = {
 		  GCallendar.setCurrent(ctrlId);
 	    var _ctrl=GCallendar.current_control;
       var _ctrlLeftTop=OAT.Dom.position(_ctrl);
-      if (OAT.Dom.isIE() == false)
+      if (OAT.Browser.isIE == false)
         GCallendar.obj.show(_ctrlLeftTop[0]+25, _ctrlLeftTop[1]+0, GCallendar.onClick);
       else
         GCallendar.obj.show(_ctrlLeftTop[0]+25, _ctrlLeftTop[1]+0, GCallendar.onClick);
