@@ -28,7 +28,7 @@ export LANG LC_ALL
 
 MODE=$1
 LOGDIR=`pwd`
-VERSION="1.1.25"
+VERSION="1.1.37"
 LOGFILE="${LOGDIR}/vad_make.log"
 STICKER_DAV="vad_dav.xml"
 STICKER_FS="vad_filesystem.xml"
@@ -224,7 +224,7 @@ sticker_init() {
   echo "  <name package=\"Polls\">" >> $STICKER
   echo "    <prop name=\"Title\" value=\"ODS Polls Manager\"/>" >> $STICKER
   echo "    <prop name=\"Developer\" value=\"OpenLink Software\"/>" >> $STICKER
-  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2010 OpenLink Software\"/>" >> $STICKER
+  echo "    <prop name=\"Copyright\" value=\"(C) 1998-2011 OpenLink Software\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.com/virtuoso\"/>" >> $STICKER
   echo "    <prop name=\"Download\" value=\"http://www.openlinksw.co.uk/virtuoso\"/>" >> $STICKER
   echo "  </name>" >> $STICKER
@@ -236,8 +236,8 @@ sticker_init() {
   echo "<dependencies>" >> $STICKER
   echo "  <require>" >> $STICKER
   echo "    <name package=\"Framework\"/>" >> $STICKER
-  echo "    <versions_later package=\"1.65.02\">" >> $STICKER
-  echo "      <prop name=\"Date\" value=\"2010-01-28 12:00\" />" >> $STICKER
+  echo "    <versions_later package=\"1.82.42\">" >> $STICKER
+  echo "      <prop name=\"Date\" value=\"2011-06-15 12:00\" />" >> $STICKER
   echo "      <prop name=\"Comment\" value=\"An incompatible version of the ODS Framework\" />" >> $STICKER
   echo "    </versions_later>" >> $STICKER
   echo "  </require>" >> $STICKER
@@ -379,8 +379,8 @@ then
 fi
 virtuoso_shutdown
 STOP_SERVER
-chmod 644 $VAD_DAV
-chmod 644 $VAD_FS
+chmod 644 ods_polls_dav.vad
+chmod 644 ods_polls_filesystem.vad
 directory_clean
 
 CHECK_LOG
