@@ -1,5 +1,5 @@
 --
---  $Id: polls-d.sql,v 1.6.2.1 2009/08/12 11:51:57 source Exp $
+--  $Id: polls-d.sql,v 1.6.2.4 2011/05/18 21:18:29 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -57,6 +57,16 @@ POLLS.WA.exec_no_error ('DROP type wa_polls');
 -- Views
 POLLS.WA.exec_no_error ('DROP view POLLS..TAGS_VIEW');
 
+-- Registry
+registry_remove ('_polls_path_');
+registry_remove ('_polls_version_');
+registry_remove ('_polls_build_');
+registry_remove ('polls_settings_update');
+registry_remove ('polls_index_version');
+registry_remove ('polls_path_upgrade2');
+registry_remove ('__ods_polls_sioc_init');
+registry_remove ('polls_services_update');
+
 -- Procedures
 create procedure POLLS.WA.drop_procedures()
 {
