@@ -1,5 +1,5 @@
 --  
---  $Id: postinstall.sql,v 1.15.2.1 2010/09/20 10:15:46 source Exp $
+--  $Id: postinstall.sql,v 1.15.2.2 2011/05/18 21:17:43 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -65,4 +65,18 @@ create procedure WV.WIKI.temp ()
 ;
 
 WV.WIKI.temp ();
+
+create procedure WV.WIKI.temp ()
+{
+  if (registry_get ('wiki_services_update') = '1')
+    return;
+
+  SIOC..fill_ods_wiki_services();
+  registry_set ('wiki_services_update', '1');
+}
+;
+
+WV.WIKI.temp ();
+
+
 drop procedure WV.WIKI.temp;
