@@ -1,5 +1,5 @@
 --
---  $Id: proc_on_api.sql,v 1.1.2.4 2009/12/08 23:25:22 source Exp $
+--  $Id: proc_on_api.sql,v 1.1.2.6 2011/03/26 14:21:51 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -697,6 +697,10 @@ create procedure ODS.ODS_API."wiki.options.set" (
   {
     WV.WIKI.SETCLUSTERPARAM (clusterName, 'new-topic-template', "value");
   }
+  else if (name = 'skinSource')
+  {
+    WV.WIKI.SETCLUSTERPARAM (clusterName, 'skin-source', "value");
+  }
   else if (name = 'primarySkin')
   {
     WV.WIKI.SETCLUSTERPARAM (clusterName, 'skin', "value");
@@ -793,6 +797,10 @@ create procedure ODS.ODS_API."wiki.options.get" (
   {
     "value" := WV.WIKI.CLUSTERPARAM (clusterName, 'index-page', 'WelcomeVisitors');
   }
+  else if (name = 'skinSource')
+  {
+    "value" := WV.WIKI.CLUSTERPARAM (clusterName, 'skin-source', 'Local');
+  }
   else if (name = 'primarySkin')
   {
     "value" := WV.WIKI.CLUSTERPARAM (clusterName, 'skin', 'default');
