@@ -19,7 +19,7 @@
 //  
 //  
 //
-// $Id: Marshaler.cs,v 1.5.2.2 2009/06/17 11:12:53 source Exp $
+// $Id: Marshaler.cs,v 1.5.2.6 2011/10/17 15:51:19 source Exp $
 //
 
 //#define IGNORE_ENCODING
@@ -141,6 +141,11 @@ namespace OpenLink.Data.Virtuoso
 				MarshalInt (stream, (int) value);
 				return;
 
+			case TypeCode.Int64:
+			        stream.WriteByte ((byte) BoxTag.DV_INT64);
+				MarshalLongInt64 (stream, (Int64) value);
+				return;
+
 			case TypeCode.Single:
 				stream.WriteByte ((byte) BoxTag.DV_SINGLE_FLOAT);
 				MarshalSingle (stream, (float) value);
@@ -190,6 +195,20 @@ namespace OpenLink.Data.Virtuoso
 				{
 					DateTimeMarshaler.MarshalDate (stream, value, DateTimeType.DT_TYPE_TIME);
 				}
+				else if (type == typeof (VirtuosoTimeSpan))
+				{
+					DateTimeMarshaler.MarshalDate (stream, value, DateTimeType.DT_TYPE_TIME);
+				}
+				else if (type == typeof(VirtuosoDateTime))
+				{
+					DateTimeMarshaler.MarshalDate (stream, value, DateTimeType.DT_TYPE_DATETIME);
+				}
+#if ADONET3
+				else if (type == typeof(VirtuosoDateTimeOffset))
+				{
+					DateTimeMarshaler.MarshalDate (stream, value, DateTimeType.DT_TYPE_DATETIME);
+				}
+#endif
 				else if (type == typeof (int[]))
 				{
 					stream.WriteByte ((byte) BoxTag.DV_ARRAY_OF_LONG);
@@ -363,7 +382,10 @@ namespace OpenLink.Data.Virtuoso
 #if IGNORE_ENCODING
 				return Encoding.GetEncoding("iso-8859-1").GetString (bytes);
 #else
-				return Decode (table, bytes);
+				if (connection.charset_utf8)
+                                  return Encoding.UTF8.GetString (bytes);
+				else
+				  return Decode (table, bytes);
 #endif
 			}
 
@@ -421,7 +443,10 @@ namespace OpenLink.Data.Virtuoso
 #if IGNORE_ENCODING
 				return Encoding.GetEncoding("iso-8859-1").GetString (bytes);
 #else
-				return Decode (table, bytes);
+				if (connection.charset_utf8)
+                                  return Encoding.UTF8.GetString (bytes);
+				else
+				  return Decode (table, bytes);
 #endif
 			}
 
@@ -544,7 +569,7 @@ namespace OpenLink.Data.Virtuoso
 			stream.WriteByte ((byte) value);
 		}
 
-		internal static void MarshalLongInt64 (Stream stream, int value)
+		internal static void MarshalLongInt64 (Stream stream, Int64 value)
 		{
 			stream.WriteByte ((byte) (value >> 56));
 			stream.WriteByte ((byte) (value >> 48));
