@@ -1,5 +1,5 @@
 //  
-// $Id: TcpConnection.cs,v 1.2.2.2 2009/06/15 13:24:12 source Exp $
+// $Id: TcpConnection.cs,v 1.2.2.4 2011/10/13 18:26:27 source Exp $
 //
 //  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 //  project.
@@ -21,6 +21,7 @@
 //  
 //  
 //
+// $Id: TcpConnection.cs,v 1.2.2.4 2011/10/13 18:26:27 source Exp $
 //
 
 using System;
@@ -66,6 +67,8 @@ namespace OpenLink.Data.Virtuoso
 			int hostIndex = 0;
 			int startIndex = 0;
 
+	                charset_utf8 = (options.Charset != null && options.Charset.ToUpper() == "UTF-8");
+
 			ArrayList ds_list = options.DataSourceList;
 			if (ds_list != null)
 			{
@@ -107,6 +110,7 @@ namespace OpenLink.Data.Virtuoso
 			try
 			{
 				session = new TcpSession (this, socket);
+                		socket.NoDelay = true;
 
 #if MONO
 				Future future = new Future (Service.CallerId, new object[] { null });
