@@ -19,7 +19,7 @@
 //  
 //  
 //
-// $Id: VirtuosoCommand.cs,v 1.7.2.2 2011/02/21 13:36:39 source Exp $
+// $Id: VirtuosoCommand.cs,v 1.7.2.4 2011/10/17 09:48:25 source Exp $
 //
 
 using System;
@@ -459,17 +459,19 @@ namespace OpenLink.Data.Virtuoso
 
 			bool schemaOnly = SchemaOnlyDataReader (behavior);
 			string text = GetCommandText ();
+		        bool isSparql = text.TrimStart(null).StartsWith("sparql", StringComparison.OrdinalIgnoreCase);
 			if (schemaOnly)
 			{
 				if (!isPrepared)
 				{
-					text = replaceNamedParams (text);
+                    			if (!isSparql)
+						text = replaceNamedParams (text);
 					innerCommand.Prepare (text);
 				}
 			}
 			else
 			{
-				if (parameters != null && parameters.Count > 0)
+				if (!isSparql && parameters != null && parameters.Count > 0)
 				{
 					VirtuosoParameterCollection _parameters = handleNamedParams (text, parameters);
 					Debug.Assert (_parameters.Count == parameters.Count, "Count mismatch in reordered parameter array");
@@ -560,6 +562,8 @@ namespace OpenLink.Data.Virtuoso
 
 		protected override void Dispose (bool disposing)
 		{
+			try
+			{
 			if (disposing)
 			{
 				DisposeStatement ();
@@ -571,6 +575,12 @@ namespace OpenLink.Data.Virtuoso
 			connection = null;
 			transaction = null;
 		}
+			catch (Exception e)
+			{
+				Debug.WriteLineIf(CLI.FnTrace.Enabled,
+					"VirtuosoCommand.Dispose caught exception: " + e.Message);
+			}
+		}
 
 		internal void OnConnectionClose ()
 		{
