@@ -1,5 +1,5 @@
 --
---  $Id: search.sql,v 1.1.2.4 2010/02/03 15:05:03 source Exp $
+--  $Id: search.sql,v 1.1.2.11 2011/09/23 19:22:36 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -35,11 +35,14 @@ create procedure label_get(in smode varchar)
   else if (smode='10') label := 'Cloud Around Person With Filtered Out Blank Nodes';
   else if (smode='11') label := 'Product Count By Manufacturer';
   else if (smode='12') label := 'Vendors with offers';
-  else if (smode='13') label := 'Places of worship around Paris with Cafe''s in close proximity';
+  else if (smode='13') label := 'Objects around Paris with Cafe''s in close proximity';
   else if (smode='14') label := 'Motorways across England & Scotland';
-  else if (smode='15') label := 'Places of worship around London with Cities in close proximity';
+  else if (smode='15') label := 'Objects around London with Cities in close proximity';
   else if (smode='16') label := 'Places with coordinates';
-  else if (smode='17') label := 'Subcategories of Protestant Churches';
+  else if (smode='17') label := 'Subcategories of History of Wisconsin';
+  else if (smode='18') label := 'Things within close proximity of New York City';
+  else if (smode='19') label := 'Distance between New York City and London, England';
+  else if (smode='20') label := 'All Educational Institutions within 10km of Oxford, UK';
   else if (smode='100') label := 'Concept Cloud';
   else if (smode='101') label := 'Social Net';
   else if (smode='102') label := 'Graphs in Social Net';
@@ -57,34 +60,6 @@ create procedure label_get(in smode varchar)
 }
 ;
 
-create procedure input_getcustom (in num varchar)
-{
-  if (num = 2)
-    return 'City Latitude';
-  else if (num = 3)
-    return 'Cafe Proximity to Place of Worship';
-  else if (num = 4)
-    return 'City Proximity to Place of Worship';
-  if (num = 5)
-    return 'Class URI';
-  if (num = 6)
-    return 'City Proximity to Place of Worship';
-  if (num = 7)
-    return 'Geometry Longitude';
-  if (num = 8)
-    return 'Person URI';
-  if (num = 9)
-    return 'Person URI';
-  if (num = 10)
-    return 'First Lang';
-  if (num = 11)
-    return 'Second Lang';
-  if (num = 12)
-    return 'Broader';
-  else return '';
-}
-;
-
 
 create procedure input_get (in num varchar)
 {
@@ -93,21 +68,24 @@ create procedure input_get (in num varchar)
   	'Search for',
 	'Search for',
 	'Search for',
-	'Interest',
+	'Name',
 	'Search for',
 	'Person URI',
 	'Property',
 	'Nickname',
-	'Nickname',
+	'Text pattern',
 	'Nickname',
 	'Number of items',
         'Price value',
 --	'Manufacturer',
         'City Longitude',
         'Latitude',
-        'Place of Worship URI',
+        'Entity URI',
         'Geometry Latitude',
-        'Narrower'
+        'Broader',
+        'Place URI',
+        'First City URI',
+        'City URI'
 	);
   t2 := vector (
   	'',
@@ -116,7 +94,7 @@ create procedure input_get (in num varchar)
 	''
 	);
   num := atoi (num) - 1;
-  if (num > -1 and num < 17)
+  if (num > -1 and num < 20)
     return t1[num];
   else if (num > 98 and num < 102)
     return t2[num - 99];
@@ -124,6 +102,46 @@ create procedure input_get (in num varchar)
 }
 ;
 
+create procedure input_getcustom (in num varchar)
+{
+  if (num = 2)
+    return 'City Latitude';
+  else if (num = 3)
+    return 'Cafe Proximity to Entity URI';
+  else if (num = 4)
+    return 'Distance';
+  if (num = 5)
+    return 'isA';
+  if (num = 6)
+    return 'Distance';
+  if (num = 7)
+    return 'Geometry Longitude';
+  if (num = 8)
+    return 'Person URI';
+  if (num = 9)
+    return 'Person URI';
+  if (num = 10)
+    return 'First Language';
+  if (num = 11)
+    return 'Second Language';
+  if (num = 12)
+    return 'Narrower';
+  if (num = 13)
+    return 'Thing(s) Proximity (km)';
+  if (num = 14)
+    return 'Language';
+  if (num = 15)
+    return 'Second City URI';
+  if (num = 16)
+    return 'Institution Proximity to the Place';
+  if (num = 17)
+    return 'Ontology URI';
+  if (num = 18)
+    return 'Language';
+  else return '';
+}
+;
+
 
 create procedure desc_get (in num varchar)
 {
@@ -132,7 +150,7 @@ create procedure desc_get (in num varchar)
   	'Show triples containing a text pattern. The bif:search_excerpt is used to format a short excerpt of the matching literal in search-engine style',
 	'What sources talk the most about a given subject? Show the top N graphs containing triples with the given text pattern. Sort by descending triple count.',
 	'What types of objects contain a text pattern. Find matches, get the type. Group by type, order by count.',
-	'What else are people interested in X interested in? What else do Harry Potter fans like?',
+	'What are the interests of a given person based on their foaf:name?',
 	'Who writes the most about a topic. Show for each author the number of works mentioning the topic and total number of works.'
 ||'<br>For all documents and posts we have extracted named entities the entity could shows the entities which occur in the works of each author.'
 ||'There are statistics about named entities occurring together, these are used for display a list of related entities. '
@@ -147,9 +165,12 @@ create procedure desc_get (in num varchar)
 --        'Show products count for given manufacturer.',
         'Show places of worship, within certain km of Paris, that have cafes in close proximity.',
         'Show motorways across England & Scotland from DBpedia.',
-        'Shows cities within cerain proximity of London.',
+        'Shows cities within certain proximity of London.',
         'Shows geometries with their coordinates.',
-        'Find entities that are subcategories of Protestant Churches, no deeper than 3 levels within the concept scheme hierarchy filtered by a specific subcategory.'
+        'Find entities that are subcategories of History of Wisconsin, no deeper than 3 levels within the concept scheme hierarchy filtered by a specific subcategory.',
+        'Shows things within certain proximity of a place.',
+        'Shows distance between 2 cities.',
+        'Find all Educational Institutions within 10km of Oxford, UK ordered by date of establishment.'
 	);
   t2 := vector (
   	'',
@@ -158,7 +179,7 @@ create procedure desc_get (in num varchar)
 	''
 	);
   num := atoi (num) - 1;
-  if (num > -1 and num < 17)
+  if (num > -1 and num < 20)
     return t1[num];
   else if (num > 98 and num < 102)
     return t2[num - 99];
@@ -179,15 +200,19 @@ create procedure head_get (in num varchar)
     vector ('Person URI', 'Graph', 'Step No.', 'Path'),
     vector ('Person', 'Nick name', 'Shared Interests', 'Total Interests'),
     vector ('Thing', 'Nick name', 'Occurrences'),
-    vector ('Thing', 'Nick name', 'Occurrences'),
+    vector ('Thing', 'Text Pattern', 'Occurrences'),
     vector ('Manifacturer URI', 'Total Products'),
-    vector ('Vendor', 'Offer', 'Business Function', 'Customer Type', 'Offer Object', 'Type of Good', 'Price'),
+    vector ('Vendor', 'Offer', 'Business Function', 'Offer Object', 'Type of Good', 'Price'),    
+--    vector ('Vendor', 'Offer', 'Business Function', 'Customer Type', 'Offer Object', 'Type of Good', 'Price'),
 --    vector ('Total Products'),
     vector ('Cafe URI', 'Latitude', 'Longitude', 'Cafe Name', 'Church Name', 'Count'),
-    vector ('Road', 'Service', 'Latitude', 'Longitude'),
-    vector ('City URI', 'Count'),
+    vector ('Road', 'Latitude', 'Longitude'),
+    vector ('City URI', 'Distance (km)'),
     vector ('Geometry URI', 'Latitude', 'Longitude'),
-    vector ('SKOS Broader', 'SKOS Narrower', 'SKOS Level', 'Entity URI', 'Entity Name', 'Geo Point')
+    vector ('SKOS Broader', 'SKOS Narrower', 'SKOS Level', 'Entity URI', 'Entity Name', 'Geo Point'),
+    vector ('Resource URI', 'Name', 'Location'),
+    vector ('First City Name', 'First City Location', 'Second City Name', 'Second City Location', 'Distance'),
+    vector ('Institution URI', 'Name', 'Established' ,'Latitude', 'Longitude')
   );
   t2 := vector (
     vector (),
@@ -197,7 +222,7 @@ create procedure head_get (in num varchar)
   );
   t3 := vector ();
   num := atoi (num) - 1;
-  if (num > -1 and num < 17)
+  if (num > -1 and num < 20)
     return t1[num];
   else if (num > 98 and num < 102)
     return t2[num - 99];
@@ -619,7 +644,7 @@ create procedure pick_query(in smode varchar, inout val any, inout query varchar
 --    option (transitive, t_distinct, t_in(?s), t_out(?o), t_min (1), t_max (4), t_step ('step_no') as ?dist) .
 --    filter (?s= <http://myopenlink.net/dataspace/person/kidehen#this>)
 --  } order by ?dist desc 3 limit 50;
-    if (isnull(val)  or val = '') val := 'http://myopenlink.net/dataspace/person/kidehen#this';
+    if (isnull(val)  or val = '') val := 'http://www.w3.org/People/Berners-Lee/card#i';
     s1 := 'sparql SELECT ?o ?dist ( ( SELECT COUNT (*) WHERE {?o foaf:knows ?xx } ) ) WHERE  { { SELECT ?s ?o  WHERE { ?s foaf:knows ?o } } OPTION (transitive, t_distinct, t_in(?s), t_out(?o), t_min (1), t_max (4), t_step (''step_no'') as ?dist ) . FILTER (?s= <';
     validate_input(val);
     s2 := val;
@@ -647,8 +672,8 @@ create procedure pick_query(in smode varchar, inout val any, inout query varchar
 --  } limit 20;
 
     if (isnull(val)  or val = '') val := 'foaf:knows';
-    if (isnull(val2)  or val2 = '') val2 := 'http://myopenlink.net/dataspace/person/kidehen#this';
-    if (isnull(val3)  or val3 = '') val3 := 'http://www.advogato.org/person/mparaz/foaf.rdf#me';
+    if (isnull(val2)  or val2 = '') val2 := 'http://www.w3.org/People/Berners-Lee/card#i';
+    if (isnull(val3)  or val3 = '') val3 := 'http://myopenlink.net/dataspace/person/kidehen#this';
 -- old variant with foaf:knows
 --  s1 := 'sparql SELECT ?link ?g ?step ?path WHERE { { SELECT ?s ?o ?g WHERE { graph ?g {?s foaf:knows ?o } } } OPTION (transitive, t_distinct, t_in(?s), t_out(?o), t_no_cycles, T_shortest_only, t_step (?s) as ?link, t_step (''path_id'') as ?path, t_step (''step_no'') as ?step, t_direction 3) . FILTER (?s= <';
     s1 := 'sparql SELECT ?link ?g ?step ?path WHERE { { SELECT ?s ?o ?g WHERE { graph ?g {?s ';
@@ -714,7 +739,7 @@ s3 := '\')) .
     }
   else if (smode = '9')
     {
-      if (isnull(val)  or val = '') val := '"SQL"';
+      if (isnull(val)  or val = '') val := '"data"';
 
 -- b3s variant:
 
@@ -734,7 +759,7 @@ s3 := '\')) .
     }
   else if (smode = '10')
     {
-      if (isnull(val)  or val = '') val := '"SQL"';
+      if (isnull(val)  or val = '') val := '"Wellington"';
 
       s1 :=
       'sparql define input:inference \'virtrdf-label\' SELECT ?s ?lbl COUNT(*) WHERE { ?s  ?p2 ?o2 .  ?o2 <http://www.w3.org/2000/01/rdf-schema#label> ?lbl . ' ||
@@ -760,13 +785,14 @@ s3 := '\')) .
     {
       if (isnull(val)  or val = '') val := '500';
 
+--' ?ab gr:eligibleCustomerTypes ?el . ' ||
+
       s1 :=
-      'sparql SELECT ?xx ?ab ?bp ?el ?b ?c ?cv WHERE {' ||
+      'sparql SELECT ?xx ?ab ?bp ?b ?c bif:ceiling(?cv*100)/100.00  WHERE {' ||
       ' ?xx a gr:BusinessEntity . ' ||
       ' ?xx gr:offers ?ab . ' ||
       ' ?ab rdf:type gr:Offering . ' ||
       ' ?ab gr:hasBusinessFunction ?bp . ' ||
-      ' ?ab gr:eligibleCustomerTypes ?el . ' ||
       ' ?ab gr:includesObject ?b . ' ||
       ' ?b rdf:type gr:TypeAndQuantityNode . ' ||
       ' ?b gr:typeOfGood ?c . ' ||
@@ -776,7 +802,7 @@ s3 := '\')) .
       ' FILTER (?cv > ';
       validate_input(val);
       s2 := val;
-      s3 := ') } LIMIT 50';
+      s3 := ') } LIMIT 10';
       query := s1 || s2 || s3;
     }
 -- for now is removed from the list
@@ -797,7 +823,7 @@ s3 := '\')) .
     if (isnull(val2)  or val2 = '') val2 := '48.853';
 
     if (isnull(val3)  or val3 = '') val3 := '5';
-    if (isnull(val4)  or val4 = '') val4 := '0.2';
+    if (isnull(val4)  or val4 = '') val4 := '0.9';
 
 
     s1 := 'sparql SELECT DISTINCT ?cafe ?lat ?long ?cafename ?churchname ' ||
@@ -828,17 +854,17 @@ s3 := '\')) .
       if (isnull(val)  or val = '') val := '52.000';
 
       s1 :=
-      'sparql SELECT ?road ?services ?lat ?long WHERE ' ||
+      'sparql SELECT ?road ?lat ?long WHERE ' ||
       '  { ' ||
       '    { ' ||
-      '     ?services dbpprop:road ?road . ' ||
+--      '     ?services dbpprop:road ?road . ' ||
       '      ?road a yago:MotorwaysInEngland . ' ||
       '      ?services dbpprop:lat ?lat . ' ||
       '      ?services dbpprop:long ?long . ' ||
       '    } ' ||
       '    UNION ' ||
       '    { ' ||
-      '      ?services dbpprop:road ?road . ' ||
+--      '      ?services dbpprop:road ?road . ' ||
       '      ?road a yago:MotorwaysInScotland . ' ||
       '      ?services dbpprop:lat ?lat . ' ||
       '      ?services dbpprop:long ?long . ' ||
@@ -852,7 +878,7 @@ s3 := '\')) .
   else if (smode='15')
   {
     if (isnull(val)  or val = '') val := 'http://dbpedia.org/resource/London';
-    if (isnull(val2)  or val2 = '') val2 := 'http://dbpedia.org/ontology/City';
+    if (isnull(val2)  or val2 = '') val2 := 'http://dbpedia.org/ontology/PopulatedPlace';
     if (isnull(val3)  or val3 = '') val3 := '10';
     s1 := 'sparql SELECT DISTINCT ?m ( bif:round(bif:st_distance (?geo, ?gm)) ) ' ||
     ' WHERE { <';
@@ -875,15 +901,16 @@ s3 := '\')) .
     s1 := 'sparql SELECT DISTINCT ?s (bif:round(?lat)) AS ?lat (bif:round(?long)) AS ?long ' ||
       ' WHERE ' ||
       '   { ' ||
-      '     { ' ||
-      '       SELECT ?g ?s WHERE  ' ||
-      '         {  ' ||
-      '           graph ?g { ' ||
-      '             ?s geo:geometry ?geo }  ' ||
-      '         } ' ||
-      '       LIMIT 100  ' ||
-      '     } ' ||
+      --'     { ' ||
+      --'       SELECT ?g ?s WHERE  ' ||
+      --'         {  ' ||
+      --'           graph ?g { ' ||
+      --'             ?s geo:geometry ?geo }  ' ||
+      --'         } ' ||
+      --'       LIMIT 100  ' ||
+      --'     } ' ||
       '     graph ?g { ' ||
+      '       ?s geo:geometry ?geo . ' ||
       '       ?s geo:lat ?lat . ' ||
       '       ?s geo:long ?long . } ' ||
       '     FILTER (datatype (?lat) IN (xsd:integer, xsd:float, xsd:double)) . ' ||
@@ -899,10 +926,10 @@ s3 := '\')) .
   }
   else if ( smode='17' )
   {
-    if (isnull(val)  or val = '') val := 'category:Protestant_churches';
+    if (isnull(val)  or val = '') val := 'category:History_of_Wisconsin';
     if (isnull(val2)  or val2 = '') val2 := 'en';
     if (isnull(val3)  or val3 = '') val3 := 'en';
-    if (isnull(val4)  or val4 = '') val4 := 'http://dbpedia.org/resource/Category:Churches_in_London';
+    if (isnull(val4)  or val4 = '') val4 := 'http://dbpedia.org/resource/Category:Metropolitan_areas_of_Wisconsin';
 
     validate_input(val);
     validate_input(val2);
@@ -935,6 +962,75 @@ s3 := '\')) .
    s5 := concat(s5, val4, '> )  } ORDER BY ASC (?dist) ');
    query := concat('',s1, s2, s3, s4, s5, '');
   }
+  else if ( smode='18' )
+  {
+    if (isnull(val)  or val = '') val := 'http://dbpedia.org/resource/New_York_City';
+    if (isnull(val2)  or val2 = '') val2 := '20';
+    if (isnull(val3)  or val3 = '') val3 := 'en';
+
+    validate_input(val);
+    validate_input(val2);
+    validate_input(val3);
+
+    s1 := 'sparql SELECT DISTINCT ?resource ?label ?location ' ||
+       ' WHERE  ' ||
+       '   { ' ||
+       '     <';
+   s2 := val;
+   s3 := '> geo:geometry ?sourcegeo . ' ||
+       '     ?resource geo:geometry ?location ; ' ||
+       '                    rdfs:label ?label . ' ||
+       '     FILTER( bif:st_intersects( ?location, ?sourcegeo, ';
+   s4 := val2;
+   s5 := concat(' ) ) . FILTER( lang( ?label ) = "', val3, '" ) } ');
+   query := concat('', s1, s2, s3, s4, s5, '');
+  }
+  else if ( smode='19' )
+  {
+    if (isnull(val)  or val = '') val := 'http://dbpedia.org/resource/New_York_City';
+    if (isnull(val2)  or val2 = '') val2 := 'http://dbpedia.org/resource/London';
+
+    validate_input(val);
+    validate_input(val2);
+
+    s1 := 'sparql SELECT ?nylLabel ?nyl ?lnLabel ?ln (bif:round ( bif:st_distance( ?nyl, ?ln ) ) ) AS ?distanceBetweenNewYorkCityAndLondon ' ||
+          ' FROM <http://dbpedia.org> ' ||
+          ' WHERE ' ||
+          '  { ' ||
+          '    <';
+    s2 := val;
+    s3 := '> geo:geometry ?nyl . ' ||
+         '<';
+    s4 := val2;
+    s5 := concaT('>  geo:geometry ?ln .  <', s2, '> rdfs:label ?nylLabel .  <', s4, '> rdfs:label ?lnLabel } ');
+    query := concat('', s1, s2, s3, s4, s5, '');
+  }
+  else if ( smode='20' )
+  {
+    if (isnull(val)    or val = '') val := 'http://dbpedia.org/resource/Oxford';
+    if (isnull(val2)  or val2 = '') val2 := '5';
+    if (isnull(val3)  or val3 = '') val3 := 'http://dbpedia.org/property/established';
+    if (isnull(val4)  or val4 = '') val4 := 'en';
+
+    validate_input(val);
+    validate_input(val2);
+    validate_input(val3);
+    validate_input(val4);
+
+    s1 := 'sparql SELECT DISTINCT ?thing AS ?uri ?thingLabel AS ?name ?date AS ?established ?lat ?long  ' ||
+          'WHERE ' ||
+          ' { ' ||
+          '   <';
+    s2 := val;
+    s3 := '> geo:geometry ?sourcegeo . ' ||
+          ' ?resource geo:geometry ?matchgeo . ' ||
+          ' ?resource geo:lat ?lat . ' ||
+          ' ?resource geo:long ?long . ' ||
+          ' FILTER( bif:st_intersects( ?matchgeo, ?sourcegeo, ';
+    s4 := concat (val2, ' ) ) . ?thing ?somelink ?resource . ?thing <', val3);
+    s5 := concat('> ?date . ?thing rdfs:label ?thingLabel . FILTER( lang( ?thingLabel ) = "', val4, '" ) } ');
+    query := concat('', s1, s2, s3, s4, s5, '');
+  }
   --smode > 99 is reserved for drill-down queries
   else if (smode = '1001' or smode = '104')
     {
@@ -977,7 +1073,7 @@ s3 := '\')) .
 --    ?o2 <http://b3s-demo.openlinksw.com/label> ?lbl .
 --  }
 --;
-    if (isnull(val)  or val = '') val := 'http://myopenlink.net/dataspace/person/kidehen#this';
+    if (isnull(val)  or val = '') val := 'http://www.w3.org/People/Berners-Lee/card#i';
     s1 := 'sparql define input:inference ''virtrdf-label'' SELECT ?lbl COUNT(*) WHERE { <';
     validate_input(val);
     s2 := val;
@@ -1002,7 +1098,7 @@ s3 := '\')) .
 --    filter (?s= <http://myopenlink.net/dataspace/person/kidehen#this>)
 --  } order by ?dist desc 3 limit 50
 --;
-    if (isnull(val)  or val = '') val := 'http://myopenlink.net/dataspace/person/kidehen#this';
+    if (isnull(val)  or val = '') val := 'http://www.w3.org/People/Berners-Lee/card#i';
     s1 := 'sparql SELECT ?o ?name ?dist ((SELECT COUNT (*) WHERE {?o foaf:knows ?xx})) WHERE { { SELECT ?s ?o ?name WHERE { ?s foaf:knows ?o . ?o foaf:name ?name } } OPTION (transitive, t_distinct, t_in(?s), t_out(?o), t_min (2), t_max (4), t_step (''step_no'') as ?dist) . FILTER (?s= <';
     validate_input(val);
     s2 := val;
@@ -1030,7 +1126,7 @@ s3 := '\')) .
 --  } limit 20
 --;
 
-    if (isnull(val)  or val = '') val := 'http://myopenlink.net/dataspace/person/kidehen#this';
+    if (isnull(val)  or val = '') val := 'http://www.w3.org/People/Berners-Lee/card#i';
     --if (isnull(val2)  or val2 = '') val2 := 'http://www.advogato.org/person/mparaz/foaf.rdf#me';
     --s1 := 'sparql select ?link ?g ?step ?path where  { { select ?s ?o ?g where { graph ?g {?s foaf:knows ?o } } } option (transitive, t_distinct, t_in(?s), t_out(?o), t_no_cycles, T_shortest_only, t_step (?s) as ?link, t_step (''path_id'') as ?path, t_step (''step_no'') as ?step, t_direction 3) . filter (?s= <';
     s1 := 'sparql define input:same-as "YES" SELECT ?g COUNT (*) WHERE { { SELECT ?s ?o ?g WHERE { graph ?g {?s foaf:knows ?o } } } OPTION (transitive, t_distinct, t_in(?s), t_out(?o), t_min (1)) .  FILTER (?s= <';
@@ -1055,7 +1151,7 @@ s3 := '\')) .
 --  ?p foaf:nick ?n
 --} order by desc 2 limit 50
 --;
-    if (isnull(val)  or val = '') val := 'http://myopenlink.net/dataspace/person/kidehen#this';
+    if (isnull(val)  or val = '') val := 'http://www.w3.org/People/Berners-Lee/card#i';
     s1 := 'sparql SELECT ?n ((SELECT COUNT (*) WHERE { ?p foaf:interest ?i . ?ps foaf:interest ?i})) ((SELECT COUNT (*) WHERE { ?p foaf:interest ?i})) WHERE { { SELECT DISTINCT ?p ?psi WHERE { ?p foaf:interest ?i . ?psi foaf:interest ?i  } } . FILTER (?psi = <';
     validate_input(val);
     s2 := val;
