@@ -1,5 +1,5 @@
 --
---  $Id: DET_RDFData.sql,v 1.16.2.1 2010/05/18 19:25:26 source Exp $
+--  $Id: DET_RDFData.sql,v 1.16.2.2 2011/07/19 19:36:41 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -641,6 +641,11 @@ create function DB.DBA."RDFData_DAV_RES_CONTENT" (in id any, inout content any,
           ses := sioc..compose_foaf (uname, type, pg);
 	  goto ret_place2;
 	}
+      else if (__proc_exists ('sioc.DBA.ods_obj_describe') is not null)
+	{
+	  ses := sioc..ods_obj_describe (iri, type, pg);
+	  goto ret_place2;
+	}
       else if (regexp_match ('https?://([^/]*)/dataspace/([^/]*)(#this|/sioc.rdf|/sioc.n3)?\x24', iri) is not null
 	  and __proc_exists ('sioc.DBA.ods_sioc_obj_describe') is not null)
 	{
