@@ -1,5 +1,5 @@
 --
---  $Id: uriqa.sql,v 1.7 2009/02/13 10:09:14 source Exp $
+--  $Id: uriqa.sql,v 1.7.2.1 2011/09/28 17:01:16 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -47,7 +47,7 @@ create function WS.WS.URIQA_CFG_ITEM_VALUE (in param_name varchar, in is_list in
   declare string_val varchar;
   declare list_val any;
   ini_path := virtuoso_ini_path ();
-  string_val := cfg_item_value (ini_path, 'URIQA', param_name);
+  string_val := virtuoso_ini_item_value ('URIQA', param_name);
   if (string_val is not null)
     {
       if (is_list)
