@@ -9,9 +9,10 @@ iSPARQL.Defaults = {
 		 "http://dbpedia.org/sparql",
                  "http://loc.openlinksw.com/sparql",
 		 "http://lod.openlinksw.com/sparql",
+                 "http://sparql.sindice.com/sparql",
 		 "http://bbc.openlinksw.com/sparql",
 		 "http://demo.openlinksw.com/sparql",
-		 "http://myopenlink.net:8890/sparql/",
+		 "http://myopenlink.net:8890/sparql",
 		 "http://linkedgeodata.org/sparql",
 		 "http://sparql.reegle.info/",
 //		 "http://www.govtrack.us/sparql",
@@ -34,16 +35,16 @@ iSPARQL.Defaults = {
 			   ['rdfs:seeAlso',    'http://www.w3.org/2000/01/rdf-schema#',true],
 			   ['owl:sameAs',      'http://www.w3.org/2002/07/owl#',       true]],
 
-    query:        'SELECT * WHERE {?s ?p ?o}',
-    sponge:       'none',
+    query:        "SELECT * WHERE {?s ?p ?o}",
+    sponge:       "none",
     grabLimit:    100,
     grabDepth:    2,
     grabAll:      false,
-    graph:        '',
-    queryTimeout: 2000, // ms
+    graph:        "",
+    timeout:      false, // ms
     auth:         {user:'dav',password:'dav'},
     tab:          0,
-    anchorMode:   0, /* 0:describe, 1:get data items,2:Open in new window */
+    anchorMode:   0, /* 0:Exec SPARQL describe, 1:get data items,2:Open in new window,3:Open "describe" page */
     maxrows:      50,
     view:         1,
     endpoint:     '/sparql',
@@ -81,6 +82,10 @@ iSPARQL.Defaults = {
 	    iSPARQL.Settings.query  = p['query']; 
 	    qp = true; 
 	}
+	if (p['qtxt']) { 
+	    iSPARQL.Settings.query  = p['qtxt']; 
+	    qp = true; 
+	}
 	if (p['sponge'])            { iSPARQL.Settings.sponge = p['sponge']; qp = true; }
 	if (p['should_sponge'])     { iSPARQL.Settings.sponge = p['should_sponge']; qp = true; }
 	if (p['view']) {
@@ -96,6 +101,8 @@ iSPARQL.Defaults = {
 	if (qp) iSPARQL.Settings.qp_override = qp;
 	if (p['__DEBUG']) iSPARQL.Settings.debug = true;
 	if (p['maxrows']) iSPARQL.Settings.maxrows = parseInt(p['maxrows']);
+	if (p['timeout']) iSPARQL.Settings.timeout = parseInt(p['timeout']);
+	if (p['amode']) iSPARQL.Settings.anchorMode = parseInt(p['amode']);
     },
 
     //
@@ -125,6 +132,8 @@ iSPARQL.Defaults = {
 			iSPARQL.Defaults.auth[authParm] = iSPARQL.serverDefaults.auth[authParm];
 		}
 		continue;
+	    } else if (defName == 'namespaces') {
+		OAT.IRIDB.insertIRIArr (iSPARQL.serverDefaults[defName]);
 	    }
 	    iSPARQL.Defaults[defName] = iSPARQL.serverDefaults[defName];
 	}
@@ -157,6 +166,7 @@ iSPARQL.Defaults = {
 	    iSPARQL.Defaults.map_type = OAT.Map.TYPE_OL;
 	    break;
 	}
+	this.handlePageParams();
     }
 };
 
