@@ -1,5 +1,5 @@
 /*
- *  $Id: execute.js,v 1.32.2.33 2011/03/29 22:51:20 source Exp $
+ *  $Id: execute.js,v 1.32.2.42 2011/09/20 12:03:50 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -201,6 +201,7 @@ var QueryExec = function(optObj) {
 		namedGraphs:[],
 		sponge:false,
 	maxrows:false,
+		timeout:false,
 	sourceQuery:false /* before macro expansion */
     };
 
@@ -211,31 +212,56 @@ var QueryExec = function(optObj) {
 	this.mini = false;
     	this.miniplnk = false;
     	this.mRDFCtr = false;
+	this.anchor_pref_c = false;
+	
+	this.makeAnchorPrefs = function () {
+		var anchor_pref_c = OAT.Dom.create("div", {className: "anchor_pref_c"});
+		var anchor_label = OAT.Dom.create("label", {htmlFor: "anchor_pref_sel"});
+		anchor_label.innerHTML = "Dereferencing:";
+		anchor_pref_sel = OAT.Dom.create ("select", {id: "anchor_pref_sel"});
+		anchor_pref_sel.options.add (new Option ("SPARQL Describe",0));
+		anchor_pref_sel.options.add (new Option ("SELECT IRI as S or O",1));
+		anchor_pref_sel.options.add (new Option ("GET Page",2));
+		anchor_pref_sel.options.add (new Option ("Use Virtuoso Web Service",3)); 
+		anchor_pref_sel.selectedIndex = iSPARQL.Settings.anchorMode;
+
+		OAT.Event.attach(anchor_pref_sel, 'change', function () {
+			iSPARQL.Settings.anchorMode = ($('anchor_pref_sel').selectedIndex);
+		});
+
+        OAT.Dom.append ([anchor_pref_c, anchor_label, anchor_pref_sel]);
+
+		return anchor_pref_c;
+	}
 
 	this.init = function() {
-		this.dom.result = OAT.Dom.create("div");
+		this.dom.result =   OAT.Dom.create("div", {className:'ep_result'});
 		this.dom.request = OAT.Dom.create("div", {className:'ep_request'}); 
 		this.dom.response = OAT.Dom.create("pre",{className:'ep_response'});
-		this.dom.query = OAT.Dom.create("pre");
+		this.dom.query =    OAT.Dom.create("pre", {className:'ep_query'});
+
 //	this.dom.select = OAT.Dom.create("select");
 //	OAT.Dom.option("Machine-readable","1",this.dom.select);
 //	OAT.Dom.option("Human-readable","0",this.dom.select);
 
 		var tabs1 = ["Result","SPARQL Params","Response","Query"];
 		var tabs2 = [self.dom.result,self.dom.request,self.dom.response,self.dom.query];
-		self.dom.tab = OAT.Dom.create("div",{padding:"5px",backgroundColor:"#fff"});
+
+		self.dom.tab = OAT.Dom.create ("div",{className:'res_tab_ctr',id: 'res_tab_ctr'});
 		self.dom.ul = OAT.Dom.create("ul",{},"tabres");
 		self.tab = new OAT.Tab(self.dom.tab,{dockMode:true,dockElement:self.dom.ul});
+
 		for (var i=0;i<tabs1.length;i++) {
 			var li = OAT.Dom.create("li");
 			self.dom.ul.appendChild(li);
 			li.innerHTML = tabs1[i];
 			self.tab.add(li,tabs2[i]);
 		}
+
 		if (self.options.div) {
 			OAT.Dom.clear(self.options.div);
 	    OAT.Dom.append([self.options.div,/*self.dom.select,*/OAT.Dom.create("br")]);
-			OAT.Dom.append([self.options.div,self.dom.ul,self.dom.tab]);
+			OAT.Dom.append([self.options.div,self.dom.ul,self.dom.tab]); 
 		}
 		self.initNav();
 
@@ -255,6 +281,44 @@ var QueryExec = function(optObj) {
 		OAT.IRIDB.insertIRI (window.defaultPrefixes[i].uri, window.defaultPrefixes[i].label);
 	}
 
+		self.load_notify = new OAT.Notify (false, {notifytype: 2});
+		self.error_notify = new OAT.Notify (false, {notifytype: 2});
+
+		self.request_count = 0;
+
+		OAT.MSG.attach ('*',
+						'iSPARQL_QE_START', 
+						function () { 
+							var msg = 'Executing query (' + self.request_count;
+
+							if (self.request_count > 1) msg += ' requests pending.)';
+							else msg += ' request pending.)';
+
+							self.load_notify.send (msg, {image: "images/notify-throbber.gif", timeout: false});
+						});
+		
+		OAT.MSG.attach ('*',
+						'iSPARQL_QE_DONE',
+						function () {
+							if (self.request_count < 1)
+								self.load_notify.hide();
+							else {
+								var msg = 'Executing query (' + self.request_count;
+
+								if (self.request_count > 1) msg += ' requests pending.)';
+								else msg += ' request pending.)';
+
+								self.load_notify.send (msg, {image: "images/notify-throbber.gif", timeout: false});
+							}
+						});
+
+		OAT.MSG.attach ('*', 
+						'iSPARQL_QE_ERROR',
+						function (m,s,l) {
+							var msg = 'Query returned error.';
+							self.error_notify.send (msg, {timeout: 3000});
+						});
+
     };
 
 	this.initNav = function() {
@@ -349,6 +413,7 @@ var QueryExec = function(optObj) {
 		cache.opts.endpoint != opts.endpoint || 
 		cache.opts.defaultGraph != opts.defaultGraph ||
 	        cache.opts.maxrows != opts.maxrows ||
+				cache.opts.timeout != opts.timeout ||
 	        cache.opts.namedGraphs != opts.namedGraphs ||
 	        cache.opts.pragmas != opts.pragmas);
     };
@@ -356,22 +421,20 @@ var QueryExec = function(optObj) {
 	this.buildRequest = function(opts) {
 		var paramsObj = {};
 
-		if (opts.defaultGraph && !opts.query.match(/from *</i)) { paramsObj["default-graph-uri"] = opts.defaultGraph; }
-	if (opts.maxrows && opts.query && !opts.query.match(/limit *[0-9].*/i)) { paramsObj["maxrows"] = opts.maxrows; }
+		if (opts.defaultGraph && !opts.query.match(/from *</i)) { 
+          paramsObj["default-graph-uri"] = opts.defaultGraph; 
+        }
+
+		if (opts.maxrows && opts.query && !opts.query.match(/limit *[0-9].*/i)) { 
+          paramsObj["maxrows"] = opts.maxrows; 
+        }
+
+		if (opts.timeout) { paramsObj["timeout"] = opts.timeout; }
 		if (opts.sponge && self.options.virtuoso) { paramsObj["should-sponge"] = opts.sponge; }
 
-		var pragmas = [];
-	
-		if (opts.pragmas) {
-			for (var i=0;i<opts.pragmas.length;i++) {
-				var pragma = opts.pragmas[i];
-				var name = pragma[0];
-				var values = pragma[1];
-				for(var j=0;j<values.length;j++) { pragmas.push(name+" "+values[j]); }
-			}
-		}
-
-		paramsObj["query"] = pragmas.join('\n') + '\n' + opts.query;
+		var pragmas = self.getPragmas(opts);
+		if (!pragmas) pragmas = "";
+		paramsObj["query"] = pragmas + '\n' + opts.query;
 
 		var arr = [];
 		for (var p in paramsObj) {
@@ -394,15 +457,27 @@ var QueryExec = function(optObj) {
     };
 
     this.resultType = function(data) {
-		if (data.documentElement == null) return iSPARQL.ResultType.ERROR; // OPERA returns XMLDocument with all members null
-		if (data.documentElement.localName == "sparql")
+
+		if (data.documentElement == null) 
+			return iSPARQL.ResultType.ERROR; // OPERA returns XMLDocument with all members null
+
+		var fElm;
+		
+		if (OAT.Browser.isIE) {
+			fElm = data.documentElement.baseName;
+		} else {
+			fElm = data.documentElement.localName;
+		}
+		
+		switch (fElm) {
+		case "sparql":
 			return iSPARQL.ResultType.RESSET;
-		if (data.documentElement.localName == "RDF")
+		case "RDF":
 			return iSPARQL.ResultType.GRAPH;
-		if (data.documentElement.localName == "parsererror")
+		default:
 			return iSPARQL.ResultType.ERROR;
+		}
     };
-	
     //
     // Cache incoming result set or graph
     //
@@ -419,9 +494,13 @@ var QueryExec = function(optObj) {
 		}
 		else {
 			rt = iSPARQL.ResultType.ERROR;
-			if (data.match (/Error SR171/))
+			if (data.match (/Error SR171/)) {
+				iSPARQL.StatusUI.errMsg = 'Query Engine Timeout on Server Detected';
 				to = true;
 		}
+			else 
+				iSPARQL.StatusUI.errMsg = 'Error executing query';
+		}
 		
 			var cacheItem = {
 				resType: rt,
@@ -440,10 +519,28 @@ var QueryExec = function(optObj) {
 			cacheItem.dom.request_c  = OAT.Dom.create ("div",{className: "request_c"});
 			cacheItem.dom.response_c = OAT.Dom.create ("div",{className: "response_c"});
 			
+     	cacheItem.dom.result_opts_c = OAT.Dom.create ("div",{className:"result_opts_c"}); 
+		cacheItem.dom.plnk_c = OAT.Dom.create("div", {className: "result_plnk_c"});
+
+		cacheItem.dom.deref_prefs = self.makeAnchorPrefs();
+		var clrfix = OAT.Dom.create ("div", {className: "clearfix"})
+		OAT.Dom.append([cacheItem.dom.result_opts_c, cacheItem.dom.plnk_c, cacheItem.dom.deref_prefs], 
+					   [cacheItem.dom.result_c, cacheItem.dom.result_opts_c, clrfix]);
+
+
 		var req_href;
 		
-		if (opts.endpoint && !opts.endpoint.match (/^http/))
-			req_href = unescape(request.substring(4));
+		// FIXME (ghard) this is wrong
+
+		if (opts.endpoint) {
+			if (opts.endpoint.match (/^http/))
+				req_href = opts.endpoint + "?" + request;
+			else
+				req_href = 
+				document.location.protocol + "//" +
+				document.location.host + 
+				opts.endpoint + "?" + request; 
+		}
 		else
 			req_href = 
 			document.location.protocol + "//" +
@@ -462,27 +559,93 @@ var QueryExec = function(optObj) {
 		if (old>=0) self.nav(old);
     };
 
+	this.makeDataLinks = function () {
+		var item = self.cache[self.cacheIndex];
+		var opts = item.opts;
+		var request = item.request;
+		var ctr = self.miniplnk;
+		
+		var nloca = document.location;
+
+		if (opts.endpoint == '/sparql') {
+			var resUriBase = nloca.protocol + 
+				"//" + nloca.host + "/sparql/?query=" + encodeURIComponent(opts.query);
+			
+			resUriBase += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
+			resUriBase += "&timeout=" + (opts.timeout ? opts.timeout : "");
+			resUriBase += "&default-graph-uri=" + (opts.defaultGraph ? opts.defaultGraph : "");
+			resUriBase += "&format=";
+
+			$('cxml_raw_lnk').href =   resUriBase + encodeURIComponent("text/cxml");
+			$('csv_raw_lnk').href =    resUriBase + encodeURIComponent("text/csv");
+
+			$('nt_raw_lnk').href =     resUriBase + encodeURIComponent("text/plain");
+			$('n3_raw_lnk').href =     resUriBase + encodeURIComponent("text/rdf+n3");
+			$('json_raw_lnk').href =   resUriBase + encodeURIComponent("application/rdf+json");
+			$('rdfxml_raw_lnk').href = resUriBase + encodeURIComponent("application/rdf+xml");
+			
+			$('odata_atom_lnk').href = resUriBase + encodeURIComponent("application/atom+xml");
+			$('odata_json_lnk').href = resUriBase + encodeURIComponent("application/odata+json");
+
+			OAT.Dom.show('data_links');
+		}
+
+	}
+
+	this.getPragmas = function (opts) {
+		var pragmas = [];
+		
+		if (opts.pragmas) { 
+			for (var i=0;i<opts.pragmas.length;i++) {
+				var pragma = opts.pragmas[i];
+				var name = pragma[0];
+				var values = pragma[1];
+				for(var j=0;j<values.length;j++) { pragmas.push(name+" "+values[j]); }
+			}
+			return (pragmas.join('\n'));
+		}
+		return false;
+	}
+
     this.makeMiniRDFPlinkURI = function (caller,msg,o) {
 	var item = self.cache[self.cacheIndex];
 	var opts = item.opts;
 	var request = item.request;
-	var plnk = self.miniplnk;
+		var ctr = self.miniplnk;
 
+		OAT.Dom.clear(ctr);
+
+		if (iSPARQL.Settings.pivotInstalled) 
+			self.makePivotPermalink(ctr);
+		
 	var nloca = document.location;
 
-	var xparm = "?query=" + encodeURIComponent(opts.query);
+		var pragmas = self.getPragmas(opts);
+		
+		var xparm = "?query=";
+
+		if (pragmas) xparm += encodeURIComponent(pragmas);
+
+		xparm += encodeURIComponent(opts.query);
 
 	if (opts.endpoint)
 	    xparm = xparm + "&endpoint="  + opts.endpoint;
 
 		xparm = xparm + "&resultview=" + item.mini.options.tabs[o.tabIndex][0];
 	xparm += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
+		xparm += "&timeout=" + (opts.timeout ? opts.timeout : "");
 		xparm += "&view=" + opts.view;
-	plnk.target = "_blank";
-	plnk.href= nloca.protocol + "//" + nloca.host + "/isparql/view/" + xparm;
+		xparm += "&amode=" + iSPARQL.Settings.anchorMode;
+		var plnk_a = OAT.Dom.create("a");
+
+		plnk_a.target = "_blank";
+		plnk_a.href= nloca.protocol + "//" + nloca.host + "/isparql/view/" + xparm;
+		plnk_a.innerHTML = "Permalink"
 
 		if (iSPARQL.Settings.shorten_uris) 
-			iSPARQL.Common.shortenURI (plnk);
+			iSPARQL.Common.shortenURI (plnk_a);
+
+		OAT.Dom.append([ctr, plnk_a]);
 				
     }
 
@@ -501,6 +664,7 @@ var QueryExec = function(optObj) {
 
 		xparm = xparm + "&resultview=" + item.mini.options.tabs[o.tabIndex][0];
 		xparm += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
+		xparm += "&timeout=" + (opts.timeout ? opts.timeout : "");
 		
 		plnk.target = "_blank";
 
@@ -682,12 +846,23 @@ var QueryExec = function(optObj) {
 
 	a = OAT.Dom.create("a");
 	a.innerHTML = "Make Pivot";
-	var xparm = document.location.protocol + '//' + document.location.host + 
-	    "/sparql/?query=" + encodeURIComponent(opts.query) + 
-	    "&endpoint=" + opts.endpoint;
+
+		var pragmas = self.getPragmas(opts);
+		var xparm;
+
+		if (opts.endpoint != '/sparql')
+			xparm = opts.endpoint + "?query=";
+		else 
+			xparm = document.location.protocol + '//' + document.location.host + 
+			"/sparql/?query=";
+
+		if (pragmas) xparm += encodeURIComponent(pragmas);
+
+		xparm += encodeURIComponent(opts.query);
 	xparm += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
-	xparm += "&default-graph-uri=" + (opts.defaultGraph ? opts.defaultGraph : "");
-	xparm += "&format=text/cxml";
+		xparm += "&timeout=" + (opts.timeout ? opts.timeout : "");
+		xparm += "&default-graph-uri=" + (opts.defaultGraph ? encodeURIComponent(opts.defaultGraph) : "");
+		xparm += "&format=text%2Fcxml";
 	a.href = document.location.protocol + '//' + document.location.host + 
 	    '/PivotViewer/' + "?url=" + encodeURIComponent(xparm);
 	a.target = "_blank";
@@ -708,12 +883,24 @@ var QueryExec = function(optObj) {
 	
 	var execURIa = OAT.Dom.create ("a");
 	execURIa.innerHTML = "Execute Permalink";
+
 	var nloca = document.location;
 
-	var xparm = "?query=" + encodeURIComponent(opts.query) + "&endpoint="  + opts.endpoint;
+		var pragmas = self.getPragmas(opts);
+		
+		var xparm = "?query=";
+
+		if (pragmas) xparm += encodeURIComponent(pragmas);
+
+		xparm += encodeURIComponent(opts.query);
+
+		xparm += "&endpoint="  + encodeURIComponent(opts.endpoint);
+		
 	xparm += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
-	xparm += "&default-graph-uri=" + (opts.defaultGraph ? opts.defaultGraph : "");
+		xparm += "&timeout=" + (opts.timeout ? opts.timeout : "");
+		xparm += "&default-graph-uri=" + (opts.defaultGraph ? encodeURIComponent(opts.defaultGraph) : "");
 		xparm += "&view=" + opts.view;
+		xparm += "&amode=" + iSPARQL.Settings.anchorMode;
 
 	execURIa.href = nloca.protocol + "//" + nloca.host + "/isparql/view/" + xparm;
 	
@@ -723,31 +910,16 @@ var QueryExec = function(optObj) {
 	execURIa.target = "_blank";
 	
 		OAT.Dom.append([ctr,execURIa]);
+		OAT.Dom.hide ('data_links')
     };
 
     this.drawSparqlResultSet = function (resSet) {
 		var item = self.cache[self.cacheIndex];
-		self.dom.plnk_ctr = OAT.Dom.create("div", {className: "result_plnk_ctr"});
 
 		if (iSPARQL.Settings.pivotInstalled) 
-			self.makePivotPermalink(self.dom.plnk_ctr);
+			self.makePivotPermalink(item.dom.plnk_c);
 
-		self.makeExecPermalink (self.dom.plnk_ctr);
-
-		var anchor_pref_ctr = OAT.Dom.create("div", {className: "anchor_pref_ctr"});
-		anchor_label = OAT.Dom.create("label", {htmlFor: "anchor_pref_sel"});
-		anchor_label.innerHTML = "Anchor behavior:";
-		anchor_pref_sel = OAT.Dom.create ("select", {id: "anchor_pref_sel"});
-		anchor_pref_sel.options.add (new Option ("Describe",0));
-		anchor_pref_sel.options.add (new Option ("Get Data Items",1));
-		anchor_pref_sel.options.add (new Option ("Open Web Page",2));
-
-		OAT.Event.attach(anchor_pref_sel, 'change', function () {
-			iSPARQL.Settings.anchorMode = ($('anchor_pref_sel').selectedIndex);
-		});
-
-		OAT.Dom.append ([item.dom.result_c, self.dom.plnk_ctr, anchor_pref_ctr]);
-        OAT.Dom.append ([anchor_pref_ctr, anchor_label, anchor_pref_sel]);
+		self.makeExecPermalink (item.dom.plnk_c);
 
 		var grid = new OAT.Grid (item.dom.result_c);
 	grid.createHeader(resSet.variables);
@@ -899,11 +1071,16 @@ var QueryExec = function(optObj) {
     this.makeErrorResp = function (data) {
 	var txt = OAT.Xml.serializeXmlDoc(data);
 
-		if (txt.length == 0 || txt == false) txt = data;
+		if (txt.length == 0 || txt == false) {
+			if (typeof data == 'object')
+ 				txt = 'Invalid response received';
+			else {
+				txt = data;
 
 		txt = txt.replace(/</g,"&lt;");
 		txt = txt.replace(/>/g,"&gt;");
-
+			} 
+		}
 	return txt;
     };
 
@@ -927,15 +1104,25 @@ var QueryExec = function(optObj) {
 // TODO store full query context obj in cache and add functions to restore
 //
 
+		if (typeof iSPARQL.dataObj == 'undefined') iSPARQL.dataObj = {};
 		iSPARQL.dataObj.query        = item.opts.query;
 		iSPARQL.dataObj.defaultGraph = item.opts.defaultGraph;
 		iSPARQL.dataObj.maxrows      = item.opts.maxrows;
+		iSPARQL.dataObj.timeout      = item.opts.timeout;
 		iSPARQL.dataObj.namedGraphs  = item.opts.namedGraphs;
 		iSPARQL.dataObj.pragmas      = item.opts.pragmas;
+		iSPARQL.dataObj.endpoint     = item.opts.endpoint;
 
+		if (typeof iSPARQL.endpointOpts != 'undefined') 
 		iSPARQL.endpointOpts.setEndpoint (null, item.opts.endpoint);
 
-		adv.redraw();
+		if (item.resType == iSPARQL.ResultType.GRAPH) 
+			OAT.Dom.show('data_links');
+		else 
+			OAT.Dom.hide('data_links');
+
+		if (typeof adv != 'undefined') adv.redraw();
+
 //		if (!item.wasError && qbe) qbe.loadFromString(iSPARQL.dataObj.query);
 
 		self.refreshNav();
@@ -968,6 +1155,7 @@ var QueryExec = function(optObj) {
 		a.innerHTML = "Query Permalink";
 	var xparm = "?query=" + encodeURIComponent(opts.query) + "&endpoint=" + opts.endpoint;
 	xparm += "&maxrows=" + (opts.maxrows ? opts.maxrows : "");
+		xparm += "&timeout=" + (opts.timeout ? opts.timeout : "");
 	xparm += "&default-graph-uri=" + (opts.defaultGraph ? opts.defaultGraph : "");
 		xparm += "&view=" + opts.view;
 	a.href = document.location.protocol + '//' + document.location.host + '/isparql/' + xparm;
@@ -1008,16 +1196,16 @@ var QueryExec = function(optObj) {
 
 			var ua = navigator.userAgent.toLowerCase();
 		
-			if (ua.search ("iphone") || 
+/*			if (ua.search ("iphone") || 
 				ua.search ("ipod") ||
 				ua.search ("android") ||
 				ua.search ("symbian") ||
 				ua.search ("S60"))
-				
+*/				
 				map_h = OAT.Dom.getViewport()[1].toString()+'px';
-			else 
+/*			else 
 				map_h = "600px";
-
+*/			
 			//	generate Result
 			
 			if (item.resType == iSPARQL.ResultType.GRAPH) { // Use RDFMini to show Graphs
@@ -1040,10 +1228,7 @@ var QueryExec = function(optObj) {
 		    lastIndex = self.parseTabIndex (opts.resultView, tabs);
 				var c_i = self.cacheIndex;
 				
-				self.plnk_ctr = OAT.Dom.create ("div",{className:"result_plnk_ctr"}); 
-				
-		self.miniplnk = OAT.Dom.create ("a");
-		self.miniplnk.innerHTML = "Permalink";
+				self.miniplnk = OAT.Dom.create ("div",{id:"rdf_plink_c"});
 
 /*					self.addthis_ctr.innerHtml = '<div id="sharelink" class="addthis_toolbox addthis_default_style "' + 
 						'addthis:url=""' + 
@@ -1056,32 +1241,37 @@ var QueryExec = function(optObj) {
 						'<a class="addthis_button_preferred_3"></a>' +
 						'<a class="addthis_button_preferred_4"></a>' +
 						'</div>'
-				}
-*/
+				}*/
 //				item.content = OAT.Dom.create ("div",{className: "rdf_mini_ctr"});
 				
 
-				if (iSPARQL.Settings.pivotInstalled) 
-					self.makePivotPermalink(self.plnk_ctr);
-
-				var mini_c = OAT.Dom.create("div",{className: "rdf_mini_ctr"});
+				var mini_c = OAT.Dom.create("div",{className: "rdf_mini_c"});
 				
+				if (item.store.getTripleCount() > 0) {
 				item.mini = new OAT.RDFMini(mini_c,{tabs:tabs,
 													showSearch:false,
 													store: item.store});
 
-				OAT.Dom.append([item.dom.result_c, self.plnk_ctr, mini_c]);
-				
+				item.mini.processLink = self.processLink;
+
 				//		self.tab.go(0); // got to do here or maps won't resize properly.
-				//		self.cache[self.cacheIndex].mini.processLink = self.processLink;
 				//		self.cache[self.cacheIndex].mini.store.addXmlDoc(data);
 				
 				item.mini.select.selectedIndex = lastIndex;
 				item.mini.redraw();
 				
-				OAT.Dom.append ([self.plnk_ctr, self.miniplnk]);
+					self.makeMiniRDFPlinkURI (false,false,{tabIndex:lastIndex});
+					OAT.MSG.attach (item.mini, 'RDFMINI_VIEW_CHANGED', self.makeMiniRDFPlinkURI);
+					self.makeDataLinks();
 				
-				var ua = navigator.userAgent;
+				} else {
+					mini_c.innerHTML = 
+						'<h2>No Data</h2>' +
+						'<p>This query returned an empty graph.</p>';
+				}
+
+				OAT.Dom.append ([item.dom.plnk_c, self.miniplnk]);
+				OAT.Dom.append ([item.dom.result_c, mini_c]);
 				
 				//				if (iSPARQL.Settings.addthis_key && !OAT.Browser.isScreenOnly) {
 				//					self.addthis_ctr = OAT.Dom.create ("a",{id: "addthis_ctr",
@@ -1092,10 +1282,6 @@ var QueryExec = function(optObj) {
 				//					OAT.MSG.attach (item.mini, 'RDFMINI_VIEW_CHANGED', self.makeAddThisURL);
 				//				}
 				
-		self.makeMiniRDFPlinkURI (false,false,{tabIndex:lastIndex});
-				OAT.MSG.attach (item.mini, 'RDFMINI_VIEW_CHANGED', self.makeMiniRDFPlinkURI);
-				
-
 			} else {
 		if (data.firstChild.tagName == 'sparql' && 
 		    data.firstChild.namespaceURI == 'http://www.w3.org/2005/sparql-results#') {		    
@@ -1130,16 +1316,14 @@ var QueryExec = function(optObj) {
 
 	this.processLink = function(domNode,href) {
 		var dereferenceRef = function(event) {
-			OAT.Event.prevent(event);
-
 			var cache = self.cache[self.cacheIndex];
 
-/*	    var q = 'define get:soft "replacing" \n'+
+			var q = 'define get:soft "replacing" \n'+
 					'define input:same-as "yes" \n'+
 					'define input:grab-seealso <http://www.w3.org/2002/07/owl#sameAs> \n'+
-		'DESCRIBE <'+href+'> FROM <' + href + '>';*/
-	    var q  = 'DESCRIBE <'+href+'>';
-			var bq = 'DESCRIBE <'+href+'>';
+		        'DESCRIBE <'+ encodeURI(href) +'> FROM <' + encodeURI(href) + '>';
+//			var q  = 'DESCRIBE <'+href+'>';
+			var bq = 'DESCRIBE <'+ encodeURI(href) +'>';
 			var o = {};
 			for (var p in cache.opts) { o[p] = cache.opts[p]; }
 			o.defaultGraph = false;
@@ -1153,8 +1337,18 @@ var QueryExec = function(optObj) {
 			self.execute(o);
  	};
 
+		var describeRef = function (event) {
+			var loc = window.location;
+			var go_to = loc.protocol + "//" + loc.hostname;
+			
+			if (loc.protocol == "https:" && loc.port != "443") go_to += ":" + loc.port;
+			if (loc.protocol == "http:" && loc.port != "80") go_to += ":" + loc.port;
+			
+			go_to += "/describe/?url=" + encodeURI(href); 
+			window.open (go_to);
+		}
+		
 		var selectRef = function(event) {
-			OAT.Event.prevent(event);
 			var cache = self.cache[self.cacheIndex];
 			var o = {};
 			for (var p in cache.opts) { o[p] = cache.opts[p]; }
@@ -1163,13 +1357,13 @@ var QueryExec = function(optObj) {
 
 			var q = 'SELECT DISTINCT * \n';
 			if (graph) { q += 'FROM <' + graph + '> \n'; }
-			q += 'WHERE { { <'+href+'> ?p ?o } UNION { ?s ?p <'+href+'> } }';
+			q += 'WHERE { { <'+ encodeURI(href)+'> ?p ?o } UNION { ?s ?p <'+ encodeURI(href) +'> } }';
 
 			o.query = q;
 			self.execute(o);
 	};
 
-		var genRef = function() {
+/*		var genRef = function() {
 			var ul = OAT.Dom.create("ul",{marginLeft:"20px",marginTop:"10px"});
 
 			var li = OAT.Dom.create("li");
@@ -1197,7 +1391,7 @@ var QueryExec = function(optObj) {
 			return ul;
 	};
 
-/*		var obj = {
+		var obj = {
 			title:"URL",
 			content:genRef,
 			newHref:href,
@@ -1208,16 +1402,21 @@ var QueryExec = function(optObj) {
 		}; */
 		
 		OAT.Event.attach (domNode, 'click', function (event) {
+            OAT.Event.prevent(event);
 			switch (iSPARQL.Settings.anchorMode) {
 			case 0:
 				dereferenceRef(event);
-				break;
+				event.cancelBubble = true;
+                return false;
 			case 1:
 				selectRef(event);
 				break;
-			default:
-				OAT.Event.prevent(event);
+			case 2:
 				window.open(href);
+				break;
+			default:
+				describeRef(event);
+				break;
 			}
 		});
 
@@ -1235,8 +1434,6 @@ var QueryExec = function(optObj) {
 //	a.target = "_blank";
 //	a.href = "/about/html/" + href;
 
-//	domNode.parentNode.appendChild(img1);
-//	domNode.parentNode.appendChild(a);
     };
 
      this.detectLocationMacros = function (q) {
@@ -1278,12 +1475,14 @@ var QueryExec = function(optObj) {
 		var request = self.buildRequest(opts);
 
 		var callback = function(data) {
+			self.request_count--;
 			self.addResponse(request,optObj,0,data);
 			if (opts.callback) { opts.callback(data); }
 	    OAT.MSG.send (self,"iSPARQL_QE_DONE",self);
 	};
 
 		var onerror = function(xhr) {
+			self.request_count--;
 			var txt = xhr.getResponseText();
 			if (txt.match(/SP031/) && optObj.backupQuery) {
 				var newO = {};
@@ -1295,6 +1494,8 @@ var QueryExec = function(optObj) {
 				self.addResponse(request,optObj,1,txt);
 				if (opts.onerror) { opts.onerror(txt); }
 			}
+			OAT.MSG.send (self, "iSPARQL_QE_DONE", self);
+			OAT.MSG.send (self, "iSPARQL_QE_ERROR", self);
 	};
 
 		var o = {
@@ -1307,6 +1508,9 @@ var QueryExec = function(optObj) {
 
 		if (!opts.endpoint) { opts.endpoint = '/sparql'; }
 
+		self.request_count++;
+		OAT.MSG.send (self, "iSPARQL_QE_START", self);
+		
 		OAT.AJAX.POST(opts.endpoint,request,callback,o);
 	}
 
