@@ -1,5 +1,5 @@
 /*
- *  $Id: main.js,v 1.45.2.31 2011/03/29 11:03:35 source Exp $
+ *  $Id: main.js,v 1.45.2.39 2011/09/08 15:14:18 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -44,7 +44,7 @@ function init() {
 	iSPARQL.Common.initAdv(); // XXX warning defines global adv
 	
 
-	if (!OAT.Browser.isIE && !OAT.Browser.isScreenOnly)
+		if (OAT.Browser.hasSVG)
 	    iSPARQL.Common.initQBE(); // XXX warning defines global qbe
 
     	iSPARQL.Common.initQE();  // XXX warning defines global qe, uses global qbe
@@ -55,12 +55,10 @@ function init() {
 
 	iSPARQL.StatusUI.hide();
 
-	if (!OAT.Browser.isIE || !OAT.Browser.isScreenOnly) {
+		if (OAT.Browser.hasSVG) {
 	    if (qbe.svgsparql) { qbe.svgsparql.reposition(); }
-	}
-
-		if (OAT.Browser.isIE || OAT.Browser.isScreenOnly && iSPARQL.Settings.view == 0)
 			tab.go (1); /* is 0-based index... */
+		}
 		else
 			tab.go (iSPARQL.Settings.view);
 
@@ -161,11 +159,11 @@ iSPARQL.QueryExec = function(optObj) {
     self.go(optObj);
 }
 
-iSPARQL.LayoutMgr = {
-	resize_h: function () {
-		
-	},
-}
+//iSPARQL.LayoutMgr = {
+//	resize_h: function () {
+//		
+//	},
+//}
 
 iSPARQL.Advanced = function () {
     var self = this;
@@ -183,8 +181,8 @@ iSPARQL.Advanced = function () {
 	$("query").value = iSPARQL.dataObj.query;
 
 	/* default graph */
+		if (iSPARQL.dataObj.defaultGraph) 
 	$("default-graph-uri").value = iSPARQL.dataObj.defaultGraph;
-
 	}
 
     this.func_load = function() {
@@ -236,7 +234,8 @@ iSPARQL.Advanced = function () {
 	    pragmas:iSPARQL.endpointOpts.pragmas,
 	    namedGraphs:iSPARQL.dataObj.namedGraphs,
 	    callback:iSPARQL.Common.setData,
-			maxrows:iSPARQL.dataObj.maxrows,
+			maxrows:iSPARQL.Settings.maxrows,
+			timeout:iSPARQL.Settings.timeout,
 			view:1
 	}
 	
@@ -307,6 +306,7 @@ iSPARQL.Advanced = function () {
 	dataObj.endpointOpts.useProxy = iSPARQL.endpointOpts.useProxy;
 	dataObj.endpointOpts.pragmas = iSPARQL.endpointOpts.pragmas;
 	dataObj.maxrows = iSPARQL.dataObj.maxrows;
+		dataObj.timeout = iSPARQL.dataObj.timeout;
 	dataObj.defaultGraph = $v('default-graph-uri');
 		dataObj.metaDataOpts = iSPARQL.mdOpts.getMetaDataObj();
 		
@@ -361,10 +361,8 @@ OAT.Observer = {
     }
 }
 
-
-
 //
-// XXX (ghard) consider saving the whole query context (default graph, endpoint, etc.)
+// TODO (ghard) consider saving the whole query context (default graph, endpoint, etc.)
 //
 
 iSPARQL.RecentQueriesUI = function () {
@@ -437,6 +435,9 @@ iSPARQL.RecentQueriesUI = function () {
 //
 // Observes SpongerOpts
 //
+ // EndPointOptsUI
+	
+
 
 iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
     var self = this;
@@ -556,6 +557,7 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
     this.grabAllChangeCB = function (elm) {
 	if (elm.target.checked) {
 	    self.opts.setGrabAll (self);
+			self.hideSeeAlsoControls();
 	}
     }
 
@@ -585,7 +587,7 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
     }
 
     this.grabPredsChangeCB = function (elm) {
-	return 0;
+		self.opts.setSeeAlso(self);
     }
 
     this.hideSeeAlsoControls = function() {
@@ -677,11 +679,11 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
 	    || window.confirm("This will remove custom added predicates. Really restore?")) {
 
 	    $("pathTravSchemesPreds").options.length = 0;
-	    $("pathTravSchemesPreds").options[0] = new Option('foaf:knows','http://xmlns.com/foaf/0.1/',true);
-	    $("pathTravSchemesPreds").options[1] = new Option('sioc:links_to','http://rdfs.org/sioc/ns#',true);
-	    $("pathTravSchemesPreds").options[2] = new Option('rdfs:isDefinedBy','http://www.w3.org/2000/01/rdf-schema#',true);
-	    $("pathTravSchemesPreds").options[3] = new Option('rdfs:seeAlso','http://www.w3.org/2000/01/rdf-schema#',true);
-	    $("pathTravSchemesPreds").options[4] = new Option('owl:sameAs','http://www.w3.org/2002/07/owl#',true);
+			$("pathTravSchemesPreds").options[0] = new Option('foaf:knows','http://xmlns.com/foaf/0.1/knows',true);
+			$("pathTravSchemesPreds").options[1] = new Option('sioc:links_to','http://rdfs.org/sioc/ns#links_to',true);
+			$("pathTravSchemesPreds").options[2] = new Option('rdfs:isDefinedBy','http://www.w3.org/2000/01/rdf-schema#isDefinedBy',true);
+			$("pathTravSchemesPreds").options[3] = new Option('rdfs:seeAlso','http://www.w3.org/2000/01/rdf-schema#seeAlso',true);
+			$("pathTravSchemesPreds").options[4] = new Option('owl:sameAs','http://www.w3.org/2002/07/owl#sameAs',true);
 	    }
 	setGrabPragma();
     }
@@ -689,7 +691,7 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
     this.setEpOptCtl = function () {
 	self.epCombo.input.value = self.opts.endpointPath;
 
-	if (self.opts.verType == iSPARQL.SERVER_TYPE_VIRTUOSO)
+		if (self.opts.serverType == iSPARQL.SERVER_TYPE_VIRTUOSO)
 	    $("endpointTypeInd").innerHtml = "(Virtuoso)";
 	else
 	    $("endpointTypeInd").innerHtml = "(Generic)";
@@ -793,7 +795,7 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
 
     self._init ();
 
-} // EndPointOptsUI
+}
 
 //
 // EndpointOpts holds global endpoint-related options, it is observed by the EndpointOptsUI
@@ -803,6 +805,7 @@ iSPARQL.EndpointOptsUI = function (optsObj, toggler, indicator, container) {
 //
 //
 
+	
 iSPARQL.EndpointOpts = function (optsObj) {
     var self = this;
 
@@ -825,7 +828,7 @@ iSPARQL.EndpointOpts = function (optsObj) {
 	var _o = {
 		endpoint: '/sparql',
 		useProxy: true,
-		pragmas: [],
+		pragmas: []
 	};
 
 	for (p in optsObj) {
@@ -885,27 +888,37 @@ iSPARQL.EndpointOpts = function (optsObj) {
     }
 
     this.endpointDetectCB = function (data, headers) {
-	if (headers["Server"].match(/Virtuoso.*/)) {
+		var srv = headers["Server"];
+		if (srv && srv.match (/Virtuoso.*/)) {
 	    self.serverType = iSPARQL.SERVER_TYPE.VIRTUOSO;
 	} else { 
 	    self.serverType = iSPARQL.SERVER_TYPE.GENERIC;
 	}
+		OAT.MSG.send ("*", "ISPARQL_SERVER_DETECTED", self.serverType);
 	OAT.Observer.notify (self.observers, null, 'detectEndpoint');
     }
 
     this.detectEndpointType = function () {
+		if (self.setEndpointTO) {
+			clearTimeout (self.setEndpointTO);
 	self.setEndpointTO = false;
+		}
 	var o = {
-	    type:OAT.AJAX.TYPE_XML
+			type:OAT.AJAX.TYPE_TEXT
 	};
 
-//	OAT.AJAX.POST (self.endpointPath, "select * where {?s ?p ?o} limit 1", self.endPointDetectCB, o);
+		// FIXME: Webkit origin policies clash with detection of Server from headers :(
+//		
+//		OAT.AJAX.HEAD (self.endpointPath, 
+//                      false,
+//                      self.endpointDetectCB, o);
     }
 
     this.setEndpointTO = false;
 
     this.setEndpoint = function (callerObj, val) {
 	self.endpointPath = val;
+		iSPARQL.Settings.endpoint = val;
 	if (self.setEndpointTO) clearTimeout (self.setEndpointTO);
 	self.setEndpointTO = setTimeout (self.detectEndpointType, 3000);
 	self.saveSes();
@@ -961,7 +974,8 @@ iSPARQL.EndpointOpts = function (optsObj) {
 	    a.push(['define get:soft',[(val)? '"'+val+'"' : val]]);
 	else {
 	    a.push(['define get:soft',[false]]);
-			//	    a.push(['define input:grab-limit',[false]]); XXX will have to test which way is best for user (what happens to existing other 
+			//	    a.push(['define input:grab-limit',[false]]); 
+            //               XXX will have to test which way is best for user (what happens to existing other 
 	    //	    a.push(['define input:grab-depth',[false]]); Sponger options if sponger is set to be off.
 	    //	    a.push(['define input:grab-all',[false]]);
 			//	    a.push(['define input:grab-seealso',[false]]);
@@ -1018,10 +1032,10 @@ iSPARQL.EndpointOpts = function (optsObj) {
 	for (var i=0;i<preds.length;i++) {
 	    var p = preds[i];
 
-	    if (p.selected) { v.push("<" + p.text + ">"); pref.push(p.text + " = " + p.value); }
+			if (p.selected) v.push("<" + p.value +">");
 	}
 
-	iSPARQL.Common.addPrefix (p);
+//		iSPARQL.Common.addPrefix (p);
 
 	a.push(['define input:grab-all',[false]]);
 	a.push(['define input:grab-seealso',v]);
@@ -1420,6 +1434,7 @@ iSPARQL.Common = {
 	if (toolkitImagesPath.match(/[^\/]$/)) { toolkitImagesPath += "/"; }
 
 	OAT.Preferences.imagePath = toolkitImagesPath;
+		OAT.Preferences.showAjax = 0;
 	OAT.AJAX.imagePath = toolkitImagesPath;
 	OAT.Anchor.imagePath = toolkitImagesPath;
 	OAT.WebDav.options.imagePath = toolkitImagesPath;
@@ -1489,7 +1504,7 @@ iSPARQL.Common = {
 							  OAT.MSG.send (self,"iSPARQL_USER_PREF_CHANGE",false);
 						  });
 
-		if (iSPARQL.serverConn.isConnected) {
+/*		if (iSPARQL.serverConn.isConnected) {
 	if (iSPARQL.serverConn.isVirtuoso) {
 	    $('about_version').innerHTML = iSPARQL.serverConn.serverVersion;
 	    $('about_date').innerHTML = iSPARQL.serverConn.serverBuildDate;
@@ -1502,19 +1517,29 @@ iSPARQL.Common = {
 		else {
 			$('about_version').innerHTML = 'unknown';
 			$('about_date').innerHTML = 'unknown';
+		} */
+
+		if (iSPARQL.Settings.sparqlCxmlInstalled) {
+			OAT.Dom.show ('cxml_raw_lnk_c');
 		}
 
+        $('about_version').innerHTML = iSPARQL.Settings.isparql_version;
 	$('about_oat_version').innerHTML = OAT.Preferences.version;
 	$('about_oat_build').innerHTML = OAT.Preferences.build;
 	$('throbber').src = OAT.Preferences.imagePath + "Dav_throbber.gif";
 
+		var iridbstats = OAT.IRIDB.getStats();
+		$('iridb_stats').innerHTML = iridbstats.iriCount;
+
+
+		// FIXME: only shows triple_count, etc. stats on the latest rdfstore.
+
 	OAT.MSG.attach ("*", 
 			"OAT_RDF_STORE_LOADED", 
 			function (m,s,l) {	
 			    var iridbstats = OAT.IRIDB.getStats();
 			    $('iridb_stats').innerHTML = iridbstats.iriCount;
 			    $('triple_count').innerHTML = m.getTripleCount();
-							$('item_count').innerHTML = m.getGraphCount();
 			    $('label_count').innerHTML = m.getLabelCount();
 			    $('label_proc_count').innerHTML = m.getLabelProcCount();
 			});
@@ -1539,19 +1564,21 @@ iSPARQL.Common = {
  			OAT.Dom.show('queryopts');
 			OAT.Dom.show('queryMetaData');
 			OAT.Dom.show('controls');
+				OAT.Dom.hide('data_links')
 	    }
 	    else if (newIndex == 1) { // Advanced
 			OAT.Dom.hide ('qry_type_ctls');
  			OAT.Dom.show('queryopts');
 			OAT.Dom.show('queryMetaData');
 			OAT.Dom.show('controls');
+				OAT.Dom.hide('data_links')
 	    }
 	    else {	// Result tab
 			OAT.Dom.hide('qry_type_ctls');
  			OAT.Dom.hide('queryopts');
 			OAT.Dom.hide('queryMetaData');
 			OAT.Dom.hide('controls');
-		
+				OAT.Dom.show('data_links')
 	    }
 	}
 
@@ -1574,14 +1601,17 @@ iSPARQL.Common = {
 	tab = new OAT.Tab ("main_col",
 						   {//dockMode:false,
 							//dockElement:"tabs",
-					goCallback:tab_goCallback,
+							goCallback:tab_goCallback
 							//onDock:onDock,
 							//onUnDock:onUnDock,
 							//dockWindowWidth:1000,
 							//dockWindowHeight:600
 						   });
 
+		if (OAT.Browser.hasSVG) {
 	tab_qbe =     tab.add ("tab_qbe",    "page_qbe");
+		}
+
 	tab_query =   tab.add ("tab_query",  "page_query");
 	tab_results = tab.add ("tab_results","page_results");
 
@@ -1596,6 +1626,7 @@ iSPARQL.Common = {
 
 	OAT.Event.attach ("browse_btn", "click", iSPARQL.Common.fileRef);
 
+		if (OAT.Browser.hasSVG) {
 	var loadToQBE = OAT.Dom.create("li",{},"nav");
 	loadToQBE.title = 'Load query into QBE';
 	var img = OAT.Dom.create("img");
@@ -1628,28 +1659,17 @@ iSPARQL.Common = {
 		       });
 
 	OAT.Dom.append([qe.dom.ul,loadToAdvanced],[loadToAdvanced,img]);
-
-	/* qbe_unsupp */
-	iSPARQL.dialogs.qbe_unsupp = new OAT.Dialog("SVG Not Available","qbe_unsupported_div",{width:400,modal:1});
-
-	iSPARQL.dialogs.qbe_unsupp.ok = function() {
-	    tab.go(1);
-	    iSPARQL.dialogs.qbe_unsupp.hide();
 	}
 
-	iSPARQL.dialogs.qbe_unsupp.cancel = iSPARQL.dialogs.qbe_unsupp.ok;
-
 	iSPARQL.StatusUI.statMsg ("Prefixes&#8230;");
 	var sel_elm = $("prefix");
 
-	for (var i=0;i<window.defaultPrefixes.length;i++) {
-	    var p_obj = window.defaultPrefixes[i];
-	    if (! (!!p_obj.hidden)) {
-		var opt_val = "PREFIX " + p_obj.label + ": <" + p_obj.uri + ">";
-		var opt_ct  = p_obj.label.toUpperCase();
+		for (var i=0;i<iSPARQL.Defaults.namespaces.length;i++) {
+			var p_obj = iSPARQL.Defaults.namespaces[i];
+			var opt_val = "PREFIX " + p_obj[1] + ": <" + p_obj[0] + ">";
+			var opt_ct  = p_obj[1].toUpperCase();
 		var opt_elm = OAT.Dom.option (opt_ct, opt_val, sel_elm);
 	    }
-	}
 
 	iSPARQL.StatusUI.statMsg ("MetaData UI&#8230;");
 	iSPARQL.mdUI = new iSPARQL.MetaDataOptsUI (iSPARQL.mdOpts,
@@ -1709,19 +1729,36 @@ iSPARQL.Common = {
 	// Help menu dismiss button
 
 	/* build info */
-	$("foot_r").innerHTML += " OAT Version " + OAT.Preferences.version + " Build " + OAT.Preferences.build;
-
+//		$("foot_r").innerHTML += " OAT Version " + OAT.Preferences.version + " Build " + OAT.Preferences.build;
 
 		$('default-graph-uri').value = iSPARQL.Settings.graph;
 		$('query').value = iSPARQL.Settings.query;
 
 		$('maxrows').value = iSPARQL.Settings.maxrows;
-	OAT.Event.attach ('maxrows', 'change',
+
+		OAT.Event.attach ('maxrows', 'blur',
 					function () {
 						var n = parseInt($v('maxrows'));
 							  iSPARQL.Settings.maxrows = isNaN(n) ? 0 : n; 
 						});
 
+		OAT.MSG.attach ("*", "iSPARQL_SERVER_DETECTED", 
+						function (m,s,l) { 
+							if (l == iSPARQL.SERVER_TYPE_VIRTUOSO) {
+								$('timeout').disabled=false;
+							}
+							else {
+								$('timeout').disabled=true;
+							}
+						});
+
+		$('timeout').value = iSPARQL.Settings.timeout ? iSPARQL.Settings.timeout : '';
+		OAT.Event.attach ('timeout', 'blur',
+						  function () {
+							  var n = parseInt($v('timeout'));
+							  iSPARQL.Settings.timeout = isNaN(n) ? false : n;
+						  });
+
 		enable_if_ubiq ($('ubiq_gem'));
 		
 		page_w = OAT.Dom.getWH('page')[0] - 20;
@@ -2004,7 +2041,7 @@ iSPARQL.Common = {
 
     setData:function(data) {
 	iSPARQL.dataObj.data = data;
-		OAT.Debug.log(0,'setData:' + iSPARQL.dataObj.data);
+		OAT.Debug.log(0,'setData');
     },
 
     // sessionStorage handling
@@ -2085,6 +2122,9 @@ iSPARQL.Common = {
 		if (qp) iSPARQL.Settings.qp_override = qp;
 		if (p['__DEBUG']) iSPARQL.Settings.debug = true;
 		if (p['maxrows']) iSPARQL.Settings.maxrows = parseInt(p['maxrows']);
+		if (p['timeout']) iSPARQL.Settings.timeout = parseInt(p['timeout']);
+		if (p['amode']) iSPARQL.Settings.anchorMode = parseInt(p['amode']);
+
     },
 
 
@@ -2130,6 +2170,7 @@ iSPARQL.Common = {
 		iSPARQL.dataObj.prefixes = []; // XXX
 	iSPARQL.dataObj.pragmas = [];
 		iSPARQL.dataObj.maxrows = iSPARQL.Settings.maxrows;
+		iSPARQL.dataObj.timeout = iSPARQL.Settings.timeout;
 	iSPARQL.dataObj.canvas = false;
 		iSPARQL.dataObj.sponge = iSPARQL.Settings.sponge;
 		iSPARQL.dataObj.tab = iSPARQL.Settings.tab;
@@ -2237,7 +2278,8 @@ function tool_put_line_start(txt){
     }
     query.value = res;
     //alert(res.charAt(start - 1 - OAT.Browser.isIE));
-    if (!((res.charAt(start - 1 - OAT.Browser.isIE) == "\n" || start == 0) && start != end)) start = start + txt.length;
+    if (!((res.charAt(start - 1 - OAT.Browser.isIE) == "\n" || start == 0) && start != end)) 
+      start = start + txt.length;
 
     if (cnt > 1) end = end + (cnt * txt.length) - (OAT.Browser.isIE * (cnt - 1));
     else end = end + txt.length;
