@@ -1,5 +1,5 @@
 /*
- *  $Id: qbe.js,v 1.36.2.11 2010/07/01 11:53:53 source Exp $
+ *  $Id: qbe.js,v 1.36.2.14 2011/09/05 15:27:50 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -427,7 +427,7 @@ iSPARQL.QBE = function (def_obj) {
 	dataObj.endpointOpts.pragmas = iSPARQL.endpointOpts.pragmas;
 	dataObj.canvas = self.svgsparql.toXML();
 	dataObj.defaultGraph = $v('qbe_graph');
-	dataObj.metaDataOpts = iSPARQL.mdOpts; // XXX check IO.Save
+	dataObj.metaDataOpts = iSPARQL.mdOpts.getMetaDataObj();
 	dataObj.maxrows = iSPARQL.dataObj.maxrows;
 
 	if(qe.cacheIndex == -1) {
@@ -1479,7 +1479,8 @@ iSPARQL.QBE = function (def_obj) {
 	    sponge:$v("qbe_sponge"),
 	    endpoint:iSPARQL.endpointOpts.endpointPath,
 	    pragmas:iSPARQL.endpointOpts.pragmas,
-	    maxrows:iSPARQL.dataObj.maxrows
+	    maxrows:iSPARQL.dataObj.maxrows,
+	    view:0
 	}
 	iSPARQL.recentQueryUI.addQuery (p.query);
 	
@@ -2152,7 +2153,7 @@ iSPARQL.QBE = function (def_obj) {
 	} else { OAT.Dom.hide("qbe_return_btn");}
 
     } else {
-	if (self.svgsparql) { self.loadFromString(self.defaults.query); }
+	if (self.svgsparql) { self.loadFromString(self.defaults.query); self.svgsparql.reposition();}
 	$('qbe_graph').value = self.defaults.graph;
 	OAT.Dom.hide("qbe_return_btn");
     }
