@@ -1,5 +1,5 @@
 /*
- *  $Id: statusui.js,v 1.1.2.3 2010/06/28 09:33:06 source Exp $
+ *  $Id: statusui.js,v 1.1.2.5 2011/02/03 10:28:23 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -24,8 +24,25 @@ iSPARQL.StatusUI = {
 	else
 	    $("statMsgElem").innerHTML = msg;
     },
+    init: function () {
+	OAT.Dom.show ("splashThrobber");
+    },
     errMsg: function (msg) {
 	$("statMsgErr").innerHTML = msg;
+    },
+    addCustomTemplate: function (div) {
+	var c = this.newTplCtr ();
+	OAT.Dom.append ([c, div]);
+    },
+    newTplCtr: function () {
+	var custCtr = OAT.Dom.create ("div",{className:"statusUICustCtr"});
+	OAT.Dom.append(["statMsgMain",custCtr]);
+	return custCtr;
+    },
+    absorb: function (ctr) {
+	var c = OAT.Dom.create ("div", {className: "statusUIAbsorbedCtr"})
+	OAT.Dom.unlink (ctr);
+	OAT.Dom.append ([c, ctr]);
     }
 };
 
