@@ -1,9 +1,9 @@
 /*
- *  $Id: ghostdrag.js,v 1.13.2.1 2010/01/14 15:57:39 source Exp $
+ *  $Id: ghostdrag.js,v 1.13.2.4 2010/04/06 16:46:12 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2005-2009 OpenLink Software
+ *  Copyright (C) 2005-2010 OpenLink Software
  *
  *  See LICENSE file for details.
  */
@@ -42,20 +42,20 @@ OAT.GhostDragData = {
 				obj.callback(t[0],x,y);
 			}
 		}
-		if (ok) { 
+		if (ok) {
 			/* mouseup at correct place - remove element */
 			OAT.Dom.unlink(elm);
-			OAT.MSG.send(obj,OAT.MSG.GD_END,elm);
+			OAT.MSG.send(obj,"GD_END",elm);
 		} else {
 			/* mouseup at wrong place - let's animate it back */
-			OAT.MSG.send(obj,OAT.MSG.GD_ABORT,elm);
+			OAT.MSG.send(obj,"GD_ABORT",elm);
 			obj.onFail();
 			var coords = OAT.Dom.position(obj.originalElement);
 			var x = coords[0];
 			var y = coords[1];
 			var sf = function() { OAT.Dom.unlink(elm); }
 			var anim = new OAT.AnimationPosition(elm,{speed:10,delay:10,left:x,top:y});
-			OAT.MSG.attach(anim.animation,OAT.MSG.ANIMATION_STOP,sf);
+			OAT.MSG.attach(anim.animation,"ANIMATION_STOP",sf);
 			anim.start();
 		}
 	}, /* OAT.GhostDragData.up() */
@@ -71,7 +71,7 @@ OAT.GhostDragData = {
 			elm.style.zIndex = 2000;
 			if (obj.process) { obj.process(elm); }
 			obj.pending = 0;
-			OAT.MSG.send(obj,OAT.MSG.GD_START,elm);
+			OAT.MSG.send(obj,"GD_START",elm);
 		}
 		/*
 			selection sukz. detect it and remove!
@@ -81,8 +81,8 @@ OAT.GhostDragData = {
 		/* ok, now move */
 		var offs_x = event.clientX - elm.mouse_x;
 		var offs_y = event.clientY - elm.mouse_y;
-		var new_x = parseInt(OAT.Dom.style(elm,"left")) + offs_x;
-		var new_y = parseInt(OAT.Dom.style(elm,"top")) + offs_y;
+		var new_x = parseInt(OAT.Style.get(elm,"left")) + offs_x;
+		var new_y = parseInt(OAT.Style.get(elm,"top")) + offs_y;
 		elm.style.left = new_x + "px";
 		elm.style.top = new_y + "px";
 		elm.mouse_x = event.clientX;
@@ -110,7 +110,7 @@ OAT.GhostDrag = function() {
 	this.callbacks = [];
 	this.targets = [];
 	this.pending = 0; /* mouse is down, waiting for move to appear */
-	
+
 	this.addSource = function(node,process,callback) {
 		var elm = $(node);
 		self.sources.push(elm);
@@ -119,7 +119,7 @@ OAT.GhostDrag = function() {
 		var cica = true;
 		var ref = function(event) {
 			OAT.Event.prevent(event);
-			var index = self.sources.find(elm);
+			var index = self.sources.indexOf(elm);
 			if (index == -1) return;
 			var x = event.clientX;
 			var y = event.clientY;
@@ -127,22 +127,22 @@ OAT.GhostDrag = function() {
 		}
 		OAT.Event.attach(elm,"mousedown",ref);
 	}
-	
+
 	this.delSource = function(node) {
 		var elm = $(node);
-		var index = self.sources.find(elm);
+		var index = self.sources.indexOf(elm);
 		if (index == -1) { return; }
 		self.sources.splice(index,1);
 		self.processes.splice(index,1);
 		self.callbacks.splice(index,1);
 	}
-	
+
 	this.clearSources = function() {
 		self.sources = [];
 		self.processes = [];
 		self.callbacks = [];
 	}
-	
+
 	this.addTarget = function(node,customTest,isLast) {
 		var elm = $(node);
 		var newTriple = [elm,customTest,isLast];
@@ -153,7 +153,7 @@ OAT.GhostDrag = function() {
 			self.targets.push(newTriple);
 		}
 	}
-	
+
 	this.delTarget = function(node) {
 		var elm = $(node);
 		var index = -1;
@@ -178,7 +178,7 @@ OAT.GhostDrag = function() {
 		var coords = OAT.Dom.position(elm);
 		obj.style.left = coords[0]+"px";
 		obj.style.top = coords[1]+"px";
-		OAT.Style.opacity(obj,0.5);
+		OAT.Style.set(obj,{opacity:0.5});
 		obj.appendChild(elm.cloneNode(true));
 		obj.mouse_x = x;
 		obj.mouse_y = y;
@@ -189,4 +189,3 @@ OAT.GhostDrag = function() {
 
 OAT.Event.attach(document,"mousemove",OAT.GhostDragData.move);
 OAT.Event.attach(document,"mouseup",OAT.GhostDragData.up);
-OAT.Loader.featureLoaded("ghostdrag");
