@@ -1,9 +1,9 @@
 /*
- *  $Id: rdfmini.js,v 1.7.2.2 2010/01/14 15:57:40 source Exp $
+ *  $Id: rdfmini.js,v 1.7.2.12 2011/08/29 10:22:17 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
- *  Copyright (C) 2005-2009 OpenLink Software
+ *  Copyright (C) 2005-2010 OpenLink Software
  *
  *  See LICENSE file for details.
  */
@@ -23,7 +23,7 @@ OAT.RDFMini = function(div,optObj) {
 			["browser","Raw Triples",{removeNS:true}],
 			["triples","Grid view",{}],
 			["svg","SVG Graph",{}],
-			["map","Yahoo Map",{provider:2}],
+			["map","Google Map",{provider:OAT.Map.TYPE_G3}],
 			["timeline","Timeline",{}],
 			["images","Images",{}],
 			["tagcloud","Tag Cloud",{}]
@@ -31,15 +31,18 @@ OAT.RDFMini = function(div,optObj) {
 		querySearchURI:false,
 		showSearch:true,
 		imagePath:OAT.Preferences.imagePath,
-		endpoint:"/sparql?query="
+	endpoint:"/sparql?query=",
+	store: false,
+	sel_ctr: false // optional place view selector in custom place
 	}
+
 	for (var p in optObj) { this.options[p] = optObj[p]; }
-	
+
 	this.parent = $(div);
-	this.content = OAT.Dom.create("div",{},"rdf_mini");
+	this.content = OAT.Dom.create("div",{className:"rdf_mini"});
 	this.tabs = [];
 	this.select = false;
-	
+
 	this.executeSparql = function(template,replace) {
 		var str = template;
 		for (var p in replace) {
@@ -49,7 +52,7 @@ OAT.RDFMini = function(div,optObj) {
 		var url = self.options.endpoint+encodeURIComponent(str)+"&format=rdf";
 		self.open(url);
 	}
-	
+
 	this.search = function(str) {
 		var s = (str ? str : $v(self.searchInput));
 		if (!s.trim()) { return; }
@@ -65,7 +68,7 @@ OAT.RDFMini = function(div,optObj) {
 			self.executeSparql('SELECT ?s ?p ?o WHERE { ?s ?p ?o . ?o bif:contains "\'{query}\'"}',{"{query}":s});
 		}
 	}
-	
+
 	this.init = function() {
 		OAT.Dom.clear(self.parent);
 		self.throbber = OAT.Dom.create("img",{styleFloat:"right",cssFloat:"right",cursor:"pointer"});
@@ -73,7 +76,7 @@ OAT.RDFMini = function(div,optObj) {
 		OAT.Event.attach(self.throbber,"click",OAT.AJAX.abortAll);
 		self.parent.appendChild(self.throbber);
 		OAT.Dom.hide(self.throbber);
-		
+
 		var s = OAT.Dom.create("div");
 		var inp = OAT.Dom.create("input",{verticalAlign:"middle"});
 		inp.type = "text";
@@ -85,7 +88,7 @@ OAT.RDFMini = function(div,optObj) {
 		OAT.Event.attach(btn,"click",self.search);
 		OAT.Event.attach(inp,"keypress",function(e) { if (e.keyCode == 13) { self.search(); } });
 		self.searchInput = inp;
-		
+
 		if (!self.options.tabs.length) {
 			var note = new OAT.Notify();
 			var msg = "No visualizations available!";
@@ -102,15 +105,43 @@ OAT.RDFMini = function(div,optObj) {
 			}
 			OAT.Event.attach(s,"change",self.redraw);
 			self.select = s;
-			OAT.Dom.append([self.parent,OAT.Dom.text("Visualization: "),s]);
+
+	    var sel_ctr;
+
+	    if (!self.options.sel_ctr) 
+		sel_ctr = OAT.Dom.create("div",{className:"rdfmini_view_sel_ctr"});
+	    else
+		sel_ctr = self.options.self_ctr;
+
+	    var sel_lbl = OAT.Dom.create("label");
+
+	    OAT.Dom.append([self.parent,sel_ctr],[sel_ctr,sel_lbl,s]);
+	    sel_lbl.innerHTML = "View:";
 		} else {
 			var t = self.options.tabs[0];
 			var obj = new OAT.RDFTabs[t[0]](self,t[2]);
 			self.tabs.push(obj);
 		}
+
+	OAT.MSG.attach ("*","MAP_NOTHING_TO_SHOW", function (_s,_m,_e) { s.selectedIndex = 0;self.redraw() })
+	
+	var ua = navigator.userAgent;
+
+	if (ua.indexOf('iPhone') != -1 || ua.indexOf('Android') != -1 ) {
+	    vp = OAT.Dom.getViewport();
+
+	    self.content.style.width = vp[1]+'px';
+	    self.content.style.height = vp[0]+'px';
+	    console.info ('viewport: '+vp[1]+'x'+vp[0]);
+	    self.content.scrollIntoView(true);
+	}
+//	else {
+//	    self.content.style.height = '600px';
+//	}
+
 		self.parent.appendChild(self.content);
 	}
-	
+
 	this.redraw = function() { /* change vis */
 	    var index = 0;
 	    if (self.select) { index = self.select.selectedIndex; }
@@ -120,36 +151,76 @@ OAT.RDFMini = function(div,optObj) {
 	    self.tabs[index].redraw();
 	    var et = {};
 	    et.tabIndex = index;
-	    OAT.MSG.send (self,OAT.MSG.RDFMINI_VIEW_CHANGED,et);
+	    OAT.MSG.send (self,"RDFMINI_VIEW_CHANGED",et);
 	}
-	
+
 	this.open = function(url) { /* open url */
 		self.store.clear();
 		self.store.addURL(url);
 	}
-	
+
 	this.reset = function() { /* url arrived */
 		for (var i=0;i<self.tabs.length;i++) {
 			self.tabs[i].reset();
 		}
 		self.redraw();
 	}
-	
+
 	var ajaxStart = function() { OAT.Dom.show(self.throbber); }
 	var ajaxEnd = function() { OAT.Dom.hide(self.throbber); }
 
-	this.store = new OAT.RDFStore(self.reset,{onstart:ajaxStart,onend:ajaxEnd});
+    if (this.options.store) {
+	this.store = this.options.store;
+	this.store.options.onstart = ajaxStart;
+	this.store.options.onend = ajaxEnd;
+	this.store.reset = this.reset;
+    }
+    else this.store = new OAT.RDFStore(this.reset,{onstart:ajaxStart,onend:ajaxEnd});
+
 	this.data = self.store.data;
 
 	this.getContent = function(data_,disabledActions) {
 		var content = false;
-		var data = (typeof(data_) == "object" ? data_.uri : data_);
-		var type = self.getContentType(data);
-		
-		switch (type) {
+	    var data;
+	var label = false;
+	var ciri;
+	var _type;
+
+	    if (data_.constructor == OAT.RDFAtom)
+		switch (data_.getTag()) {
+		case OAT.RDFTag.IRI:
+		    data = data_.getIRI();
+		ciri = OAT.IRIDB.resolveCIRI(data_.getValue());
+		content = OAT.Dom.create("span");
+		var a = OAT.Dom.create("a");
+		a.innerHTML = (label ? label : ciri);
+		a.href = data;
+		content.appendChild(a);
+		self.processLink(a,data,disabledActions);
+		return content;
+		case OAT.RDFTag.LIT:
+		    data = data_.getValue();
+		    break;
+	    }
+	else if (typeof data_ == 'object') {
+	    ciri = OAT.IRIDB.resolveCIRI(data_.iid);
+	    data = OAT.IRIDB.getIRI(data_.iid);
+	    label = data_.label;
+	    content = OAT.Dom.create("span");
+	    var a = OAT.Dom.create("a");
+	    a.innerHTML = (label ? label : ciri);
+	    a.href = data;
+	    content.appendChild(a);
+	    self.processLink(a,data,disabledActions);
+	    return content;
+	}
+	
+	_type = self.getContentType(data); // Only literals should be left something that may be deref'd?
+
+	switch (_type) { // XXX this should be pruned with extreme prejudice
 			case 3:
 				content = OAT.Dom.create("img");
-				content.title = data;
+	    content.title = (label ? label : data);
 				content.src = data;
 				self.processLink(content,data);
 			break;
@@ -162,14 +233,19 @@ OAT.RDFMini = function(div,optObj) {
 			case 1:
 				content = OAT.Dom.create("span");
 				var a = OAT.Dom.create("a");
-				a.innerHTML = data;
+	    a.innerHTML = (label ? label : ciri);
 				a.href = data;
 				content.appendChild(a);
 				self.processLink(a,data,disabledActions);
 			break;
 			default:
 				content = OAT.Dom.create("span");
-				content.innerHTML = data;
+
+	    if (data.match(/(#this$|#me$)/))
+		content.innerHTML = (label ? label : ciri);
+	    else
+		content.innerHTML = (label ? label : data);
+
 				/* create dereference a++ lookups for all anchors */
 				var anchors_ = content.getElementsByTagName("a");
 				var anchors = [];
@@ -177,14 +253,14 @@ OAT.RDFMini = function(div,optObj) {
 				for (var j=0;j<anchors.length;j++) {
 					var a = anchors[j];
 					if (a.href.match(/^http/)) {
-						self.processLink(a,a.href); 
+						self.processLink(a,a.href);
 					}
 				}
 			break;
 		} /* switch */
 		return content;
 	}
-	
+
 	this.simplify = self.store.simplify;
 	this.getContentType = self.store.getContentType;
 	this.getTitle = self.store.getTitle;
@@ -193,4 +269,3 @@ OAT.RDFMini = function(div,optObj) {
 
 	this.init();
 }
-OAT.Loader.featureLoaded("rdfmini");
