@@ -1,5 +1,5 @@
 /*
- *  $Id: rdfstore.js,v 1.32.2.4 2010/06/28 09:35:57 source Exp $
+ *  $Id: rdfstore.js,v 1.32.2.9 2011/08/21 12:50:21 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -58,15 +58,23 @@ OAT.RDFStoreData = {
 OAT.RDFStore = function(tripleChangeCallback, optObj) {
     var self = this;
 
+    this.preferredClasses = {
+	'http://xmlns.com/foaf/0.1/Person': 0,
+	'http://': 1
+    };
+    
     // properties used as labels - in order of preference
 
-    this.labelProps = {"http://xmlns.com/foaf/0.1/name": 0,
-		       "http://xmlns.com/foaf/0.1/nick": 1,
-		       "http://www.w3.org/2000/01/rdf-schema#label": 2,
-		       "http://purl.org/dc/elements/1.1/title": 3,
-		       "http://dbpedia.org/property/name":4,
-		       "http://www.w3.org/2004/02/skos/core#prefLabel": 5,
-		       "http://www.w3.org/2002/12/cal/ical#summary": 6};
+    this.labelProps = {
+	"http://www.w3.org/2000/01/rdf-schema#label": 0,
+	"http://www.w3.org/2004/02/skos/core#prefLabel": 1,
+        "http://www.openlinksw.com/schemas/virtrdf#label": 1,
+	"http://xmlns.com/foaf/0.1/name": 2,
+	"http://xmlns.com/foaf/0.1/nick": 3,
+	"http://purl.org/dc/elements/1.1/title": 4,
+	"http://dbpedia.org/property/name":5,
+	"http://www.w3.org/2002/12/cal/ical#summary": 6
+    };
 
     self.labelPropLookup = []; // label predicates by iid
 
@@ -115,14 +123,14 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
     this.filtersURI = [];
     this.filtersProperty = [];
 
-    this.items = [];
+    this.graphs = [];
 
     this.getTripleCount = function () {
 	return self.data.triples.length;
     }
 
-    this.getItemCount = function () {
-	return self.items.length;
+    this.getGraphCount = function () {
+	return self.graphs.length;
     }
 
     this.getLabelCount = function () {
@@ -170,12 +178,9 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 	    if (item.type.indexOf(t[2]) == -1) item.type.push(t[2]);
     }
 
-    //
-    // FIXME
-//
 
     this.nsPrefixPredHandler = function (item, t, opt) {
-	OAT.IRIDB.addNs (t[1], t[2]);
+	OAT.IRIDB.addNs (t[0], t[2].getValue());
     }
 
     this.dequeueURL = function (url, success)
@@ -218,7 +223,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
     //
 
     for (i in self.labelProps)
-	self.addPredHandler (i, self.labelPredHandler, self.labelProps[i])
+	self.addPredHandler (i, self.labelPredHandler, self.labelProps[i]);
 
     //
     // Add predicate handler for type
@@ -313,11 +318,11 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
     }
 
     this.addXmlDoc = function(xmlDoc,href) {
-	OAT.MSG.send(self,"OAT_RDF_STORE_LOADING",{url:xmlDoc.baseURI});
-	var triples = OAT.RDF.parse (xmlDoc);
+	OAT.MSG.send(self,"OAT_RDF_STORE_LOADING",{url:href});
+	var triples = OAT.RDF.parse (xmlDoc, href);
 	var ncount = 0;
 
-	if (!!window.console) window.console.log("addXmlDoc: Got " + triples.length + " triples.");
+	if (!!window.console && !!window.__isparql_debug) window.console.log("addXmlDoc: Got " + triples.length + " triples.");
 
 	/* sanitize triples */
 /*	for (var i=0;i<triples.length;i++) {
@@ -345,7 +350,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 	if (!!window.console) window.console.log("numbers in o: " + ncount);
 */
 	self.addTriples(triples,href);
-	OAT.MSG.send(self,"OAT_RDF_STORE_LOADED",{url:xmlDoc.baseURI});
+	OAT.MSG.send(self,"OAT_RDF_STORE_LOADED",{url:href});
     }
 
     this.addTripleList = function(triples,href,title) {
@@ -361,20 +366,21 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 	    enabled:true,
 	    title:title
 	}
-	self.items.push(o);
+	
+	self.graphs.push(o);
 	self.rebuild(false);
     }
 
     this.findIndex = function(url) {
-	for (var i=0;i<self.items.length;i++) {
-	    var item = self.items[i];
+	for (var i=0;i<self.graphs.length;i++) {
+	    var item = self.graphs[i];
 	    if (item.href == url) { return i; }
 	}
 	return -1;
     }
 
     this.clear = function() {
-	self.items = [];
+	self.graphs = [];
 	self.rebuild(true);
 	OAT.MSG.send(self, "OAT_RDF_STORE_CLEARED",{});
     }
@@ -382,7 +388,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
     this.remove = function(uri) {
 	var index = self.findIndex(uri);
 	if (index == -1) { return; }
-	self.items.splice(index,1);
+	self.graphs.splice(index,1);
 	self.rebuild(true);
 	OAT.MSG.send(self,"OAT_RDF_STORE_URI_REMOVED",{uri:uri});
     }
@@ -390,35 +396,35 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
     this.enable = function(uri) {
 	var index = self.findIndex(uri);
 	if (index == -1) { return; }
-	self.items[index].enabled = true;
+	self.graphs[index].enabled = true;
 	self.rebuild(true);
 	OAT.MSG.send(self,"OAT_RDF_STORE_ENABLED",{uri:uri});
     }
 
     this.enableAll = function() {
-	for (var i=0;i<self.items.length;i++)
-	    self.enable(self.items[i].href);
+	for (var i=0;i<self.graphs.length;i++)
+	    self.enable(self.graphs[i].href);
     }
 
     this.disable = function(url) {
 	var index = self.findIndex(url);
 	if (index == -1) { return; }
-	self.items[index].enabled = false;
+	self.graphs[index].enabled = false;
 	self.rebuild(true);
 	OAT.MSG.send(self,"OAT_RDF_STORE_DISABLED",{uri:uri});
     }
 
     this.disableAll = function() {
-	for (var i=0;i<self.items.length;i++)
-	    self.disable(self.items[i].href);
+	for (var i=0;i<self.graphs.length;i++)
+	    self.disable(self.graphs[i].href);
     }
 
     this.invertSel = function() {
-	for (var i=0;i<self.items.length;i++)
-	    if (self.items[i].enabled) {
-		self.disable(self.items[i].href);
+	for (var i=0;i<self.graphs.length;i++)
+	    if (self.graphs[i].enabled) {
+		self.disable(self.graphs[i].href);
 	    } else {
-		self.enable(self.items[i].href);
+		self.enable(self.graphs[i].href);
 	    }
     }
 
@@ -448,7 +454,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 	    var p = triple[1];
 	    var o = triple[2];
 
-	    if (!!window.console) window.console.log ("<"+OAT.IRIDB.getIRI(s)+">"+"<"+OAT.IRIDB.getIRI(p)+">"+o.toString());
+	    //	    if (!!window.console) window.console.log ("<"+OAT.IRIDB.getIRI(s)+">"+"<"+OAT.IRIDB.getIRI(p)+">"+o.toString());
 
 	    var cnt = self.data.all.length;
 
@@ -504,8 +510,8 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 
 	if (complete) { /* complete = all */
 	    self.data.all = [];
-	    for (var i=0;i<self.items.length;i++) { // Item is sort-of synonymous to graph
-		var item = self.items[i];
+	    for (var i=0;i<self.graphs.length;i++) { // Item is sort-of synonymous to graph
+		var item = self.graphs[i];
 		if (item.enabled) { todo.push([item.triples,item.href,item.title]); }
 	    }
 	} else { /* not complete - only last item */
@@ -513,7 +519,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 		var item = self.data.all[i];
 		conversionTable[item.uri] = item;
 	    }
-	    var item = self.items[self.items.length-1];
+	    var item = self.graphs[self.graphs.length-1];
 	    todo.push([item.triples,item.href,item.title]);
 	}
 	for (var i=0;i<todo.length;i++) {
@@ -537,7 +543,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 		    }
 		}
 	    } /* predicates */
-	} /* items */
+	} /* graphs */
 
 	/* 3. apply filters: create self.data.structured + 4. convert filtered data back to triples */
 
@@ -619,6 +625,8 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 
 	self.data.triples = [];
 	
+// XXX
+	
 	for (var i=0;i<self.data.structured.length;i++) {
 	    var item = self.data.structured[i];
 	    for (var p in item.preds) {
@@ -697,21 +705,7 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 
 	_iid = (iid.constructor == OAT.RDFAtom) ? iid._value : iid;
 
-	var ciri = OAT.IRIDB.resolveCIRI (_iid);
-
-	if (!!ciri) 
-	    return ciri;
-	else 
-	{
-	    var iri = OAT.IRIDB.getIRI(_iid);
-
-	    spliri = OAT.IRIDB.splitIRI(iri);
-
-	    if (!!spliri) 
-		return (spliri[1]);
-	    else 
-		return iri;
-	    }
+	return OAT.IRIDB.resolveCIRI (_iid);
 	    }
 
     this.getTitle = function(item) {
@@ -740,10 +734,5 @@ OAT.RDFStore = function(tripleChangeCallback, optObj) {
 
     this.simplify = function(str) {
 	return (self.getCIRIorSplit (OAT.IRIDB.insertIRI(str)));
-//	var r = str.match(/([^\/#]+)[\/#]?$/);
-//	if (r && r[1] == "this") {
-//	    r = str.match(/([^\/#]+)#[^#]*$/);
-//	}
-//	return (r ? r[1] : str);
     }
 }
