@@ -1,5 +1,5 @@
 /*
- *  $Id: rdftabs.js,v 1.63.2.20 2011/03/29 11:02:10 source Exp $
+ *  $Id: rdftabs.js,v 1.63.2.25 2011/09/20 12:03:04 source Exp $
  *
  *  This file is part of the OpenLink Software Ajax Toolkit (OAT) project.
  *
@@ -159,15 +159,7 @@ OAT.RDFTabs.browser = function(parent,optObj) {
 	    var p_ctr = OAT.Dom.create("div",{className:"rdf_p"});
 	    var p_a = OAT.Dom.create("a");
 	    var p_iri = OAT.IRIDB.getIRI(p);
-	    var p_ciri = OAT.IRIDB.getCIRIByID(p);
-
-	    if (!p_ciri) {
-		var spiri = OAT.IRIDB.splitIRI(p_iri);
-		if (!spiri) 
-		    p_ciri = p_iri;
-		else
-		    p_ciri = spiri[0];
-	    }
+	    var p_ciri = OAT.IRIDB.resolveCIRI(p);
 
 	    p_a.href = p_iri;
 
@@ -381,9 +373,10 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
 
     this.options = {
 	limit:5,
-	description:"This module is used to navigate through locally cached data, one resource at a time. Note that filters are not applied here.",
+	description:"This module is used to navigate through locally cached data, one resource at a time. Filters are not applied.",
 	desc:"Navigate through locally cached data (Filters not applied)"
     }
+
     for (var p in optObj) { self.options[p] = optObj[p]; }
 
     this.initialized = false;
@@ -473,6 +466,8 @@ OAT.RDFTabs.navigator = function(parent,optObj) {
     this.getTypeObject = function() { /* object of resource types */
 	var obj = [];
 	var noTypeArr = [];
+
+	OAT.IRIDB.insertIRI('http://openlinksw.com/schemas/oat/rdftabs#','oatrdftabs');
 	var ntIID = OAT.IRIDB.insertIRI('http://openlinksw.com/schemas/oat/rdftabs#isReferencedBy');
 
 	var data = self.parent.data.all;
@@ -1040,13 +1035,19 @@ OAT.RDFTabs.triples = function(parent,optObj) {
 		iid = atom; // this is a plain IID value
 	}
 
-	var a = OAT.Dom.create("a");
+
+	try { 	// Dirty data does exist, you see...
 	var iri = decodeURIComponent(OAT.IRIDB.getIRI(iid));
-	a.innerHTML = self.parent.store.getCIRIorSplit(iid);
-	a.href = iri;
+            var col_v_elm = OAT.Dom.create("a");
+	    col_v_elm.innerHTML = self.parent.store.getCIRIorSplit(iid);
+	    col_v_elm.href = iri;
+	    self.parent.processLink(col_v_elm, iri);
+        } catch (e) {
+	    col_v_elm = OAT.Dom.create("span",{className:"error_col"});
+            col_v_elm.innerHTML = OAT.IRIDB.getIRI(iid) + " (invalid URI)";
+	}
 	OAT.Dom.clear(v);
-	v.appendChild(a);
-	self.parent.processLink(a,iri);
+	v.appendChild(col_v_elm);
     }
 
     this.redraw = function() {
@@ -1100,9 +1101,9 @@ OAT.RDFTabs.svg = function(parent,optObj) {
 	var cnt = self.parent.data.triples.length;
 
 	if (cnt > self.options.limit) {
-	    var note = new OAT.Notify();
+	    var note = new OAT.Notify(false, {notifyType: 2});
 	    var msg = "Note: Display limited to " + self.options.limit + " triples."
-	    note.send (msg,{delayIn:10,width:350,height:50,timeout:5000});
+	    note.send (msg,{delayIn:10,timeout:5000});
 	    cnt = self.options.limit;
 	}
 
@@ -1308,6 +1309,10 @@ OAT.RDFTabs.PointList = function (opts) {
     }
 }
 
+//
+// Sends MAP_NOTHING_TO_SHOW if there's nothing to show on map
+//
+
 OAT.RDFTabs.map = function(parent,optObj) {
     var self = this;
     OAT.RDFTabs.parent(self);
@@ -1320,7 +1325,9 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	desc:"Plots all geodata onto a map",
 	clickPopup:true,
 	hoverPopup:true,
-	height: "600px"
+	height: "600px",
+	useMobileOpts:false,
+	supportedMobileDetected:false
     }
 
     for (var p in optObj) { self.options[p] = optObj[p]; }
@@ -1337,6 +1344,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
     if (useragent.indexOf('iPhone') != -1 || useragent.indexOf('Android') != -1 ) {
 	self.elm.style.width = '100%';
 	self.elm.style.height = '100%';
+	self.options.supportedMobileDetected = true;
     } else {
     this.elm.style.height = self.options.height;
     }
@@ -1358,7 +1366,8 @@ OAT.RDFTabs.map = function(parent,optObj) {
 						     "http://www.w3.org/2003/01/geo/latitude", 
 						     "http://www.w3.org/2006/vcard/ns#latitude",
 						     "http://www.w3.org/2001/vcard-rdf/3.0#latitude",
-						     "http://dbpedia.org/property/lat"]);
+						     "http://dbpedia.org/property/lat",
+						     "http://www.openlinksw.com/schemas/zillow#latitude"]);
 
     this.lonProperties    = OAT.IRIDB.insertIRIArr (["http://www.w3.org/2003/01/geo/lng",
 						     "http://www.w3.org/2003/01/geo/wgs84_pos#long",
@@ -1367,7 +1376,8 @@ OAT.RDFTabs.map = function(parent,optObj) {
 						     "http://www.w3.org/2003/01/geo/longitude",
 						     "http://www.w3.org/2006/vcard/ns#longitude",
 						     "http://www.w3.org/2001/vcard-rdf/3.0#longitude",
-						     "http://dbpedia.org/property/long"]);
+						     "http://dbpedia.org/property/long",
+						     "http://www.openlinksw.com/schemas/zillow#longitude"]);
 
     this.lookupProperties = OAT.IRIDB.insertIRIArr (["http://xmlns.com/foaf/0.1/name",
 						     "http://xmlns.com/foaf/0.1/location"]); /* interesting to be put into lookup pin */
@@ -1434,18 +1444,19 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	var coords = [0,0];
 
 	if (item.type) {
-	    for (var i=0;i<item.type.length;i++)
+	    for (var i=0;i<item.type.length;i++) {
 		if (self.pointTypes.indexOf(item.type[i].getValue()) != -1)
 	    coords = self.extractCoords (item.preds);
+	}
 	}
 
 	    for (var p in preds) {
 		var pred = preds[p];
 	    if (!!(p = parseInt(p))) {	    
 		if (self.keyProperties.indexOf(p) != -1) { 
+		    if (pred[0] instanceof OAT.RDFAtom && pred[0].isLit())
 		    pointResource = pred[0].getValue(); 
 		} /* resource containing geo coordinates */
-		
 		if (self.locProperties.indexOf(p) != -1) { 
 		    locValue = pred[0].getValue(); 
 		} /* resource containing geo coordinates */
@@ -1470,7 +1481,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	
 	if (!pointResource && locValue) { /* geocode location */
 	    if (!!window.console) window.console.log ('geocoding: '+locValue);
-		self.geoCode(locValue,item);
+	    self.geoCode(locValue,item);
 		return;
 	    }
 	
@@ -1480,7 +1491,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 		    coords[0] = cmatches[2];
 		    coords[1] = cmatches[1];
 		    if (coords[0] == 0 || coords[1] == 0) { return; }
-		    self.attachMarker(coords,item);
+		self.attachMarker(coords,item);
 		    return;
 		}
 	    
@@ -1494,8 +1505,6 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	    }
 	    }
 	
-	if (pointResource.isLit()) { return; } /* not a reference */
-
 	    self.usedBlanknodes.push(pointResource);
 
 	    /* normal marker add */
@@ -1503,7 +1512,7 @@ OAT.RDFTabs.map = function(parent,optObj) {
 
 	if (coords[0] == 0 || coords[1] == 0) { return; }
 
-	self.attachMarker(coords,item);
+	self.attachMarker(coords, item);
     } /* tryItem */
 
     this.trySimple = function(item) {
@@ -1594,13 +1603,54 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	return false;
 	}
 
+    // an item isCoordinateContainer if it's referred by another item and
+    // its type property values are all geo point types and/or it only has geo coordinate properties
+
+    this.isCoordinateContainer = function (item) {
+	type_iid = OAT.IRIDB.getIRIID("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+	if (!item.back.length) 
+	    return false;
+	var preds = item.preds;
+	var nc_cnt = 0;
+	for (var p in preds) {
+	    var pVal = parseInt(p);
+	    if (pVal == type_iid) { // go through types - pointTypes don't count
+		for (var i=0;i<preds[p].length;i++) {
+		    if (self.pointTypes.find(preds[p][i].getIID()) == -1)
+			return false;
+		}
+	    } else {
+		if (self.lonProperties.find(pVal) != -1 ||
+		    self.latProperties.find(pVal) != -1 || 
+		    self.keyProperties.find(pVal) != -1) {
+		    continue;
+		}
+		else return false; // got a non-geo-property
+	    }
+	}
+	return true;
+    }
+
+    this.drawReferences = function (item, container) {
+        var p_table = OAT.Dom.create ("table");
+	for (i=0;i<item.back.length;i++) { // include backreferences;
+	    var predC = OAT.Dom.create("tr",{className:"predicate"});
+	    var predT = OAT.Dom.create("td",{className:"pred_title"});
+	    predT.innerHTML = "Referenced by";
+	    var predV = OAT.Dom.create("td",{className:"pred_value"});
+	    var content = self.parent.getContent(item.back[i], "replace");
+	    OAT.Dom.append([predV,content],[predC,predT,predV], [p_table,predC]);
+	}
+	OAT.Dom.append ([container,p_table]);
+    }
+
 //
 // Return marker content for item
 //    
 
-    this.drawAllProps = function (item) {
-	var div = OAT.Dom.create ("div",{className:"all_props_ctr"});
+    this.drawAllProps = function (item, container) {
 	var preds = item.preds;
+        var p_table = OAT.Dom.create ("table");
 	for (var p in preds) {
 	    if (self.markerPredBlacklist.find(parseInt(p)) != -1) continue; // Not all predicates are created equal
 	    var pred = preds[p];
@@ -1611,26 +1661,53 @@ OAT.RDFTabs.map = function(parent,optObj) {
 		predT.innerHTML = simple;
 		var predV = OAT.Dom.create("td",{className:"pred_value"});
 		var content = self.parent.getContent(pred[0],"replace");
-		OAT.Dom.append([predV,content],[predC,predT,predV],[div,predC]);
+		OAT.Dom.append([predV,content],[predC,predT,predV],[p_table,predC]);
 	    } /* only interesting data */
 	} /* for all predicates */
-	return div;
+        OAT.Dom.append([container,p_table])
+	return container;
     }
 
+    this.showMarkerPopup = function () {
+
+    }
+
+    this.hideMarkerPopup = function () {
+
+    }
+
+    this.makeAnchor = function (ctr, content, href) {
+	var a = OAT.Dom.create ("a");
+	a.href=href;
+        
+    }
     this.drawMarker = function (item) {
+	if (self.isCoordinateContainer(item))
+	    var s_item = item.back[0];
+	else 
+	    s_item = false;
+
 	var titleH = OAT.Dom.create("h2",{className:"markerTitle"});
 
-	var title = self.parent.getTitle(item);
+	var title = self.parent.getTitle(s_item ? s_item : item);
 	var titleHref='';
+	var popup_ctr;
 
+	if (self.options.supportedMobileDetected && self.options.useMobileOpts) {
+	    var titleA = OAT.Dom.create ("a");
+	    OAT.Dom.attach (titleA,"click",self.showMarkerPopup);
+	}
+	else {
 	if (title.match(/^http/i)) {
-	    self.parent.processLink(titleH,title);
-	    titleH.style.cursor = "pointer";
-	titleH.innerHTML = title;
+	        var titleA = OAT.Dom.create("a",{href:titleHref,target:"_blank"});
+		self.parent.processLink(titleA, title);
+		titleA.innerHTML = title;
+		OAT.Dom.append ([titleH, titleA]);		
 	} else {
-	    var titleHref = self.parent.getURI(item);
+		var titleHref = self.parent.getURI(s_item ? s_item : item);
 	    if (titleHref) {
 		var titleA = OAT.Dom.create("a",{href:titleHref,target:"_blank"});
+		    self.parent.processLink(titleA, titleHref);
 		titleA.innerHTML = title;
 		OAT.Dom.append ([titleH, titleA]);
 	    } else {
@@ -1638,23 +1715,27 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	    }
 	}
 
-	var ctr = OAT.Dom.create("div",{overflow:"auto",className:'marker_ctr'});
-	var abstr = self.getAbstract(item);
+	    var ctr = OAT.Dom.create("div",{className:'marker_ctr'});
+	    var abstr = self.getAbstract(s_item ? s_item : item);
 
 	if (abstr) {
 	var abstrC = OAT.Dom.create("div",{className:'abstract'});
 	    abstrC.innerHTML = abstr;
 	}	
+	}
 	
 	//	if (self.parent.store.itemHasType (item, "")) { }
 	//	if (self.parent.store.itemHasType (item, "")) { }
 
-	var ap = self.drawAllProps (item);
+	var props_ctr = OAT.Dom.create("div", {className: "props_ctr"});
 
 	if (abstr)
-	OAT.Dom.append([ctr,titleH,abstrC,ap]);
+	    OAT.Dom.append([ctr,titleH,abstrC,props_ctr]);
 	else 
-	    OAT.Dom.append([ctr,titleH,ap]);
+	    OAT.Dom.append([ctr,titleH,props_ctr]);
+
+	self.drawAllProps (s_item ? s_item : item, props_ctr);
+	self.drawReferences (s_item ? s_item : item, props_ctr);
 
 	return ctr;
     }
@@ -1744,9 +1825,10 @@ OAT.RDFTabs.map = function(parent,optObj) {
 	function tryList() {
 	    if (!self.pointListLock && self.map_loaded) {
 		if (!self.pointList.length()) {
-		    var note = new OAT.Notify();
+		    var note = new OAT.Notify (false, {notifyType: 2});
 		    var msg = "Current data set contains nothing that could be displayed on the map.";
-		    note.send(msg);
+		    note.send(msg, {timeout: 4000});
+		    OAT.MSG.send (self, "MAP_NOTHING_TO_SHOW", false);
 		}
 		self.map.optimalPosition(self.pointList.makePointsArray(false));
  	    } else {
@@ -1871,7 +1953,7 @@ OAT.RDFTabs.timeline = function(parent,optObj) {
 	if (!uris.length) {
 	    var note = new OAT.Notify();
 	    var msg = "Current data set contains nothing that could be displayed on the timeline.";
-	    note.send(msg);
+	    note.send(msg, {timeout: 4000});
 	}
     }
 }
@@ -2062,7 +2144,7 @@ OAT.RDFTabs.images = function(parent,optObj) {
 		} else {
 		    /* look for predicates that are/contain image links */
 		    for (var j=0;j<pred.length;j++) {
-			if (item.constructor != OAT.RDFAtom ||
+			if (pred[j].constructor != OAT.RDFAtom ||
 			    pred[j].getTag() != OAT.RDFTag.IRI) { continue; }
 			var value = pred[j].getIRI();
 			if (self.parent.getContentType(value) == 3) {
