@@ -1,6 +1,6 @@
 --
 --
---  $Id: virt_rdf_label.sql,v 1.1.4.4 2011/03/23 12:13:19 source Exp $
+--  $Id: virt_rdf_label.sql,v 1.1.4.7 2011/10/13 18:36:28 source Exp $
 --
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
@@ -39,6 +39,8 @@ TTLP (
 @prefix og: <http://opengraphprotocol.org/schema/> .
 @prefix dv: <http://rdf.data-vocabulary.org/> .
 @prefix c: <http://www.w3.org/2002/12/cal/icaltzd#> .
+@prefix oplzllw: <http://www.openlinksw.com/schemas/zillow#> .
+@prefix oplgp: <http://www.openlinksw.com/schemas/googleplus#> .
 
 dc:title rdfs:subPropertyOf virtrdf:label .
 rdfs:label rdfs:subPropertyOf virtrdf:label .
@@ -96,6 +98,14 @@ dv:photo rdfs:subPropertyOf virtrdf:image .
 <http://www.w3.org/1999/02/22-rdf-syntax-ns#value> rdfs:subPropertyOf virtrdf:label .
 <http://rdf.alchemyapi.com/rdf/v1/s/aapi-schema.rdf#Name> rdfs:subPropertyOf virtrdf:label .
 <http://rdf.freebase.com/ns/common.topic.alias> rdfs:subPropertyOf virtrdf:label .
+<http://commontag.org/ns#label> rdfs:subPropertyOf virtrdf:label .
+oplzllw:latitude owl:equivalentProperty geo:lat .
+oplzllw:longitude owl:equivalentProperty geo:long .
+oplgp:organization_name rdfs:subPropertyOf virtrdf:label .
+oplgp:displayName rdfs:subPropertyOf virtrdf:label .
+oplgp:activity_title rdfs:subPropertyOf virtrdf:label .
+oplgp:actor_displayName rdfs:subPropertyOf virtrdf:label .
+oplgp:attachment_displayName rdfs:subPropertyOf virtrdf:label .
 ', '', 'virtrdf-label');
 
 rdfs_rule_set ('virtrdf-label', 'virtrdf-label');
