@@ -2,7 +2,7 @@
 #
 #  mkdemo.sh
 #
-#  $Id: mkdemo.sh,v 1.34 2009/07/09 13:01:00 source Exp $
+#  $Id: mkdemo.sh,v 1.33.2.4 2010/09/21 13:12:36 source Exp $
 #
 #  Creates a demo database
 #  
@@ -64,7 +64,6 @@ else
     RM="rm -rf"
 fi
 
-BPEL=$HOME/binsrc/bpel/
 VOS=0
 if [ "z$SERVER" = "z" ]  
 then
@@ -247,17 +246,18 @@ if [ $VOS -eq 1 ]
 then
     if [ "x$HOST_OS" = "x" ]
     then
-	(cd $BPEL; make)
-	(cd $HOME/binsrc/samples/sparql_demo ; make)
+	(cd $HOME/appsrc ; make)
+	(cd $HOME/binsrc/b3s ; make)
+	(cd $HOME/binsrc/bpel; make)
 	(cd $HOME/binsrc/isparql ; make)
-	(cd $HOME/binsrc/tutorial ; make)
-	(cd $HOME/binsrc/yacutia ; make)
 	(cd $HOME/binsrc/rdf_mappers ; make)
 	(cd $HOME/binsrc/samples/image_magick ; make)
-	(cd $HOME/appsrc ; make)
+	(cd $HOME/binsrc/samples/sparql_demo ; make)
+	(cd $HOME/binsrc/tutorial ; make)
+	(cd $HOME/binsrc/yacutia ; make)
     fi
 else
-    (cd $BPEL; make )
+    (cd $HOME/binsrc/bpel; make )
     [ -f doc_dav.vad ] || (chmod +x mkdoc.sh ; ./mkdoc.sh)
 fi
 
@@ -338,14 +338,16 @@ fi
 BREAK
 ECHO "Collecting and installing VAD packages"
 
-$LN $BPEL/bpel_dav.vad .
-$LN $HOME/binsrc/samples/sparql_demo/sparql_demo_dav.vad .
+$LN $HOME/binsrc/b3s/fct_dav.vad .
+$LN $HOME/binsrc/bpel/bpel_dav.vad .
 $LN $HOME/binsrc/isparql/isparql_dav.vad .
+$LN $HOME/binsrc/rdf_mappers/rdf_mappers_dav.vad .
+$LN $HOME/binsrc/samples/sparql_demo/sparql_demo_dav.vad .
 $LN $HOME/binsrc/tutorial/tutorial_dav.vad .
 $LN $HOME/binsrc/yacutia/conductor_dav.vad .
-$LN $HOME/binsrc/rdf_mappers/rdf_mappers_dav.vad .
 
 [ -f conductor_dav.vad ] && DO_COMMAND "vad_install ('conductor_dav.vad')" dba dba
+[ -f fct_dav.vad ] && DO_COMMAND "vad_install ('fct_dav.vad')" dba dba
 [ -f doc_dav.vad ] && DO_COMMAND "vad_install ('doc_dav.vad')" dba dba
 [ -f rdf_mappers_dav.vad ] && DO_COMMAND "vad_install ('rdf_mappers_dav.vad')" dba dba
 [ -f isparql_dav.vad ] && DO_COMMAND "vad_install ('isparql_dav.vad')" dba dba
@@ -377,8 +379,8 @@ then
     [ -f ods_wiki_dav.vad ] && DO_COMMAND "vad_install ('ods_wiki_dav.vad')" dba dba
 fi
 
-[ -f demo_dav.vad ] && DO_COMMAND "vad_install ('demo_dav.vad')" dba dba
 [ -f tutorial_dav.vad ] && DO_COMMAND "vad_install ('tutorial_dav.vad')" dba dba
+[ -f demo_dav.vad ] && DO_COMMAND "vad_install ('demo_dav.vad')" dba dba
 
 DO_COMMAND "delete from wa_domains where WD_DOMAIN = 'localhost'" dba dba
 
@@ -423,10 +425,6 @@ else
 fi
 
 
-ECHO "Cleanups"
-
-rm -f demo.trx demo.log virtuoso.ini
-chmod 644 demo.db
 
 #
 #  Show final results of run
@@ -438,6 +436,10 @@ then
 	exit 1
 fi
 
+ECHO "Cleanups"
+chmod 644 demo.db
+rm -f demo.trx demo.log virtuoso.ini
+
 BANNER "COMPLETED DEMO DATABASE (mkdemo.sh)"
 
 exit 0
