@@ -1,5 +1,5 @@
 /*
- *  $Id: VirtuosoRepository.java,v 1.1.2.7 2010/06/28 10:32:02 source Exp $
+ *  $Id: VirtuosoRepository.java,v 1.1.2.9 2011/05/17 13:56:31 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -27,9 +27,11 @@ import java.io.File;
 import java.sql.DriverManager;
 import java.sql.SQLException;
 
+import org.openrdf.model.BNodeFactory;
 import org.openrdf.model.ValueFactory;
 import org.openrdf.model.URIFactory;
 import org.openrdf.model.LiteralFactory;
+import org.openrdf.model.impl.BNodeFactoryImpl;
 import org.openrdf.model.impl.ValueFactoryImpl;
 import org.openrdf.model.impl.LiteralFactoryImpl;
 import org.openrdf.model.impl.URIFactoryImpl;
@@ -57,6 +59,7 @@ public class VirtuosoRepository implements Repository {
 	
 	private URIFactory uf = new URIFactoryImpl();
 	private LiteralFactory lf = new LiteralFactoryImpl();
+	private BNodeFactory bf = new BNodeFactoryImpl();
 
 	File dataDir;
 	
@@ -74,6 +77,7 @@ public class VirtuosoRepository implements Repository {
 
 	boolean useLazyAdd = false;
 	int prefetchSize = 200;
+	int queryTimeout = 0;
 	private boolean initialized = false;
 	String ruleSet;
     
@@ -343,6 +347,23 @@ public class VirtuosoRepository implements Repository {
 	}
 
 
+	/**
+	 * Set the query timeout(default 0) 
+	 * 
+	 * @param seconds
+	 *        queryTimeout seconds, 0 - unlimited.
+	 */
+	public void setQueryTimeout(int seconds) {
+		this.queryTimeout = seconds;
+	}
+
+	/**
+	 * Get the query timeout seconds
+	 */
+	public int getQueryTimeout() {
+		return this.queryTimeout;
+	}
+
 	/**
 	 * Set the RoundRobin state for connection(default false) 
 	 * 
@@ -376,7 +397,7 @@ public class VirtuosoRepository implements Repository {
 	 */
 	@Deprecated
 	public ValueFactory getValueFactory() {
-		return new ValueFactoryImpl(uf, lf);
+		return new ValueFactoryImpl(bf, uf, lf);
 	}
 
 	/**
