@@ -1,7 +1,7 @@
 /*
  *  chil.c
  *
- *  $Id: chil.c,v 1.5.2.2 2009/05/14 18:32:25 source Exp $
+ *  $Id: chil.c,v 1.5.2.3 2011/10/13 18:29:57 source Exp $
  *
  *  Server main
  *
@@ -41,6 +41,7 @@
 #include <sqlnode.h>
 #include "wiservic.h"		/* Needed also in Unix builds. Includes wi.h. */
 #include "sqlver.h"
+#include "wi.h"
 
 #if defined(__GNUC__) && __GNUC__ == 3 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1)
 /* When compiling with optimization printf is defined as a macro and
@@ -54,6 +55,7 @@
 
 char *f_config_file = "wi.cfg";
 char *f_license_file = NULL;
+PCONFIG pconfig = NULL;
 int f_read_from_rebuilt_database = 0;
 dk_session_t *listening;
 extern unsigned long int cfg_autocheckpoint;	/* Defined in disk.c */
