@@ -1,5 +1,5 @@
 --
---  $Id: tdav_meta.sql,v 1.4.2.1 2009/04/20 21:19:39 source Exp $
+--  $Id: tdav_meta.sql,v 1.4.2.2 2010/03/04 23:48:20 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -79,7 +79,7 @@ create procedure TDAV_META_CHECK (in resname varchar, in propuri varchar, in pro
     result (sprintf ('PASSED: triplet S=%s P=%s V=%s', resname, propuri, propval));
   else
     {
-      declare actual_value;
+      declare actual_value any;
       actual_value := coalesce ((
     select top 1 DRI_CATVALUE
     from WS.WS.SYS_DAV_RDF_INVERSE, WS.WS.SYS_DAV_RES, WS.WS.SYS_RDF_PROP_NAME
