@@ -1,7 +1,7 @@
 /*
  *  time.c
  *
- *  $Id: time.c,v 1.4 2010/03/01 00:06:43 source Exp $
+ *  $Id: time.c,v 1.2.2.1 2009/04/20 21:18:31 source Exp $
  *
  *  Timing functions
  *
@@ -177,22 +177,6 @@ get_msec_count ()
 #endif
 
 
-void
-sleep_msec (int msec)
-{
-#if !defined (WIN32)
-  struct timeval tv;
-
-  tv.tv_sec = msec / 1000;
-  tv.tv_usec = (msec % 1000) * 1000;
-  select (0, NULL, NULL, NULL, &tv);
-#else
-  /* The Windows Sleep suspends current thread execution */
-  Sleep (msec);
-#endif
-}
-
-
 /* Random function
    TODO Probably make this a bif (PmN) */
 
