@@ -1,7 +1,7 @@
 --
 --  tpcc.sql
 --
---  $Id: tpcc.sql,v 1.4 2010/03/01 00:06:43 source Exp $
+--  $Id: tpcc.sql,v 1.2.2.1 2009/04/20 21:18:31 source Exp $
 --
 --  TPC-C Benchmark transactions as stored procedures.
 --
@@ -193,7 +193,19 @@ create procedure payment (
         from customer
 	where c_last = _c_last
 	  and c_d_id = _d_id
-	  and c_w_id = _c_w_id;
+	  and c_w_id = _w_id;
+
+
+
+      select count(C_ID) into namecnt2
+        from customer
+	where c_last = _c_last
+	  and c_d_id = _d_id
+	  and c_w_id = _w_id;
+
+
+	if (namecnt <> namecnt2)
+	  dbg_printf ('Bad count for last %s d %d c1 %d c2 %d', _c_last, _d_id, namecnt, namecnt2);
 	      declare c_byname cursor for
 	select c_id
 	  from customer
@@ -454,8 +466,7 @@ create procedure ol_insert (
 }
 
 
-
-
+-- cust_info - part of New Order transaction. Return customer info.
 create procedure cust_info (
     in w_id integer,
     in d_id integer,
