@@ -1,7 +1,7 @@
 /*
  *  viconfig.c
  *
- *  $Id: viconfig.c,v 1.38.2.8 2010/01/28 00:36:51 source Exp $
+ *  $Id: viconfig.c,v 1.38.2.28 2011/10/28 21:49:35 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -107,9 +107,11 @@ extern char *vdb_odbc_error_file; /* from sqlrrun.c */
 extern char *vdb_trim_trailing_spaces; /* from sqlrrun.c */
 extern long cfg_disable_vdb_stat_refresh;
 extern char *www_maintenance_page;
+extern char *http_proxy_address;
 extern char *http_cli_proxy_server;
 extern char *http_cli_proxy_except;
 extern int32 http_enable_client_cache;
+extern int32 log_proc_overwrite;
 
 #ifdef _SSL
 extern char *https_port;
@@ -173,13 +175,16 @@ extern int uriqa_dynamic_local;
 extern int lite_mode;
 extern int rdf_obj_ft_rules_size;
 extern int it_n_maps;
+extern int32 ric_samples_sz;
+extern int32 enable_p_stat;
 extern int aq_max_threads;
 
 char * http_log_file_check (struct tm *now); /* http log name checking */
 
 int32 c_txn_after_image_limit;
 int32 c_n_fds_per_file;
-int32 c_syslog;
+int32 c_syslog = 0;
+char *c_syslog_facility = NULL;
 /* These are the config variables as read here */
 char *c_error_log_file;
 char *c_database_file;
@@ -203,6 +208,8 @@ int32 c_null_unspecified_params;
 int32 c_prefix_resultnames;
 int32 c_disable_mt_write;
 int32 c_bad_parent_links;
+extern int32 dbs_check_extent_free_pages;
+extern int32 dbs_cache_check_enable;
 #if 0/*obsoleted*/
 int32 c_bad_dtp;
 int32 c_atomic_dive;
@@ -249,11 +256,6 @@ dk_set_t c_stripes;
 int32 c_log_segments_num;
 log_segment_t *c_log_segments;
 int32 c_log_audit_trail;
-#if REPLICATION_SUPPORT
-int32 c_repl_queue_max = 50000;
-char *c_db_name;
-char *c_repl_server_enable;
-#endif
 int32 c_use_array_params = 0;
 int32 c_num_array_params = 10;
 int32 c_server_thread_sz = 60000;
@@ -322,9 +324,11 @@ int32 c_http_thread_sz = 280000;
 int32 c_http_keep_hosting = 0;
 extern long http_keep_hosting; /* from http.c */
 char *c_ucm_load_path = 0;
+int32 c_lh_xany_normalization_flags = 0;
 int32 c_i18n_wide_file_names = 0;
 char *c_i18n_volume_encoding = NULL;
 char *c_i18n_volume_emergency_encoding = NULL;
+extern int lh_xany_normalization_flags;
 extern int i18n_wide_file_names;
 extern struct encoding_handler_s *i18n_volume_encoding;
 extern struct encoding_handler_s *i18n_volume_emergency_encoding;
@@ -337,6 +341,7 @@ int32 c_lite_mode = 0;
 int32 c_uriqa_dynamic_local = 0;
 int32 c_rdf_obj_ft_rules_size = 0;
 int32 c_it_n_maps = 0;
+extern int32 c_dense_page_allocation;
 
 /* externs about client configuration */
 extern int32 cli_prefetch;
@@ -347,6 +352,7 @@ extern int32 cli_not_c_char_escape;
 extern int32 cli_utf8_execs;
 extern int32 cli_binary_timestamp;
 extern int32 cli_no_system_tables;
+extern int32 cli_max_cached_stmts;
 
 int32 c_cli_encryption_on_password;
 extern long cli_encryption_on_password;
@@ -515,18 +521,112 @@ virtuoso_cfg_next_string (char **pkey, char **pret)
 
 LOG *startup_log = NULL;
 
+#ifdef HAVE_SYSLOG
+struct _syslog_code {
+	char    *c_name;
+	int     c_val;
+} syslog_facilitynames[] =
+{
+#ifdef LOG_AUTH
+	{ "AUTH",	LOG_AUTH },
+#endif
+#ifdef LOG_AUTHPRIV
+	{ "AUTHPRIV",	LOG_AUTHPRIV },
+#endif
+#ifdef LOG_CRON
+	{ "CRON",	LOG_CRON },
+#endif
+#ifdef LOG_DAEMON
+	{ "DAEMON",	LOG_DAEMON },
+#endif
+#ifdef LOG_FTP
+	{ "FTP",	LOG_FTP },
+#endif
+#ifdef LOG_INSTALL
+	{ "INSTALL",	LOG_INSTALL },
+#endif
+#ifdef LOG_KERN
+	{ "KERN",	LOG_KERN },
+#endif
+#ifdef LOG_LPR
+	{ "LPR",	LOG_LPR },
+#endif
+#ifdef LOG_MAIL
+	{ "MAIL",	LOG_MAIL },
+#endif
+#ifdef LOG_NETINFO
+	{ "NETINFO",	LOG_NETINFO },
+#endif
+#ifdef LOG_NEWS
+	{ "NEWS",	LOG_NEWS },
+#endif
+#ifdef LOG_RAS
+	{ "RAS",	LOG_RAS },
+#endif
+#ifdef LOG_REMOTEAUTH
+	{ "REMOTEAUTH",	LOG_REMOTEAUTH },
+#endif
+#ifdef LOG_SYSLOG
+	{ "SYSLOG",	LOG_SYSLOG },
+#endif
+#ifdef LOG_USER
+	{ "USER",	LOG_USER },
+#endif
+#ifdef LOG_UUCP
+	{ "UUCP",	LOG_UUCP },
+#endif
+#ifdef LOG_LOCAL0
+	{ "LOCAL0",	LOG_LOCAL0 },
+#endif
+#ifdef LOG_LOCAL1
+	{ "LOCAL1",	LOG_LOCAL1 },
+#endif
+#ifdef LOG_LOCAL2
+	{ "LOCAL2",	LOG_LOCAL2 },
+#endif
+#ifdef LOG_LOCAL3
+	{ "LOCAL3",	LOG_LOCAL3 },
+#endif
+#ifdef LOG_LOCAL4
+	{ "LOCAL4",	LOG_LOCAL4 },
+#endif
+#ifdef LOG_LOCAL5
+	{ "LOCAL5",	LOG_LOCAL5 },
+#endif
+#ifdef LOG_LOCAL6
+	{ "LOCAL6",	LOG_LOCAL6 },
+#endif
+#ifdef LOG_LOCAL7
+	{ "LOCAL7",	LOG_LOCAL7 },
+#endif
+	{ "DEFAULT",	LOG_USER }
+};
+#endif
+
+
 LOG *
-cfg_open_syslog (int level)
+cfg_open_syslog (int level, char *c_facility)
 {
 #ifdef HAVE_SYSLOG
+  int facility = LOG_USER;
+  int i;
+
+  if (!c_facility) c_facility = "default";
+
+  for (i = 0; i < sizeof (syslog_facilitynames) / sizeof (syslog_facilitynames[0]); i++)
+    {
+      if (!stricmp (syslog_facilitynames[i].c_name, c_facility))
+	{
+	  facility = syslog_facilitynames[i].c_val;
+	  break;
+	}
+    }
+
       return log_open_syslog ("Virtuoso",
 	  LOG_CONS | LOG_NOWAIT | LOG_PID,
-	  LOG_USER,
-	  level, L_MASK_ALL,
-            f_debug ?
-                L_STYLE_LEVEL | L_STYLE_GROUP | L_STYLE_TIME :
-		L_STYLE_GROUP | L_STYLE_TIME
-	  );
+      facility, level,
+	L_MASK_ALL,
+	f_debug ? L_STYLE_LEVEL | L_STYLE_GROUP : L_STYLE_GROUP);
 #else
       return NULL;
 #endif
@@ -596,6 +696,9 @@ cfg_setup (void)
   if (cfg_getlong (pconfig, section, "Syslog", &c_syslog) == -1)
     c_syslog = 0;
 
+  if (cfg_getstring (pconfig, section, "SyslogFacility", &c_syslog_facility) == -1)
+    c_syslog_facility = "default";
+
   if (c_file_extend < DP_INSERT_RESERVE + 5)
     c_file_extend = DP_INSERT_RESERVE + 5;
 
@@ -617,8 +720,9 @@ cfg_setup (void)
       if (startup_log)
         log_close (startup_log);
       startup_log = NULL;
+
       if (c_syslog)
-	cfg_open_syslog (c_error_log_level);
+    cfg_open_syslog (c_error_log_level, c_syslog_facility);
 
   if (cfg_getstring (pconfig, section, "xa_persistent_file", &c_xa_persistent_file) == -1)
     c_xa_persistent_file = s_strdup (setext (prefix, "pxa", EXT_SET));
@@ -852,7 +956,11 @@ cfg_setup (void)
     sqlo_compiler_exceeds_run_factor = 0;
 
   if (cfg_getlong (pconfig, section, "MaxMemPoolSize", &sqlo_max_mp_size) == -1)
-    sqlo_max_mp_size = 500000000;
+    sqlo_max_mp_size = 200000000;
+
+#ifdef POINTER_64
+  sqlo_max_mp_size *= 2;
+#endif
 
   if (sqlo_max_mp_size != 0 && sqlo_max_mp_size < 5000000)
     sqlo_max_mp_size = 5000000;
@@ -886,6 +994,23 @@ cfg_setup (void)
   if (cfg_getlong (pconfig, section, "BuffersAllocation", &malloc_bufs) == -1)
     malloc_bufs = 0;
 
+  if (cfg_getlong (pconfig, section, "SwapGuard", &swap_guard_on) == -1)
+    swap_guard_on = 0;
+
+  /*  ExtentReadThreshold, ExtentReadWindow, ExtentReadStartupThreshold, ExtentReadStartupWindow */
+  if (cfg_getlong (pconfig, section, "ExtentReadThreshold", &em_ra_threshold) == -1)
+   em_ra_threshold = 2;
+  if (cfg_getlong (pconfig, section, "ExtentReadWindow", &em_ra_window) == -1)
+    em_ra_window = 1000;
+  if (cfg_getlong (pconfig, section, "ExtentReadStartupThreshold", &em_ra_startup_threshold) == -1)
+    em_ra_startup_threshold = 0;
+  if (cfg_getlong (pconfig, section, "ExtentReadStartupWindow", &em_ra_startup_window) == -1)
+   em_ra_startup_window = 40000;
+
+  if (cfg_getlong (pconfig, section, "LogProcOverwrite", &log_proc_overwrite) == -1)
+    log_proc_overwrite = 1;
+
+
   {
     int nbdirs;
     dk_set_t bd = NULL;
@@ -1031,9 +1156,20 @@ cfg_setup (void)
   if (cfg_getlong (pconfig, section, "RdfFreeTextRulesSize", &c_rdf_obj_ft_rules_size) == -1)
     c_rdf_obj_ft_rules_size = 0;
 
+  if (cfg_getlong (pconfig, section, "RdfInferenceSampleCacheSize", &ric_samples_sz) == -1)
+    ric_samples_sz = 0;
+
   if (cfg_getlong (pconfig, section, "IndexTreeMaps", &c_it_n_maps) == -1)
     c_it_n_maps = 0;
 
+  if (cfg_getlong (pconfig, section, "DensePageAllocation", &c_dense_page_allocation) == -1)
+    c_dense_page_allocation = 0;
+
+  if (cfg_getlong (pconfig, section, "PageMapCheck", &dbs_cache_check_enable) == -1)
+    dbs_cache_check_enable = 0;
+
+  if (cfg_getlong (pconfig, section, "MaxOpenClientStatements", &cli_max_cached_stmts) == -1)
+    cli_max_cached_stmts = 10000;
 
   section = "HTTPServer";
 
@@ -1193,6 +1329,9 @@ cfg_setup (void)
   if (cfg_getstring (pconfig, section, "MaintenancePage", &www_maintenance_page) == -1)
     www_maintenance_page = NULL;
 
+  if (cfg_getstring (pconfig, section, "GatewayIpAddress", &http_proxy_address) == -1)
+    http_proxy_address = NULL;
+
   if (cfg_getlong (pconfig, section, "RDFContentNegotiation", &c_http_check_rdf_accept) == -1)
     c_http_check_rdf_accept = 1;
 
@@ -1219,6 +1358,8 @@ cfg_setup (void)
     c_bad_parent_links = 0;
   if (cfg_getlong (pconfig, section, "DuplicateCheckpointRemaps", &cpt_remap_recovery) == -1)
     cpt_remap_recovery = 0;
+  if (cfg_getlong (pconfig, section, "CheckExtentFreePages", &dbs_check_extent_free_pages) == -1)
+    dbs_check_extent_free_pages = 0;
 
 
 #if 0/*obsoleted*/
@@ -1304,6 +1445,10 @@ cfg_setup (void)
     c_sparql_result_set_max_rows = 0;
   if (cfg_getlong (pconfig, section, "MaxMemInUse", &c_sparql_max_mem_in_use) == -1)
     c_sparql_max_mem_in_use = 0;
+  if (cfg_getlong (pconfig, section, "TransitivityCacheEnabled", &tn_cache_enable) == -1)
+    tn_cache_enable = 0;
+  if (cfg_getlong (pconfig, section, "EnablePstats", &enable_p_stat) == -1)
+    enable_p_stat = 1;
 
   /* Now open the HTTP log */
   if (http_log_file)
@@ -1321,18 +1466,6 @@ cfg_setup (void)
     }
 
   srv_client_defaults_init ();
-#if REPLICATION_SUPPORT
-  if (cfg_getstring (pconfig, "Replication", "ServerName", &c_db_name) == -1)
-    c_db_name = NULL;
-  if (cfg_getstring (pconfig, "Replication", "ServerEnable", &c_repl_server_enable) == -1)
-    c_repl_server_enable = NULL;
-  if (c_repl_server_enable &&
-      strcmp (c_repl_server_enable, "1") && stricmp (c_repl_server_enable, "On"))
-    c_repl_server_enable = NULL;
-
-  if (cfg_getlong (pconfig, "Replication", "QueueMax", &c_repl_queue_max) == -1)
-    c_repl_queue_max = 50000;
-#endif
 
   /*
    *  VDB related parameters
@@ -1406,13 +1539,8 @@ cfg_setup (void)
   else if (c_unremap_quota < 500)
     c_unremap_quota = 500;
 
-#if REPLICATION_SUPPORT
-  if (c_server_threads > MAX_THREADS - 5)
-    c_server_threads = MAX_THREADS - 5;
-#else
   if (c_server_threads > MAX_THREADS - 3)
     c_server_threads = MAX_THREADS - 3;
-#endif
 
   /* Initialization of UCMs */
 
@@ -1457,6 +1585,8 @@ cfg_setup (void)
   /* Initialization of national filesystems */
   
   section = "I18N";
+  if (cfg_getlong (pconfig, section, "XAnyNormalization", &c_lh_xany_normalization_flags) == -1)
+    c_lh_xany_normalization_flags = 0;
   if (cfg_getlong (pconfig, section, "WideFileNames", &c_i18n_wide_file_names) == -1)
     c_i18n_wide_file_names = 0;
   if (cfg_getstring (pconfig, section, "VolumeEncoding", &c_i18n_volume_encoding) == -1)
@@ -1630,7 +1760,7 @@ new_db_read_cfg (dbe_storage_t * ignore, char *mode)
   aq_max_threads = c_aq_max_threads;
   if (aq_max_threads > 100)
     aq_max_threads = 100;
-  if (aq_max_threads < 10)
+  if (aq_max_threads < 10 && aq_max_threads > 0)
     aq_max_threads = 10;
 #ifdef _SSL
   https_port = c_https_port;
@@ -1668,11 +1798,6 @@ new_db_read_cfg (dbe_storage_t * ignore, char *mode)
   sqlo_max_layouts = c_sqlo_max_layouts;
   sql_proc_use_recompile = c_sql_proc_use_recompile;
 
-#if REPLICATION_SUPPORT
-  repl_queue_max = c_repl_queue_max;
-  repl_server_enable = c_repl_server_enable;
-  db_name = box_string (c_db_name);
-#endif
 
   callstack_on_exception = c_callstack_on_exception;
   log_file_line = c_log_file_line;
@@ -1719,6 +1844,7 @@ new_db_read_cfg (dbe_storage_t * ignore, char *mode)
   sparql_max_mem_in_use = c_sparql_max_mem_in_use;
   cli_encryption_on_password = c_cli_encryption_on_password;
 
+  lh_xany_normalization_flags = c_lh_xany_normalization_flags;
   i18n_wide_file_names = c_i18n_wide_file_names;
   if (NULL != c_i18n_volume_encoding)
     {
@@ -2127,7 +2253,8 @@ failed:
   log (L_ERR, "If you are absolutely sure that this is not the case, please try");
   log (L_ERR, "to remove the file %s and start again.", c_lock_file);
 
-  close (lck_fd);
+  if (lck_fd >= 0)
+    close (lck_fd);
   lck_fd = -1;
 
   return -1;
