@@ -1,7 +1,7 @@
 /*
  *  viunix.c
  *
- *  $Id: viunix.c,v 1.11.4.3 2011/10/28 16:00:40 source Exp $
+ *  $Id: viunix.c,v 1.11.2.4 2011/10/28 21:49:35 source Exp $
  *
  *  OpenLink Virtuoso VDBMS Server main
  *  
@@ -706,14 +706,6 @@ main (int argc, char **argv)
   if (!strchr (f_mode, 'b'))
     {
       http_init_part_two ();
-#if REPLICATION_SUPPORT
-      if (f_read_from_rebuilt_database)	/* if booting from crash log, */
-	{				/* go read the account levels from db */
-	  repl_read_db_levels ();
-	}
-
-      /*repl_sync_server (NULL, NULL);*/
-#endif
     }
 #ifdef _RENDEZVOUS
   start_rendezvous ();
