@@ -1,7 +1,7 @@
 /*
  *  viwin32.c
  *
- *  $Id: viwin32.c,v 1.11 2009/04/14 11:53:45 source Exp $
+ *  $Id: viwin32.c,v 1.11.2.2 2011/10/26 09:34:24 source Exp $
  *
  *  OpenLink Virtuoso DBMS Server
  *  Main code for Win32
@@ -157,7 +157,7 @@ extern int ob_just_report;
 /*
  *  Globals for virtuoso
  */
-PCONFIG	pconfig;			/* configuration file */
+PCONFIG	pconfig = NULL;			/* configuration file */
 
 /*
  *  Module locals
@@ -1256,7 +1256,7 @@ usage (void)
       PACKAGE_VERSION, DBMS_SRV_GEN_MAJOR, DBMS_SRV_GEN_MINOR, build_thread_model, build_date);
   p = stpcpy (p, line);
 
-  sprintf (line, "Compiled for 32 Bit Windows Operating Environments\n");
+  sprintf (line, "Compiled for %s (%s)\n", build_opsys_id, build_host_id);
   p = stpcpy (p, line);
 
   if (build_special_server_model && strlen(build_special_server_model) > 1)
@@ -1546,14 +1546,6 @@ ApplicationMain (int argc, char **argv)
   if (!strchr (f_mode, 'b'))
     {
       http_init_part_two ();
-#if REPLICATION_SUPPORT
-      if (f_read_from_rebuilt_database)	/* if booting from crash log, */
-	{				/* go read the account levels from db */
-	  repl_read_db_levels ();
-	}
-
-      /*repl_sync_server (NULL, NULL);*/
-#endif
     }
 #ifdef _RENDEZVOUS
   start_rendezvous ();
