@@ -1,7 +1,7 @@
 --
 --  vfs.sql
 --
---  $Id: vfs.sql,v 1.10.2.23 2011/03/23 12:05:12 source Exp $
+--  $Id: vfs.sql,v 1.10.2.28 2011/08/01 14:17:06 source Exp $
 --
 --  Site-copy robot.
 --  
@@ -77,19 +77,33 @@ create procedure WS.WS.VFS_ENSURE_NEW_SITE (in _host varchar, in _root varchar,
 }
 ;
 
+create procedure WS.WS.VFS_GUESS_CONTENT_TYPE (in origin_uri varchar, in ret_content_type varchar, inout ret_body any)
+{
+  declare ret_begin, ret_html any;
+  ret_begin := subseq (ret_body, 0, 4096);
+  if (isstring_session (ret_begin))
+    ret_begin := string_output_string (ret_begin);
+  ret_html := xtree_doc (ret_begin, 2);
+  if (xpath_eval ('/sitemapindex|/urlset', ret_html) is not null)
+    return 'text/xml';
+  return ret_content_type;
+}
+;
+
 create procedure WS.WS.COPY_PAGE_1 (in _host varchar, in _urls any, in _root varchar,
     in _upd integer, in _dbg integer)
 {
   declare _header, _etag, _http_resp_code, _start_url varchar;
   declare _del, _desc, _t_urls, _opts, _c_type varchar;
   declare _dav_method, _d_imgs, _opage varchar;
-  declare _dav_enabled, _other, ua varchar;
+  declare _dav_enabled, _other, ua, cust_headers varchar;
   declare dt, redir_flag, store_flag, try_to_get_rdf integer;
   declare _since datetime;
   declare _udata, ext_hook, store_hook, _header_arr, _resps, xp_exp any;
-  declare n_urls, conv_html, is_sitemap, accept_rdf int;
+  declare n_urls, conv_html, is_sitemap, accept_rdf, time_out int;
 
   conv_html := 1;
+  time_out := null;
   n_urls := position (0, _urls) - 1;
   if (n_urls < 0)
     n_urls := length (_urls);
@@ -98,26 +112,28 @@ create procedure WS.WS.COPY_PAGE_1 (in _host varchar, in _urls any, in _root var
   select VS_NEWER, VS_OPTIONS, coalesce (VS_METHOD, ''), VS_URL, VS_SRC, coalesce (VS_OPAGE, ''),
          coalesce (VS_REDIRECT, 1), coalesce (VS_STORE, 1), coalesce (VS_DLOAD_META, 0), 
 	 deserialize (VS_UDATA), VS_EXTRACT_FN, VS_STORE_FN, coalesce (VS_DEL, ''), coalesce (VS_OTHER, ''), 
-	 VS_CONVERT_HTML, VS_IS_SITEMAP, VS_XPATH, VS_ACCEPT_RDF
+	 VS_CONVERT_HTML, VS_IS_SITEMAP, VS_XPATH, VS_ACCEPT_RDF, VS_TIMEOUT, VS_HEADERS
       into _since, _opts, _dav_method, _start_url, _d_imgs, _opage, 
       	 redir_flag, store_flag, try_to_get_rdf, 
 	 _udata, ext_hook, store_hook, _del, _other, 
-	 conv_html, is_sitemap, xp_exp, accept_rdf
+	 conv_html, is_sitemap, xp_exp, accept_rdf, time_out, cust_headers
       from VFS_SITE where VS_HOST = _host and VS_ROOT = _root;
 nf_opt:
 
-  _header := '';
+  if (length (cust_headers) > 2)
+    cust_headers := rtrim (cust_headers, ' \r\n') || '\r\n';
+  else
+    cust_headers := null;
+   
+  _header := coalesce (cust_headers, '');
   if (isstring (_opts) and strchr (_opts, ':') is not null)
     _header := sprintf ('Authorization: Basic %s\r\n', encode_base64(_opts));
 
-  if (_upd = 1 and _since is not null)
-    _header := concat (_header, 'If-Modified-Since: ', soap_print_box (_since, '', 1), '\r\n');
-
-  if (accept_rdf)
+  if (accept_rdf and strstr (_header, 'Accept:') is null)
     _header := _header || 'Accept: application/rdf+xml, text/n3, text/rdf+n3, */*\r\n'; -- /* rdf formats */
   -- global setting for crawler UA  
   ua := registry_get ('vfs_ua');
-  if (isstring (ua) and length (ua))
+  if (isstring (ua) and length (ua) and strstr (_header, 'User-Agent:') is null)
     {
       ua := trim (ua, ' \r\n');
       _header := _header || 'User-Agent: ' || ua || '\r\n';
@@ -129,12 +145,23 @@ nf_opt:
       for (declare i int, i := 0; i < n_urls; i := i + 1)
         {
 	  declare _url, _hdr varchar;
+	  declare _dt datetime;
 	  _url := _urls[i];
-	  _etag := (select VU_ETAG from VFS_URL where VU_HOST = _host and VU_URL = _url and VU_ROOT = _root);
-	  if (_etag is not null and isstring (_etag))
+	  -- take Etag if present
+	  _etag := null;
+	  _dt := _since;
+	  for select VU_ETAG, VU_CPTIME from VFS_URL where VU_HOST = _host and VU_URL = _url and VU_ROOT = _root do
+	    {
+	      _etag := VU_ETAG;
+	      _dt := VU_CPTIME;
+	    }
+	  if (_etag is not null and isstring (_etag) and length (_etag))
 	    _hdr := concat (_header,'If-None-Match: ', _etag, '\r\n');
   else
 	    _hdr := _header;
+	  -- check against last fetch  
+	  if (_upd = 1 and _dt is not null)
+	    _hdr := concat (_hdr, 'If-Modified-Since: ', soap_print_box (_dt, '', 1), '\r\n');
 	  _header_arr[i] := _hdr;  
 	}
     }
@@ -201,7 +228,7 @@ get_again:
     if (n_urls = 1)
       {
 	declare _resp, _content any;
-        _content := http_get (_t_urls[0], _resp, 'GET', case when _upd = 1 then _header_arr[0] else _header_arr end);
+        _content := http_get (_t_urls[0], _resp, 'GET', case when _upd = 1 then _header_arr[0] else _header_arr end, null, null, redir_flag, time_out);
 	-- when single request we can check here and re-try if no header is received
 	if (isarray(_resp) and length (_resp) and not isstring (_resp [0]))
 	  {
@@ -254,11 +281,13 @@ get_again:
 	  goto end_crawl;
   }
 
-  _c_type := coalesce (http_request_header (_resp, 'Content-Type'), '');
+      if (_http_resp_code = '200' and (isstring (_content) or __tag (_content) = 185))
+	{
+	  _c_type := http_request_header (_resp, 'Content-Type', null, '');
+      _c_type := DB.DBA.RDF_SPONGE_GUESS_CONTENT_TYPE (_url, _c_type, _content);
+	  _c_type := WS.WS.VFS_GUESS_CONTENT_TYPE (_url, _c_type, _content);
       _etag := http_request_header (_resp, 'ETag', null, '');
 
-      if (_http_resp_code = '200' and (isstring (_content) or __tag (_content) = 185))
-    {
       if (ext_hook is not null and __proc_exists (ext_hook))
 	    call (ext_hook) (_host, _url, _root, _content, _c_type, lev + 1);
 	  else if ((_url like '%.htm%' or _url like '%/' or _c_type like 'text/html%' or _c_type like 'application/%xml' or _c_type = 'text/xml' or _url like '%.xml' or _url like '%.xml.gz') 
@@ -328,11 +357,12 @@ create procedure WS.WS.VFS_STATUS_SET (in _tgt varchar, in _root varchar, in _st
 }
 ;
 
--- /* top level procedure for processing queues */
+-- /* top level procedure for processing queues, don't grant NEVER to any body else */
 create procedure WS.WS.SERV_QUEUE_TOP (in _tgt varchar, in _root varchar, in _upd integer,
     in _dbg integer, in _fn varchar, in _clnt_data any, in threads int := null, in batch_size int := 1)
 {
   declare _msg, _stat, oq varchar;
+  set_user_id ('dba');
 do_again:
   _stat := '00000';
   _msg := '';
@@ -376,6 +406,7 @@ create procedure WS.WS.SERV_QUEUE (in _tgt varchar, in _root varchar, in _upd in
   declare aq_list, aq, url_batch any;
   declare err any;
   declare pid, thr_conf, care_bot int;
+  declare delay_sec float;
 
   _total := 0;
   registry_set ('WEB_COPY', 'X __sequence_set (''WEB_COPY_SSHUT'', datediff (''second'', stringdate (''1980-01-01''), now ()), 0)');
@@ -387,15 +418,15 @@ create procedure WS.WS.SERV_QUEUE (in _tgt varchar, in _root varchar, in _upd in
   commit work;
 
   whenever not found goto n_site;
-  select VS_URL, VS_METHOD, VS_THREADS, VS_BOT 
-      into _tgt_url, _dav_method, thr_conf, care_bot 
+  select VS_URL, VS_METHOD, VS_THREADS, VS_BOT, VS_DELAY 
+      into _tgt_url, _dav_method, thr_conf, care_bot, delay_sec 
       from VFS_SITE where VS_HOST = _tgt and VS_ROOT = _root with (exclusive);
+  if (care_bot)  
+    VFS_ROBOTS_GET (_tgt, _root, delay_sec);
   if (thr_conf is not null and thr_conf > 0)
     nthreads := thr_conf;
   if (nthreads is null or nthreads < 0)
     nthreads := 1;
-  if (care_bot)  
-    VFS_ROBOTS_GET (_tgt, _root);
   commit work;
   -- if it is an update 
   if (_upd = 1)
@@ -439,11 +470,19 @@ create procedure WS.WS.SERV_QUEUE (in _tgt varchar, in _root varchar, in _upd in
 	  {
 	    ERR_MAIL_SEND (_tgt, vector (), _root, __SQL_STATE, __SQL_MESSAGE);
 	    rollback work;
+	    __SQL_STATE := cast (__SQL_STATE as varchar);
+	    if (__SQL_STATE = '40001')
+	      {
+		resignal;
+	      }
 	    goto fn_end;
 	  };
 	found_one := 0; ndone := 0;
 	if (WS.WS.VFS_STATUS_GET (_tgt, _root) = 'stopped')
 	  goto fn_end;
+	commit work;
+	if (delay_sec > 0)
+	  delay (delay_sec);
 	for (declare i int, i := 0; i < batch_size; i := i + 1)
 	      {
 	     _rc := call (url_fn) (_tgt, _root, _next_url, _clnt_data);
@@ -533,6 +572,9 @@ create procedure WS.WS.LOCAL_STORE (in _host varchar, in _url varchar, in _root
     {
       insert soft VFS_URL (VU_HOST, VU_URL, VU_CHKSUM, VU_CPTIME, VU_ETAG, VU_ROOT)
 	  values (_host, _url, md5 (_content), now (), _s_etag, _root);
+      if (row_count () = 0)
+	update WS.WS.VFS_URL set VU_CHKSUM = md5 (_content), VU_CPTIME = now (), VU_ETAG = _s_etag where
+	    VU_HOST = _host and VU_URL = _url and VU_ROOT = _root;
       return 0;
     }
 
@@ -638,7 +680,7 @@ create procedure WS.WS.GET_URLS (in _host varchar, in _url varchar, in _root var
   declare _flw, _nflw, _method, _delete, xp_exp, base, robots, care_of_bot varchar;
   declare _newer datetime;
   declare _own integer;
-  declare frames, _urls_arr any;
+  declare frames, _urls_arr, origin_iri, urls any;
 
   if (WS.WS.SITEMAP_PROCESS (_host, _url, _root, _content, ctype, lev))
     return;
@@ -668,6 +710,8 @@ create procedure WS.WS.GET_URLS (in _host varchar, in _url varchar, in _root var
 
   _inx := 0;
   base := WS.WS.VFS_URI_COMPOSE (vector ('http', _host, _url, '', '', ''));
+  origin_iri := iri_to_id (base, 1);
+  urls := make_array (_urls_arr_len, 'any');
   while (_inx < _urls_arr_len)
 	    {
       declare hi, sch any;
@@ -691,16 +735,18 @@ create procedure WS.WS.GET_URLS (in _host varchar, in _url varchar, in _root var
 	  _tmp_url := WS.WS.VFS_URI_COMPOSE (hi);  -- local part & URL parameters
 	  if (_tmp_host <> _host)
 	    _tmp_url := _tmp;
+	  urls [_inx] := _tmp_url;  
 	      --dbg_obj_print ('LINK: ',_tmp_host,_tmp_url);
 	  if ((get_keyword (_tmp_url, frames) is not null or WS.WS.FOLLOW (_host, _root, _tmp_url, _flw, _nflw, _start_url, _d_imgs)))
 			{
-	      insert soft VFS_QUEUE (VQ_HOST, VQ_TS, VQ_URL, VQ_STAT, VQ_ROOT, VQ_LEVEL)
-		  values (_host, now (), _tmp_url, 'waiting', _root, lev);
+	      insert soft VFS_QUEUE (VQ_HOST, VQ_TS, VQ_URL, VQ_STAT, VQ_ROOT, VQ_LEVEL, VQ_ORIGIN)
+		  values (_host, now (), _tmp_url, 'waiting', _root, lev, origin_iri);
 		}
 		}
       next_url:
       _inx := _inx + 1;
     }
+  delete from WS.WS.VFS_QUEUE where VQ_HOST = _host and VQ_ROOT = _root and VQ_ORIGIN = origin_iri and not position (VQ_URL, urls);
 no_site_rec:  
   return;
 }
@@ -748,21 +794,21 @@ create procedure WS.WS.SITEMAP_PROCESS (in _host varchar, in _url varchar, in _r
 		}
 	      xp := ar;
 	    }
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0);
 	  return 1;
 	}
       else if (xpath_eval ('/sitemapindex/sitemap/loc', xt) is not null)
 	{
 	  --xp := xpath_eval ('/sitemapindex/sitemap/loc/text()', xt, 0);
 	  xp := WS.WS.SITEMAP_GET_LOC (xt, '/sitemapindex/sitemap', './loc/text()', './lastmod/text()');
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0);
 	  return 1;
 	}
       else if (xpath_eval ('/urlset/url/loc', xt) is not null)
 	{
 	  --xp := xpath_eval ('/urlset/url/loc/text()', xt, 0);
 	  xp := WS.WS.SITEMAP_GET_LOC (xt, '/urlset/url', './loc/text()', './lastmod/text()');
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 1);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 1);
 	  return 1;
 	}
     }
@@ -1047,7 +1093,7 @@ create procedure WS.WS.FIND_URI (in _content varchar, in _d_imgs varchar,
 
   if (tidy_external () and ctype like 'text/html')
     _content := tidy_html (_content, 'output-xhtml:yes\r\ntidy-mark:no');
-  if (ctype like 'application/%+xml' or ctype = 'text/xml')
+  if (ctype like 'application/%xml' or ctype = 'text/xml')
     _xml_tree := xml_tree (_content, 0);
   else  
   _xml_tree := xml_tree (_content, 2);
@@ -1775,7 +1821,7 @@ create procedure WS.WS.VFS_EXTRACT_RDF (in _host varchar, in _root varchar, in _
 {
   declare mime_type, _graph, _base, out_arr, tmp varchar;
   declare html_start, xd any;
-  declare rc int;
+  declare rc, deadl int;
 
   html_start := null;
 
@@ -1784,9 +1830,20 @@ create procedure WS.WS.VFS_EXTRACT_RDF (in _host varchar, in _root varchar, in _
   _graph := WS.WS.MAKE_URL (_host, _start_path);
   _base := WS.WS.MAKE_URL (_host, url);
 
+  commit work;
+  deadl := 6;
+
   declare exit handler for sqlstate '*'
   {
-    return;
+    rollback work;
+    __SQL_STATE := cast (__SQL_STATE as varchar);
+    if (__SQL_STATE = '40001')
+      {
+	deadl := deadl - 1;
+	if (deadl > 0)
+	  goto again;
+      }
+    resignal;
   };
 
   if (url like '*.gz')
@@ -1801,7 +1858,7 @@ create procedure WS.WS.VFS_EXTRACT_RDF (in _host varchar, in _root varchar, in _
     }
   -- RDF/XML or RDF/N3 depends on option
   mime_type := DB.DBA.RDF_SPONGE_GUESS_CONTENT_TYPE (url, ctype, content);
-
+again:
   -- RDF formats 
     {
       if (strstr (mime_type, 'application/rdf+xml') is not null)
@@ -1888,12 +1945,16 @@ create procedure WS.WS.SITEMAP_ENSURE_NEW_SITE (in _host varchar, in _root varch
 ;
 
 
-create procedure WS.WS.SITEMAP_URLS_REGISTER (in _host varchar, in _root varchar, inout xp any, in lev int := 0, in sm int := 0)
+create procedure WS.WS.SITEMAP_URLS_REGISTER (in _host varchar, in _root varchar, in src_url varchar, inout xp any, in lev int := 0, in sm int := 0, in delta int := 1)
 {
-  declare _flw_s, _nflw_s, _url, _img varchar;
+  declare _flw_s, _nflw_s, _url, _img, origin_iri varchar;
+  declare urls, inx any;
   whenever not found goto no_ini;
   select VS_FOLLOW, VS_NFOLLOW, VS_URL, VS_SRC into _flw_s, _nflw_s, _url, _img
       from VFS_SITE where VS_HOST = _host and VS_ROOT = _root;
+  origin_iri := iri_to_id (WS.WS.MAKE_URL (_host, src_url), 1);
+  urls := make_array (length (xp), 'any');
+  inx := 0;
   foreach (any u in xp) do
     {
       declare hf, host, url, abs_url varchar;
@@ -1917,16 +1978,20 @@ create procedure WS.WS.SITEMAP_URLS_REGISTER (in _host varchar, in _root varchar
       url := WS.WS.VFS_URI_COMPOSE (hf);
       if (host <> _host)
 	url := abs_url;
+      urls [inx] := url;	
+      inx := inx + 1;
       if (WS.WS.FOLLOW (_host, _root, url, _flw_s, _nflw_s, _url, _img))
 	    {
-	  insert soft WS.WS.VFS_QUEUE (VQ_HOST, VQ_TS, VQ_URL, VQ_STAT, VQ_ROOT, VQ_LEVEL, VQ_VIA_SITEMAP) 
-	      values (_host, now (), url, 'waiting', _root, lev, sm); 
+	  insert soft WS.WS.VFS_QUEUE (VQ_HOST, VQ_TS, VQ_URL, VQ_STAT, VQ_ROOT, VQ_LEVEL, VQ_VIA_SITEMAP, VQ_ORIGIN) 
+	      values (_host, now (), url, 'waiting', _root, lev, sm, origin_iri); 
 	  if (row_count () = 0)
 	    {
 	      update WS.WS.VFS_QUEUE set VQ_STAT = 'waiting' where VQ_HOST = _host and VQ_ROOT = _root and VQ_URL = url and VQ_TS < ts;
 	    }
 	}
     }
+  if (delta)
+    delete from WS.WS.VFS_QUEUE where VQ_HOST = _host and VQ_ROOT = _root and VQ_ORIGIN = origin_iri and not position (VQ_URL, urls);
   if (length (xp))
     update WS.WS.VFS_SITE set VS_IS_SITEMAP = 1 where VS_HOST = _host and VS_ROOT = _root and VS_IS_SITEMAP = 0;
   commit work;
@@ -1980,7 +2045,7 @@ create procedure WS.WS.SITEMAP_XML_PARSE (in _host varchar, in _url varchar, in
   if (_url like '%.xml' or _url like '%.xml.gz' or _c_type = 'text/xml' or _c_type = 'application/xml' or _c_type = 'application/sparql-results+xml')
     {
       xt := xtree_doc (_content);
-      if (xpath_eval ('/urlset/dataset', xt) is not null)
+      if (xpath_eval ('/urlset/dataset/dataDumpLocation', xt) is not null)
 	{
 	  declare ts any;
 	  xp := xpath_eval ('/urlset/dataset/dataDumpLocation/text()', xt, 0);
@@ -1999,29 +2064,29 @@ create procedure WS.WS.SITEMAP_XML_PARSE (in _host varchar, in _url varchar, in
 		}
 	      xp := ar;
 	    }
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0);
 	}
       else if (xpath_eval ('/sitemapindex/sitemap/loc', xt) is not null)
 	{
 	  --xp := xpath_eval ('/sitemapindex/sitemap/loc/text()', xt, 0);
 	  xp := WS.WS.SITEMAP_GET_LOC (xt, '/sitemapindex/sitemap', './loc/text()', './lastmod/text()');
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0);
 	}
       else if (_c_type = 'application/sparql-results+xml')
 	{
 	  xp := xpath_eval ('/sparql/results/result/binding/uri/text()', xt, 0);
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0);
 	}
       else if (xpath_eval ('/urlset/url/loc', xt) is not null)
 	{
 	  --xp := xpath_eval ('/urlset/url/loc/text()', xt, 0);
 	  xp := WS.WS.SITEMAP_GET_LOC (xt, '/urlset/url', './loc/text()', './lastmod/text()');
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 1);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 1);
 	}
-      if (xpath_eval ('/urlset/dataset/sampleURI', xt) is not null)
+      else if (xpath_eval ('/urlset/dataset/sampleURI', xt) is not null)
 	{
 	  xp := xpath_eval ('/urlset/dataset/sampleURI/text()', xt, 0);
-	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 1);
+	  WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 1);
 	}
     }
 }
@@ -2086,14 +2151,14 @@ create procedure WS.WS.SITEMAP_RDF_STORE (in _host varchar, in _url varchar, in
             arr[i] := iri_to_id (arr[i], 0);  
 	}
       objs := (select DB.DBA.VECTOR_AGG (id_to_iri (O)) from DB.DBA.RDF_QUAD where G = iri_to_id (graph, 0) and P in (arr) and isiri_id (O));
-      WS.WS.SITEMAP_URLS_REGISTER (_host, _root, objs, lev, 0);
+      WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, objs, lev, 0, 0);
     } 
   if (_c_type = 'text/html' and isvector (udata) and 1 = get_keyword ('follow-meta', udata, 0))
     {
       declare xt, xp any;
       xt := xtree_doc (_content, 2);
       xp := xpath_eval ('/html/head/link[@rel="meta"]/@href', xt, 0);
-      WS.WS.SITEMAP_URLS_REGISTER (_host, _root, xp, lev, 0);
+      WS.WS.SITEMAP_URLS_REGISTER (_host, _root, _url, xp, lev, 0, 0);
     }
   insert soft WS.WS.VFS_URL (VU_HOST, VU_URL, VU_CHKSUM, VU_CPTIME, VU_ETAG, VU_ROOT)
       values (_host, _url, md5 (_content), now (), _s_etag, _root);
@@ -2104,7 +2169,7 @@ create procedure WS.WS.SITEMAP_RDF_STORE (in _host varchar, in _url varchar, in
 }
 ;
 
-create procedure VFS_ROBOTS_PARSE (in txt varchar, in ua varchar)
+create procedure VFS_ROBOTS_PARSE (in txt varchar, in ua varchar, out delay_sec float)
 {
   declare s any;
   declare v1 any;
@@ -2112,6 +2177,7 @@ create procedure VFS_ROBOTS_PARSE (in txt varchar, in ua varchar)
   s := string_output ();
   http (txt, s);
   v1 := '';
+  delay_sec := 0;
   while (1)
     {
       declare l, arr any;
@@ -2138,12 +2204,17 @@ create procedure VFS_ROBOTS_PARSE (in txt varchar, in ua varchar)
 	      exact_match := 1;
 	      v1 := '';
 	      fill := 1;
+	      delay_sec := 0;
 	    }
 	}
       else if (fill and lower (arr[0]) = 'disallow')
 	{
 	  v1 := v1 || trim (arr[1]) || '%;';
 	}
+      else if (fill and lower (arr[0]) = 'crawl-delay')
+	{
+	  delay_sec := atof (trim (arr[1]));
+	}
     }
   ex:
   v1 := trim (v1);
@@ -2153,9 +2224,9 @@ create procedure VFS_ROBOTS_PARSE (in txt varchar, in ua varchar)
 }
 ;
 
-create procedure VFS_ROBOTS_GET (in _host varchar, in _root varchar)
+create procedure VFS_ROBOTS_GET (in _host varchar, in _root varchar, inout site_delay_sec float)
 {
-  declare url, ret, head, me, robots any;
+  declare url, ret, head, me, robots, delay_sec any;
   url := vector ('http', _host, '/robots.txt', '', '', '');
   url := vfs_uri_compose (url);
   robots := null;
@@ -2168,9 +2239,11 @@ create procedure VFS_ROBOTS_GET (in _host varchar, in _root varchar)
     goto en;
   me := coalesce (cfg_item_value (virtuoso_ini_path (), 'HTTPServer', 'ClientIdString'), 
   	'Mozilla/4.0 (compatible; OpenLink Virtuoso)');
-  robots := vfs_robots_parse (ret, me);
+  robots := VFS_ROBOTS_PARSE (ret, me, delay_sec);
+  if (delay_sec > site_delay_sec)
+    site_delay_sec := delay_sec;
   en:
-  update VFS_SITE set VS_ROBOTS = robots where VS_HOST = _host and VS_ROOT = _root;
+  update VFS_SITE set VS_ROBOTS = robots, VS_DELAY = delay_sec where VS_HOST = _host and VS_ROOT = _root;
   return robots;
 }
 ;
