@@ -1,7 +1,7 @@
 --
 --  vfsddk.sql
 --
---  $Id: vfsddk.sql,v 1.5.2.12 2011/03/08 13:39:52 source Exp $
+--  $Id: vfsddk.sql,v 1.5.2.16 2011/08/01 14:17:06 source Exp $
 --
 --  Site-copy robot DB.
 --  
@@ -56,15 +56,18 @@ create table WS.WS.VFS_QUEUE (
     VQ_LEVEL	int default 0,
     VQ_VIA_SITEMAP int default 0,
     VQ_DT	timestamp,
+    VQ_ORIGIN	IRI_ID_8,
     primary key (VQ_HOST, VQ_URL, VQ_ROOT))
 create index VQ_HOST_ROOT on WS.WS.VFS_QUEUE (VQ_HOST, VQ_ROOT)
 create index VQ_HOST_TIME on WS.WS.VFS_QUEUE (VQ_HOST, VQ_ROOT, VQ_STAT, VQ_TS, VQ_URL)
 create index VQ_TS on WS.WS.VFS_QUEUE (VQ_TS)
+create index VQ_ORIGIN on WS.WS.VFS_QUEUE (VQ_ORIGIN)    
 ;
 
 
 -- Site setting
 create table WS.WS.VFS_SITE (
+    VS_ID	integer identity,	
     VS_DESCR	varchar,
     VS_HOST 	varchar,
     VS_URL	varchar,
@@ -95,6 +98,9 @@ create table WS.WS.VFS_SITE (
     VS_ACCEPT_RDF int default 0,
     VS_THREADS  int default 1,
     VS_ROBOTS long varchar default null,
+    VS_DELAY	float default 0,
+    VS_TIMEOUT	float default null,
+    VS_HEADERS  long varchar default null,
     primary key (VS_HOST, VS_ROOT))
 create index VS_HOST_ROOT on WS.WS.VFS_SITE (VS_HOST, VS_URL, VS_ROOT)
 ;
@@ -186,6 +192,18 @@ alter table WS.WS.VFS_SITE add VS_THREADS int default 1
 alter table WS.WS.VFS_SITE add VS_ROBOTS long varchar default null
 ;
 
+alter table WS.WS.VFS_SITE add VS_DELAY float default 0
+;
+
+alter table WS.WS.VFS_SITE add VS_TIMEOUT  float default null
+;
+
+alter table WS.WS.VFS_SITE add VS_HEADERS  long varchar default null
+;
+
+alter table WS.WS.VFS_SITE add VS_ID integer identity
+;
+
 alter table WS.WS.VFS_QUEUE add VQ_LEVEL int default 0
 ;
 
@@ -195,5 +213,32 @@ alter table WS.WS.VFS_QUEUE add VQ_VIA_SITEMAP int default 0
 alter table WS.WS.VFS_QUEUE add VQ_DT timestamp
 ;
 
+alter table WS.WS.VFS_QUEUE add VQ_ORIGIN IRI_ID_8
+;
+
 alter table WS.WS.VFS_URL add VU_RES_ID int
 ;
+
+create procedure WS.WS.VFS_UPGRADE ()
+{
+  declare inx int;
+  declare arr any;
+  if (not exists (select 1 from WS.WS.VFS_SITE where VS_ID is null))
+    return;
+  inx := 1;
+  arr := (select DB.DBA.VECTOR_AGG (vector (VS_HOST, VS_ROOT)) from WS.WS.VFS_SITE);
+  foreach (any x in arr) do
+    {
+      declare host, root varchar;
+      host := x[0];
+      root := x[1];
+      update WS.WS.VFS_SITE set VS_ID = inx where VS_HOST = host and VS_ROOT = root;
+      inx := inx + 1;
+    }
+  DB.DBA.SET_IDENTITY_COLUMN ('WS.WS.VFS_SITE', 'VS_ID', inx);
+}
+;
+
+--!AFTER
+WS.WS.VFS_UPGRADE ()
+;
