@@ -1,7 +1,7 @@
 /*
  *  Dkbox.c
  *
- *  $Id: Dkbox.c,v 1.22.2.5 2009/12/09 10:58:42 source Exp $
+ *  $Id: Dkbox.c,v 1.22.2.9 2011/10/25 19:36:36 source Exp $
  *
  *  Boxes
  *
@@ -41,6 +41,9 @@
 /*#define DV_UNAME_UNIT_DEBUG*/
 /*#define DV_UNAME_STATS*/
 
+#ifndef DOUBLE_ALIGN
+#error boxes must be aligned at 8
+#endif
 
 #ifdef _DEBUG
 long box_types_alloc[256];	/* implicit zero-fill assumed */
@@ -159,21 +162,22 @@ dk_alloc_box (size_t bytes, dtp_t tag)
   if (bytes & ~0xffffff)
     GPF_T1 ("box to allocate is too large");
 #endif
+
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes);
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
 #else
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-#ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dk_alloc (align_bytes) + 4;
-#else
-#error boxes must be aligned at 8
-  align_bytes += 4;
   ptr = (unsigned char *) dk_alloc (align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
+
+#ifdef DOUBLE_ALIGN
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[(unsigned) tag]++;
 #endif
@@ -192,21 +196,22 @@ dk_alloc_box_long (size_t bytes, dtp_t tag)
   if (bytes > 100000000)
     GPF_T1 ("malloc debug only check for large boxes, over 100M");
 #endif
+
   /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes);
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
 #else
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
 
-#ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dk_alloc (align_bytes) + 4;
-#else
-#error boxes must be aligned at 8
-  align_bytes += 4;
   ptr = (unsigned char *) dk_alloc (align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
+
+#ifdef DOUBLE_ALIGN
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[(unsigned) tag]++;
 #endif
@@ -228,25 +233,25 @@ dk_try_alloc_box (size_t bytes, dtp_t tag)
   if (bytes & ~0xffffff)
     GPF_T1 ("box to allocate is too large");
 #endif
-  /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
 
+  /* This assumes dk_alloc aligns at least at 4 */
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
+#else
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
+#endif
+
   ptr = (unsigned char *) dk_try_alloc (align_bytes);
   if (!ptr)
     return (box_t) ptr;
-  ptr += 4;
-#else
-  align_bytes += 4;
-  ptr = (unsigned char *) dk_try_alloc (align_bytes);
-#endif
+
 #ifdef _DEBUG
   box_types_alloc[(unsigned) tag]++;
 #endif
 
-  if (!ptr)
-    return (box_t) ptr;
+#ifdef DOUBLE_ALIGN
+  ptr += 4;
+#endif
 
   WRITE_BOX_HEADER (ptr, bytes, tag);
   /* memset (ptr, 0, bytes); */
@@ -262,15 +267,20 @@ dk_alloc_box_zero (size_t bytes, dtp_t tag)
   size_t align_bytes;
 
   /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+#ifdef DOUBLE_ALIGN
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
+#else
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
+#endif
+
+  ptr = (unsigned char *) dk_alloc (align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
 
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dk_alloc (align_bytes) + 4;
-#else
-  align_bytes += 4;
-  ptr = (unsigned char *) dk_alloc (align_bytes);
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[tag]++;
 #endif
@@ -290,15 +300,20 @@ dbg_dk_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
   uint32 align_bytes;
 
   /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+#ifdef DOUBLE_ALIGN
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
+#else
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
+#endif
+
+  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
 
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes) + 4;
-#else
-  align_bytes += 4;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[tag]++;
 #endif
@@ -317,15 +332,20 @@ dbg_dk_alloc_box_long (DBG_PARAMS size_t bytes, dtp_t tag)
   uint32 align_bytes;
 
   /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+#ifdef DOUBLE_ALIGN
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
+#else
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
+#endif
+
+  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
 
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes) + 4;
-#else
-  align_bytes += 4;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[tag]++;
 #endif
@@ -346,17 +366,20 @@ dbg_dk_try_alloc_box (DBG_PARAMS size_t bytes, dtp_t tag)
   uint32 align_bytes;
 
   /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
-
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes) + 4;
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
 #else
-  align_bytes += 4;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
 #endif
+
+  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
   if (!ptr)
     return (box_t) ptr;
+
+#ifdef DOUBLE_ALIGN
+  ptr += 4;
+#endif
+
 #ifdef _DEBUG
   box_types_alloc[tag]++;
 #endif
@@ -375,15 +398,20 @@ dbg_dk_alloc_box_zero (DBG_PARAMS size_t bytes, dtp_t tag)
   uint32 align_bytes;
 
   /* This assumes dk_alloc aligns at least at 4 */
-  align_bytes = IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes);
+#ifdef DOUBLE_ALIGN
+  align_bytes = 8 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_8 (bytes));
+#else
+  align_bytes = 4 + (IS_STRING_ALIGN_DTP (tag) ? ALIGN_STR (bytes) : ALIGN_4 (bytes));
+#endif
+
+  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  if (!ptr)
+    return (box_t) ptr;
 
 #ifdef DOUBLE_ALIGN
-  align_bytes += 8;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes) + 4;
-#else
-  align_bytes += 4;
-  ptr = (unsigned char *) dbg_malloc (DBG_ARGS align_bytes);
+  ptr += 4;
 #endif
+
 #ifdef _DEBUG
   box_types_alloc[tag]++;
 #endif
@@ -564,7 +592,11 @@ dk_free_box (box_t box)
       if (box_destr[tag])
 	if (0 != box_destr[tag] (box))
 	  return 0;
+#ifdef DOUBLE_ALIGN
+      len = ALIGN_8 (len);
+#else
       len = ALIGN_4 (len);
+#endif
     }
 
 #ifndef NDEBUG
@@ -678,7 +710,11 @@ dk_free_tree (box_t box)
       if (box_destr[tag])
 	if (0 != box_destr[tag] (box))
 	  return 0;
+#ifdef DOUBLE_ALIGN
+      len = ALIGN_8 (len);
+#else
       len = ALIGN_4 (len);
+#endif
     }
 
 #ifndef NDEBUG
@@ -711,7 +747,7 @@ box_reuse (caddr_t box, ccaddr_t data, size_t len, dtp_t dtp)
 }
 
 
-#ifdef DEBUG
+#ifdef DK_ALLOC_BOX_DEBUG
 void
 dk_check_tree_iter (box_t box, box_t parent, dk_hash_t * known)
 {
@@ -723,12 +759,10 @@ dk_check_tree_iter (box_t box, box_t parent, dk_hash_t * known)
   tag = box_tag (box);
   if ((DV_UNAME == tag) || (DV_REFERENCE == tag))
     return;
-#ifndef NDEBUG
   if (TAG_FREE == tag)
     GPF_T1 ("Tree contains a pointer to a freed box");
   if (TAG_BAD == tag)
     GPF_T1 ("Tree contains a pointer to a box marked bad");
-#endif
   if (!box_can_appear_twice_in_tree[tag])
     {
       box_t other_parent = gethash (box, known);
@@ -765,12 +799,10 @@ dk_check_tree_heads_iter (box_t box, box_t parent, dk_hash_t * known, int count_
   tag = box_tag (box);
   if ((DV_UNAME == tag) || (DV_REFERENCE == tag))
     return;
-#ifndef NDEBUG
   if (TAG_FREE == tag)
     GPF_T1 ("Tree contains a pointer to a freed box");
   if (TAG_BAD == tag)
     GPF_T1 ("Tree contains a pointer to a box marked bad");
-#endif
   if (!box_can_appear_twice_in_tree[tag])
     {
       box_t other_parent = gethash (box, known);
@@ -809,12 +841,10 @@ dk_check_domain_of_connectivity_iter (box_t box, box_t parent, dk_hash_t * known
   tag = box_tag (box);
   if ((DV_UNAME == tag) || (DV_REFERENCE == tag))
     return;
-#ifndef NDEBUG
   if (TAG_FREE == tag)
     GPF_T1 ("Domain of connectivity contains a pointer to a freed box");
   if (TAG_BAD == tag)
     GPF_T1 ("Domain of connectivity contains a pointer to a box marked bad");
-#endif
   if (IS_NONLEAF_DTP (tag))
     {
       box_t *obj = (box_t *) box;
@@ -1311,12 +1341,18 @@ rbb_allocate (void)
 caddr_t 
 rbb_from_id (int64 n)
 {
-  rdf_box_t * rb = (rdf_box_t*)rbb_allocate ();
-  rb->rb_ro_id = n;
-  rb->rb_is_outlined = 1;
-  rb->rb_type = RDF_BOX_DEFAULT_TYPE;
-  rb->rb_lang = RDF_BOX_DEFAULT_LANG;
-  return (caddr_t)rb;
+  rdf_bigbox_t * rbb = rbb_allocate ();
+  rbb->rbb_base.rb_ro_id = n;
+  rbb->rbb_base.rb_is_outlined = 1;
+#if 0
+  rbb->rbb_base.rb_type = RDF_BOX_ILL_TYPE;
+  rbb->rbb_base.rb_lang = RDF_BOX_ILL_LANG;
+#else
+  rbb->rbb_base.rb_type = RDF_BOX_DEFAULT_TYPE;
+  rbb->rbb_base.rb_lang = RDF_BOX_DEFAULT_LANG;
+#endif
+  rbb->rbb_box_dtp = DV_STRING;
+  return (caddr_t)rbb;
 }
 
 
@@ -1329,6 +1365,8 @@ rdf_box_audit_impl (rdf_box_t * rb)
   if ((0 == rb->rb_ro_id) && (0 == rb->rb_is_complete))
     GPF_T1 ("RDF box is too incomplete");
 #endif
+  if (rb->rb_is_complete)
+    rb_dt_lang_check(rb);
 }
 
 
@@ -1405,6 +1443,17 @@ DBG_NAME (box_dv_short_strconcat) (DBG_PARAMS const char *str1, const char *str2
 }
 
 
+box_t
+DBG_NAME (box_dv_wide_nchars) (DBG_PARAMS const wchar_t *buf, size_t buf_wchar_count)
+{
+  wchar_t *box;
+  box = (wchar_t *)DBG_NAME (dk_alloc_box) (DBG_ARGS (uint32) ((buf_wchar_count + 1) * sizeof (wchar_t)), DV_WIDE);
+  memcpy (box, buf, buf_wchar_count * sizeof (wchar_t));
+  box[buf_wchar_count] = (wchar_t)'\0';
+  return (box_t)box;
+}
+
+
 caddr_t
 DBG_NAME (box_vsprintf) (DBG_PARAMS size_t buflen_eval, const char *format, va_list tail)
 {
@@ -2426,6 +2475,14 @@ box_float (float d)
 }
 
 
+#undef box_dv_wide_nchars
+box_t
+box_dv_wide_nchars (const wchar_t *buf, size_t buf_wchar_count)
+{
+  return dbg_box_dv_wide_nchars (__FILE__, __LINE__, buf, buf_wchar_count);
+}
+
+
 #undef box_vsprintf
 caddr_t
 box_vsprintf (size_t buflen_eval, const char *format, va_list tail)
