@@ -1,7 +1,7 @@
 /*
  *  Dkbox.h
  *
- *  $Id: Dkbox.h,v 1.32.2.9 2011/03/08 13:41:56 source Exp $
+ *  $Id: Dkbox.h,v 1.32.2.11 2011/10/25 19:36:36 source Exp $
  *
  *  Boxes
  *
@@ -251,12 +251,14 @@ ptr += 4
 
 #ifdef DOUBLE_ALIGN
 
+#define ALIGN_LIKE_BOX(x)		ALIGN_8(x)
 #define STATIC_DV_NULL 			{0,0,0,0,0,0,0,(char)DV_DB_NULL}
 #define BOX_AUTO_OVERHEAD 		8
 #define BOX_BEGIN_IN_AREA(area) 	(((char *) (~((ptrlong)7) & (ptrlong)(area))) + BOX_AUTO_OVERHEAD)
 
 #else /* DOUBLE_ALIGN */
 
+#define ALIGN_LIKE_BOX(x)		ALIGN_4(x)
 #define STATIC_DV_NULL 			{0,0,0,(char)DV_DB_NULL}
 #define BOX_AUTO_OVERHEAD 		4
 #define BOX_BEGIN_IN_AREA(area) 	(((char *) area) + BOX_AUTO_OVERHEAD)
@@ -602,6 +604,8 @@ uname_blk_t;
 #define RDF_BOX_DEFAULT_LANG 		0x0101
 #define RDF_BOX_MAX_TYPE 		0x7F01
 #define RDF_BOX_MAX_LANG 		0x7F01
+#define RDF_BOX_ILL_TYPE 		0x7F02
+#define RDF_BOX_ILL_LANG 		0x7F03
 #define RDF_BOX_GEO 			0x100
 #define RDF_BOX_INTERVAL 		0xff
 #define RDF_BOX_STRING_ID 		0xfe /* Like a type 257 but no string inlined, collates by lang and id alone */
@@ -621,6 +625,15 @@ typedef struct rdf_box_s
   caddr_t 		rb_box;
 } rdf_box_t;
 
+#ifndef NDEBUG
+#define rb_dt_lang_check(rb) do { \
+    if (RDF_BOX_ILL_TYPE == (rb)->rb_type) GPF_T1("Bad rb_type"); \
+    if (RDF_BOX_ILL_LANG == (rb)->rb_lang) GPF_T1("Bad rb_lang"); \
+  } while (0)
+#else
+#define rb_dt_lang_check(rb)
+#endif
+
 #define RB_MAX_INLINED_CHARS 		20
 
 #define RBS_OUTLINED			0x01
@@ -697,6 +710,7 @@ EXE_EXPORT (box_t, box_dv_uname_nchars, (const char *buf, size_t buf_len));
 EXE_EXPORT (box_t, box_dv_uname_substr, (ccaddr_t box, int n1, int n2));
 EXE_EXPORT (box_t, box_double, (double d));
 EXE_EXPORT (box_t, box_float, (float d));
+EXE_EXPORT (box_t, box_dv_wide_nchars, (const wchar_t *buf, size_t buf_wchar_count));
 #ifdef _DKSYSTEM_H
 EXE_EXPORT (caddr_t, box_vsprintf, (size_t buflen_eval, const char *format, va_list tail));
 #endif
@@ -761,6 +775,7 @@ box_t dbg_box_dv_uname_nchars (const char *file, int line, const char *buf, size
 box_t dbg_box_dv_uname_substr (const char *file, int line, ccaddr_t box, int n1, int n2);
 box_t dbg_box_double (const char *file, int line, double d);
 box_t dbg_box_float (const char *file, int line, float d);
+box_t dbg_box_dv_wide_nchars (const char *file, int line, const wchar_t *buf, size_t buf_wchar_count);
 #ifdef _DKSYSTEM_H
 caddr_t dbg_box_vsprintf (const char *file, int line, size_t buflen_eval, const char *format, va_list tail);
 #endif
@@ -790,6 +805,7 @@ caddr_t dbg_box_vsprintf (const char *file, int line, size_t buflen_eval, const
 #define box_dv_uname_substr(S,N1,N2)		dbg_box_dv_uname_substr (__FILE__, __LINE__, (S), (N1), (N2))
 #define box_double(D)				dbg_box_double (__FILE__, __LINE__, (D))
 #define box_float(D)				dbg_box_float (__FILE__, __LINE__, (D))
+#define box_dv_wide_nchars(B,WCHAR_COUNT)	dbg_box_dv_wide_nchars (__FILE__, __LINE__, (B), (WCHAR_COUNT))
 #define box_vsprintf(L,F,T)			dbg_box_vsprintf (__FILE__, __LINE__, (L), (F), (T))
 
 #endif
