@@ -1,5 +1,5 @@
 /*
- *  $Id: VirtuosoConnection.java,v 1.9.2.11 2010/07/09 14:22:28 source Exp $
+ *  $Id: VirtuosoConnection.java,v 1.9.2.13 2011/10/28 21:37:06 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -128,7 +128,8 @@ public class VirtuosoConnection implements Connection
    private boolean readOnly = false;
 
    // The timeout for I/O
-   protected int timeout = 60;
+   protected int timeout_def = 60;
+   protected int timeout = 0;
    protected int txn_timeout = 0;
 
    protected int fbs = VirtuosoTypes.DEFAULTPREFETCH;
@@ -358,6 +359,7 @@ public class VirtuosoConnection implements Connection
    public boolean isConnectionLost() 
    {
      try{
+	pingStatement.setQueryTimeout(1);
         pingStatement.execute();
         return false;
      } catch (Exception e ) {
@@ -533,7 +535,9 @@ public class VirtuosoConnection implements Connection
 	else
 #endif
 	 socket = new Socket(host,port);
-	 socket.setSoTimeout(timeout*1000);
+
+	 if (timeout > 0)
+	   socket.setSoTimeout(timeout*1000);
 	 socket.setTcpNoDelay(true);
 #if JDK_VER >= 12
          socket.setReceiveBufferSize(recvbs);
@@ -656,13 +660,23 @@ public class VirtuosoConnection implements Connection
 			 //  System.err.println (client_charset.elementAt(0).toString());
 			 //else
 			 //  System.err.println ("<NULL>");
-			 timeout = (int) (cdef_param (client_defaults, "SQL_QUERY_TIMEOUT", timeout * 1000) / 1000);
-			 //System.err.println ("timeout = " + timeout);
-			 socket.setSoTimeout(timeout*1000);
-			 txn_timeout = (int) (cdef_param (client_defaults, "SQL_TXN_TIMEOUT", txn_timeout * 1000)/ 1000);
-			 //System.err.println ("txn timeout = " + txn_timeout);
-			 trxisolation = (int) cdef_param (client_defaults, "SQL_TXN_ISOLATION", trxisolation);
-			 //System.err.println ("txn isolation = " + trxisolation);
+
+			 if (timeout <= 0) {
+			   timeout = (int) (cdef_param (client_defaults, "SQL_QUERY_TIMEOUT", timeout_def * 1000) / 1000);
+			   //System.err.println ("timeout = " + timeout);
+			 }
+                         if (timeout > 0)
+			   socket.setSoTimeout(timeout*1000);
+
+			 if (txn_timeout <= 0) {
+			   txn_timeout = (int) (cdef_param (client_defaults, "SQL_TXN_TIMEOUT", txn_timeout * 1000)/ 1000);
+			   //System.err.println ("txn timeout = " + txn_timeout);
+			 }
+
+			 if (trxisolation <= 0) {
+			   trxisolation = (int) cdef_param (client_defaults, "SQL_TXN_ISOLATION", trxisolation);
+			   //System.err.println ("txn isolation = " + trxisolation);
+			 }
 
 			 utf8_execs = cdef_param (client_defaults, "SQL_UTF8_EXECS", 0) != 0;
 			 //System.err.println ("utf8_execs = " + utf8_execs);
@@ -1618,7 +1632,7 @@ public class VirtuosoConnection implements Connection
    protected byte[] charsetBytes1(String source, String from, String to) throws VirtuosoException
     {
        byte ans[] = new byte[0];
-       System.err.println ("charsetBytes1(" + from + " , " + to);
+       //System.err.println ("charsetBytes1(" + from + " , " + to);
        //System.err.println ("charsetBytes1 src len=" + source.length());
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream( source.length() );
        try
