@@ -1,5 +1,5 @@
 /*
- *  $Id: VirtuosoConnectionPoolDataSource.java,v 1.1.1.1.2.3 2010/01/25 15:42:53 source Exp $
+ *  $Id: VirtuosoConnectionPoolDataSource.java,v 1.1.1.1.2.6 2011/08/18 09:26:38 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -234,6 +234,8 @@ public class VirtuosoConnectionPoolDataSource
     synchronized(initLock) {
       if (!isInitialized) {
         isInitialized = true;
+        if (initialPoolSize == 0)
+          initialPoolSize = minPoolSize;
         if (initialPoolSize != 0) {
           OpenHelper initThread = new OpenHelper(initialPoolSize, info);
           initThread.start();
@@ -321,6 +323,8 @@ public class VirtuosoConnectionPoolDataSource
     synchronized(initLock) {
       if (!isInitialized) {
         isInitialized = true;
+        if (initialPoolSize == 0)
+          initialPoolSize = minPoolSize;
         if (initialPoolSize != 0) {
           OpenHelper initThread = new OpenHelper(initialPoolSize, info);
           initThread.start();
@@ -365,7 +369,7 @@ public class VirtuosoConnectionPoolDataSource
   public void setMinPoolSize(int parm) throws SQLException
   {
     try {
-      Field fld = getClass().getField(n_minPoolSize);
+      Field fld = getClass().getField(this.n_minPoolSize);
       setField(fld, parm);
     } catch (Exception e) {
       throw new VirtuosoException("Error: "+e.toString(), VirtuosoException.OK);
@@ -398,7 +402,7 @@ public class VirtuosoConnectionPoolDataSource
   public void setMaxPoolSize(int parm) throws SQLException
   {
     try {
-      Field fld = getClass().getField(n_maxPoolSize);
+      Field fld = getClass().getField(this.n_maxPoolSize);
       setField(fld, parm);
     } catch (Exception e) {
       throw new VirtuosoException("Error: "+e.toString(), VirtuosoException.OK);
@@ -429,7 +433,7 @@ public class VirtuosoConnectionPoolDataSource
   public void setInitialPoolSize(int parm) throws SQLException
   {
     try {
-      Field fld = getClass().getField(n_initialPoolSize);
+      Field fld = getClass().getField(this.n_initialPoolSize);
       setField(fld, parm);
     } catch (Exception e) {
       throw new VirtuosoException("Error: "+e.toString(), VirtuosoException.OK);
@@ -461,7 +465,7 @@ public class VirtuosoConnectionPoolDataSource
   public void setMaxIdleTime(int parm) throws SQLException
   {
     try {
-      Field fld = getClass().getField(n_maxIdleTime);
+      Field fld = getClass().getField(this.n_maxIdleTime);
       setField(fld, parm);
     } catch (Exception e) {
       throw new VirtuosoException("Error: "+e.toString(), VirtuosoException.OK);
@@ -515,7 +519,7 @@ public class VirtuosoConnectionPoolDataSource
   public void setMaxStatements(int parm) throws SQLException
   {
     try {
-      Field fld = getClass().getField(n_maxStatements);
+      Field fld = getClass().getField(this.n_maxStatements);
       setField(fld, parm);
     } catch (Exception e) {
       throw new VirtuosoException("Error: "+e.toString(), VirtuosoException.OK);
@@ -571,7 +575,7 @@ public class VirtuosoConnectionPoolDataSource
     }
 
     public void run() {
-      if (connPool.cacheSize >= count || (maxPoolSize != 0 && connPool.cacheSize > maxPoolSize))
+      if (connPool.cacheSize >= count || (maxPoolSize != 0 && connPool.cacheSize >= maxPoolSize))
         return;
 
       for(int i = 0; i < count; i++)
@@ -579,7 +583,8 @@ public class VirtuosoConnectionPoolDataSource
 
           VirtuosoConnection conn = new VirtuosoConnection (conn_url, "localhost", 1111, info);
           connPool.addPooledConnection(new VirtuosoPooledConnection(conn, connKey));
-          if (connPool.cacheSize >= count || (maxPoolSize != 0 && connPool.cacheSize > maxPoolSize))
+          if ((minPoolSize != 0 && connPool.cacheSize >= minPoolSize) 
+              || (maxPoolSize != 0 && connPool.cacheSize >= maxPoolSize))
             return;
         } catch (Exception e) {
         }
@@ -590,14 +595,14 @@ public class VirtuosoConnectionPoolDataSource
 
   private class CloseHelper extends Thread {
 
-    private ArrayList connList;
+    private LinkedList connList;
     private PooledConnection pconn;
 
     private CloseHelper() {
       setName("Virtuoso CloseHelper");
     }
 
-    protected CloseHelper(ArrayList _connList) {
+    protected CloseHelper(LinkedList _connList) {
       this();
       connList = _connList;
     }
@@ -609,7 +614,7 @@ public class VirtuosoConnectionPoolDataSource
 
     public void run() {
       if (connList != null) {
-        for(Iterator i = connList.iterator(); i.hasNext(); )
+        for(ListIterator i = connList.listIterator(); i.hasNext(); )
           try {
             ((VirtuosoPooledConnection)i.next()).close();
           } catch (Exception e) { }
@@ -728,17 +733,40 @@ public class VirtuosoConnectionPoolDataSource
     }
 
     private VirtuosoPooledConnection lookup(String _Key) {
+#if JDK_VER >= 16
+      LinkedList<Object> closeTmp = new LinkedList<Object>();
+#else
+      LinkedList closeTmp = new LinkedList();
+#endif
       VirtuosoPooledConnection pooledConn;
       int _hashKey = _Key.hashCode();
 
-      for(Iterator iterator = unUsed.iterator(); iterator.hasNext(); ) {
-        pooledConn = (VirtuosoPooledConnection)iterator.next();
-        if (pooledConn.hashConnURL == _hashKey && pooledConn.connURL.equals(_Key)) {
+      try {
+        for(ListIterator iterator = unUsed.listIterator(); iterator.hasNext(); ) {
+          pooledConn = (VirtuosoPooledConnection)iterator.next();
+          if (pooledConn.hashConnURL == _hashKey && pooledConn.connURL.equals(_Key)) {
             iterator.remove();
-            return pooledConn;
+	    if (pooledConn.isConnectionLost()) {
+	      synchronized(this) {              
+                if (cacheSize > 1) 
+                  --cacheSize;
+                closeTmp.add(pooledConn);
+              }
+	    } else {
+              return pooledConn;
+	    }
+          }
+        }
+
+        return null;
+
+      } finally {
+        if (closeTmp.size() > 0) {
+          // close connections
+          CloseHelper helpThread = new CloseHelper(closeTmp);
+          helpThread.start();
         }
       }
-      return null;
     }
 
 
@@ -760,48 +788,49 @@ public class VirtuosoConnectionPoolDataSource
         }
       }
     // if couldn't found an unused Connection
-      if (maxPoolSize == 0 || cacheSize < maxPoolSize) {
-       // establish a new Connection
-        synchronized(this) {
+      synchronized(this) { 
+        if (maxPoolSize == 0 || cacheSize < maxPoolSize) {
+         // establish a new Connection
           conn = new VirtuosoConnection (conn_url, "localhost", 1111, info);
           cacheSize++;
           pconn = new VirtuosoPooledConnection(conn, connKey, cpds);
           in_Use.put(pconn, pconn);
+          return pconn;
         }
-        return pconn;
 
-      } else {
+      } 
+
       // wait a free Connection
-        long start = System.currentTimeMillis();
-        long _timeout = loginTimeout * 1000L;
-        Thread thr = Thread.currentThread();
-        while (pconn == null) {
-//      System.out.println("Thread "+thr+" begin a waiting...");
-          synchronized(this) {
-            if (!unUsed.isEmpty() && (pconn = lookup(connKey)) != null) {
-//            System.out.println("Thread "+thr+" has found a free connection");
-              pconn.init(cpds);
-              in_Use.put(pconn, pconn);
-              return pconn;
-            }
+      long start = System.currentTimeMillis();
+      long _timeout = loginTimeout * 1000L;
+      Thread thr = Thread.currentThread();
+      while (pconn == null) {
+//    System.out.println("Thread "+thr+" begin a waiting...");
+        synchronized(this) {
+          if (!unUsed.isEmpty() && (pconn = lookup(connKey)) != null) {
+//          System.out.println("Thread "+thr+" has found a free connection");
+            pconn.init(cpds);
+            in_Use.put(pconn, pconn);
+            return pconn;
+          }
 
-            try {
-              if (loginTimeout > 0) {
-                wait(_timeout);
-                _timeout -= (System.currentTimeMillis() - start);
-                if (_timeout < 0) {
-//                System.out.println("Thread "+thr+" : loginTimeout has expired");
-                  throw new VirtuosoException("Connection failed loginTimeout has expired", VirtuosoException.TIMEOUT);
-                }
-              } else {
-                wait();
+          try {
+            if (loginTimeout > 0) {
+              wait(_timeout);
+              _timeout -= (System.currentTimeMillis() - start);
+              if (_timeout < 0) {
+//              System.out.println("Thread "+thr+" : loginTimeout has expired");
+                throw new VirtuosoException("Connection failed loginTimeout has expired", VirtuosoException.TIMEOUT);
               }
-//            System.out.println("Thread "+thr+" has woken ");
-            } catch (InterruptedException e) {
+            } else {
+              wait();
             }
+//          System.out.println("Thread "+thr+" has woken ");
+          } catch (InterruptedException e) {
           }
         }
       }
+
       return null;
     }
 
@@ -809,9 +838,9 @@ public class VirtuosoConnectionPoolDataSource
     private void checkPool() {
       VirtuosoPooledConnection pooledConn;
 #if JDK_VER >= 16
-      ArrayList<Object> closeTmp = null;
+      LinkedList<Object> closeTmp = new LinkedList<Object>();
 #else
-      ArrayList closeTmp = null;
+      LinkedList closeTmp = new LinkedList();
 #endif
       ListIterator l_iter;
 
@@ -820,14 +849,8 @@ public class VirtuosoConnectionPoolDataSource
         long minTime = System.currentTimeMillis() - maxIdleTime * 1000L;
 
         synchronized(this) {
-          int count = unUsed.size();
-#if JDK_VER >= 16
-          closeTmp = new ArrayList<Object>(count);
-#else
-          closeTmp = new ArrayList(count);
-#endif
-          for(l_iter = unUsed.listIterator(); l_iter.hasPrevious(); ) {
-            pooledConn = (VirtuosoPooledConnection)l_iter.previous();
+          for(l_iter = unUsed.listIterator(); l_iter.hasNext(); ) {
+            pooledConn = (VirtuosoPooledConnection)l_iter.next();
             if (pooledConn.tmClosed < minTime) {
                closeTmp.add(pooledConn);
                l_iter.remove();
@@ -837,17 +860,12 @@ public class VirtuosoConnectionPoolDataSource
         }
       }
 
-      if (maxPoolSize != 0 && cacheSize > maxPoolSize) {
-       //remove connections
-        synchronized(this) {
+      synchronized(this) {
+        if (maxPoolSize != 0 && cacheSize > maxPoolSize) {
+         //remove connections
           int count = cacheSize - maxPoolSize;
-#if JDK_VER >= 16
-          closeTmp = new ArrayList<Object>(count);
-#else
-          closeTmp = new ArrayList(count);
-#endif
-          for(l_iter = unUsed.listIterator(); l_iter.hasPrevious() && count > 0; count--) {
-            closeTmp.add(l_iter.previous());
+          for(l_iter = unUsed.listIterator(); l_iter.hasNext() && count > 0; count--) {
+            closeTmp.add(l_iter.next());
             l_iter.remove();
             cacheSize--;
           }
@@ -860,12 +878,14 @@ public class VirtuosoConnectionPoolDataSource
         helpThread.start();
       }
 
-      if (minPoolSize != 0 && cacheSize < minPoolSize) {
-       //add connections
-        Properties info = createConnProperties();
-        int count = minPoolSize - cacheSize;
-        OpenHelper helpThread = new OpenHelper(count, info);
-        helpThread.start();
+      synchronized(this) {
+        if (minPoolSize != 0 && cacheSize < minPoolSize) {
+         //add connections
+          Properties info = createConnProperties();
+          int count = minPoolSize - cacheSize;
+          OpenHelper helpThread = new OpenHelper(count, info);
+          helpThread.start();
+        }
       }
     }
 
