@@ -1,7 +1,7 @@
 /*
  *  thread_int.h
  *
- *  $Id: thread_int.h,v 1.6.2.1 2010/11/09 11:01:45 source Exp $
+ *  $Id: thread_int.h,v 1.6.2.3 2011/10/13 18:34:19 source Exp $
  *
  *  Thread internals
  *  
@@ -132,6 +132,7 @@ struct thread_s
   void *		thr_sql_scs;
   int                   thr_attached;
   caddr_t		thr_dbg;
+  struct lock_trx_s *	thr_lt; /* use to access lt during checkpoint wait */
 #ifndef NDEBUG
   void *		thr_pg_dbg;
 #endif  
@@ -225,6 +226,7 @@ struct mutex_s
     long		mtx_spin_waits;
     long		mtx_waits;
     long		mtx_enters;
+    long long		mtx_wait_clocks;
 #endif
     int			mtx_type;
   };
