@@ -1,7 +1,7 @@
 /*
  *  2pc.c
  *
- *  $Id: 2pc.c,v 1.15.2.7 2009/08/19 07:29:52 source Exp $
+ *  $Id: 2pc.c,v 1.15.2.17 2010/10/05 09:39:17 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -132,12 +132,21 @@ def_commit_done (void *res, int trx_status)
   semaphore_leave (future->ft_sem);
   return 0;
 }
+
 static unsigned long
 def_abort_done (void *res, int trx_status)
 {
   tp_future_t *future = (tp_future_t *) res;
   if (res)
-    semaphore_leave (future->ft_sem);
+    {
+      if (!future->ft_release)
+	semaphore_leave (future->ft_sem);
+      else			/* the other end marked to be release as it will not wait for it */
+	{
+	  semaphore_free (future->ft_sem);
+	  dk_free (future, sizeof (tp_future_t));
+	}
+    }
   _2pc_printf (("abort_done... %d\n", trx_status));
   return 0;
 }
@@ -215,7 +224,7 @@ mq_create_message (int type, void *resource, void *client_v)
   NEW_VARZ (tp_message_t, mm);
 
   mm->mm_type = type;
-  mm->mm_resource = resource;
+  mm->mm_resource = (tp_future_t *)resource;
   mm->mm_trx = client->cli_trx;
   mm->mm_tp_data = client->cli_tp_data;
   mm->vtbl = &tp_vtbl;
@@ -241,7 +250,7 @@ mq_create_xa_message (int type, void *resource, void *tp_data)
   NEW_VARZ (tp_message_t, mm);
 
   mm->mm_type = type;
-  mm->mm_resource = resource;
+  mm->mm_resource = (tp_future_t *)resource;
   mm->mm_trx = tpd->cli_tp_lt;
   mm->mm_tp_data = tpd;
   mm->vtbl = &xa_tp_vtbl;
@@ -512,22 +521,33 @@ cli_2pc_transact (lock_trx_t * lt, int operation)
       LEAVE_TXN;
       return LTE_DEADLOCK;
     }
+  lt_wait_checkpoint ();
   lt->lt_2pc._2pc_prepared = 0;
   if (operation == SQL_ROLLBACK)
     lt->lt_status = LT_BLOWN_OFF;
 
   if (lt->lt_status == LT_COMMITTED && !LT_IS_RUNNING (lt))
+    {
+      lt_threads_inc_inner (lt);
+      LT_CLOSE_ACK_THREADS(lt);
+      lt->lt_close_ack_threads++;
     lt_2pc_commit (lt);
+    }
   else
     lt_kill_other_trx (lt, NULL, NULL, LT_KILL_ROLLBACK);
 
   if (lt->lt_error != LTE_OK)
-    lt_log_debug (("cli_2pc_transact op=%d result=%d lt=%p cli=%p", operation,
+    {
+      lt_log_debug (("cli_2pc_transact op=%d result=%d lt=%p cli=%p", operation,
 	    (int) lt->lt_error, lt, lt->lt_client));
+    }
+  lt->lt_2pc._2pc_wait_commit = 0; /* commit happened */
   LEAVE_TXN;
   return lt->lt_error;
 }
 
+extern dk_mutex_t * log_write_mtx;
+
 int
 lt_2pc_prepare (lock_trx_t * lt)
 {
@@ -536,24 +556,26 @@ lt_2pc_prepare (lock_trx_t * lt)
   _2pc_printf (("lt_2pc_prepare\n"));
   lt->lt_status = LT_PREPARE_PENDING;
 
-  if (LTE_OK != lt_log_replication (lt))
-    {
-      rc = LTE_LOG_FAILED;
-      goto failed;
-    }
 
+  LEAVE_TXN;
+  mutex_enter (log_write_mtx);
   if (LTE_OK != log_commit (lt))
     {
       rc = LTE_LOG_FAILED;
+      mutex_leave (log_write_mtx);
+      IN_TXN;
       goto failed;
     }
+  mutex_leave (log_write_mtx);
+  IN_TXN;
 
   lt->lt_status = LT_COMMITTED;
+  lt->lt_2pc._2pc_wait_commit = 1;
   if (lt->lt_2pc._2pc_prepared)
     {
       *((unsigned long *) lt->lt_2pc._2pc_prepared) = LT_PREPARED;
       lt->lt_2pc._2pc_prepared = 0;
-    };
+    }
 
 failed:
   if (rc != LTE_OK)
@@ -585,7 +607,6 @@ lt_2pc_commit (lock_trx_t * lt)
   lt->lt_status = LT_COMMITTED;
 
   log_final_transact (lt, 1);
-  lt_send_repl_cast (lt);
   if (lt->lt_mode == TM_SNAPSHOT)
     {
       lt_close_snapshot (lt);
@@ -615,6 +636,7 @@ lt_2pc_commit (lock_trx_t * lt)
 #ifdef MSDTC_DEBUG
   lt->lt_in_mts = 0;
 #endif
+  lt_resume_waiting_end (lt);
   lt_restart (lt, TRX_CONT_LT_LEAVE);
   IN_TXN;
 
@@ -661,7 +683,7 @@ xa_wait_commit (tp_data_t * tpd)
   if (tpd)
     {
       semaphore_enter (tpd->cli_tp_sem2);
-      /* tp_data_free (tpd); */
+      tp_data_free (tpd);
     }
   return 0;
 }
@@ -742,7 +764,6 @@ tp_data_free (tp_data_t * tpd)
 	    break;
 	  }
 	case TP_XA_TYPE:
-	  dk_free_box ((box_t) tpd->cli_tp_trx);
 	  break;
 	case TP_VIRT_TYPE:
 	  break;
@@ -752,7 +773,8 @@ tp_data_free (tp_data_t * tpd)
     }
 
   dk_free_box (tpd->tpd_trx_cookie);
-  semaphore_free (tpd->cli_tp_sem2);
+  if (tpd->cli_tp_sem2)
+    semaphore_free (tpd->cli_tp_sem2);
   dk_free (tpd, sizeof (tp_data_t));
 }
 
@@ -1275,6 +1297,13 @@ global_xa_init ()
 
 }
 
+void
+virt_xa_tp_set_xid (tp_data_t * tpd, void *xid)
+{
+  tpd->cli_tp_trx = xid;
+  tpd->tpd_trx_cookie = box_copy ((caddr_t) xid);
+}
+
 int
 virt_xa_add_trx (void *xid, lock_trx_t * lt)
 {
@@ -1286,16 +1315,18 @@ virt_xa_add_trx (void *xid, lock_trx_t * lt)
   if (x == 0)
     {
       xa_id_t *xx = (xa_id_t *) dk_alloc (sizeof (xa_id_t));
-      tp_data_t * tpd = (tp_data_t*)dk_alloc (sizeof (tp_data_t));;
+      tp_data_t * tpd = (tp_data_t*)dk_alloc (sizeof (tp_data_t));
+      memset (tpd, 0, sizeof (tp_data_t));
       memcpy (&xx->xid, xid, sizeof (virtXID));
+      virt_xa_tp_set_xid (tpd, xid);
+      lt->lt_2pc._2pc_log = box_copy (xid);
+      dk_free_tree (lt->lt_2pc._2pc_xid);
+      lt->lt_2pc._2pc_xid = xid;
       xid = (void *) &xx->xid;
       xx->xid_sem = 0;
       xx->xid_cli = NULL;
-      memset (tpd, 0, sizeof (tp_data_t));
       tpd->cli_tp_enlisted = CONNECTION_PREPARED;
       tpd->cli_tp_sem2 = semaphore_allocate (0);
-      tpd->cli_tp_trx = xid;
-      tpd->tpd_trx_cookie = (caddr_t) xid;
       tpd->cli_tp_lt = lt;
       tpd->cli_trx_type = lt->lt_2pc._2pc_type = TP_XA_TYPE;
       xx->xid_tp_data = tpd;
@@ -1353,7 +1384,7 @@ virt_xa_set_client (void *xid, struct client_connection_s *cli)
 	  return rc;
 #else
 	  mutex_leave (global_xa_map->xm_mtx);
-	  return VXA_AGAIN;
+	  return VXA_ERROR;
 #endif
 	}
     }
@@ -1375,7 +1406,7 @@ virt_xa_suspend_lt (void *xid, struct client_connection_s *cli)
 int
 virt_xa_client (void *xid, client_connection_t * cli, struct tp_data_s **tpd, int op)
 {
-  xa_id_t **xx;
+  xa_id_t **xx, *x;
 
   if (!xid)
     return -1;
@@ -1392,7 +1423,7 @@ virt_xa_client (void *xid, client_connection_t * cli, struct tp_data_s **tpd, in
 	}
       else
 	{
-	  xa_id_t *x = (xa_id_t *) dk_alloc (sizeof (xa_id_t));
+	  x = (xa_id_t *) dk_alloc (sizeof (xa_id_t));
 	  memcpy (&x->xid, xid, sizeof (virtXID));
 	  xid = (void *) &x->xid;
 	  x->xid_sem = semaphore_allocate (0);
@@ -1408,22 +1439,34 @@ virt_xa_client (void *xid, client_connection_t * cli, struct tp_data_s **tpd, in
 	}
       return 0;
     }
+  x = xx[0];
 
   if (op == SQL_XA_ROLLBACK)
-    xx[0]->xid_op = SQL_XA_ROLLBACK;
+    x->xid_op = SQL_XA_ROLLBACK;
 
-  tpd[0] = xx[0]->xid_tp_data;
+  tpd[0] = x->xid_tp_data;
   mutex_leave (global_xa_map->xm_mtx);
 
   IN_TXN;
-  if (cli->cli_trx != xx[0]->xid_tp_data->cli_tp_lt)
+  lt_wait_checkpoint ();
+  if (cli->cli_trx != x->xid_tp_data->cli_tp_lt)
     {
       lt_kill_other_trx (cli->cli_trx, NULL, NULL, LT_KILL_ROLLBACK);
+      lt_done (cli->cli_trx);
       cli->cli_trx = NULL;
-      cli_set_trx (cli, xx[0]->xid_tp_data->cli_tp_lt);
+      cli_set_trx (cli, x->xid_tp_data->cli_tp_lt);
     }
-  cli->cli_tp_data = xx[0]->xid_tp_data;
-  xx[0]->xid_cli = cli;
+  cli->cli_tp_data = x->xid_tp_data;
+  if (x->xid_cli && x->xid_cli != cli)
+    {
+      x->xid_cli->cli_tp_data = NULL;
+      if (x->xid_cli->cli_trx == cli->cli_trx)
+	{
+	  x->xid_cli->cli_trx = NULL;
+	  cli_set_new_trx (x->xid_cli);
+	}
+    }
+  x->xid_cli = cli;
   LEAVE_TXN;
   return 0;
 }
@@ -1440,8 +1483,12 @@ virt_xa_remove_xid (void *xid)
   xx = (xa_id_t **) id_hash_get (global_xa_map->xm_xids, (caddr_t) & xid);
   if (xx)
     {
-      dk_free (xx[0], sizeof (xa_id_t));
+      xa_id_t * x = xx[0];
+      x->xid_cli->cli_tp_data = NULL;
       id_hash_remove (global_xa_map->xm_xids, (caddr_t) & xid);
+      if (x->xid_sem)
+	semaphore_free (x->xid_sem);
+      dk_free (x, sizeof (xa_id_t));
     }
   mutex_leave (global_xa_map->xm_mtx);
 }
@@ -1779,25 +1826,25 @@ txa_add_entry (txa_entry_t * e)
 void
 txa_remove_entry (void *xid, int check)
 {
-  if (txa_search_trx (xid))
+  txa_entry_t * e;
+  mutex_enter (global_xa_map->xm_mtx);
+  if (NULL != (e = txa_search_trx (xid)))
     {
       uint32 sz = box_length (txi.txi_parsed_info) - sizeof (txa_entry_t *);
-      txa_entry_t **new_v =
-	  (txa_entry_t **) dk_alloc_box (sz, DV_ARRAY_OF_POINTER);
+      txa_entry_t **new_v = (txa_entry_t **) dk_alloc_box (sz, DV_ARRAY_OF_POINTER);
       int idx, nidx = 0;
       for (idx = 0; idx < (int) (sz / sizeof (txa_entry_t *) + 1); idx++)
 	{
-	  if (memcmp (xid, txi.txi_parsed_info[idx]->txe_id,
-		  box_length (xid)))
+	  if (memcmp (xid, txi.txi_parsed_info[idx]->txe_id, box_length (xid)))
 	    {
-	      memcpy (new_v + nidx, txi.txi_parsed_info + idx,
-		  sizeof (txa_entry_t *));
+	      memcpy (new_v + nidx, txi.txi_parsed_info + idx, sizeof (txa_entry_t *));
 	      nidx++;
 	    }
 	}
       if (nidx != (idx - 1))
-	GPF_T;
+	GPF_T1 ("Inconsistent XA info");
       dk_free_box ((box_t) txi.txi_parsed_info);
+      txa_free_entry (e);
       txi.txi_parsed_info = new_v;
     }
 #if 0
@@ -1808,6 +1855,8 @@ txa_remove_entry (void *xid, int check)
       dk_free_box (xid_str);
     }
 #endif
+  txa_write ();
+  mutex_leave (global_xa_map->xm_mtx);
 }
 
 caddr_t *
@@ -1835,7 +1884,7 @@ txa_open (char *file_name)
   int fd;
   file_set_rw (file_name);
   fd = fd_open (file_name, OPEN_FLAGS);
-  if (fd > 0)
+  if (fd >= 0)
     {
       txi.txi_trx_file = box_string (file_name);
       txi.txi_fd = fd;
@@ -1854,13 +1903,15 @@ txa_write ()
   dk_free_tree ((box_t) txi.txi_info);
   txi.txi_info = txa_serialize (txi.txi_parsed_info);
   fd = fd_open (txi.txi_trx_file, OPEN_FLAGS);
-  if (fd > 0)
+  if (fd >= 0)
     {
       FTRUNCATE (fd, 0);
       txa_write_info (fd, txi.txi_info);
       fd_close (fd, txi.txi_trx_file);
       return 0;
     }
+  else
+    log_error ("Can not open %s file", txi.txi_trx_file);
   return -1;
 }
 
@@ -1885,14 +1936,10 @@ void
 txa_from_trx (lock_trx_t * lt, char *log_file_name)
 {
   txa_entry_t *e;
-  client_connection_t *cli = lt->lt_client;
-  tp_data_t *tpd = cli->cli_tp_data;
-
-  e = txa_search_trx (tpd->cli_tp_trx);
+  e = txa_search_trx (lt->lt_2pc._2pc_xid);
   if (!e)
     {
-      e = txa_create_entry (tpd->cli_tp_trx,
-	  log_file_name, (ptrlong) lt->lt_2pc._2pc_logged, (char *) "PRP");
+      e = txa_create_entry (lt->lt_2pc._2pc_xid, log_file_name, (ptrlong) lt->lt_2pc._2pc_logged, (char *) "PRP");
       if (e)
 	{
 	  txa_add_entry (e);
@@ -1900,7 +1947,11 @@ txa_from_trx (lock_trx_t * lt, char *log_file_name)
 	}
     }
   else
-    GPF_T;
+    {
+      dk_free_box (e->txe_offset);
+      e->txe_offset = box_num (lt->lt_2pc._2pc_logged);
+      txa_write ();
+    }
 }
 
 static caddr_t
