@@ -1,7 +1,7 @@
 /*
  *  CLI.h
  *
- *  $Id: CLI.h,v 1.19 2009/06/03 12:34:47 source Exp $
+ *  $Id: CLI.h,v 1.17.2.4 2011/07/14 19:14:14 source Exp $
  *
  *  SQL client data structures
  *
@@ -34,7 +34,6 @@
 #include "sqlcomp.h"
 #include "numeric.h"
 #include "blobio.h"
-#include "widv.h"
 #include "wirpce.h"
 #include "date.h"
 #include "datesupp.h"
@@ -117,6 +116,8 @@ typedef struct cli_connection_s
     con_defaults_t	con_defs;
     wcharset_t *	con_charset;
     caddr_t		con_charset_name;
+    int 		con_wide_as_utf16;
+    int			con_string_is_utf8;
 #ifdef VIRTTP
     caddr_t con_d_trx_id; /* connection is enlisted in Virtuoso TP transaction */
 #endif
@@ -408,8 +409,8 @@ SQLRETURN str_box_to_buffer(char *box, char *buffer, int buffer_length, void *st
 void str_box_to_place (char *box, char *place, int max, int *sz);
 int dv_to_sqlc_default (caddr_t xx);
 int vector_to_text (caddr_t vec, size_t box_len, dtp_t vectype, char *dest, size_t dest_size);
-SQLLEN dv_to_str_place (caddr_t it, dtp_t dtp, SQLLEN max, caddr_t place, SQLLEN *len_ret, SQLLEN str_from_pos, cli_stmt_t *stmt, int nth_col, SQLLEN box_len, int c_type, SQLSMALLINT sql_type);
-SQLLEN dv_to_place (caddr_t it, int c_type, SQLSMALLINT sql_type, SQLLEN max, caddr_t place, SQLLEN *len_ret, SQLLEN str_from_pos, cli_stmt_t *stmt, int nth_col);
+SQLLEN dv_to_str_place (caddr_t it, dtp_t dtp, SQLLEN max, caddr_t place, SQLLEN *len_ret, SQLLEN str_from_pos, cli_stmt_t *stmt, int nth_col, SQLLEN box_len, int c_type, SQLSMALLINT sql_type, SQLLEN *out_chars);
+SQLLEN dv_to_place (caddr_t it, int c_type, SQLSMALLINT sql_type, SQLLEN max, caddr_t place, SQLLEN *len_ret, SQLLEN str_from_pos, cli_stmt_t *stmt, int nth_col, SQLLEN *out_chars);
 #ifndef MAP_DIRECT_BIN_CHAR
 void bin_dv_to_str_place (unsigned char *str, char *place, size_t nbytes);
 void bin_dv_to_wstr_place (unsigned char *str, wchar_t *place, size_t nbytes);
@@ -901,7 +902,7 @@ if (wsz##param) \
     if ((con)->con_defs.cdef_utf8_execs) \
       { \
 	SQLSMALLINT len1; \
-	len1 = (SQLSMALLINT) cli_utf8_to_narrow (con->con_charset, sz##param, _cb##param, wsz##param, cb##param); \
+	len1 = (SQLSMALLINT) cli_utf8_to_narrow (con->con_charset, sz##param, _vpcb##param, wsz##param, cb##param); \
 	if (pcb##param) \
 	  *pcb##param = *_pcb##param; \
 	dk_free_box ((box_t) sz##param); \
