@@ -1,7 +1,7 @@
 /*
  *  bif_crypto.c
  *
- *  $Id: bif_crypto.c,v 1.8.2.2 2009/08/19 07:29:52 source Exp $
+ *  $Id: bif_crypto.c,v 1.8.2.5 2011/01/27 14:19:53 source Exp $
  *
  *  Cryptography functions
  *
@@ -904,6 +904,158 @@ bif_smime_sign (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return ret;
 }
 
+static caddr_t
+bif_smime_encrypt (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  char * me = "smime_encrypt";
+  caddr_t msg = bif_string_arg (qst, args, 0, me);
+  caddr_t scerts = bif_array_arg (qst, args, 1, me);
+  caddr_t cipher_name = bif_string_arg (qst, args, 2, me);
+  caddr_t ret = NULL;
+  caddr_t err = NULL;
+  BIO *out_bio = NULL, *in_bio = NULL;
+  PKCS7 *p7 = NULL;
+  X509_STORE *store = NULL;
+  STACK_OF (X509) * certs = NULL;
+  int inx;
+  char err_buf[512];
+  char *ptr = NULL;
+  int flags = 0;
+  const EVP_CIPHER *cipher = NULL;
+
+  cipher = EVP_get_cipherbyname (cipher_name);
+  if (!cipher)
+    sqlr_new_error ("42000", "CR006", "Cannot find cipher");
+  store = smime_get_store_from_array (scerts, &err);
+  if (err)
+    sqlr_resignal (err);
+  if (!store)
+    sqlr_new_error ("42000", "CR006", "No recipient certificates");
+
+  certs = sk_X509_new_null ();
+  if (store && store->objs)
+    {
+      for (inx = 0; inx < sk_X509_OBJECT_num (store->objs); inx++)
+	{
+	  X509_OBJECT *obj = sk_X509_OBJECT_value (store->objs, inx);
+	  if (obj->type == X509_LU_X509)
+	    sk_X509_push (certs, X509_dup (obj->data.x509));
+	}
+    }
+  if (store)
+    X509_STORE_free (store);
+  in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
+  if (in_bio)
+    {
+      p7 = PKCS7_encrypt(certs, in_bio, cipher, flags);
+      BIO_free (in_bio);
+      in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
+    }
+  sk_X509_pop_free (certs, X509_free);
+
+  if (!p7)
+    {
+      if (in_bio)
+	BIO_free (in_bio);
+      sqlr_new_error ("42000", "CR009", "Cannot generate PKCS7 structure. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+
+  out_bio = BIO_new (BIO_s_mem ());
+  if (!out_bio)
+    {
+      if (p7)
+	PKCS7_free (p7);
+      if (in_bio)
+	BIO_free (in_bio);
+      sqlr_new_error ("42000", "CR010", "Cannot allocate output storage. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+
+  SMIME_write_PKCS7 (out_bio, p7, in_bio, flags);
+  PKCS7_free (p7);
+  BIO_free (in_bio);
+
+  ret = dk_alloc_box (BIO_get_mem_data (out_bio, &ptr) + 1, DV_SHORT_STRING);
+  memcpy (ret, ptr, box_length (ret) - 1);
+  ret[box_length (ret) - 1] = 0;
+
+  BIO_free (out_bio);
+  return ret;
+}
+
+static caddr_t
+bif_smime_decrypt (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  char * me = "smime_decrypt";
+  caddr_t msg = bif_string_arg (qst, args, 0, me);
+  caddr_t cert = bif_string_arg (qst, args, 1, me);
+  caddr_t privatekey = bif_string_arg (qst, args, 2, me);
+  caddr_t privatepass = bif_string_or_null_arg (qst, args, 3, me);
+  int flags = 0;
+  caddr_t ret = NULL;
+  BIO *out_bio = NULL, *in_bio = NULL, *data_bio = NULL;
+  PKCS7 *p7 = NULL;
+  X509 *recip_cert = NULL;
+  EVP_PKEY *recip_key = NULL;
+  int rc;
+  char err_buf[512];
+  char *ptr = NULL;
+
+  recip_cert = x509_get_cert_from_buffer (cert);
+  if (!recip_cert)
+    {
+      sqlr_new_error ("42000", "CR007", "Error reading the recipient certificate. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+
+  recip_key = x509_get_pkey_from_buffer (privatekey, privatepass);
+  if (!recip_key)
+    {
+      X509_free (recip_cert);
+      sqlr_new_error ("42000", "CR008", "Error reading the recipient private key. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+
+  in_bio = BIO_new_mem_buf (msg, box_length (msg) - 1);
+  if (in_bio)
+    {
+      p7 = SMIME_read_PKCS7 (in_bio, &data_bio);
+      BIO_free (in_bio);
+    }
+  if (!p7)
+    {
+      X509_free (recip_cert);
+      EVP_PKEY_free (recip_key);
+      if (data_bio) BIO_free (data_bio);
+      sqlr_new_error ("42000", "CR004", "Cannot read PKCS7 attached signature. SSL Error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+  out_bio = BIO_new (BIO_s_mem ());
+  if (!out_bio)
+    {
+      X509_free (recip_cert);
+      EVP_PKEY_free (recip_key);
+      PKCS7_free (p7);
+      if (data_bio) BIO_free (data_bio);
+      sqlr_new_error ("42000", "CR010", "Cannot allocate output storage. SSL error : %s", get_ssl_error_text (err_buf, sizeof (err_buf)));
+    }
+
+  rc = PKCS7_decrypt(p7, recip_key, recip_cert, out_bio, flags);
+
+  X509_free (recip_cert);
+  EVP_PKEY_free (recip_key);
+
+  if (rc)
+    {
+      ret = dk_alloc_box (BIO_get_mem_data (out_bio, &ptr) + 1, DV_SHORT_STRING);
+      memcpy (ret, ptr, box_length (ret) - 1);
+      ret[box_length (ret) - 1] = 0;
+    }
+  else
+    ret = NEW_DB_NULL;
+
+  PKCS7_free (p7);
+  if (data_bio) BIO_free (data_bio);
+  BIO_free (out_bio);
+  return ret;
+}
+
 
 static caddr_t
 bif_pem_certificates_to_array (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -1104,6 +1256,7 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
   long type = bif_long_arg (qst, args, 0, "get_certificate_info");
   caddr_t scert = BOX_ELEMENTS (args) > 1 ? bif_string_or_null_arg (qst, args, 1, "get_certificate_info") : NULL;
   int internal = 0;
+  char buffer[4096];
 
   if (qi->qi_client->cli_ws)
     ssl = (SSL *) tcpses_get_ssl (qi->qi_client->cli_ws->ws_session->dks_session);
@@ -1173,7 +1326,6 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	X509_NAME *subj = X509_get_subject_name (cert);
 	if (subj)
 	  {
-	    char buffer[4096];
 	    X509_NAME_oneline (subj, buffer, sizeof (buffer));
 	    ret = box_dv_short_string (buffer);
 	    break;
@@ -1185,7 +1337,6 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	X509_NAME *subj = X509_get_issuer_name (cert);
 	if (subj)
 	  {
-	    char buffer[4096];
 	    X509_NAME_oneline (subj, buffer, sizeof (buffer));
 	    ret = box_dv_short_string (buffer);
 	    break;
@@ -1313,6 +1464,43 @@ bif_get_certificate_info (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args
 	  *err_ret = srv_make_new_error ("42000", "XXXXX", "Can not read the public key from the certificate");
 	break;
       }
+    case 10:
+      {
+	char *attr = BOX_ELEMENTS (args) > 4 ? bif_string_arg (qst, args, 4, "get_certificate_info") : "CN";
+	X509_NAME *subj = X509_get_subject_name (cert);
+	X509_NAME_ENTRY *ne, *ne_ret = NULL;
+	int n, i, len;
+	char *s, *data_ptr;
+	BIO *mem = BIO_new (BIO_s_mem ());
+	for (i = 0; NULL != subj && i < sk_X509_NAME_ENTRY_num(subj->entries); i++)
+	  {
+	    ne = sk_X509_NAME_ENTRY_value(subj->entries,i);
+	    n = OBJ_obj2nid (ne->object);
+	    if ((n == NID_undef) || ((s = OBJ_nid2sn (n)) == NULL))
+	      {
+		i2t_ASN1_OBJECT (buffer, sizeof (buffer), ne->object);
+		s = buffer;
+	      }
+	    if (!strcmp (s, attr))
+	      {
+		ne_ret = ne;
+		break;
+	      }
+	  }
+	if (ne_ret)
+	  {
+	    ASN1_STRING_print (mem, ne_ret->value);
+	    len = BIO_get_mem_data (mem, &data_ptr);
+	    if (len > 0 && data_ptr)
+	      {
+		ret = dk_alloc_box (len + 1, DV_SHORT_STRING);
+		memcpy (ret, data_ptr, len);
+		ret[len] = 0;
+	      }
+	  }
+	BIO_free (mem);
+	break;
+      }
     default:
       {
 	if (!internal)
@@ -1368,6 +1556,8 @@ bif_crypto_init (void)
   bif_define_typed ("tree_hmac", bif_tree_hmac, &bt_varchar);
   bif_define_typed ("smime_verify", bif_smime_verify, &bt_varchar);
   bif_define_typed ("smime_sign", bif_smime_sign, &bt_varchar);
+  bif_define_typed ("smime_encrypt", bif_smime_encrypt, &bt_varchar);
+  bif_define_typed ("smime_decrypt", bif_smime_decrypt, &bt_varchar);
   bif_define_typed ("pem_certificates_to_array", bif_pem_certificates_to_array, &bt_any);
   bif_define_typed ("get_certificate_info", bif_get_certificate_info, &bt_any);
   bif_define_typed ("x509_certificate_verify", bif_x509_certificate_verify, &bt_any);
