@@ -1,7 +1,7 @@
 /*
  *  bif_date.c
  *
- *  $Id: bif_date.c,v 1.12.2.1 2009/04/18 21:55:09 source Exp $
+ *  $Id: bif_date.c,v 1.12.2.5 2011/09/23 15:20:37 source Exp $
  *
  *  Bifs for date
  *
@@ -227,8 +227,11 @@ dt_scan_from_buffer (const char *buf, int mode, caddr_t *dt_ret, const char **er
             } while (isdigit (tail[0]));
           ts.fraction = acc * (msec_factor ? msec_factor : 1);
         }
-      err_msg_ret[0] = "Colon is expected after minute";
-      return 0;
+      if ('Z' != tail[0] && strncmp (tail, " GMT", 4))
+	{
+	  err_msg_ret[0] = "Colon or time zone is expected after minute";
+	  return 0;
+	}
     }
   else
     {
@@ -266,7 +269,7 @@ scan_tz:
     DT_SET_MINUTE (dt_ret[0], ts.minute);
     DT_SET_SECOND (dt_ret[0], ts.second);
     DT_SET_FRACTION (dt_ret[0], ts.fraction);
-    DT_SET_TZ (dt_ret[0], 0);
+    DT_SET_TZ (dt_ret[0], dt_local_tz);
   }
   SET_DT_TYPE_BY_DTP (dt_ret[0], (ymd_found ? (hms_found ? DV_DATETIME : DV_DATE) : DV_TIME));
   return (tail - buf);
@@ -456,6 +459,49 @@ bif_timezone (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   return box_num (DT_TZ (arg));
 }
 
+#define NASA_TJD_OFFSET (2440000 - 1721423)
+
+caddr_t
+bif_nasa_tjd_number (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t arg = bif_date_arg (qst, args, 0, "nasa_tjd_number");
+  long n = DT_DAY (arg) - NASA_TJD_OFFSET;
+  return box_num (n);
+}
+
+caddr_t
+bif_nasa_tjd_fraction (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  caddr_t arg = bif_date_arg (qst, args, 0, "nasa_tjd_fraction");
+  double f = (((DT_HOUR (arg) * (boxint)60 + DT_MINUTE (arg)) * (boxint)60 + DT_SECOND (arg)) * (boxint)1000000 + DT_FRACTION (arg)) / (60*60*24*1000000.0);
+  return box_double (f);
+}
+
+caddr_t
+bif_merge_nasa_tjd_to_datetime (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  boxint num = bif_long_arg (qst, args, 0, "merge_nasa_tjd_to_datetime");
+  caddr_t res = dk_alloc_box_zero (DT_LENGTH, DV_DATETIME);
+  DT_SET_DAY (res, num + NASA_TJD_OFFSET);
+  if (1 < BOX_ELEMENTS (args))
+    {
+      double frac = bif_double_arg (qst, args, 1, "merge_nasa_tjd_to_datetime");
+      boxint frac_microsec = frac * (60*60*24*1000000.0);
+      if ((0 > frac_microsec) || (60*60*24*1000000 <= frac_microsec))
+        sqlr_new_error ("22023", "SR644", "Fraction of julian day should be nonnegative and less than 1");
+      DT_SET_FRACTION (res, (frac_microsec % 1000000) * 1000);
+      frac_microsec = frac_microsec / 1000000;
+      DT_SET_SECOND (res, (frac_microsec % 60));
+      frac_microsec = frac_microsec / 60;
+      DT_SET_MINUTE (res, (frac_microsec % 60));
+      frac_microsec = frac_microsec / 60;
+      DT_SET_HOUR (res, frac_microsec);
+      DT_SET_DT_TYPE (res, DT_TYPE_DATETIME);
+    }
+  else
+    DT_SET_DT_TYPE (res, DT_TYPE_DATE);
+  return res;
+}
 
 caddr_t
 bif_date_add (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
@@ -485,48 +531,44 @@ bif_date_diff (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t unit = bif_string_arg (qst, args, 0, "datediff");
   caddr_t dt1 = bif_date_arg (qst, args, 1, "datediff");
   caddr_t dt2 = bif_date_arg (qst, args, 2, "datediff");
-  TIMESTAMP_STRUCT ts1;
-  TIMESTAMP_STRUCT ts2;
-  boxint s1 = (boxint)DT_DAY (dt1) * 24 * 60 * 60 + DT_HOUR (dt1) * 60 * 60 + DT_MINUTE (dt1) * 60 + DT_SECOND (dt1);
-  boxint s2 = (boxint)DT_DAY (dt2) * 24 * 60 * 60 + DT_HOUR (dt2) * 60 * 60 + DT_MINUTE (dt2) * 60 + DT_SECOND (dt2);
-  int tz_tweak = DT_TZ (dt1);
-
+  boxint s1 =
+      (boxint) DT_DAY (dt1) * 24 * 60 * 60 + (boxint) DT_HOUR (dt1) * 60 * 60 + (boxint) DT_MINUTE (dt1) * 60 + DT_SECOND (dt1);
+  boxint s2 =
+      (boxint) DT_DAY (dt2) * 24 * 60 * 60 + (boxint) DT_HOUR (dt2) * 60 * 60 + (boxint) DT_MINUTE (dt2) * 60 + DT_SECOND (dt2);
+  int frac1, frac2;
+  int diffyear, diffmonth;
   if (0 == stricmp (unit, "day"))
-    return box_num ((boxint)DT_DAY (dt2) - (boxint)DT_DAY (dt1));
-
+    return box_num ((boxint) DT_DAY (dt2) - (boxint) DT_DAY (dt1));
   if (0 == stricmp (unit, "hour"))
     return box_num ((s2 - s1) / (60 * 60));
-
   if (0 == stricmp (unit, "minute"))
     return box_num ((s2 - s1) / 60);
-
   if (0 == stricmp (unit, "second"))
     return box_num (s2 - s1);
-
-  dt_to_GMTimestamp_struct (dt2, &ts2);
-  dt_to_GMTimestamp_struct (dt1, &ts1);
-
-  if (0 == stricmp (unit, "month"))
+  diffyear = !stricmp (unit, "year");
+  diffmonth = (diffyear ? 0 : !stricmp (unit, "month"));
+  if (diffyear || diffmonth)
     {
+      TIMESTAMP_STRUCT ts1;
+      TIMESTAMP_STRUCT ts2;
+      int tz_tweak = DT_TZ (dt1);
+      dt_to_GMTimestamp_struct (dt2, &ts2);
+      dt_to_GMTimestamp_struct (dt1, &ts1);
       ts_add (&ts1, tz_tweak, "minute");
       ts_add (&ts2, tz_tweak, "minute");
-      return box_num ((boxint)(ts2.year * 12 + ts2.month) - (boxint)(ts1.year * 12 + ts1.month));
+      if (diffyear)
+	return box_num ((boxint) ts2.year - (boxint) ts1.year);
+      if (diffmonth)
+	return box_num ((boxint) (ts2.year * 12 + ts2.month) - (boxint) (ts1.year * 12 + ts1.month));
     }
-
-  if (0 == stricmp (unit, "year"))
-    {
-      ts_add (&ts1, tz_tweak, "minute");
-      ts_add (&ts2, tz_tweak, "minute");
-      return box_num ((boxint)ts2.year - (boxint)ts1.year);
-    }
-
+  frac1 = DT_FRACTION (dt1);
+  frac2 = DT_FRACTION (dt2);
   if (0 == stricmp (unit, "millisecond"))
-    return box_num ((s2 - s1) * 1000 + (ts2.fraction / 1000000 - ts1.fraction / 1000000));
+    return box_num ((s2 - s1) * (boxint) 1000 + (frac2 / 1000000 - frac1 / 1000000));
   if (0 == stricmp (unit, "microsecond"))
-    return box_num ((s2 - s1) * (boxint)1000000 + (ts2.fraction / 1000 - ts1.fraction / 1000));
+    return box_num ((s2 - s1) * (boxint) 1000000 + (frac2 / 1000 - frac1 / 1000));
   if (0 == stricmp (unit, "nanosecond"))
-    return box_num ((s2 - s1) * (boxint)1000000000 + (ts2.fraction - ts1.fraction));
-
+    return box_num ((s2 - s1) * (boxint) 1000000000 + (frac2 - frac1));
   sqlr_new_error ("22023", "DT002", "Bad unit in datediff: %s.", unit);
   return NULL;
 }
@@ -558,11 +600,13 @@ bif_timestampadd (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   ptrlong part = bif_long_arg (qst, args, 0, "timestampadd");
   int n = (int) bif_long_arg (qst, args, 1, "timestampadd");
   caddr_t dt = bif_date_arg (qst, args, 2, "timestampadd");
+  int saved_tz = DT_TZ (dt);
   GMTIMESTAMP_STRUCT ts;
   dt_to_GMTimestamp_struct (dt, &ts);
   ts_add (&ts, n, interval_odbc_to_text (part, "timestampadd"));
   res = dk_alloc_box (DT_LENGTH, DV_DATETIME);
   GMTimestamp_struct_to_dt (&ts, res);
+  DT_SET_TZ (res, saved_tz);
   return res;
 }
 
@@ -814,6 +858,9 @@ bif_date_init ()
   bif_define_typed ("minute", bif_minute, &bt_integer);
   bif_define_typed ("second", bif_second, &bt_integer);
   bif_define_typed ("timezone", bif_timezone, &bt_integer);
+  bif_define_typed ("nasa_tjd_number", bif_nasa_tjd_number, &bt_integer);
+  bif_define_typed ("nasa_tjd_fraction", bif_nasa_tjd_fraction, &bt_double);
+  bif_define_typed ("merge_nasa_tjd_to_datetime", bif_merge_nasa_tjd_to_datetime, &bt_datetime);
 
   bif_define_typed ("now", bif_timestamp, &bt_timestamp);	/* This is standard name */
   bif_define_typed ("getdate", bif_timestamp, &bt_datetime);	/* This is standard name? */
