@@ -1,7 +1,7 @@
 /*
  *  bif_explain.c
  *
- *  $Id: bif_explain.c,v 1.17.2.10 2010/02/02 14:16:14 source Exp $
+ *  $Id: bif_explain.c,v 1.17.2.14 2011/06/07 12:30:28 source Exp $
  *
  *  Implements bif 'explain'
  *
@@ -50,6 +50,7 @@
 #include "sqlcstate.h"
 #include "sqlo.h"
 #include "rdfinf.h"
+#include "rdf_core.h"
 
 /* sqlprt.c */
 void trset_start (caddr_t *qst);
@@ -62,6 +63,28 @@ static void qr_print (query_t * qr);
 static void node_print (data_source_t * node);
 
 
+
+caddr_t
+dv_iri_short_name (caddr_t x)
+{
+  caddr_t pref, local, r;
+  iri_id_t iid = unbox_iri_id (x);
+  caddr_t name = key_id_to_iri ((query_instance_t*)THR_ATTR (THREAD_CURRENT_THREAD, TA_REPORT_QST), iid);
+  if (!name)
+    return NULL;
+  if (iri_split (name, &pref, &local))
+    {
+      dk_free_box (name);
+      dk_free_box (pref);
+      r = box_dv_short_string (local + 4);
+      dk_free_box (local);
+      return r;
+    }
+  dk_free_box (name);
+  return NULL;
+}
+
+
 static void
 ssl_print (state_slot_t * ssl)
 {
@@ -90,6 +113,7 @@ ssl_print (state_slot_t * ssl)
 
     case SSL_CONSTANT:
 	{
+	  dtp_t dtp = DV_TYPE_OF (ssl->ssl_constant);
 	  caddr_t err_ret = NULL;
 	  if (DV_TYPE_OF (ssl->ssl_constant) == DV_DB_NULL)
 	    stmt_printf (("<constant DB_NULL>"));
@@ -100,7 +124,26 @@ ssl_print (state_slot_t * ssl)
 		  NUMERIC_MAX_PRECISION, NUMERIC_MAX_SCALE,
 		  &err_ret);
 	      if (!err_ret && strval)
-		stmt_printf (("<constant (" EXPLAIN_LINE_MAX_STR_FORMAT ")>", strval));
+		{
+		  switch (dtp)
+		    {
+		      case DV_IRI_ID:
+			    {
+			      caddr_t str = dv_iri_short_name (ssl->ssl_constant);
+			      if (str)
+				{
+				  stmt_printf ((" #" EXPLAIN_LINE_MAX_STR_FORMAT " ", str));
+				  dk_free_box (str);
+				  break;
+				}
+			    }
+		      case DV_LONG_INT: case DV_NUMERIC: case DV_SINGLE_FLOAT: case DV_DOUBLE_FLOAT:
+			  stmt_printf ((" " EXPLAIN_LINE_MAX_STR_FORMAT " ", strval));
+			  break;
+		      default:
+			  stmt_printf (("<c " EXPLAIN_LINE_MAX_STR_FORMAT ">", strval));
+		    }
+		}
 	      else
 		stmt_printf (("<constant>"));
 	      if (err_ret)
@@ -423,7 +466,7 @@ ks_print (key_source_t * ks)
   stmt_printf (("\n%s", ks->ks_spec.ksp_key_cmp ? " inlined " : ""));
 
   sp_list_print (ks->ks_spec.ksp_spec_array);
-  stmt_printf (("\n"));
+  stmt_printf (("%s\n", ks->ks_copy_search_pars ? " [copies params]" : ""));
 
   if (ks->ks_row_spec)
     {
@@ -820,6 +863,15 @@ node_print (data_source_t * node)
       insert_node_t *ins = (insert_node_t *) node;
       stmt_printf (("Insert %s ", ins->ins_table->tb_name));
       ssl_list_print (ins->ins_values);
+      if (ins->ins_key_only)
+	stmt_printf ((" only key %s ", ins->ins_key_only));
+      if (ins->ins_daq)
+	{
+	  stmt_printf ((" DAQ ("));
+	  ssl_print (ins->ins_daq);
+	  stmt_printf ((")"));
+	}
+      stmt_printf (("\n"));
     }
   else if (in == (qn_input_fn) select_node_input_subq)
     {
@@ -1306,7 +1358,7 @@ xsql_print_stmt (sql_comp_t * sc, comp_table_t * ct, ST * tree, char * text, siz
       sprintf_more (text, tlen, fill, " %s ", (caddr_t) tree);
       return 1;
     }
-  if (ST_P (tree, COL_DOTTED))
+  if (ST_COLUMN (tree, COL_DOTTED))
     {
       caddr_t pref = tree->_.col_ref.prefix;
       sprintf_more (text, tlen, fill, " ");
