@@ -1,7 +1,7 @@
 /*
  *  bif_ldapcli.c
  *
- *  $Id: bif_ldapcli.c,v 1.3.2.1 2009/04/18 21:55:10 source Exp $
+ *  $Id: bif_ldapcli.c,v 1.3.2.3 2011/05/19 13:15:16 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -455,7 +455,7 @@ ldap_print_sresult (LDAP *ld, dk_set_t * s)
 	      case LDAP_RES_SEARCH_REFERENCE:
 		    { /* 'reference', (<ref>, ...) */
 		      char **refs = NULL;
-		      LDAPControl **ctrls;
+		      LDAPControl **ctrls = NULL;
 		      int rc, i;
 		      dk_set_t p = NULL;
 		      rc = LDAP_PARSE_REFERENCE (ld, msg, &refs, &ctrls, 0);
@@ -583,6 +583,31 @@ bif_ldap_search (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   dk_set_t s = NULL;
   caddr_t * ret;
   caddr_t alloced_passwd = NULL;
+  int is_null;
+#if !defined (WIN32)
+  struct timeval tv;
+  long secs = 0;
+#endif
+  /* version */
+  if (BOX_ELEMENTS (args) > 6)
+    {
+      long ver;
+      ver = bif_long_or_null_arg (qst, args, 6, "ldap_search", &is_null);
+      if (!is_null && ver >= LDAP_VERSION_MIN && ver <= LDAP_VERSION_MAX)
+	version = ver;
+    }
+#if !defined (WIN32)
+  if (BOX_ELEMENTS (args) > 7)
+    {
+      secs = bif_long_or_null_arg (qst, args, 7, "ldap_search", &is_null);
+      if (secs > 0)
+	{
+	  tv.tv_sec = secs;
+	  tv.tv_usec = 0;
+	}
+    }
+#endif
+
 
 #ifdef WIN32
   if (!ldap_module)
@@ -609,6 +634,11 @@ bif_ldap_search (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 	"Failed to set LDAP version option: %s (%d)", ldap_err2string (rc),
 	rc);
 
+#if !defined (WIN32)
+  if (secs > 0 && (rc = ldap_set_option (ld, LDAP_OPT_NETWORK_TIMEOUT, &tv)) != LDAP_OPT_SUCCESS)
+    sqlr_new_error ("2E000", "LD006", "Failed to set LDAP version option: %s (%d)", ldap_err2string (rc), rc);
+#endif
+
 #ifndef WIN32
   if (tls && (rc = ldap_start_tls_s (ld, NULL, NULL)) != LDAP_SUCCESS)
     sqlr_new_error ("2E000", "LD016", "Failed to start TLS: %s (%d)",
