@@ -1,7 +1,7 @@
 /*
  *  eqlcomp.c
  *
- *  $Id: eqlcomp.c,v 1.16.2.7 2010/02/01 12:53:43 source Exp $
+ *  $Id: eqlcomp.c,v 1.16.2.14 2011/10/13 10:59:22 source Exp $
  *
  *  SQL Query Node Constructors.
  *
@@ -416,6 +416,8 @@ qr_free (query_t * qr)
 	}
       id_hash_free (qr->qr_call_counts);
     }
+  if (qr->qr_stats_mtx)
+    mutex_free (qr->qr_stats_mtx);
 #endif
 #if defined (MALLOC_DEBUG) || defined (VALGRIND)
   if ((NULL != qr->qr_static_prev) || (NULL != qr->qr_static_next) || (qr == static_qr_dllist))
@@ -1321,7 +1323,7 @@ ts_free (table_source_t * ts)
 {
   if (TS_ALT_POST == ts->ts_is_alternate)
     {
-      /* an alternate ts refers to the after tests and code of the primary ts.  Set the refs to nulll */
+      /* an alternate ts refers to the after tests and code of the primary ts.  Set the refs to null */
       ts->src_gen.src_after_test = NULL;
       ts->src_gen.src_after_code = NULL;
       if (ts->ts_order_ks)
@@ -1709,6 +1711,7 @@ key_source_om (comp_context_t * cc, key_source_t * ks)
     return;
   om = (out_map_t *) dk_alloc_box (sizeof (out_map_t) * n_out, DV_BIN);
   memset (om, 0, n_out * sizeof (out_map_t));
+  ks->ks_out_map = om;
   DO_SET (dbe_column_t *, col, &ks->ks_out_cols)
     {
       if (ks->ks_key->key_bit_cl && col->col_id == ks->ks_key->key_bit_cl->cl_col_id)
@@ -1716,10 +1719,15 @@ key_source_om (comp_context_t * cc, key_source_t * ks)
       else if (CI_ROW == (ptrlong) col)
 	om[inx++].om_is_null = OM_ROW;
       else
-	om[inx++].om_cl = *key_find_cl (ks->ks_key, col->col_id);
+	{
+	  dbe_col_loc_t * cl = key_find_cl (ks->ks_key, col->col_id);
+	  if (cl)
+	    om[inx++].om_cl = *cl;
+	  else
+	    SQL_GPF_T1 (cc, "cannot find column in index");
+	}
     }
   END_DO_SET();
-  ks->ks_out_map = om;
 }
 
 
@@ -2504,6 +2512,7 @@ eql_compile_eql (const char *string, client_connection_t * cli, caddr_t * err)
     eql_stmt_comp (&cc, (caddr_t) text, &head, &tail);
 
     qr->qr_head_node = head;
+    qr->qr_text = box_string (string);
 
     qr_add_current_of_output (qr);
     QR_POST_COMPILE (qr, (&cc));
