@@ -1,5 +1,5 @@
 --
---  $Id: ftp.sql,v 1.9 2009/07/14 19:04:33 source Exp $
+--  $Id: ftp.sql,v 1.7.2.2 2011/09/28 17:01:16 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -18,13 +18,11 @@
 --  You should have received a copy of the GNU General Public License along
 --  with this program; if not, write to the Free Software Foundation, Inc.,
 --  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
---
---
 
--- Note the FTP_POP_WRITE debugging function.
--- To debug, replace all 'pop_write<whitespace>(' with 'FTP_POP_WRITE<whitespace>('.
--- But do NOT replace all 'pop_write' with 'FTP_POP_WRITE'.
--- See the code of FTP_POP_WRITE and FTP_WRITE to find out the reason :)
+--  Note the FTP_POP_WRITE debugging function.
+--  To debug, replace all 'pop_write<whitespace>(' with 'FTP_POP_WRITE<whitespace>('.
+--  But do NOT replace all 'pop_write' with 'FTP_POP_WRITE'.
+--  See the code of FTP_POP_WRITE and FTP_WRITE to find out the reason :)
 
 --
 -- SERVER
@@ -120,7 +118,7 @@ create procedure WS.WS.FTP_SRV (in path any, in params any, in lines any)
 
   if (FTP_ANONYMOUS_CHECK (ftp_user))
     {
-       home_dir := cfg_item_value(virtuoso_ini_path(), 'HTTPServer', 'FTPServerAnonymousHome');
+       home_dir := virtuoso_ini_item_value ('HTTPServer', 'FTPServerAnonymousHome');
        if (home_dir is NULL)
 	 home_dir := '/DAV/';
        else if (DAV_HIDE_ERROR (DAV_SEARCH_ID (home_dir, 'C')) is null)
@@ -210,7 +208,7 @@ create procedure FTP_GET_COMMAND (in _in varchar, inout command varchar, inout a
 create procedure FTP_ANONYMOUS_CHECK (in _user varchar)
 {
   if (upper (_user) = 'ANONYMOUS' and
-      cfg_item_value(virtuoso_ini_path(), 'HTTPServer', 'FTPServerAnonymousLogin') = '1')
+      virtuoso_ini_item_value ('HTTPServer', 'FTPServerAnonymousLogin') = '1')
     return 1;
 
   return 0;
@@ -225,7 +223,7 @@ FTP_AUTHENTICATE (in id any, in what char(1), in req varchar, in a_uname varchar
     {
       if (upper (a_uname) = 'ANONYMOUS')
         {
-          if (cfg_item_value (virtuoso_ini_path(), 'HTTPServer', 'FTPServerAnonymousLogin') = '1')
+          if (virtuoso_ini_item_value ('HTTPServer', 'FTPServerAnonymousLogin') = '1')
             return DAV_AUTHENTICATE (id, what, req, 'anonymous', a_pwd, 1);
           return -12;
         }
@@ -1327,8 +1325,8 @@ create procedure GET_FREE_PORT ()
 {
    declare _last, _min, _max any;
 
-   _min := cfg_item_value(virtuoso_ini_path(), 'HTTPServer', 'FTPServerMinFreePort');
-   _max := cfg_item_value(virtuoso_ini_path(), 'HTTPServer', 'FTPServerMaxFreePort');
+   _min := virtuoso_ini_item_value ('HTTPServer', 'FTPServerMinFreePort');
+   _max := virtuoso_ini_item_value ('HTTPServer', 'FTPServerMaxFreePort');
 
    if (_min is NULL) _min := 20000;
    if (_max is NULL) _max := 30000;
@@ -1435,7 +1433,7 @@ create procedure FTP_WRITE (in l_user varchar, in w_str varchar, in command varc
   declare log_file varchar;
   -- dbg_obj_princ ('FTP_WRITE (', l_user, w_str, command, len, ')');
 
-  log_file := cfg_item_value(virtuoso_ini_path(), 'HTTPServer', 'FTPServerLogFile');
+  log_file := virtuoso_ini_item_value ('HTTPServer', 'FTPServerLogFile');
 
   if (log_file is NULL)
     goto finish;
