@@ -1,7 +1,7 @@
 /*
  *  gate.c
  *
- *  $Id: gate.c,v 1.20.2.3 2010/01/30 09:48:39 source Exp $
+ *  $Id: gate.c,v 1.20.2.9 2011/06/10 22:45:37 source Exp $
  *
  *  Hyperspace and gate.
  *
@@ -100,9 +100,13 @@ itc_adaptive_read_inc (it_cursor_t * itc, buffer_desc_t * dest_buf)
 #define ADAPTIVE_READ_INC(itc, dest_buf) \
   itc_adaptive_read_inc (itc, dest_buf)
 
-#ifdef MTX_DEBUG
+#ifdef PAGE_DEBUG
 #define AL_WAIT_SET_WRITER(buf) \
-  { if ((buf) && (buf)->bd_is_write) (buf)->bd_writer = THREAD_CURRENT_THREAD;}
+  { \
+    if ((buf) && (buf)->bd_is_write) \
+      (buf)->bd_writer = THREAD_CURRENT_THREAD; \
+    BUF_DBG_ENTER (buf); \
+  }
 #else
 #define AL_WAIT_SET_WRITER(buf)
 #endif
@@ -118,13 +122,15 @@ itc_adaptive_read_inc (it_cursor_t * itc, buffer_desc_t * dest_buf)
 buffer_desc_t * bounds_check_buf;
 
 int
-page_wait_access (it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from, buffer_desc_t ** buf_ret, int mode, int max_change)
+DBGP_NAME (page_wait_access) (DBGP_PARAMS it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from,
+    buffer_desc_t ** buf_ret, int mode, int max_change)
 {
   buffer_desc_t decoy;
   buffer_desc_t *buf;
   dp_addr_t phys_dp;
   itc->itc_to_reset = RWG_NO_WAIT;
   itc->itc_max_transit_change = max_change;
+  itc->itc_must_kill_trx = 0;
   if (!dp)
     GPF_T1 ("Zero DP in page_fault_map_sem");
 
@@ -155,6 +161,7 @@ page_wait_access (it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from, bu
 	  else
 	    {
 	      *buf_ret = PF_OF_DELETED;
+	      itc->itc_must_kill_trx = 1;
 	      itc->itc_to_reset = RWG_WAIT_ANY;
 	      ITC_LEAVE_MAPS (itc);
 	      return RWG_WAIT_ANY;
@@ -214,6 +221,7 @@ page_wait_access (it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from, bu
       BUF_BOUNDS_CHECK (buf);
       buf_ext_check (buf);
       *buf_ret = buf;
+      BUF_DBG_ENTER (buf);
       return itc->itc_to_reset;
     }
   if (buf->bd_being_read)
@@ -264,6 +272,7 @@ page_wait_access (it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from, bu
 	  BUF_BOUNDS_CHECK (buf);
 	  *buf_ret = buf;
 	  BUF_TOUCH (buf);
+	  BUF_DBG_ENTER (buf);
 	  return  itc->itc_to_reset;
 	}
       else
@@ -287,6 +296,7 @@ page_wait_access (it_cursor_t * itc, dp_addr_t dp,  buffer_desc_t * buf_from, bu
 	  itc->itc_pl = buf->bd_pl;
 	  *buf_ret = buf;
 	  BUF_TOUCH (buf);
+	  BUF_DBG_ENTER (buf);
 	  return itc->itc_to_reset;
 	}
       else
@@ -340,7 +350,7 @@ page_release_read (buffer_desc_t * buf)
 	  if (PA_WRITE == waiting->itc_dive_mode)
 	    {
 	      /* if adaptive landing made this a write, then free no more cursors. */
-#ifdef MTX_DEBUG
+#ifdef PAGE_DEBUG
 	      buf->bd_writer = waiting->itc_thread;
 #endif
 	      buf->bd_read_waiting = next;
@@ -384,7 +394,7 @@ page_release_writes (buffer_desc_t * buf)
 	  BUF_BOUNDS_CHECK (buf);
 	  waiting->itc_pl = buf->bd_pl;
 	  BD_SET_IS_WRITE (buf, 1);
-#ifdef MTX_DEBUG
+#ifdef PAGE_DEBUG
 	  buf->bd_writer = waiting->itc_thread;
 #endif
 	  buf->bd_write_waiting = next;
@@ -414,13 +424,14 @@ page_release_writes (buffer_desc_t * buf)
 
 
 void
-page_leave_inner (buffer_desc_t * buf)
+DBGP_NAME (page_leave_inner) (DBGP_PARAMS buffer_desc_t * buf)
 {
 #ifdef MTX_DEBUG
   if (!is_crash_dump && buf->bd_tree)
     ASSERT_IN_MAP (buf->bd_tree, buf->bd_page);
 #endif
   BUF_BOUNDS_CHECK(buf);
+  BUF_DBG_LEAVE (buf);
   if (buf->bd_readers)
     {
       buf->bd_readers--;
@@ -446,7 +457,7 @@ page_leave_inner (buffer_desc_t * buf)
 	    page_release_read (buf);
 	}
     }
-  /* set the space to nnull for deld buffer after all else.  This is so that buffer replacement will wait for the page map so it does not do dirty reads on intermediate states of releasing the buffer */
+  /* set the space to null for deld buffer after all else.  This is so that buffer replacement will wait for the page map so it does not do dirty reads on intermediate states of releasing the buffer */
 }
 
 
@@ -558,7 +569,7 @@ itc_try_land (it_cursor_t * itc, buffer_desc_t ** buf_ret)
       ITC_SEM_WAIT (itc);
       if (itc->itc_to_reset <= RWG_WAIT_DATA)
 	{
-#ifdef MTX_DEBUG
+#ifdef PAGE_DEBUG
 	  buf->bd_writer = THREAD_CURRENT_THREAD;
 #endif
 	  itc->itc_pl = buf->bd_pl;
@@ -662,6 +673,12 @@ retry:
   if (ctl_itc->itc_to_reset > RWG_WAIT_KEY)
     {
       TC (tc_reentry_split);
+      if (ctl_itc->itc_must_kill_trx) /* reference to free remap, kill trx */
+	{
+	  ctl_itc->itc_must_kill_trx = 0;
+	  if (!wi_inst.wi_checkpoint_atomic && ctl_itc->itc_ltrx)
+	    itc_bust_this_trx (ctl_itc, NULL, ITC_BUST_THROW);
+	}
       goto retry;
     }
   if (PF_OF_DELETED == buf)
@@ -792,7 +809,10 @@ itc_dive_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
       if (itm1 != itm2)
 	mutex_leave (&itm2->itm_mtx);
       if (BUF_NONE_WAITING (old_buf))
+	{
+	  BUF_DBG_LEAVE_INL (old_buf);
 	old_buf->bd_readers--;
+	}
       else
 	page_leave_inner (old_buf);
       /* If the itms were different, the target itm is already exited */
@@ -822,7 +842,10 @@ itc_dive_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
       dest_buf->bd_readers++;
       mutex_leave (&itm2->itm_mtx);
       if (BUF_NONE_WAITING (old_buf))
+	{
+	  BUF_DBG_LEAVE_INL (old_buf);
 	old_buf->bd_readers--;
+	}
       else
 	page_leave_inner (old_buf);
       mutex_leave (&itm1->itm_mtx);
@@ -854,7 +877,10 @@ itc_dive_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
       ADAPTIVE_READ_INC (itc, dest_buf);
       mutex_leave (&itm2->itm_mtx);
       if (BUF_NONE_WAITING (old_buf))
+	{
+	  BUF_DBG_LEAVE_INL (old_buf);
 	old_buf->bd_readers--;
+	}
       else
 	page_leave_inner (old_buf);
       mutex_leave (&itm1->itm_mtx);
@@ -883,6 +909,7 @@ itc_dive_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
   goto check_link_2;
  check_link:
   BUF_TOUCH (dest_buf);
+  BUF_DBG_ENTER_INL (dest_buf);
  check_link_2:
   back_link = LONG_REF (dest_buf->bd_buffer + DP_PARENT);
   itc->itc_page = dest_buf->bd_page;
@@ -893,11 +920,19 @@ itc_dive_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
   if ((*buf_ret)->bd_readers <= 0 && !(*buf_ret)->bd_is_write)
     GPF_T1 ("dive transit ends in bd_readers <= 0 or is_write > 0");
 #endif
+  if ((*buf_ret)->bd_is_dirty && (*buf_ret)->bd_physical_page != (*buf_ret)->bd_page)
+    {
+      it_map_t * itm = IT_DP_MAP (tree, to);
+      mutex_enter (&itm->itm_mtx);
+      if (!gethash ((void*)(void*)(ptrlong)to, &itm->itm_remap))
+	GPF_T;
+      mutex_leave (&itm->itm_mtx);
+    }
 }
 
 
 void
-itc_landed_down_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
+DBGP_NAME (itc_landed_down_transit) (DBGP_PARAMS it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
 {
   buffer_desc_t *old_buf = *buf_ret;
   int waited = 0;
@@ -911,7 +946,7 @@ itc_landed_down_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t
   if (!old_buf->bd_is_write) GPF_T1 ("landed transit got no write");
 #endif
   itc->itc_is_on_row = 1;
-  page_wait_access (itc, to, *buf_ret, buf_ret, PA_WRITE, RWG_NO_WAIT);
+  DBGP_NAME (page_wait_access) (DBGP_ARGS itc, to, *buf_ret, buf_ret, PA_WRITE, RWG_NO_WAIT);
   if (itc->itc_to_reset > RWG_NO_WAIT)
     {
       /* there was a wait and the itc was registered.  To unregister, reenter the page and go back to itc_search to see where to goo. The leaf could have been deld or split */
@@ -954,10 +989,10 @@ itc_landed_down_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t
 }
 
 void
-itc_down_transit (it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
+DBGP_NAME (itc_down_transit) (DBGP_PARAMS it_cursor_t * itc, buffer_desc_t ** buf_ret, dp_addr_t to)
 {
   if (itc->itc_landed)
-    itc_landed_down_transit (itc, buf_ret, to);
+    DBGP_NAME (itc_landed_down_transit) (DBGP_ARGS itc, buf_ret, to);
   else
     itc_dive_transit (itc, buf_ret, to);
 }
@@ -989,7 +1024,7 @@ up_again:
     parent_buf = NULL;
   page_wait_access (itc, up, *buf_ret, &parent_buf, PA_WRITE, RWG_WAIT_NO_ENTRY_IF_WAIT);
   /* the no entry case applies only to a wait. Also, if there is a wait, the original buffer will stay occupied by this itc.
-  * So if wegot no buffer, there was a wait and if we got one the transit had no wait but could have disk read. */
+  * So if we got no buffer, there was a wait and if we got one the transit had no wait but could have disk read. */
   if (!parent_buf)
     {
       up_ctr ++;
@@ -1162,7 +1197,7 @@ it_root_image_invalidate (index_tree_t * tree)
 
 
 buffer_desc_t *
-itc_reset (it_cursor_t * it)
+DBGP_NAME (itc_reset) (DBGP_PARAMS it_cursor_t * it)
 {
   /* Enter to root in read mode and return the buffer */
   buffer_desc_t *buf;
@@ -1183,7 +1218,7 @@ itc_reset (it_cursor_t * it)
       if (it->itc_is_registered)
 	GPF_T1 ("should not be registered when resetting to a hi");
       ITC_IN_KNOWN_MAP (it, it->itc_tree->it_hash_first);
-        page_wait_access (it, it->itc_tree->it_hash_first, NULL, &buf, PA_WRITE, RWG_WAIT_ANY);
+        DBGP_NAME (page_wait_access) (DBGP_ARGS it, it->itc_tree->it_hash_first, NULL, &buf, PA_WRITE, RWG_WAIT_ANY);
 	ITC_LEAVE_MAPS (it);
 	it->itc_map_pos = DP_DATA + HASH_HEAD_LEN; /* offset into the page, not inx of page map */
       it->itc_page = buf->bd_page;
@@ -1201,7 +1236,8 @@ itc_reset (it_cursor_t * it)
       if (buf)
 	return buf;
     }
-  if (tree->it_is_single_page)
+  if (tree->it_is_single_page
+    )
     it->itc_dive_mode = PA_WRITE;
 
   for (;;)
@@ -1235,7 +1271,7 @@ itc_reset (it_cursor_t * it)
 	    }
 	}
       TC (tc_root_cache_miss);
-      page_wait_access (it, dp, NULL, &buf, it->itc_dive_mode, RWG_WAIT_KEY);
+      DBGP_NAME (page_wait_access) (DBGP_ARGS it, dp, NULL, &buf, it->itc_dive_mode, RWG_WAIT_KEY);
       if (buf == PF_OF_DELETED)
 	GPF_T1 ("The root page of an index is free do a crash dump for recovery.");
       if (it->itc_to_reset > RWG_WAIT_KEY)
@@ -1245,7 +1281,8 @@ itc_reset (it_cursor_t * it)
     }
   BUF_TOUCH (buf);
   if (!tree->it_is_single_page && !tree->it_root_image_version
-      && tree->it_key->key_id != KI_TEMP)
+      && tree->it_key->key_id != KI_TEMP
+     )
     {
       it_new_root_image (tree, buf);
     }
@@ -1384,7 +1421,7 @@ itc_register (it_cursor_t * itc, buffer_desc_t * buf)
     GPF_T1 ("must have write on the buffer for registration");
   rdbg_printf_2 (("  register itc=%x on L=%d \n", it, it->itc_page));
   itc->itc_is_registered = 1;
-#if 1
+#ifndef NDEBUG
   {
     it_cursor_t * ck = buf->bd_registered;
     while (ck)
