@@ -1,7 +1,7 @@
 /*
  *  hash.c
  *
- *  $Id: hash.c,v 1.19.2.10 2009/12/22 15:59:24 source Exp $
+ *  $Id: hash.c,v 1.19.2.19 2011/06/01 09:20:51 source Exp $
  *
  *  Hash Index
  *
@@ -714,7 +714,11 @@ hash_cast (query_instance_t * qi, hash_area_t * ha, int inx, state_slot_t * ssl,
       || (DV_IRI_ID_8 == target_dtp && DV_IRI_ID == dtp))
     return data;
   if (SSL_CONSTANT == ssl->ssl_type)
+#ifndef NDEBUG
     GPF_T1 ("constant ssl in hash_cast");
+#else
+    sqlr_new_error ("42000", ".....", "constant ssl in hash_cast, please report statement compiled");
+#endif
   if (IS_BLOB_DTP (target_dtp))
     target_dtp = DV_BLOB_INLINE_DTP (target_dtp);  /* non blob value for blob col. Will be inlined */
   data = box_cast_to (qst, data, dtp,
@@ -772,6 +776,8 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
   rd.rd_allocated = RD_AUTO;
   rd.rd_itc = itc;
   rd.rd_key = ha->ha_key;
+  if (HA_PROC_FILL == ha->ha_op || HA_GROUP == ha->ha_op)
+    rd.rd_any_ser_flags = DKS_TO_HA_DISK_ROW;
   if (!bp_ref_itc)
     {
       index_tree_t * ha_tree = (index_tree_t *) QST_GET_V (qst, ha->ha_tree);
@@ -793,6 +799,8 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
     {
       int var_len_org = var_len;
       rd.rd_non_comp_max = MAX_ROW_BYTES;
+      if (THR_IS_STACK_OVERFLOW (THREAD_CURRENT_THREAD, &hm_val, (PAGE_DATA_SZ+2000)))
+        sqlr_new_error ("42000", "SR483", "Stack Overflow");
     try_with_blobs_outlined:
       rd.rd_non_comp_len = var_len + key->key_row_var_start[0];
       for (inx = ha->ha_n_keys; ha->ha_key_cols[inx].cl_col_id; inx++)
@@ -840,7 +848,7 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
 	       cl->cl_sqt.sqt_dtp == DV_OBJECT))
 	    {
 	      caddr_t err = NULL;
-	      caddr_t serialized_value = box_to_any (value, &err);
+	      caddr_t serialized_value = box_to_any_1 (value, &err, NULL, rd.rd_any_ser_flags);
 	      if (err != NULL)
 		{
 		  dk_free_tree (err);
@@ -910,6 +918,7 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
   hash_row = hash_buf->bd_buffer + hb_fill;
   IE_ROW_VERSION (hash_row) = 0;
   IE_SET_KEY_VERSION (hash_row, 1);
+  memset (hash_row + key->key_null_flag_start[0], 0, key->key_null_flag_bytes[0]);
   rf.rf_row = hash_row;
   rf.rf_space = row_len;
 
@@ -926,6 +935,8 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
 
   hi_bp_set (tree->it_hi, bp_ref_itc, code, hash_buf->bd_page, hb_fill);
   rd.rd_non_comp_len = key->key_row_var_start[0];
+  if (HA_GROUP == ha->ha_op)
+    rd.rd_any_ser_flags = 0;
   itc_free_owned_params (itc);
   ITC_START_SEARCH_PARS (itc);
   for (inx = 0; ha->ha_key_cols[inx].cl_col_id; inx++)
@@ -937,6 +948,8 @@ itc_ha_disk_row (it_cursor_t * itc, buffer_desc_t * buf, hash_area_t * ha, caddr
       else
 	value = QST_GET (qst, ssl);
       err = NULL;
+      if (inx >= ha->ha_n_keys && HA_GROUP == ha->ha_op)
+	rd.rd_any_ser_flags = DKS_TO_HA_DISK_ROW;
       hash_row_set_col (&rd, &rf, &ha->ha_key_cols[inx], value, feed_temp_blobs);
 
       if (err)
@@ -1454,6 +1467,8 @@ ha_rehash (caddr_t * inst, hash_area_t * ha, index_tree_t * it)
     {
       if (inx >= ha->ha_n_keys)
 	break;
+      if (ssl->ssl_type == SSL_CONSTANT)
+	continue;
       qst_set (inst, ssl, save[inx]);
     }
   END_DO_BOX;
@@ -1576,7 +1591,7 @@ itc_ha_feed (itc_ha_feed_ret_t *ret, hash_area_t * ha, caddr_t * qst, unsigned l
 	}
       next_link = HA_FILL == ha->ha_op ? 1 : 0;
       hmk.hmk_var_len = var_len;
-      hmk.hmk_ha = ha; /* after the store is done, the hash cabn be retained and theese will be refs to free mem */
+      hmk.hmk_ha = ha; /* after the store is done, the hash can be retained and these will be refs to free mem */
       if (hi->hi_pool)
 	{
 	  hmk.hmk_data = (caddr_t *) mp_full_box_copy_tree (hi->hi_pool, (caddr_t) hmk.hmk_data);
@@ -1691,12 +1706,14 @@ setp_print_input (setp_node_t * setp, caddr_t * qst)
 void
 setp_group_row (setp_node_t * setp, caddr_t * qst)
 {
+  dtp_t row_image[MAX_ROW_BYTES];
   hash_area_t * ha = setp->setp_ha;
   itc_ha_feed_ret_t ihfr;
   int rc = itc_ha_feed (&ihfr, ha, qst, 0);
   index_tree_t * tree;
   hash_index_t * hi;
   LOCAL_RF (rf, 0, 0, ha->ha_key);
+  rf.rf_large_row = &row_image[0];
   /*setp_print_input (setp, qst);*/
   if (DVC_MATCH == rc)
     goto runX_begin; /* see below */
@@ -2001,7 +2018,11 @@ setp_order_row (setp_node_t * setp, caddr_t * qst)
     {
       tree = it_temp_allocate (wi_inst.wi_temp);
       tree->it_key = ha->ha_key;
-      it_temp_tree (tree);
+      if (!it_temp_tree (tree))
+	{
+	  it_free (tree);
+	  sqlr_new_error ("42000", "SR...", "Can't allocate tree for temp space.");
+	}
       qst_set (qst, ha->ha_tree, (caddr_t) tree);
     }
   ins_itc = (it_cursor_t *) QST_GET_V (qst, ha->ha_ref_itc);
