@@ -1,7 +1,7 @@
 /*
  *  log.c
  *
- *  $Id: log.c,v 1.16.2.7 2010/01/31 20:41:04 source Exp $
+ *  $Id: log.c,v 1.16.2.23 2011/10/13 11:09:41 source Exp $
  *
  *  Transaction log write and recovery
  *
@@ -159,6 +159,18 @@ log_set_server_version_check (int in_txn)
 
   sprintf (tmp, "server_version_check ('%s')", DBMS_SRV_VER);
   log_set_compatibility_check (in_txn, tmp);
+  if (wi_inst.wi_master->dbs_id[0])
+    {
+      int inx;
+      char hex[33];
+      for (inx = 0; inx < sizeof (wi_inst.wi_master->dbs_id); inx ++)
+	{
+	  sprintf (hex + (inx * 2), "%02x", (unsigned char) (wi_inst.wi_master->dbs_id[inx]));
+	}
+      hex[32] = 0;
+      sprintf (tmp, "server_id_check ('%s')", hex);
+      log_set_compatibility_check (in_txn, tmp);
+    }
 }
 
 
@@ -257,10 +269,11 @@ log_commit (lock_trx_t * lt)
   long bytes = strses_length (lt->lt_log);
   caddr_t *cbox;
   if (lt->lt_replicate == REPL_NO_LOG
-      || (LT_CL_PREPARED != lt->lt_status && !bytes))
+      || (LT_CL_PREPARED != lt->lt_status && !bytes) || cl_non_logged_write_mode)
     return LTE_OK;
   if (dbf_log_no_disk)
     return LTE_LOG_FAILED;
+  ASSERT_IN_MTX (log_write_mtx);
   prev_length = dbs->dbs_log_length;
   log_ses = dbs->dbs_log_session;
   if (!dbs->dbs_log_session)
@@ -293,7 +306,7 @@ log_commit (lock_trx_t * lt)
     }
   else
     cbox[LOGH_CL_2PC] = 0;
-  if (!lt->lt_branch_of && lt->lt_client->cli_user)
+  if (!lt->lt_branch_of && lt->lt_client && lt->lt_client->cli_user)
     cbox[LOGH_USER] = box_string (lt->lt_client->cli_user->usr_name);
   else
     cbox[LOGH_USER] = box_string ("");
@@ -339,8 +352,8 @@ log_commit (lock_trx_t * lt)
 	    + log_ses->dks_out_fill - 1;
 	if (lt->lt_2pc._2pc_log)
 	  print_object ((caddr_t) lt->lt_2pc._2pc_log, log_ses, NULL, NULL);
-	dk_free_box(lt->lt_2pc._2pc_log);
-	lt->lt_2pc._2pc_log = 0;
+	/* dk_free_box(lt->lt_2pc._2pc_log);
+	lt->lt_2pc._2pc_log = 0; */
 	if (lt->lt_2pc._2pc_type == TP_XA_TYPE)
 	  {
 	    mutex_enter (global_xa_map->xm_mtx);
@@ -410,7 +423,7 @@ log_text_array_sync (lock_trx_t * lt, caddr_t box)
   int rc;
   dk_session_t * lt_log;
   dk_set_t blob_log;
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return LTE_OK;
   ASSERT_IN_MTX (log_write_mtx);
   lt_log = lt->lt_log;
@@ -437,7 +450,7 @@ log_insert_sync (lock_trx_t * lt, row_delta_t * rd, int flag)
   int rc;
   dk_session_t * lt_log;
   dk_set_t blob_log;
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return LTE_OK;
   mutex_enter (log_write_mtx);
   lt_log = lt->lt_log;
@@ -464,7 +477,7 @@ log_sequence_sync (lock_trx_t * lt, caddr_t seq, boxint count)
   int rc;
   dk_session_t * lt_log;
   dk_set_t blob_log;
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return LTE_OK;
   mutex_enter (log_write_mtx);
   lt_log = lt->lt_log;
@@ -494,9 +507,9 @@ log_final_transact(lock_trx_t* lt, int is_commit)
   dbe_storage_t * dbs = wi_inst.wi_master;
   if (lt->lt_2pc._2pc_logged)
     {
-      struct stat st;
+      STAT_T st;
       int fd = tcpses_get_fd(dbs->dbs_log_session->dks_session);
-      fstat (fd, &st);
+      V_FSTAT (fd, &st);
       LSEEK (fd, lt->lt_2pc._2pc_logged, SEEK_SET);
       write (fd, is_commit ? LOG_2PC_COMMIT_S : LOG_2PC_ABORT_S ,1);
       LSEEK (fd, 0, SEEK_END);
@@ -563,7 +576,7 @@ log_2pc_archive (int64 trx_id)
       int fd;
       file_set_rw (dbs->dbs_2pc_file_name);
       fd = fd_open (dbs->dbs_2pc_file_name, LOG_OPEN_FLAGS);
-      if (fd <= 0)
+      if (fd < 0)
 	{
 	  int errn = errno;
 	  log_error ("Cannot open 2pc log %s, error : %s.  Exiting", dbs->dbs_2pc_file_name, virt_strerror (errn));
@@ -599,7 +612,7 @@ log_2pc_archive_check (int64 trx_id, int64 * max_id_ret)
     return 0;
   file_set_rw (dbs->dbs_2pc_file_name);
   fd  = fd_open (dbs->dbs_2pc_file_name, OPEN_FLAGS_RO);
-  if (fd <= 0)
+  if (fd < 0)
     {
       *max_id_ret = 0;
       return 0;
@@ -644,7 +657,7 @@ log_cl_final(lock_trx_t* lt, int is_commit)
   dbe_storage_t * dbs = wi_inst.wi_master;
   if (lt->lt_commit_flag_offset)
     {
-      struct stat st;
+      STAT_T st;
       int fd;
       char s = SQL_COMMIT == is_commit ? LOG_2PC_COMMIT : LOG_2PC_ABORT;
       ASSERT_OUTSIDE_MTX (wi_inst.wi_txn_mtx);
@@ -654,12 +667,16 @@ log_cl_final(lock_trx_t* lt, int is_commit)
 	{
 	  OFF_T end;
 	  fd = tcpses_get_fd(dbs->dbs_log_session->dks_session);
-	  fstat(fd,&st);
+	  V_FSTAT(fd,&st);
 	  if (lt->lt_commit_flag_offset !=  LSEEK(fd,lt->lt_commit_flag_offset,SEEK_SET))
 	    GPF_T1 ("failed lseek in cl commit final");
 	  write(fd, &s, 1);
 	  if (dbs->dbs_log_length != (end = LSEEK(fd,0,SEEK_END)))
-	    GPF_T1 ("dbs_log_length and end of log file differ");
+	    {
+	      log_error ("Logging at offset " OFF_T_PRINTF_FMT " where end of log file is " OFF_T_PRINTF_FMT ".",
+		  	(OFF_T_PRINTF_DTP) end, (OFF_T_PRINTF_DTP) dbs->dbs_log_length);
+	      GPF_T1 ("dbs_log_length and end of log file differ");
+	    }
 	  lt->lt_commit_flag_offset = 0;
 	  if (SQL_COMMIT == is_commit && QFID_HOST (lt->lt_w_id) == local_cll.cll_this_host)
 	    log_2pc_archive (lt->lt_w_id);
@@ -695,7 +712,7 @@ log_insert (lock_trx_t * lt, row_delta_t * rd, int flag)
   int inx;
   dk_session_t *log;
   lt_hi_row_change (lt, rd->rd_key->key_super_id, LOG_INSERT, NULL);
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   if (LOG_SYNC & flag)
     {
@@ -731,7 +748,7 @@ void
 log_delete (lock_trx_t * lt, row_delta_t * rd, int this_key_only)
 {
   lt_hi_row_change (lt, rd->rd_key->key_super_id, LOG_DELETE, NULL);
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
 
@@ -748,7 +765,7 @@ void
 log_text (lock_trx_t * lt, char *text)
 {
   caddr_t box;
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
 
@@ -764,7 +781,7 @@ log_text (lock_trx_t * lt, char *text)
 void
 log_text_array (lock_trx_t * lt, caddr_t box)
 {
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
   session_buffered_write_char (LOG_TEXT, lt->lt_log);
@@ -776,7 +793,7 @@ log_text_array (lock_trx_t * lt, caddr_t box)
 void
 log_text_array_as_user (user_t * usr, lock_trx_t * lt, caddr_t box)
 {
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
   session_buffered_write_char (LOG_USER_TEXT, lt->lt_log);
@@ -791,7 +808,7 @@ log_text_array_as_user (user_t * usr, lock_trx_t * lt, caddr_t box)
 void
 log_sequence (lock_trx_t * lt, char *text, boxint count)
 {
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
   session_buffered_write_char (LOG_SEQUENCE_64, lt->lt_log);
@@ -816,7 +833,7 @@ log_update (lock_trx_t * lt, row_delta_t * rd,
     update_node_t * upd, caddr_t * qst)
 {
   int inx;
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
   mutex_enter (lt->lt_log_mtx);
   session_buffered_write_char (LOG_UPDATE, lt->lt_log);
@@ -887,7 +904,7 @@ log_sc_change_2 (lock_trx_t * lt)
 void
 log_dd_change (lock_trx_t * lt, char * tb)
 {
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
 
   if (tb)
@@ -906,7 +923,7 @@ log_dd_change (lock_trx_t * lt, char * tb)
 void
 log_dd_type_change (lock_trx_t * lt, char * udt_name, caddr_t tree)
 {
-  if (!lt || lt->lt_replicate == REPL_NO_LOG)
+  if (!lt || lt->lt_replicate == REPL_NO_LOG || cl_non_logged_write_mode)
     return;
 
   if (tree)
@@ -1087,7 +1104,7 @@ log_replay_text (lock_trx_t * lt, dk_session_t * in, int is_pushback, int use_st
   int n_args = 0;
   caddr_t *entry = (caddr_t *) scan_session (in);
   dtp_t dtp = DV_TYPE_OF (entry);
-  caddr_t text = DV_ARRAY_OF_POINTER == dtp ? entry[0] : (caddr_t) entry;
+  caddr_t text = DV_ARRAY_OF_POINTER == dtp && BOX_ELEMENTS (entry) > 0 ? entry[0] : (caddr_t) entry;
   caddr_t err = NULL;
   caddr_t stmt_id = box_dv_short_string ("repl_stmt");
   query_t *qr;
@@ -1097,6 +1114,12 @@ log_replay_text (lock_trx_t * lt, dk_session_t * in, int is_pushback, int use_st
   caddr_t *arr = NULL;
   LOG_REPL_OPTIONS (opts);
 
+  if (!DV_STRINGP (text))
+    {
+      dk_free_box ((box_t) entry);
+      return srv_make_new_error ("42000", "TR100", "log_replay_text: invalid query text");
+    }
+
   if (!is_pushback)
     {
       if (dtp == DV_ARRAY_OF_POINTER)
@@ -1398,7 +1421,7 @@ log_replay_text (lock_trx_t * lt, dk_session_t * in, int is_pushback, int use_st
 cr_done:
   if (use_stmt_cache)
     {
-      sst = cli_get_stmt_access (lt->lt_client, stmt_id, GET_EXCLUSIVE);
+      sst = cli_get_stmt_access (lt->lt_client, stmt_id, GET_EXCLUSIVE, NULL);
       err = stmt_set_query (sst, lt->lt_client, text, opts);
       LEAVE_CLIENT (lt->lt_client);
       if (err != NULL)
@@ -1643,6 +1666,7 @@ try_again:
 	  /*session_buffered_read(in,transact,in->dks_in_fill - in->dks_in_read);
 	  id_hash_set (global_xa_map->xm_log_xids, (caddr_t) & xid, (caddr_t) & transact);*/
 	  virt_xa_add_trx (xid, lt);
+	  lt->lt_replicate = REPL_LOG;
 	  is_xa = 1;
 	  _xa_log_ctr++;
 	}
@@ -1728,9 +1752,21 @@ try_again:
     {
       cli->cli_trx = NULL;
       lt->lt_client = NULL;
+      lt->lt_status = LT_PREPARED;
+      lt->lt_2pc._2pc_wait_commit = 1;
       cli_set_new_trx (cli);
       rc = LTE_OK;
     }
+  if (in_log_replay)
+    {
+      wi_free_old_qrs ();
+      DO_SET (dbe_schema_t *, sc, &wi_inst.wi_free_schemas)
+	{
+	  dk_set_delete (&wi_inst.wi_free_schemas, (void *) sc);
+	  dbe_schema_free (sc);
+	}
+      END_DO_SET ();
+    }
   LEAVE_TXN;
   if (LTE_LOG_FAILED == rc)
     {
@@ -1751,46 +1787,6 @@ int level_print = 0;
 void
 logh_set_level (lock_trx_t * lt, caddr_t * logh)
 {
-#if REPLICATION_SUPPORT
-  if (logh && logh[LOGH_REPLICATION])
-    {
-      caddr_t *replh = (caddr_t *) logh[LOGH_REPLICATION];
-      repl_acct_t *ra;
-      ra = ra_find (replh[REPLH_SERVER], replh[REPLH_ACCOUNT]);
-      if (level_print)
-	dbg_printf ((" Rfwd level %s %s " BOXINT_FMT "\n", replh [REPLH_SERVER], replh [REPLH_ACCOUNT] ? replh [REPLH_ACCOUNT] : "-", unbox (replh [REPLH_LEVEL])));
-      if (!ra)
-	{
-	  ra = ra_add (replh[REPLH_SERVER], replh[REPLH_ACCOUNT], 1, 0, 0);
-	}
-
-      if (replh[REPLH_LEVEL])
-	{
-	  caddr_t log_array;
-#ifdef UNIX
-	  gettimeofday (&ra->ra_last_txn, NULL);
-#else
-	  ra->ra_last_txn.tv_sec = (long) time (NULL);
-#endif
-	  log_array = list (4, box_string ("sequence_set (?, ?, ?)"),
-		box_string (ra->ra_sequence),
-		box_num (unbox (replh[REPLH_LEVEL])), box_num (SET_ALWAYS));
-	  ASSERT_IN_TXN;
-	  sequence_set (ra->ra_sequence, (long) unbox (replh[REPLH_LEVEL]), SET_ALWAYS, INSIDE_MAP);
-	  log_text_array (lt, log_array);
-	  dk_free_tree (log_array);
-	  /* having replayed a replication record, log its level as a sequence set in the roll forward
-	   * log of this txn since the replication feed record itself will not be logged */
-	}
-      else
-	{
-	  /* A zero level on an account means the replicator is now synced */
-	  ra->ra_synced = RA_IN_SYNC;
-	  log_info ("Replication Account %s %s IN sync, level %ld.\n",
-		    ra->ra_server, ra->ra_account, sequence_set (ra->ra_sequence, 0, SEQUENCE_GET, INSIDE_MAP));
-	}
-    }
-#endif
 }
 
 
@@ -1905,7 +1901,7 @@ log_check_trx (int64 trx_no)
   if (log_2pc_archive_check (trx_no, &max))
     return 1;
   fd = fd_open (dbs->dbs_log_name, OPEN_FLAGS_RO);
-  if (fd <= 0)
+  if (fd < 0)
     return 0;
   file_in = dk_session_allocate (SESCLASS_TCPIP);
   tcpses_set_fd (file_in->dks_session, fd);
@@ -2106,7 +2102,10 @@ read_again:
       dk_free_tree ((caddr_t) header);
 
       if (0 == rfwd_ctr % 1000)
-	REPORT_PROGRESS;
+	{
+	  REPORT_PROGRESS;
+	  clear_old_root_images ();
+	}
     }
   if (rfwd_ctr)
     REPORT_PROGRESS;
@@ -2129,6 +2128,7 @@ log_checkpoint (dbe_storage_t * dbs, char *new_log, int shutdown)
     {
       if (dbs->dbs_log_session)
 	{
+	  mutex_enter (log_write_mtx);
 	  LSEEK (tcpses_get_fd (dbs->dbs_log_session->dks_session), 0, SEEK_SET);
 	  FTRUNCATE (tcpses_get_fd (dbs->dbs_log_session->dks_session), (OFF_T) (0));
 	  dbs->dbs_log_length = 0;
@@ -2137,6 +2137,7 @@ log_checkpoint (dbe_storage_t * dbs, char *new_log, int shutdown)
 	      log_set_byte_order_check (1);
 	      log_set_server_version_check (1);
 	    }
+	  mutex_leave (log_write_mtx);
 	  log_info ("Checkpoint finished, log reused");
 	}
       else
@@ -2151,7 +2152,7 @@ log_checkpoint (dbe_storage_t * dbs, char *new_log, int shutdown)
 	  int new_fd;
 	  file_set_rw (new_log);
 	  new_fd = fd_open (new_log, LOG_OPEN_FLAGS);
-	  if (-1 == new_fd)
+	  if (new_fd < 0)
 	    {
 	      log_error ("Cannot change to log file %s", new_log);
 	      call_exit (1);
@@ -2168,31 +2169,42 @@ log_checkpoint (dbe_storage_t * dbs, char *new_log, int shutdown)
 	  dbs->dbs_log_name = box_string (new_log);
 	}
       cfg_replace_log (new_log);
+      mutex_enter (log_write_mtx);
       dbs->dbs_log_length = 0;
       if (CPT_SHUTDOWN != shutdown)
 	{
 	  log_set_byte_order_check (1);
 	  log_set_server_version_check (1);
 	}
+      mutex_leave (log_write_mtx);
       log_info ("Checkpoint finished, new log is %s", new_log);
     }
   DO_SET (lock_trx_t *, lt, &all_trxs)
     {
-      if ((LT_CL_PREPARED == lt->lt_status || LT_2PC_PENDING == lt->lt_status)
+      if (((LT_PREPARED == lt->lt_status && lt->lt_2pc._2pc_wait_commit) || LT_2PC_PENDING == lt->lt_status)
 	  && strses_length (lt->lt_log))
 	{
+	  int status = lt->lt_status;
 	  int rc;
+	  mutex_enter (log_write_mtx);
 	  if (!dbs->dbs_log_length)
 	    {
 	      log_set_byte_order_check (1);
 	      log_set_server_version_check (1);
 	    }
-	  mutex_enter (log_write_mtx);
+	  if (LT_PREPARED == status && lt->lt_2pc._2pc_wait_commit)
+	    lt->lt_status = LT_PREPARE_PENDING;
 	  rc = log_commit (lt);
+	  lt->lt_status = status;
 	  if (LTE_OK != rc)
 	    log_error ("Checkpoint interrupted 2pc transaction between prepare and final.  The log of the prepared state could not be written to the post-checkpoint transaction log.  This means that, if the transaction commits and ought to be replayed from log, the transaction will be lost. trx no %d:%d", QFID_HOST (lt->lt_w_id), (uint32)lt->lt_w_id);
 	  mutex_leave (log_write_mtx);
 	}
+      else if (lt->lt_log_2pc && lt->lt_commit_flag_offset)
+	{
+	  log_info ("2pc txn with status %d during cpt log", lt->lt_status);
+	  lt->lt_commit_flag_offset = 0;
+	}
     }
   END_DO_SET ();
 }
@@ -2209,7 +2221,7 @@ log_init (dbe_storage_t * dbs)
       int log_fd;
       file_set_rw (dbs->dbs_log_name);
       log_fd = fd_open (dbs->dbs_log_name, LOG_OPEN_FLAGS);
-      if (log_fd == -1)
+      if (log_fd < 0)
 	{
 	  log_error ("Can't open log : %m");
 	  call_exit (1);
@@ -2238,7 +2250,7 @@ log_init (dbe_storage_t * dbs)
 	  log_info ("Processing log segment %s", ls->ls_file);
 	  file_set_rw (ls->ls_file);
 	  log_fd = fd_open (ls->ls_file, LOG_OPEN_FLAGS);
-	  if (log_fd == -1)
+	  if (log_fd < 0)
 	    {
 	      log_error ("Can't open log segment : %m");
 	      call_exit (1);
