@@ -1,7 +1,7 @@
 /*
  *  neodisk.c
  *
- *  $Id: neodisk.c,v 1.19.2.14 2010/02/01 01:45:06 source Exp $
+ *  $Id: neodisk.c,v 1.19.2.23 2011/05/10 12:20:55 source Exp $
  *
  *  Neodisk Checkpoint
  *
@@ -137,9 +137,15 @@ cpt_rollback (int may_freeze)
 	case LT_BLOWN_OFF:
 	case LT_CLOSING:
 	case LT_COMMITTED:
+#ifdef VIRTTP
+	  if (stat == LT_COMMITTED && lt->lt_2pc._2pc_wait_commit)
+	    break;
+#endif
 	  rdbg_printf (("trx %lx killed by checkpoint while closing\n", lt));
 	  lt_kill_other_trx (lt, NULL, NULL, may_freeze);
-	  goto next;
+	  if (lt->lt_threads)
+	    goto next;
+	  break;
 	case LT_FREEZE:
 	  if (!lt->lt_close_ack_threads && lt->lt_threads && !lt->lt_vdb_threads && !lt->lt_lw_threads)
 	    {
@@ -183,6 +189,7 @@ cpt_rollback (int may_freeze)
 	  break;
 	case LT_1PC_PENDING:
 	case LT_2PC_PENDING:
+	case LT_PREPARE_PENDING:
 	  if (CPT_ATOMIC_PENDING == wi_inst.wi_is_checkpoint_pending)
 	    {
 	      TC (tc_atomic_wait_2pc);
@@ -228,6 +235,8 @@ lt_wait_checkpoint_1 (int cl_listener_also)
       LEAVE_TXN;
       semaphore_enter (self->thr_sem);
       IN_TXN;
+      if (self->thr_lt)
+	break;
     }
 }
 
@@ -247,6 +256,15 @@ cpt_over (void)
   cpt_thread = NULL;
   DO_SET (du_thread_t *, thr, &wi_inst.wi_waiting_checkpoint)
   {
+    if (thr->thr_lt)
+      {
+	lock_trx_t * lt = thr->thr_lt;
+	if (LT_FREEZE == lt->lt_status)
+	  {
+	    lt->lt_close_ack_threads = 0;
+	    lt->lt_status = LT_PENDING;
+	  }
+      }
     semaphore_leave (thr->thr_sem);
   }
   END_DO_SET();
@@ -1073,7 +1091,7 @@ buf_unremap (buffer_desc_t * buf)
 
 
 void
-cpt_unremap_ram (int target)
+cpt_unremap_ram (int target, int * bufs_done_total)
 {
   /* take buffers that already happen to be in memory and unremap them, stop if quota met */
   DO_SET (index_tree_t *, it, &cpt_dbs->dbs_trees)
@@ -1087,6 +1105,7 @@ cpt_unremap_ram (int target)
 	      if (buf->bd_page != buf->bd_physical_page)
 		{
 		  buf_unremap (buf);
+		  *bufs_done_total += 1;
 		  if (cpt_dbs->dbs_cpt_remap->ht_count <= target)
 		    {
 		      mutex_leave (&it->it_maps[inx].itm_mtx);
@@ -1104,7 +1123,7 @@ cpt_unremap_ram (int target)
 
 
 void
-cpt_em_unremap_read (extent_map_t * em, index_tree_t * it, int * bufs_done, int buf_quota, dk_set_t * bufs_list)
+cpt_em_unremap_read (extent_map_t * em, index_tree_t * it, int * bufs_done, int buf_quota, dk_set_t * bufs_list, int *bufs_done_total)
 {
   DO_EXT (ext, em)
     {
@@ -1116,6 +1135,7 @@ cpt_em_unremap_read (extent_map_t * em, index_tree_t * it, int * bufs_done, int
 	continue;
       dk_set_push (bufs_list, (void*) bufs);
       *bufs_done += BOX_ELEMENTS (bufs);
+      *bufs_done_total += BOX_ELEMENTS (bufs);
       if (*bufs_done > buf_quota)
 	goto enough_read;
     }
@@ -1156,12 +1176,12 @@ cpt_unremap_bufs (it_cursor_t * itc, dk_set_t * bufs_list)
 
 
 void
-em_unremap (index_tree_t * it, it_cursor_t * itc, extent_map_t * em, int * bufs_done, int buf_quota, int target, dk_set_t * bufs_list)
+em_unremap (index_tree_t * it, it_cursor_t * itc, extent_map_t * em, int * bufs_done, int buf_quota, int target, dk_set_t * bufs_list, int *bufs_done_total)
 {
   for (;;)
     {
       int l1 = dk_set_length (*bufs_list);
-      cpt_em_unremap_read (em, it, bufs_done, buf_quota, bufs_list);
+      cpt_em_unremap_read (em, it, bufs_done, buf_quota, bufs_list, bufs_done_total);
       if (l1 == dk_set_length (*bufs_list))
 	break;
       if (*bufs_done > buf_quota)
@@ -1209,24 +1229,40 @@ cpt_place_buffers ()
 
 
 void dbs_cache_check (dbe_storage_t * dbs, int mode);
+long cpt_remap_free_logical_pages;
+long cpt_reamp_free_physical_pages;
+long cpt_reamp_free_pages;
 
 void
 cpt_unremap (dbe_storage_t * dbs, it_cursor_t * itc)
 {
-  int bufs_done = 0, buf_quota = main_bufs / 4, target;
+  int bufs_done = 0, bufs_done_total, buf_quota = main_bufs / 4, target;
   dk_set_t bufs_list = NULL;
   cpt_remap_reverse = hash_table_allocate (cpt_dbs->dbs_cpt_remap->ht_actual_size);
+  cpt_remap_free_logical_pages = cpt_reamp_free_physical_pages = cpt_reamp_free_pages = 0;
   DO_HT (ptrlong, log, ptrlong, phys, dbs->dbs_cpt_remap)
     {
+#ifndef NDEBUG
+      int pf, lf;
+      pf = dbs_is_free_page (dbs, phys);
+      lf = dbs_is_free_page (dbs, log);
+      if (pf && lf)
+	cpt_reamp_free_pages++;
+      else if (pf)
+	cpt_reamp_free_physical_pages++;
+      else if (lf)
+	cpt_remap_free_logical_pages++;
+#endif
       sethash ((void*)phys, cpt_remap_reverse, (void*)log);
     }
   END_DO_HT;
   target = MIN (dbs->dbs_max_cp_remaps, (dbs->dbs_cpt_remap->ht_count / 20) * 19);
  again:
-  cpt_unremap_ram (target);
+  bufs_done_total = 0;
+  cpt_unremap_ram (target, &bufs_done_total);
   if (cpt_dbs->dbs_cpt_remap->ht_count > target)
     {
-      em_unremap (dbs->dbs_cpt_tree, itc, dbs->dbs_extent_map, &bufs_done, buf_quota, target, &bufs_list);
+      em_unremap (dbs->dbs_cpt_tree, itc, dbs->dbs_extent_map, &bufs_done, buf_quota, target, &bufs_list, &bufs_done_total);
       cpt_unremap_bufs (itc, &bufs_list);
       bufs_done = 0;
       cpt_place_buffers ();
@@ -1234,7 +1270,7 @@ cpt_unremap (dbe_storage_t * dbs, it_cursor_t * itc)
 	{
 	  if (it->it_extent_map != dbs->dbs_extent_map && it->it_extent_map)
 	    {
-	      em_unremap (it, itc, it->it_extent_map, &bufs_done, buf_quota, target, &bufs_list);
+	      em_unremap (it, itc, it->it_extent_map, &bufs_done, buf_quota, target, &bufs_list, &bufs_done_total);
 	      if (cpt_dbs->dbs_cpt_remap->ht_count <= target)
 		break;
 	    }
@@ -1243,7 +1279,7 @@ cpt_unremap (dbe_storage_t * dbs, it_cursor_t * itc)
       cpt_unremap_bufs (itc, &bufs_list);
     }
   bp_flush_all ();
-  if (cpt_dbs->dbs_cpt_remap->ht_count > target)
+  if (bufs_done_total && cpt_dbs->dbs_cpt_remap->ht_count > target)
     goto again;
   iq_shutdown (IQ_STOP);
   /* verify all unremaps are written.  shutdown exityy with bufs in iq will lose the pages, very corrupt, often visible as bad parent linkand sometimes as just lost updates.  */
@@ -1399,10 +1435,12 @@ wi_write_dirty (void)
 
 
 void
-dbs_backup_check (dbe_storage_t * dbs)
+dbs_backup_check (dbe_storage_t * dbs, int flag)
 {
 #if 1
   int n;
+  if (flag != CPT_INC_RESET)
+    return;
   for (n = 0; n < dbs->dbs_n_pages; n++)
     {
       int fl;
@@ -1725,7 +1763,7 @@ dbs_cpt_backup (void)
 	continue;
       cpt_bkp = hash_table_allocate (101);
       cpt_log_fd = fd_open (dbs->dbs_cpt_file_name, OPEN_FLAGS);
-      ftruncate (cpt_log_fd, 0);
+      FTRUNCATE (cpt_log_fd, 0);
       LSEEK (cpt_log_fd, 0, SEEK_SET);
       tcpses_set_fd (ses->dks_session, cpt_log_fd);
       dbs->dbs_cpt_recov_ses = ses;
@@ -1872,7 +1910,7 @@ dbs_checkpoint (char *log_name, int shutdown)
   DO_SET (dbe_storage_t *, dbs, &wi_inst.wi_master_wd->wd_storage)
     {
       dbs_cache_check (dbs, IT_CHECK_ALL);
-      dbs_backup_check (dbs);
+      dbs_backup_check (dbs, shutdown);
     }
   END_DO_SET();
 
@@ -1970,7 +2008,7 @@ dbs_checkpoint (char *log_name, int shutdown)
   DO_SET (dbe_storage_t *, dbs, &wi_inst.wi_master_wd->wd_storage)
     {
       dbs_cache_check (dbs, IT_CHECK_POST);
-      dbs_backup_check (dbs);
+      dbs_backup_check (dbs, shutdown);
     }
   END_DO_SET();
   if (CPT_NORMAL == shutdown)
@@ -2056,8 +2094,8 @@ srv_global_lock (query_instance_t * qi, int flag)
       lt->lt_is_excl = 1;
       lt->lt_replicate = REPL_NO_LOG;
       IN_TXN;
-      cpt_rollback (LT_KILL_ROLLBACK);
       server_lock.sl_owner = THREAD_CURRENT_THREAD;
+      cpt_rollback (LT_KILL_ROLLBACK);
       lt_threads_set_inner (lt, 1);
       LEAVE_TXN;
       server_lock.sl_ac_save = lt->lt_client->cli_row_autocommit;
