@@ -1,7 +1,7 @@
 /*
  *  rdfinf.c
  *
- *  $Id: rdfinf.c,v 1.19.2.28 2011/03/25 12:20:03 source Exp $
+ *  $Id: rdfinf.c,v 1.19.2.33 2011/10/13 14:06:54 source Exp $
  *
  *  RDF Inference
  *
@@ -322,7 +322,7 @@ iri_ensure (caddr_t * qst, caddr_t name, int flag, caddr_t * err_ret)
 
 char * sas_1_text = "select S from DB.DBA.RDF_QUAD where G = ? and O = ? and P = ? option (quietcast)";
 char * sas_2_text = "select O from DB.DBA.RDF_QUAD where G = ? and S = ? and P = ? option (quietcast)";
-char * sas_tn_text = "select O from DB.DBA.RDF_QUAD where S = :0 and P = rdf_sas_iri () and G = :1 union all select S from DB.DBA.RDF_QUAD where O = :0 and P = rdf_sas_iri () and G = :1 option (quietcast, array)";
+char * sas_tn_text = "select O from DB.DBA.RDF_QUAD where S = :0 and P = rdf_sas_iri () and G = :1 and isiri_id (O) union all select S from DB.DBA.RDF_QUAD where O = :0 and P = rdf_sas_iri () and G = :1 option (quietcast, array)";
 char * sas_tn_no_graph_text = "select O from DB.DBA.RDF_QUAD where S = :0 and P = rdf_sas_iri () union all select S from DB.DBA.RDF_QUAD where O = :0 and P = rdf_sas_iri () option (quietcast, array)";
 char * tn_ifp_text =
   " select S from DB.DBA.RDF_QUAD "
@@ -445,20 +445,22 @@ hash_queue_get (id_hash_t * ht, query_instance_t * qi, int next, int last)
 }
 
 
-void rdf_sas_ensure (caddr_t * qst);
+void rdf_sas_ensure (caddr_t * qst, caddr_t * err_ret);
 
 void
 ri_same_as_iri (rdf_inf_pre_node_t * ri, query_instance_t * qi, caddr_t iri, query_t * qr)
 {
   caddr_t * qst = (caddr_t *)qi;
   ptrlong one = 1;
-  caddr_t err;
+  caddr_t err = NULL;
   local_cursor_t * lc;
   id_hash_t * reached = (id_hash_t *) QST_GET (qi, ri->ri_sas_reached);
   id_hash_t * out = (id_hash_t *) QST_GET (qi, ri->ri_sas_out);
   id_hash_t * follow = (id_hash_t *) QST_GET (qi, ri->ri_sas_follow);
   int ginx;
-  rdf_sas_ensure (qst);
+  rdf_sas_ensure (qst, &err);
+  if (err)
+    sqlr_resignal (err);
   DO_BOX (state_slot_t *, g_ssl, ginx, ri->ri_sas_g)
     {
       caddr_t g = qst_get (qst, g_ssl);
@@ -790,7 +792,6 @@ bif_ctx_arg (caddr_t * qst, state_slot_t ** args, int nth, char * name, int crea
   if (!place)
     {
       caddr_t n2 = box_copy (ctx_name);
-      NEW_VARZ (rdf_inf_ctx_t, c1);
       return ric_allocate (n2);
     }
   else
@@ -1269,7 +1270,7 @@ bif_rdf_owl_iri (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 
 
 void
-rdf_sas_ensure (caddr_t * qst)
+rdf_sas_ensure (caddr_t * qst, caddr_t * err_ret)
 {
 }
 
@@ -1277,7 +1278,7 @@ rdf_sas_ensure (caddr_t * qst)
 caddr_t
 bif_rdf_sas_iri (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
-  rdf_sas_ensure (qst);
+  rdf_sas_ensure (qst, err_ret);
   return box_copy_tree (same_as_iri);
 }
 
@@ -1341,7 +1342,7 @@ char * cl_rdf_init_srv =
 "   DB.DBA.RDF_MAKE_IID_OF_QNAME_SAFE (null); \n"
 "   DB.DBA.RDF_QNAME_OF_IID (null); \n"
 "   rdf_inf_const_init (); \n"
-"   select  count (*) into c from sys_rdf_schema where 0 = rdfs_load_schema (rs_name, rs_uri); \n"
+"   select count (*) into c from (select distinct s.RS_NAME from sys_rdf_schema s) sub where 0 = rdfs_load_schema (sub.RS_NAME); \n"
 "   JSO_LOAD_AND_PIN_SYS_GRAPH_RO  (); \n"
 "   commit work; \n"
 "   rdf_init_thread (0); \n"
@@ -2261,7 +2262,7 @@ sqlg_cl_bracket_outer (sqlo_t * so, data_source_t * first)
   sql_comp_t * sc = so->so_sc;
   outer_seq_end_node_t * ose;
   dk_hash_t * res = hash_table_allocate (11);
-  data_source_t * first1 = qn_next (first);
+  data_source_t * first1 = first ? qn_next (first) : NULL;
   data_source_t * org_first = first1;
   void * dp;
   SET_THR_ATTR (THREAD_CURRENT_THREAD, TA_SQLC_ASG_SET, res);
