@@ -1,7 +1,7 @@
 /*
  *  recovery.c
  *
- *  $Id: recovery.c,v 1.12.2.6 2009/09/22 09:35:36 source Exp $
+ *  $Id: recovery.c,v 1.12.2.10 2011/06/13 09:29:12 source Exp $
  *
  *  Backup & Recovery procedures
  *
@@ -476,7 +476,10 @@ key_is_recoverable (key_id_t key_id)
 void
 row_log (it_cursor_t * itc, buffer_desc_t * buf, int map_pos, dbe_key_t * row_key, row_delta_t * rd)
 {
+  union {
+  void * dummy;
   dtp_t temp[4096];
+  } temp_un;
   if (row_key->key_is_bitmap)
     {
       db_buf_t bm;
@@ -491,8 +494,8 @@ row_log (it_cursor_t * itc, buffer_desc_t * buf, int map_pos, dbe_key_t * row_ke
       pl_set_at_bit ((placeholder_t *) itc, bm, bm_len, bm_start, BITNO_MIN, 0);
       itc->itc_bp.bp_at_end = 0;
       do {
-	rd->rd_temp = temp;
-	rd->rd_temp_max = sizeof (temp);
+	rd->rd_temp = &(temp_un.temp[0]);
+	rd->rd_temp_max = sizeof (temp_un.temp);
 	page_row_bm (buf, map_pos, rd, RO_ROW, itc);
 	rd->rd_n_values--; /*no bitmap string */
 	log_insert (itc->itc_ltrx, rd, LOG_KEY_ONLY | INS_REPLACING);
@@ -503,8 +506,8 @@ row_log (it_cursor_t * itc, buffer_desc_t * buf, int map_pos, dbe_key_t * row_ke
     }
   else
     {
-	rd->rd_temp = temp;
-	rd->rd_temp_max = sizeof (temp);
+	rd->rd_temp = &(temp_un.temp[0]);
+	rd->rd_temp_max = sizeof (temp_un.temp);
       page_row (buf, map_pos, rd, RO_ROW);
       log_insert (itc->itc_ltrx, rd, LOG_KEY_ONLY | INS_REPLACING);
       log_rd_blobs (itc, rd);
@@ -829,7 +832,7 @@ backup_prepare (query_instance_t * qi, char * file)
        */
 
       file_set_rw (fname);
-      if ((fd = fd_open (fname, LOG_OPEN_FLAGS)) == -1)
+      if ((fd = fd_open (fname, LOG_OPEN_FLAGS)) < 0)
 	{
 	  char *errmsg = strerror(errno);
 
@@ -842,7 +845,7 @@ backup_prepare (query_instance_t * qi, char * file)
         }
     }
 
-  ftruncate (fd, 0);
+  FTRUNCATE (fd, 0);
   tcpses_set_fd (ses->dks_session, fd);
 
   log_info ("Backup to %s started", fname);
@@ -909,6 +912,7 @@ bif_backup_prepare (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   query_instance_t *qi = (query_instance_t *) qst;
   caddr_t file = bif_string_arg (qst, args, 0, "backup_prepare");
+  sec_check_dba (qi, "backup_prepare");
   backup_prepare (qi, file);
   return NULL;
 }
@@ -927,6 +931,7 @@ bif_backup_row (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   caddr_t row = bif_arg (qst, args, 0, "backup_row");
   dtp_t tag = DV_TYPE_OF (row);
   ITC_INIT (itc, qi->qi_space, qi->qi_trx);
+  sec_check_dba (qi, "backup_row");
   if (!lt->lt_backup)
     sqlr_new_error ("42000", "SR112", "Transaction not in backup mode");
   if (tag != DV_SHORT_CONT_STRING && tag != DV_LONG_CONT_STRING)
@@ -953,6 +958,7 @@ bif_backup_flush (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
   int rc;
   query_instance_t *qi = (query_instance_t *) qst;
   lock_trx_t *lt = qi->qi_trx;
+  sec_check_dba (qi, "backup_flush");
   if (!lt->lt_backup)
     sqlr_new_error ("42000", "SR114", "Transaction not in backup mode");
 
@@ -975,12 +981,14 @@ bif_backup_close (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
 {
   query_instance_t *qi = (query_instance_t *) qst;
   lock_trx_t *lt = qi->qi_trx;
+  sec_check_dba (qi, "backup_close");
   if (!lt->lt_backup)
     sqlr_new_error ("42000", "SR116", "Transaction not in backup mode");
 
   backup_close (qi->qi_trx);
   if (!srv_have_global_lock(THREAD_CURRENT_THREAD))
     LEAVE_CPT(qi->qi_trx);
+  log_info ("Backup finished.");
   return NULL;
 }
 
@@ -1224,6 +1232,44 @@ db_check (query_instance_t * qi)
   return 1;
 }
 
+static caddr_t
+bif_log_index (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args)
+{
+  query_instance_t *qi = (query_instance_t *) qst;
+  dbe_key_t * key = bif_key_arg (qst, args, 0, "log_index");
+  buffer_desc_t *buf;
+  it_cursor_t *itc;
+
+  sec_check_dba (qi, "backup_index");
+  memset (levels, 0, sizeof (levels));
+
+  itc = itc_create (NULL , qi->qi_trx);
+  itc_from (itc, key);
+  itc->itc_isolation = ISO_UNCOMMITTED;
+  ITC_FAIL (itc)
+    {
+      itc->itc_random_search = RANDOM_SEARCH_ON; /* do not use root image cache */
+      buf = itc_reset (itc);
+      itc->itc_random_search = RANDOM_SEARCH_OFF;
+      itc_try_land (itc, &buf);
+      /* the whole traversal is in landed (PA_WRITE() mode. page_transit_if_can will not allow mode change in transit */
+      if (!buf->bd_content_map)
+	{
+	  log_error ("Blog ref'referenced as index tree top node dp=%d key=%s\n", buf->bd_page, itc->itc_insert_key->key_name);
+	}
+      else
+	walk_dbtree (itc, &buf, 0, log_page, 0);
+      itc_page_leave (itc, buf);
+    }
+  ITC_FAILED
+    {
+      itc_free (itc);
+    }
+  END_FAIL (itc);
+  itc_free (itc);
+  return NULL;
+}
+
 void
 recovery_init (void)
 {
@@ -1231,6 +1277,7 @@ recovery_init (void)
   bif_define ("backup_row", bif_backup_row);
   bif_define ("backup_flush", bif_backup_flush);
   bif_define ("backup_close", bif_backup_close);
+  bif_define ("backup_index", bif_log_index);
 #if 0
   bif_define ("crash_recovery_log_check", bif_crash_recovery_log_check);
 #endif
