@@ -1,7 +1,7 @@
 /*
  *  rltrx.c
  *
- *  $Id: rltrx.c,v 1.18.2.19 2011/03/14 15:47:01 source Exp $
+ *  $Id: rltrx.c,v 1.18.2.21 2011/10/19 15:14:10 source Exp $
  *
  *  Locking concurrency control
  *
@@ -1491,6 +1491,8 @@ lt_transact (lock_trx_t * lt, int op)
       rdbg_printf (("Host %d:  Own rollback with lte=%d of %d:%d\n", local_cll.cll_this_host, lt->lt_error, LT_W_NO (lt)));
     }
   ASSERT_IN_TXN;
+  if (lt != wi_inst.wi_cpt_lt && wi_inst.wi_checkpoint_atomic)
+    log_error ("transact while cpt atomic, trx no %d, cpt trx no %d", lt->lt_trx_no, (wi_inst.wi_cpt_lt ? wi_inst.wi_cpt_lt->lt_trx_no : 0));
   if (lt->lt_threads != lt->lt_lw_threads + lt->lt_close_ack_threads + lt->lt_vdb_threads)
     {
       lt_log_debug ((
@@ -1519,9 +1521,9 @@ lt_transact (lock_trx_t * lt, int op)
       char user[16];
       char peer[32];
       dks_client_ip (lt->lt_client, from, user, peer, sizeof (from), sizeof (user), sizeof (peer));
-      log_info ("LTRS_1 %s %s %s %s transact %p %li", user, from, peer,
+      log_info ("LTRS_1 %s %s %s %s transact %p %d", user, from, peer,
 	  op == SQL_COMMIT ? "Commit" : "Rollback", lt,
-          lt->lt_client ? lt->lt_client->cli_autocommit : 0);
+          (int) (lt->lt_client ? lt->lt_client->cli_autocommit : 0));
     }
   lt->lt_status = LT_CLOSING;
 #ifdef PAGE_TRACE
@@ -1543,6 +1545,8 @@ lt_transact (lock_trx_t * lt, int op)
 	    pl_finalize_page (pl, itc);
 	  else
 	    pl_rollback_page (pl, itc);
+	  if (lt != wi_inst.wi_cpt_lt && wi_inst.wi_checkpoint_atomic)
+	    log_error ("transact while cpt atomic, trx no: %d, n locks %d", lt->lt_trx_no, n_locks);
 	  ITC_LEAVE_MAPS (itc);
 	}
       if (pl_arr != (page_lock_t**) &pl_arr_auto)
@@ -1581,6 +1585,8 @@ lt_transact (lock_trx_t * lt, int op)
   LT_CLOSE_ACK_THREADS(lt);
   lt->lt_close_ack_threads = 0;
   lt_resume_waiting_end (lt);
+  if (wi_inst.wi_cpt_lt != lt)
+    lt_wait_checkpoint ();
 #ifdef VIRTTP
   if (lt->lt_2pc._2pc_info)
     {
