@@ -1,7 +1,7 @@
 /*
  *  search.c
  *
- *  $Id: search.c,v 1.35.2.12 2010/02/01 12:53:44 source Exp $
+ *  $Id: search.c,v 1.35.2.34 2011/06/20 14:15:43 source Exp $
  *
  *  Search
  *
@@ -528,13 +528,15 @@ dv_compare (db_buf_t dv1, db_buf_t dv2, collation_t *collation, unsigned short o
   dtp_t dv1_flags = 0, dv2_flags = 0;
   if (DV_BOX_FLAGS == dtp1)
     {
-      dv1_flags = dv1[1];
-      dv1 += 2;
+      dv1_flags = dv1[4];
+      dv1 += 5;
+      dtp1 = *dv1;
     }
   if (DV_BOX_FLAGS == dtp2)
     {
-      dv2_flags = dv2[1];
-      dv2 += 2;
+      dv2_flags = dv2[4];
+      dv2 += 5;
+      dtp2 = *dv2;
     }
   if (dtp1 == dtp2)
     {
@@ -713,7 +715,7 @@ dv_compare (db_buf_t dv1, db_buf_t dv2, collation_t *collation, unsigned short o
 	break;
 
       case DV_IRI_ID:
-	ln1 = (iri_id_t) (unsigned long) LONG_REF_NA (dv1 + 1) + offset;
+	ln1 = (iri_id_t) (unsigned int32) LONG_REF_NA (dv1 + 1) + offset;
 	break;
       case DV_IRI_ID_8:
 	dtp1 = DV_IRI_ID;
@@ -1256,7 +1258,7 @@ itc_row_check (it_cursor_t * itc, buffer_desc_t * buf)
 	      else
 		{
 		  dbe_column_t * col = sch_id_to_column (wi_inst.wi_schema, sp->sp_cl.cl_col_id);
-		  if (col && col->col_default)
+		  if (col)
 		    {
 		      if (DVC_CMP_MASK & op)
 			{
@@ -1319,6 +1321,8 @@ itc_row_check (it_cursor_t * itc, buffer_desc_t * buf)
 	  out_map_t * om = ITC_OUT_MAP (itc);
 	  DO_SET (state_slot_t *, ssl, &ks->ks_out_slots)
 	    {
+	      if (ssl->ssl_type == SSL_CONSTANT)
+		continue;
 	      if (om[inx].om_is_null)
 		{
 		  if (OM_NULL == om[inx].om_is_null)
@@ -1334,7 +1338,7 @@ itc_row_check (it_cursor_t * itc, buffer_desc_t * buf)
 		      if (!cl)
 			{
 			  dbe_column_t * col = sch_id_to_column (wi_inst.wi_schema, om[inx].om_cl.cl_col_id);
-			  if (col && col->col_default)
+			  if (col)
 			    qst_set (itc->itc_out_state, ssl, box_copy_tree (col->col_default));
 			  else
 			    qst_set_bin_string (itc->itc_out_state, ssl, (db_buf_t) "", 0, DV_DB_NULL);
@@ -1368,6 +1372,105 @@ itc_row_check (it_cursor_t * itc, buffer_desc_t * buf)
   return DVC_MATCH;
 }
 
+int
+itc_sample_row_check (it_cursor_t * itc, buffer_desc_t * buf)
+{
+  dbe_key_t *row_key = NULL;
+  /* Check the key id's and non-key columns. */
+  search_spec_t *sp;
+  dbe_key_t *key = itc->itc_insert_key;
+
+#if 0
+  if (itc->itc_insert_key && itc->itc_insert_key->key_is_bitmap && !itc->itc_no_bitmap)
+    return DVC_LESS;
+#endif
+
+  if (IE_KEY_VERSION (itc->itc_row_data) == itc->itc_insert_key->key_version)
+    itc->itc_row_key = itc->itc_insert_key;
+  else
+    {
+      ITC_REAL_ROW_KEY (itc);
+      if (!sch_is_subkey (isp_schema (NULL), itc->itc_row_key->key_id, itc->itc_insert_key->key_id))
+	return DVC_LESS;	/* Key specified but this ain't it */
+      row_key = itc->itc_row_key;
+    }
+  sp = itc->itc_row_specs;
+  if (sp)
+    {
+      do
+	{
+	  int op = sp->sp_min_op;
+	  search_spec_t sp_auto;
+
+	  if (row_key)
+	    {
+	      dbe_col_loc_t *cl = key_find_cl (row_key, sp->sp_cl.cl_col_id);
+	      if (cl)
+		{
+		  memcpy (&sp_auto, sp, sizeof (search_spec_t));
+		  sp = &sp_auto;
+		  sp->sp_cl = *cl;
+		}
+	      else
+		{
+		  dbe_column_t * col = sch_id_to_column (wi_inst.wi_schema, sp->sp_cl.cl_col_id);
+		  if (col)
+		    {
+		      if (DVC_CMP_MASK & op)
+			{
+			  if (0 == (op & cmp_boxes (col->col_default, itc->itc_search_params[sp->sp_min], sp->sp_collation, sp->sp_collation)))
+		            return DVC_LESS;
+			}
+		      else if (op == CMP_LIKE)
+			{
+			  caddr_t v = itc->itc_search_params[sp->sp_min];
+			  int st = LIKE_ARG_CHAR, pt = LIKE_ARG_CHAR;
+			  dtp_t rtype = DV_TYPE_OF (v);
+			  dtp_t ltype = DV_TYPE_OF (col->col_default);
+			  if (DV_WIDE == rtype || DV_LONG_WIDE == rtype)
+			    pt = LIKE_ARG_WCHAR;
+			  if (DV_WIDE == ltype || DV_LONG_WIDE == ltype)
+			    st = LIKE_ARG_WCHAR;
+			  if (DVC_MATCH != cmp_like (col->col_default, v, sp->sp_collation, sp->sp_like_escape, st, pt))
+			    return DVC_LESS;
+			}
+		      if (sp->sp_max_op != CMP_NONE
+			  && (0 == (sp->sp_max_op & cmp_boxes (col->col_default, itc->itc_search_params[sp->sp_max],
+				sp->sp_collation, sp->sp_collation))))
+			return DVC_LESS;
+		      goto next_sp;
+		    }
+		  return DVC_LESS;
+		}
+	    }
+
+	  if (ITC_NULL_CK (itc, sp->sp_cl))
+	    return DVC_LESS;
+	  if (DVC_CMP_MASK & op)
+	    {
+	      int res = page_col_cmp_1 (buf, itc->itc_row_data, &sp->sp_cl, itc->itc_search_params[sp->sp_min]);
+	      if (0 == (op & res) || (DVC_NOORDER & res))
+		return DVC_LESS;
+	    }
+	  else if (op == CMP_LIKE)
+	    {
+	      if (DVC_MATCH != itc_like_compare (itc, buf, itc->itc_search_params[sp->sp_min], sp))
+		return DVC_LESS;
+	      goto next_sp;
+	    }
+	  if (sp->sp_max_op != CMP_NONE)
+	    {
+	      int res = page_col_cmp_1 (buf, itc->itc_row_data, &sp->sp_cl, itc->itc_search_params[sp->sp_max]);
+	      if (0 == (sp->sp_max_op & res) || (DVC_NOORDER & res))
+		return DVC_LESS;
+	    }
+	next_sp:
+	  sp = sp->sp_next;
+	} while (sp);
+    }
+  return DVC_MATCH;
+}
+
 
 int
 itc_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
@@ -1748,7 +1851,7 @@ itc_hash_page_search (it_cursor_t * itc, buffer_desc_t ** buf_ret)
     {
       key_ver_t kv;
       if (ITC_AT_END == itc->itc_map_pos)
-	return DVC_INDEX_END; /* the next in itc_next can move this to end.  Must not ref the kv, else can get by accidentr a 1 and think it's a row */
+	return DVC_INDEX_END; /* the next in itc_next can move this to end.  Must not ref the kv, else can get by accident a 1 and think it's a row */
       row = page + itc->itc_map_pos;
       kv = IE_KEY_VERSION (row);
       if (kv!= 1)
@@ -1909,7 +2012,7 @@ itc_page_search (it_cursor_t * it, buffer_desc_t ** buf_ret, dp_addr_t * leaf_re
 	    }
 	  if (res == DVC_LESS)
 	    {
-		  /*  column is too small.  If there is a leaf, go ther else search at end */
+		  /*  column is too small.  If there is a leaf, go there else search at end */
 	      if (ITC_NULL_CK(it, sp->sp_cl))
 		{
 		  if (!leaf)
@@ -2035,6 +2138,13 @@ pg_key_compare (buffer_desc_t * buf, int pos, it_cursor_t * it)
   return DVC_MATCH;
 }
 
+#if defined(__GNUC__) && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 10))
+#define PREFETCH \
+  guess = (at_or_above + below) / 2; \
+  __builtin_prefetch (page + map->pm_entries[guess])
+#else
+#define PREFETCH
+#endif
 
 int
 itc_page_split_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
@@ -2051,6 +2161,7 @@ itc_page_split_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
   int guess;
   int at_or_above_res = -100;
   key_ver_t kv;
+  PREFETCH;
   if (it->itc_dive_mode == PA_READ ? buf->bd_is_write : !buf->bd_is_write)
     GPF_T1 ("split search supposed to be in read mode");
   if (map->pm_count == 0)
@@ -2111,6 +2222,7 @@ itc_page_split_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
 	{
 	case DVC_LESS:
 	  at_or_above = guess;
+	  PREFETCH;
 	  at_or_above_res = res;
 	  break;
 	case DVC_MATCH:	/* row found, dependent not checked */
@@ -2123,10 +2235,12 @@ itc_page_split_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
 	    {
 	      below = guess;
 	    }
+	  PREFETCH;
 	  break;
 
 	case DVC_GREATER:
 	  below = guess;
+	  PREFETCH;
 	  break;
 	default: GPF_T1 ("key_cmp_t can't return that");
 	}
@@ -2195,6 +2309,7 @@ itc_page_insert_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
       it->itc_map_pos = ITC_AT_END;
       return DVC_GREATER;
     }
+  PREFETCH;
   for (;;)
     {
       if ((below - at_or_above) <= 1)
@@ -2248,6 +2363,7 @@ itc_page_insert_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
 	{
 	case DVC_LESS:
 	  at_or_above = guess;
+	  PREFETCH;
 	  at_or_above_res = res;
 	  break;
 	case DVC_MATCH:	/* row found, dependent not checked */
@@ -2269,6 +2385,7 @@ itc_page_insert_search (it_cursor_t * it, buffer_desc_t ** buf_ret)
 	  return res;
 	case DVC_GREATER:
 	  below = guess;
+	  PREFETCH;
 	  break;
 	    default: GPF_T1 ("can't have this res for key_cmp_t");
 	}
@@ -2486,6 +2603,7 @@ itc_read_ahead1 (it_cursor_t * itc, buffer_desc_t ** buf_ret)
 
 
 long tc_read_aside;
+int enable_iq_always = 0;
 
 
 void
@@ -2493,7 +2611,7 @@ itc_read_ahead_blob (it_cursor_t * itc, ra_req_t *ra, int flags)
 {
   int inx;
   buffer_pool_t * action_bp = NULL;
-  if (!itc || !ra || ra->ra_fill < 2)
+  if (!itc || !ra || (ra->ra_fill < 2 && !enable_iq_always))
     goto fin;
 
   if (!iq_is_on ())
@@ -2586,7 +2704,9 @@ itc_read_ahead_blob (it_cursor_t * itc, ra_req_t *ra, int flags)
       ra_count++;
       ra_pages += ra->ra_bfill;
       if (ra->ra_nsiblings > 1)
-	dbg_printf (("RA %d sibling %d pages %d leaves\n", ra->ra_nsiblings, ra->ra_bfill, ra->ra_fill));
+        {
+	  dbg_printf (("RA %d sibling %d pages %d leaves\n", ra->ra_nsiblings, ra->ra_bfill, ra->ra_fill));
+        }
       iq_schedule (ra->ra_bufs, ra->ra_bfill);
     }
   ITC_LEAVE_MAPS (itc);
@@ -2605,46 +2725,54 @@ itc_read_ahead (it_cursor_t * itc, buffer_desc_t ** buf_ret)
 }
 
 int enable_read_aside = 1;
-int em_ra_window = 1000;
-int em_ra_threshold = 2;
+int32 em_ra_window = 1000;
+int32 em_ra_threshold = 2;
+int32 em_ra_startup_window = 40000;
+int32 em_ra_startup_threshold = 0;
 
-ra_req_t *
-itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
+
+int
+em_trigger_ra (extent_map_t * em, dp_addr_t ext_dp, uint32 now, int window, int threshold)
 {
-  /* mark the read history of the extent.  If more than so many inside a short time, do the whole extent
-  * the buf is the parent of the dp. */
-  extent_map_t * em = itc->itc_tree->it_extent_map;
-  extent_t * ext;
-  int inx, fill = 0;
-  dp_addr_t leaves[EXTENT_SZ];
-  dp_addr_t ext_dp = EXT_ROUND (dp);
-  uint32 now, rh;
-  ra_req_t *ra=NULL;
-  if (!enable_read_aside || em == em->em_dbs->dbs_extent_map)
-    return NULL; /* do not apply to the sys ext map */
-  now = get_msec_real_time ();
+  ptrlong rh;
+  if (main_bufs < 10000)
+    return 0;
+  if (em == em->em_dbs->dbs_extent_map)
+    return 0;
+  if (0 == threshold)
+    return 1;
   mutex_enter (&em->em_read_history_mtx);
   rh = (ptrlong) gethash (DP_ADDR2VOID(ext_dp), em->em_read_history);
   if (!rh)
     {
       sethash (DP_ADDR2VOID(ext_dp), em->em_read_history, (void*)(ptrlong)(now << 8));
       mutex_leave (&em->em_read_history_mtx);
-      return NULL;
+      return 0;
     }
-  if (now - (rh >> 8) > em_ra_window)
+  if (now - (rh >> 8) > window)
     {
       sethash (DP_ADDR2VOID(ext_dp), em->em_read_history, (void*)(ptrlong)(now << 8));
       mutex_leave (&em->em_read_history_mtx);
-      return NULL;
+      return 0;
     }
   if ((rh & 0xff) < em_ra_threshold)
     {
       sethash (DP_ADDR2VOID(ext_dp), em->em_read_history, (void*)(ptrlong)(rh + 1));
       mutex_leave (&em->em_read_history_mtx);
-      return NULL;
+      return 0;
     }
   remhash (DP_ADDR2VOID(ext_dp), em->em_read_history);
   mutex_leave (&em->em_read_history_mtx);
+  return 1;
+}
+
+int
+em_ext_ra_pages (extent_map_t * em, it_cursor_t * itc, dp_addr_t ext_dp, dp_addr_t * leaves, int max, dp_addr_t except_dp, dk_hash_t * except)
+{
+  extent_t * ext;
+  int fill = 0, inx;
+  if (max <= 0)
+    return 0;
   mutex_enter (em->em_mtx);
   ext = EM_DP_TO_EXT (em, ext_dp);
   if (!ext)
@@ -2653,9 +2781,9 @@ itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
       mutex_enter (em->em_dbs->dbs_extent_map->em_mtx);
       ext = EM_DP_TO_EXT (em->em_dbs->dbs_extent_map, ext_dp);
       if (!ext)
-	log_error ("in read aside, ext for dp %d not in the ext mapand not in sys ext map ", ext_dp);
+	log_error ("in read aside, ext for dp %d not in the ext map and not in sys ext map ", ext_dp);
       mutex_leave (em->em_dbs->dbs_extent_map->em_mtx);
-      return NULL;
+      return 0;
     }
 
   mutex_leave (em->em_mtx);
@@ -2667,7 +2795,9 @@ itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
 	{
 	  dp_addr_t other_dp = ext_dp + (BITS_IN_LONG * inx) + b_idx, phys_dp;
 	  int is_allocd = 0;
-	  if (dp == other_dp)
+	  if (except_dp == other_dp)
+	    continue;
+	  if (except && gethash (DP_ADDR2VOID (other_dp), except))
 	    continue;
 	  mutex_enter (em->em_mtx);
 	  if (0 == (ext->ext_pages[inx] & (1 << b_idx)))
@@ -2688,11 +2818,48 @@ itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
 	  if (phys_dp != other_dp)
 	    continue;
 	  leaves[fill++] = other_dp;
+	  if (fill >= max)
+	    return fill;
 	}
     }
+  return fill;
+}
+
+extern long tc_new_page;
+
+ra_req_t *
+itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
+{
+  /* mark the read history of the extent.  If more than so many inside a short time, do the whole extent
+  * the buf is the parent of the dp. */
+  int window, threshold;
+  extent_map_t * em = itc->itc_tree->it_extent_map;
+  int fill = 0;
+  dp_addr_t leaves[EXTENT_SZ];
+  dp_addr_t ext_dp = EXT_ROUND (dp);
+  uint32 now;
+  ra_req_t *ra=NULL;
+  if (disk_reads + tc_new_page < main_bufs)
+    {
+      threshold = em_ra_startup_threshold;
+      window = em_ra_startup_window;
+	}
+  else
+    {
+      threshold = em_ra_threshold;
+      window = em_ra_window;
+    }
+  if (!enable_read_aside || em == em->em_dbs->dbs_extent_map)
+    return NULL; /* do not apply to the sys ext map */
+  now = get_msec_real_time ();
+  if (!em_trigger_ra (em, ext_dp, now, window, threshold))
+    return NULL;
+  fill = em_ext_ra_pages (em, itc, ext_dp, leaves, EXTENT_SZ, dp, NULL);
   if (!fill)
     return NULL;
   tc_read_aside += fill;
+  if (itc->itc_ltrx)
+    itc->itc_ltrx->lt_client->cli_activity.da_spec_disk_reads += fill;
   ra= (ra_req_t *) dk_alloc_box(sizeof(ra_req_t),DV_CUSTOM);
   memset (ra, 0, sizeof (*ra));
   memcpy (&ra->ra_dp, leaves, fill * sizeof (dp_addr_t));
@@ -2704,6 +2871,7 @@ itc_read_aside (it_cursor_t * itc, buffer_desc_t * buf, dp_addr_t dp)
 void
 dbs_timeout_read_history (dbe_storage_t * dbs)
 {
+  int window = disk_reads < main_bufs ? em_ra_startup_window : em_ra_window;
   int now = approx_msec_real_time (), inx, nth;
   if (wi_inst.wi_checkpoint_atomic)
     return;
@@ -2724,7 +2892,7 @@ dbs_timeout_read_history (dbe_storage_t * dbs)
       mutex_enter (&em->em_read_history_mtx);
       DO_HT (void*, dp, ptrlong, rh, em->em_read_history)
 	{
-      if (now - (rh >> 8)  > 4 * em_ra_window)
+      if (now - (rh >> 8)  > 4 * window)
 	{
 	  pages[fill++] = (dp_addr_t)((ptrlong)(dp));
 	  if (fill >= 100)
@@ -2845,24 +3013,43 @@ itc_col_stat_free (it_cursor_t * itc, int upd_col, float est)
 void
 itc_row_col_stat (it_cursor_t * itc, buffer_desc_t * buf)
 {
+  int n_data = 1;
   db_buf_t row = BUF_ROW (buf, itc->itc_map_pos);
+  dbe_key_t * key = itc->itc_insert_key;
   key_ver_t kv = IE_KEY_VERSION (row);
+  int len_limit = -1, first_match = 0;
   if (!kv ||  KV_LEFT_DUMMY == kv)
     return;
-  itc->itc_st.n_sample_rows++;
   itc->itc_row_data = row;
   itc->itc_row_key = itc->itc_insert_key->key_versions[kv];
   DO_SET (dbe_column_t *, col, &itc->itc_row_key->key_parts)
     {
       col_stat_t * col_stat;
+      caddr_t * data_col = NULL;
       caddr_t data = NULL;
       dbe_column_t * current_col;
       int len, is_data = 0;
       ptrlong * place;
-      dbe_col_loc_t *cl  = key_find_cl (itc->itc_row_key, col->col_id);
+      dbe_col_loc_t *cl;
+      if (key->key_is_col)
+	cl = cl_list_find (key->key_row_var, col->col_id);
+      else
+	cl = key_find_cl (itc->itc_row_key, col->col_id);
       if (!IS_BLOB_DTP (col->col_sqt.sqt_dtp))
 	{
+	  if (key->key_bit_cl && col->col_id == key->key_bit_cl->cl_col_id)
+	    {
+	      data_col = itc_bm_array (itc, buf);
+	    }
+	  else
+	    {
 	  data = itc_box_column (itc, buf, col->col_id, cl);
+	      if (col == (dbe_column_t*)key->key_parts->data && 1 == itc->itc_search_par_fill && !box_equal (data, itc->itc_search_params[0]))
+		{
+		  dk_free_tree (data);
+		return;
+	    }
+	    }
 	  is_data = 1;
 	}
       current_col = sch_id_to_column (wi_inst.wi_schema, col->col_id);
@@ -2873,12 +3060,26 @@ itc_row_col_stat (it_cursor_t * itc, buffer_desc_t * buf)
 	  NEW_VARZ (col_stat_t, cs);
 	  sethash ((void*)current_col, itc->itc_st.cols, (void*) cs);
 	  cs->cs_distinct = id_hash_allocate (1001, sizeof (caddr_t), sizeof (caddr_t), treehash, treehashcmp);
+          id_hash_set_rehash_pct (cs->cs_distinct, 200);
 	  col_stat = cs;
 	}
 
       if (is_data)
 	{
-	  if (cl->cl_fixed_len > 0)
+	  int data_inx;
+	  n_data = data_col ? BOX_ELEMENTS (data_col) : 1;
+	  if (-1 != len_limit)
+	    n_data = MIN (BOX_ELEMENTS (data_col), len_limit);
+	  for (data_inx = first_match; data_inx < n_data; data_inx++)
+	    {
+	      if (data_col)
+		{
+		  data = data_col[data_inx];
+		  data_col[data_inx] = NULL;
+		}
+	      if (key->key_is_col && dtp_is_fixed (col->col_sqt.sqt_dtp))
+		len = 4;
+	      else if (cl->cl_fixed_len > 0)
 	    len = cl->cl_fixed_len;
 	  else
 	    len = IS_BOX_POINTER (data) ? box_length (data) : 8;
@@ -2900,10 +3101,51 @@ itc_row_col_stat (it_cursor_t * itc, buffer_desc_t * buf)
 	      id_hash_set (col_stat->cs_distinct, (caddr_t) &data, (caddr_t)&one);
 	    }
 	}
+	  if (data_col)
+	    {
+	      dk_free_tree ((caddr_t)data_col);
+	      data_col = NULL;
+	    }
+	}
     }
   END_DO_SET();
+  itc->itc_st.n_sample_rows += n_data;
 }
 
+
+void
+itc_n_p_matches_in_col (it_cursor_t * itc, caddr_t * data_col, int * first, int *last)
+{
+  /* in column wise inx sample can hit a seg with many different values of the 1st key part.  If so, count how many there are. */
+  caddr_t param = itc->itc_search_params[0];
+  int len = BOX_ELEMENTS (data_col);
+  int inx = 0;
+  for (inx = 0; inx < len; inx++)
+    {
+      if (box_equal (param, data_col[inx]))
+	{
+	  *first = inx;
+	  break;
+	}
+    }
+  if (inx == len)
+    {
+      *last = *first = 0;
+      return;
+    }
+  for (inx = len - 1; inx >= 0; inx--)
+	{
+	  if (box_equal (param, data_col[inx]))
+	    {
+	      *last = inx + 1;
+	      break;
+	    }
+	  if (inx == -1)
+	    *first = *last = 0;
+}
+}
+
+
 void
 itc_page_col_stat (it_cursor_t * itc, buffer_desc_t * buf)
 {
@@ -3017,7 +3259,7 @@ itc_random_leaf (it_cursor_t * itc, buffer_desc_t *buf, dp_addr_t * leaf_ret)
     nth = sqlbif_rnd (&inx_rnd_seed) % pm->pm_count;
   else
     {
-      log_error ("itc>_sample: should not get pages with 0 entries in pm");
+      log_error ("itc_sample: should not get pages with 0 entries in pm");
       return DVC_INDEX_END;
     }
   row = page + pm->pm_entries[nth];
@@ -3045,7 +3287,7 @@ itc_matches_on_page (it_cursor_t * itc, buffer_desc_t * buf, int * leaf_ctr_ret,
   int have_left_leaf = 0, was_left_leaf = 0;
   int pos = itc->itc_map_pos; /* itc is at leftmost match. Nothing at left of the itc */
   int save_pos = itc->itc_map_pos;
-  int ctr = 0, leaf_ctr = 0, row_ctr = 0;
+  int ctr = 0, leaf_ctr = 0, row_ctr = 0, row_match_ctr = 0;
   *ends_with_match = 0;
   for (pos = pos; pos < pm->pm_count; pos++)
     {
@@ -3073,6 +3315,9 @@ itc_matches_on_page (it_cursor_t * itc, buffer_desc_t * buf, int * leaf_ctr_ret,
 		break;
 	      sp = sp->sp_next;
 	    }
+	  if (r_kv && DVC_MATCH == res && itc->itc_row_specs && DVC_MATCH == itc_sample_row_check (itc, buf)) /* check dependant cols */
+	    row_match_ctr ++;
+
 	  if (DVC_MATCH == res)
 	    {
 	      if (r_kv)
@@ -3117,6 +3362,14 @@ itc_matches_on_page (it_cursor_t * itc, buffer_desc_t * buf, int * leaf_ctr_ret,
       int nth = (leaf_ctr * angle) / 1000;
       *alt_leaf_ret = leaves[MIN (nth, leaf_ctr - 1)];
     }
+  /* adjust estimate */
+  if (itc->itc_row_specs && row_ctr)
+    {
+      if (row_match_ctr)
+	ctr = (row_match_ctr * ctr) / row_ctr;
+      else
+	ctr = (int) ((float)ctr * MAX (0.05, ((float) row_ctr / ctr)));
+    }
   return ctr;
 }
 
@@ -3174,8 +3427,7 @@ itc_sample_1 (it_cursor_t * it, buffer_desc_t ** buf_ret, int64 * n_leaves_ret,
 	level_of_single_leaf_match = level;
     }
   if (leaf_estimate)
-    leaf_estimate = (((float)leaf_estimate) - 0.5) * (*buf_ret)->bd_content_map->pm_count * rows_per_bm
-+ leaf_ctr;
+    leaf_estimate = (((float)leaf_estimate) - 0.5) * (*buf_ret)->bd_content_map->pm_count * rows_per_bm + leaf_ctr;
   else if (leaf_ctr > 1)
     leaf_estimate = leaf_ctr - 1;
   if (rnd_leaf)
@@ -3279,6 +3531,63 @@ samples_stddev (int64 * samples, int n_samples, float * mean_ret, float * stddev
   *mean_ret = mean;
 }
 
+int32 enable_p_stat = 1;
+
+int
+itc_sample_is_rdf_p (it_cursor_t * itc)
+{
+  return enable_p_stat && strstr (itc->itc_insert_key->key_table->tb_name, "RDF_QUAD")
+    &&  0 == strcmp  (((dbe_column_t*)itc->itc_insert_key->key_parts->data)->col_name, "P")
+    && itc->itc_key_spec.ksp_spec_array && !itc->itc_key_spec.ksp_spec_array->sp_next
+    && CMP_EQ == itc->itc_key_spec.ksp_spec_array->sp_min_op;
+}
+
+
+
+
+void
+itc_record_rdf_p (it_cursor_t * itc, int64 est)
+{
+  /* store the stats of the sog fpr the given p. */
+  dbe_key_t * key = itc->itc_insert_key;
+  float distincts[4];
+  iri_id_t p = unbox_iri_id (itc->itc_search_params[0]);
+  int fill = 1;
+  col_stat_t * cs;
+  mutex_enter (alt_ts_mtx); /*any mtx that is never enterd, not worth one of its own */
+  if (!key->key_p_stat)
+    {
+      key->key_p_stat = id_hash_allocate (201, sizeof (iri_id_t), 4 * sizeof (float), boxint_hash, boxint_hashcmp);
+      id_hash_set_rehash_pct (key->key_p_stat, 200);
+    }
+  distincts[0] = est;
+  DO_SET (dbe_column_t *, col, &key->key_parts->next)
+    {
+      cs = gethash ((void*)col, itc->itc_st.cols);
+      distincts[fill++] = (float)cs->cs_distinct->ht_count * (float)est / itc->itc_st.n_sample_rows;
+      DO_IDHASH (caddr_t, k, caddr_t, ign,  cs->cs_distinct)
+	dk_free_box (k);
+      END_DO_IDHASH;
+      id_hash_free (cs->cs_distinct);
+      dk_free ((caddr_t)cs, sizeof (col_stat_t));
+      if (fill >= 4)
+	break;
+    }
+  END_DO_SET();
+  /* free stats on P */
+  cs = gethash (key->key_parts->data, itc->itc_st.cols);
+  DO_IDHASH (caddr_t, k, caddr_t, ign,  cs->cs_distinct)
+      dk_free_box (k);
+  END_DO_IDHASH;
+  id_hash_free (cs->cs_distinct);
+  dk_free ((caddr_t)cs, sizeof (col_stat_t));
+
+  hash_table_free (itc->itc_st.cols);
+  itc->itc_st.cols = NULL;
+  id_hash_set (key->key_p_stat, (caddr_t)&p, (caddr_t)&distincts);
+  mutex_leave (alt_ts_mtx);
+}
+
 #define MAX_SAMPLES 20
 
 int64
@@ -3289,6 +3598,7 @@ itc_local_sample (it_cursor_t * itc)
   float mean, stddev;
   int64 samples[MAX_SAMPLES];
   int64 n_leaves, sample, tb_count;
+  int is_rdf_p = itc_sample_is_rdf_p (itc);
   dbe_table_t * tb = itc->itc_insert_key->key_table;
   int n_samples = 1;
   itc->itc_random_search = RANDOM_SEARCH_ON;
@@ -3306,6 +3616,10 @@ itc_local_sample (it_cursor_t * itc)
     return samples[0];
   {
     int angle, step = 248, offset = 5;
+    if (is_rdf_p)
+      {
+	itc->itc_st.cols = hash_table_allocate (11);
+      }
     for (;;)
       {
 	for (angle = step + offset; angle < 1000; angle += step)
@@ -3332,6 +3646,8 @@ itc_local_sample (it_cursor_t * itc)
   }
   if (CL_RUN_SINGLE_CLUSTER == cl_run_local_only && itc->itc_insert_key->key_partition && itc->itc_insert_key->key_partition->kpd_map != clm_replicated)
     mean *= key_n_partitions (itc->itc_insert_key);
+  if (is_rdf_p)
+    itc_record_rdf_p (itc, mean);
   return ((int64) mean);
 }
 
