@@ -1,5 +1,5 @@
 --
---  $Id: soap.sql,v 1.11.2.2 2010/01/07 16:58:12 source Exp $
+--  $Id: soap.sql,v 1.11.2.5 2011/09/28 17:01:17 source Exp $
 --
 --  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 --  project.
@@ -663,7 +663,7 @@ WS_SOAP (in path any, in params any, in lines any)
       _fwd := xpath_eval ('fwd', rp, 1);
       _rev := xpath_eval ('rev', rp, 1);
       --this_host := http_request_header (lines, 'Host', null, 'localhost');
-      --this_host := 'imitko:6666'; -- findout how to get port if not supplied
+      --this_host := 'imitko:6666'; -- find out how to get port if not supplied
       --this_path := http_path();
       --this_point := sprintf ('http://%s%s', this_host, this_path);
       this_point := soap_current_url ();
@@ -2284,7 +2284,7 @@ nxt:;
 		      </Error>', __SQL_STATE, __SQL_MESSAGE));
 	goto err;
       };
-      if (atoi (cfg_item_value (virtuoso_ini_path(), 'HTTPServer', 'ServerThreads')) < 2)
+      if (atoi (virtuoso_ini_item_value ('HTTPServer', 'ServerThreads')) < 2)
 	{
 	    result := xml_tree_doc (sprintf ('<Error>
 			  <Code>%V
@@ -2324,7 +2324,7 @@ err:;
 ?>
 </TD></TR></TABLE>
 </DIV>
-<DIV class="foot"><SPAN class="foot">Virtuoso Universal Server <?=sys_stat('st_dbms_ver')?> - Copyright&copy; 1998-2010 OpenLink Software.</SPAN></DIV>
+<DIV class="foot"><SPAN class="foot">Virtuoso Universal Server <?=sys_stat('st_dbms_ver')?> - Copyright&copy; 1998-2011 OpenLink Software.</SPAN></DIV>
  </BODY>
 </HTML>
 <?vsp
