@@ -1,7 +1,7 @@
 /*
  *  space.c
  *
- *  $Id: space.c,v 1.7.2.6 2010/01/31 20:32:05 source Exp $
+ *  $Id: space.c,v 1.7.2.9 2011/05/10 18:37:35 source Exp $
  *
  *  Delta spaces
  *
@@ -42,6 +42,7 @@ int space_rehash_threshold = 2;
 
 
 
+int32 dbs_cache_check_enable = 0;
 
 void
 it_cache_check (index_tree_t * it, int mode)
@@ -68,7 +69,7 @@ it_cache_check (index_tree_t * it, int mode)
 	      /* This can be legitimate if a thread is in freeze mode and one itc is on a table scan and another is in order by or hash fill, so that the freeze is in the temp space operation . */
 	      /* error = 1; */
 	    }
-	  if (DPF_INDEX == SHORT_REF (buf->bd_buffer + DP_FLAGS)
+	  if (dbs_cache_check_enable && DPF_INDEX == SHORT_REF (buf->bd_buffer + DP_FLAGS)
 	      && IT_CHECK_FAST != mode)
 	    pg_check_map_1 (buf);
 	    if (buf->bd_is_dirty && !gethash (DP_ADDR2VOID (buf->bd_page), &itm->itm_remap))
@@ -191,6 +192,7 @@ itc_delta_this_buffer (it_cursor_t * itc, buffer_desc_t * buf, int stay_in_map)
   return buf;
 }
 
+long tc_new_page;
 
 buffer_desc_t *
 it_new_page (index_tree_t * it, dp_addr_t addr, int type, int in_pmap,
@@ -242,7 +244,7 @@ it_new_page (index_tree_t * it, dp_addr_t addr, int type, int in_pmap,
 	  return NULL;
 	}
       if (5 == n_tries)
-	log_info ("Failed to get a buffer for a new page. Retrying.  If the failure repeats, an out of disk error will be signnalled.  The cause of this is having too many buffers wired down for preread, flush or group by/hash join temp space.  To correct, increase the number of buffers in the configuration file.  If this repeats in spite of having hundreds of thousands  of buffers, please report to support.");
+	log_info ("Failed to get a buffer for a new page. Retrying.  If the failure repeats, an out of disk error will be signalled.  The cause of this is having too many buffers wired down for preread, flush or group by/hash join temp space.  To correct, increase the number of buffers in the configuration file.  If this repeats in spite of having hundreds of thousands  of buffers, please report to support.");
       if (n_tries > 4)
 	virtuoso_sleep (0, 50000 * (n_tries - 4));
     }
@@ -303,6 +305,7 @@ it_new_page (index_tree_t * it, dp_addr_t addr, int type, int in_pmap,
   buf_set_dirty (buf);
   DBG_PT_PRINTF (("New page L=%d B=%p FL=%d K=%s \n", buf->bd_page, buf, type,
 		 it->it_key ? (it->it_key->key_name ? it->it_key->key_name : "unnamed key") : "no key"));
+  TC (tc_new_page);
   return buf;
 }
 
