@@ -1,5 +1,5 @@
 /*
- *  $Id: sparul2sql.c,v 1.20.2.8 2010/02/01 12:53:45 source Exp $
+ *  $Id: sparul2sql.c,v 1.20.2.15 2011/09/12 21:05:03 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -142,7 +142,9 @@ sparp_gp_trav_find_isect_with_ctor (sparp_t *sparp, SPART *curr, sparp_trav_stat
         return SPAR_GPT_COMPLETED;
       return SPAR_GPT_NODOWN;
     case SPAR_GP:
-      if (SELECT_L == curr->_.gp.subtype)
+      switch (curr->_.gp.subtype)
+        {
+        case SELECT_L:
         {
 #if 1 /*!!! TBD: implement rewriting of ctor fields so that a field that correspond to an alias of subquery's retval is replaced with the expression of the alias. Then use the branch that is currently not in use */
           return SPAR_GPT_COMPLETED;
@@ -156,6 +158,9 @@ sparp_gp_trav_find_isect_with_ctor (sparp_t *sparp, SPART *curr, sparp_trav_stat
           return SPAR_GPT_NODOWN;
 #endif
         }
+        case SERVICE_L:
+          return 0; /* remote triples can not interfere with local data manipulations. This may change in the future if SERVICE group ctor template is added in SPARUL */
+        }
       break;
     }
   return 0;
@@ -347,6 +352,12 @@ bnode_found_or_added_for_big_ssl:
           switch (fld_type)
             {
             case SPAR_VARIABLE:
+              if (SPART_VARNAME_IS_GLOB (fld->_.var.vname))
+                {
+                  tvector_args [(fld_ctr-1)*2] = (SPART *)t_box_num_nonull (CTOR_OPCODE_CONST_OR_EXPN);
+                  tvector_args [(fld_ctr-1)*2 + 1] = fld;
+                  break;
+                }
               var_ctr = spar_cve_find_or_add_variable (sparp, cve, fld);
               tvector_args [(fld_ctr-1)*2] = (SPART *)t_box_num_nonull (CTOR_OPCODE_VARIABLE);
               tvector_args [(fld_ctr-1)*2 + 1] = (SPART *)t_box_num_nonull (var_ctr);
@@ -412,7 +423,8 @@ args_ready:
       (SPART **)t_list (4, arg0, arg1, var_vector_arg, arg3) );
   else
     ctor_call = spar_make_funcall (sparp, 1, funname,
-      (SPART **)t_list (4, arg1, var_vector_arg, arg3, t_box_num (use_limits)) );
+      /* Names arg1 and arg3 become slightly misleading when arg0 is not provided, e.g., in case of SPARQL_CONSTRUCT */
+      (SPART **)t_list (4, arg1, var_vector_arg, arg3, t_box_num_nonull (use_limits)) );
   if (cve->cve_limofs_var_alias)
     {
       SPART *alias = spartlist (sparp, 4, SPAR_ALIAS, var_vector_expn, cve->cve_limofs_var_alias, SSG_VALMODE_AUTO);
@@ -458,6 +470,27 @@ spar_simplify_graph_to_patch (sparp_t *sparp, SPART *g)
   return g->_.graph.expn;
 }
 
+int
+spar_find_sc_for_big_ssl_const (sparp_t *sparp, sql_comp_t **sc_ret)
+{
+  if (sparp->sparp_disable_big_const)
+    {
+      sc_ret[0] = NULL;
+      return 0;
+    }
+  sc_ret[0] = sparp->sparp_sparqre->sparqre_super_sc;
+  if (NULL == sc_ret[0])
+    {
+      spar_error (sparp, "The query can be compiled and executed but not translated to an accurate SQL text, add 'define sql:big-data-const 0' for workaround");
+    }
+  else
+    {
+      while (NULL != sc_ret[0]->sc_super)
+        sc_ret[0]->sc_super = sc_ret[0]->sc_super;
+    }
+  return 1;
+}
+
 void
 spar_compose_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top, SPART *graph_to_patch, SPART *ctor_gp)
 {
@@ -482,21 +515,7 @@ spar_compose_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top, SPART *gra
       cve.cve_limofs_var_alias = t_box_dv_short_string ("ctor-1");
     }
   if (big_ssl_const_mode)
-    {
-      sc_for_big_ssl_const = sparp->sparp_sparqre->sparqre_super_sc;
-      if (NULL == sc_for_big_ssl_const)
-        {
-          big_ssl_const_mode = 0;
-#ifdef NDEBUG
-          spar_error (sparp, "The query can be compiled and executed but not translated to an accurate SQL text");
-#endif
-        }
-      else
-        {
-          while (NULL != sc_for_big_ssl_const->sc_super)
-            sc_for_big_ssl_const->sc_super = sc_for_big_ssl_const->sc_super;
-        }
-    }
+    big_ssl_const_mode = spar_find_sc_for_big_ssl_const (sparp, &sc_for_big_ssl_const);
   if ((INSERT_L != top->_.req_top.subtype) && (SPARUL_INSERT_DATA != top->_.req_top.subtype))
     cve.cve_bnodes_are_prohibited = 1;
   spar_compose_retvals_of_ctor (sparp, ctor_gp, "sql:SPARQL_CONSTRUCT", sc_for_big_ssl_const, NULL, NULL,
@@ -595,8 +614,8 @@ spar_emulate_ctor_field (sparp_t *sparp, SPART *opcode, SPART *oparg, SPART **va
       {
         if (NULL == bnode_emulation)
 #ifdef DEBUG
-          bnode_emulation = box_copy_tree (spartlist (sparp, 6 + (sizeof (rdf_val_range_t) / sizeof (caddr_t)), SPAR_BLANK_NODE_LABEL,
-            NULL, NULL, NULL, NULL, NULL, SPART_RVR_LIST_OF_NULLS ) );
+          bnode_emulation = box_copy_tree (spartlist (sparp, 7 + (sizeof (rdf_val_range_t) / sizeof (caddr_t)), SPAR_BLANK_NODE_LABEL,
+            NULL, NULL, NULL, NULL, NULL, SPART_RVR_LIST_OF_NULLS, NULL ) );
 #else
           bnode_emulation = box_copy_tree (spartlist (sparp, 1, SPAR_BLANK_NODE_LABEL));
 #endif
@@ -667,32 +686,46 @@ spar_tr_fields_are_similar (sparp_t *sparp, SPART *fld1, SPART *fld2)
   return 0; /* to keep compiler happy */
 }
 
+SPART *
+spar_dealias (sparp_t *sparp, SPART *expn, int expected_type)
+{
+  if (SPAR_ALIAS == SPART_TYPE (expn))
+    expn = expn->_.alias.arg;
+  if ((expected_type > 0) && (expected_type != SPART_TYPE (expn)))
+    spar_internal_error (sparp, "spar_" "deailas(): unexpected type of expression");
+  return expn;
+}
+
 int
 spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
 {
   SPART *triple;
-  SPART *ctor;
+  SPART *emu;
   SPART **known_vars;
   SPART **retvals = top->_.req_top.retvals;
   int retvals_count = BOX_ELEMENTS (retvals);
   SPART **var_triples, **args;
-  SPART *graph_expn, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
+  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
   if (NULL != sparp->sparp_env->spare_output_route_name)
     return 0; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (retvals[0])) && (5 == BOX_ELEMENTS (retvals[0]->_.funcall.argtrees)));
   triple = spar_find_single_physical_triple_pattern (sparp, top->_.req_top.pattern);
   if (NULL == triple)
     return 0; /* nontrivial pattern, can not be optimized this way */
-  graph_expn		= retvals[0]->_.funcall.argtrees[0];
-  ctor			= retvals[0]->_.funcall.argtrees[1];
-  uid_expn		= retvals[0]->_.funcall.argtrees[2];
-  log_mode_expn		= retvals[0]->_.funcall.argtrees[3];
-  compose_report_expn	= retvals[0]->_.funcall.argtrees[4];
+  arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
+  graph_expn		= arg0->_.funcall.argtrees[0];
+  ctor			= arg0->_.funcall.argtrees[1];
+  uid_expn		= arg0->_.funcall.argtrees[2];
+  log_mode_expn		= arg0->_.funcall.argtrees[3];
+  compose_report_expn	= arg0->_.funcall.argtrees[4];
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ctor)) && (4 == BOX_ELEMENTS (ctor->_.funcall.argtrees)));
   dbg_assert (DELETE_L == top->_.req_top.subtype);
   var_triples = ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
   if (1 < retvals_count)
-    known_vars = retvals [retvals_count-1]->_.funcall.argtrees;
+    {
+      SPART *known_vars_vector = spar_dealias (sparp, retvals [retvals_count-1], SPAR_FUNCALL);
+      known_vars = known_vars_vector->_.funcall.argtrees;
+    }
   else
     known_vars = ctor->_.funcall.argtrees[1]->_.funcall.argtrees;
   if (1 != BOX_ELEMENTS (var_triples))
@@ -703,15 +736,18 @@ spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
     (CTOR_OPCODE_BNODE == unbox ((caddr_t)(args[4]))) )
     return 0; /* bnodes in constructor can not be optimized this way (BTW that is blab when inside DELETE) */
   if (!spar_tr_fields_are_similar (sparp, graph_expn,
-      triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX] ) ||
-    !spar_tr_fields_are_similar (sparp,
-      spar_emulate_ctor_field (sparp, args[0], args[1], known_vars),
-      triple->_.triple.tr_fields[SPART_TRIPLE_SUBJECT_IDX] ) ||
-    !spar_tr_fields_are_similar (sparp,
-      spar_emulate_ctor_field (sparp, args[2], args[3], known_vars),
-      triple->_.triple.tr_fields[SPART_TRIPLE_PREDICATE_IDX] ) ||
-    !spar_tr_fields_are_similar (sparp,
-      spar_emulate_ctor_field (sparp, args[4], args[5], known_vars),
+      triple->_.triple.tr_fields[SPART_TRIPLE_GRAPH_IDX] ) )
+    return 0;
+  emu = spar_emulate_ctor_field (sparp, args[0], args[1], known_vars);
+  if (!spar_tr_fields_are_similar (sparp, emu,
+      triple->_.triple.tr_fields[SPART_TRIPLE_SUBJECT_IDX] ) )
+    return 0;
+  emu = spar_emulate_ctor_field (sparp, args[2], args[3], known_vars);
+  if (!spar_tr_fields_are_similar (sparp, emu,
+      triple->_.triple.tr_fields[SPART_TRIPLE_PREDICATE_IDX] ) )
+    return 0;
+  emu = spar_emulate_ctor_field (sparp, args[4], args[5], known_vars);
+  if (!spar_tr_fields_are_similar (sparp, emu,
       triple->_.triple.tr_fields[SPART_TRIPLE_OBJECT_IDX] ) )
     return 0;
   good_ctor_call = spar_make_funcall (sparp, 1, "sql:SPARQL_DELETE_CTOR",
@@ -729,7 +765,6 @@ spar_optimize_delete_of_single_triple_pattern (sparp_t *sparp, SPART *top)
 void
 spar_optimize_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top)
 {
-  SPART *ctor;
   SPART **known_vars;
   SPART **retvals = top->_.req_top.retvals;
   int retvals_count = BOX_ELEMENTS (retvals);
@@ -740,19 +775,23 @@ spar_optimize_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top)
   dk_set_t bad_triples = NULL;
   int all_triple_count, bad_triple_count, tctr;
   const char *fname;
-  SPART *graph_expn, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
+  SPART *arg0, *graph_expn, *ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
   if (NULL != sparp->sparp_env->spare_output_route_name)
     return; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (retvals[0])) && (5 == BOX_ELEMENTS (retvals[0]->_.funcall.argtrees)));
-  graph_expn		= retvals[0]->_.funcall.argtrees[0];
-  ctor			= retvals[0]->_.funcall.argtrees[1];
-  uid_expn		= retvals[0]->_.funcall.argtrees[2];
-  log_mode_expn		= retvals[0]->_.funcall.argtrees[3];
-  compose_report_expn	= retvals[0]->_.funcall.argtrees[4];
+  arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
+  graph_expn		= arg0->_.funcall.argtrees[0];
+  ctor			= arg0->_.funcall.argtrees[1];
+  uid_expn		= arg0->_.funcall.argtrees[2];
+  log_mode_expn		= arg0->_.funcall.argtrees[3];
+  compose_report_expn	= arg0->_.funcall.argtrees[4];
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ctor)) && (4 == BOX_ELEMENTS (ctor->_.funcall.argtrees)));
   var_triples = ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
   if (1 < retvals_count)
-    known_vars = retvals [retvals_count-1]->_.funcall.argtrees;
+    {
+      SPART *known_vars_vector = spar_dealias (sparp, retvals [retvals_count-1], SPAR_FUNCALL);
+      known_vars = known_vars_vector->_.funcall.argtrees;
+    }
   else
     known_vars = ctor->_.funcall.argtrees[1]->_.funcall.argtrees;
   all_triple_count = bad_triple_count = BOX_ELEMENTS (var_triples);
@@ -811,7 +850,6 @@ spar_optimize_retvals_of_insert_or_delete (sparp_t *sparp, SPART *top)
 void
 spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
 {
-  SPART *del_ctor, *ins_ctor;
   SPART **known_vars;
   SPART **retvals = top->_.req_top.retvals;
   int retvals_count = BOX_ELEMENTS (retvals);
@@ -824,16 +862,17 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
   dk_set_t bad_ins_triples = NULL;
   int all_del_triple_count, bad_del_triple_count, del_const_count, del_tctr;
   int all_ins_triple_count, bad_ins_triple_count, ins_tctr;
-  SPART *graph_expn, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
+  SPART *arg0, *graph_expn, *del_ctor, *ins_ctor, *uid_expn, *log_mode_expn, *good_ctor_call, *compose_report_expn;
   if (NULL != sparp->sparp_env->spare_output_route_name)
     return; /* If an output may go outside the default storage then there's no way of avoiding the complete filling of the result dictionary */
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (retvals[0])) && (6 == BOX_ELEMENTS (retvals[0]->_.funcall.argtrees)));
-  graph_expn		= retvals[0]->_.funcall.argtrees[0];
-  del_ctor		= retvals[0]->_.funcall.argtrees[1];
-  ins_ctor		= retvals[0]->_.funcall.argtrees[2];
-  uid_expn		= retvals[0]->_.funcall.argtrees[3];
-  log_mode_expn		= retvals[0]->_.funcall.argtrees[4];
-  compose_report_expn	= retvals[0]->_.funcall.argtrees[5];
+  arg0 = spar_dealias (sparp, retvals[0], SPAR_FUNCALL);
+  graph_expn		= arg0->_.funcall.argtrees[0];
+  del_ctor		= arg0->_.funcall.argtrees[1];
+  ins_ctor		= arg0->_.funcall.argtrees[2];
+  uid_expn		= arg0->_.funcall.argtrees[3];
+  log_mode_expn		= arg0->_.funcall.argtrees[4];
+  compose_report_expn	= arg0->_.funcall.argtrees[5];
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (del_ctor)) && (4 == BOX_ELEMENTS (del_ctor->_.funcall.argtrees)));
   dbg_assert ((SPAR_FUNCALL == SPART_TYPE (ins_ctor)) && (4 == BOX_ELEMENTS (ins_ctor->_.funcall.argtrees)));
   del_var_triples = del_ctor->_.funcall.argtrees[0]->_.funcall.argtrees;
@@ -841,7 +880,10 @@ spar_optimize_retvals_of_modify (sparp_t *sparp, SPART *top)
   del_const_triples = del_ctor->_.funcall.argtrees[2]->_.funcall.argtrees;
   del_const_count = BOX_ELEMENTS (del_const_triples);
   if (1 < retvals_count)
-    known_vars = retvals [retvals_count-1]->_.funcall.argtrees;
+    {
+      SPART *known_vars_vector = spar_dealias (sparp, retvals [retvals_count-1], SPAR_FUNCALL);
+      known_vars = known_vars_vector->_.funcall.argtrees;
+    }
   else
     known_vars = ins_ctor->_.funcall.argtrees[1]->_.funcall.argtrees;
 /* Part 1. Collecting optimized data for DELETE ctor */
