@@ -1,7 +1,7 @@
 /*
  *  sqlcomp2.c
  *
- *  $Id: sqlcomp2.c,v 1.21.2.4 2010/02/01 12:53:45 source Exp $
+ *  $Id: sqlcomp2.c,v 1.21.2.10 2011/09/13 14:50:10 source Exp $
  *
  *  Dynamic SQL Compiler, part 2
  *
@@ -63,7 +63,7 @@ sqlc_mark_last_ts_in_join (sql_comp_t * sc, comp_table_t * ct)
 void
 sqlc_opt_last_joins (sql_comp_t * sc)
 {
-  /* tha last table and all tables right of the
+  /* the last table and all tables right of the
      last table generating multiple rows need no
      reference counts for output cols */
   comp_table_t **cts = sc->sc_tables;
@@ -652,6 +652,7 @@ yy_new_error (const char *s, const char *state, const char *native)
   int is_semi;
   int this_lineno = scn3_lineno;
   char buf_for_next [2000];
+  scn3_include_fragment_t *outer;
   if (scn3_inside_error_reporter)
     goto jmp; /* see below */
   nlen = scn3_sprint_curr_line_loc (sql_err_text, sizeof (sql_err_text));
@@ -691,6 +692,12 @@ yy_new_error (const char *s, const char *state, const char *native)
   sql_err_text [sizeof (sql_err_text)-1] = '\0';
 
 jmp:
+  outer = scn3_include_stack + scn3_include_depth;
+  if (outer->_.sif_skipped_part)
+    {
+      dk_free_box (outer->_.sif_skipped_part);
+      outer->_.sif_skipped_part = NULL;
+    }
   longjmp_splice (&parse_reset, 1);
 }
 
@@ -1305,37 +1312,6 @@ sqlc_assign_unknown_dtps (query_t *qr)
 }
 
 
-int
-qr_proc_repl_check_valid (query_t *qr, caddr_t *err)
-{
-  if (qr->qr_proc_repl_acct)
-    { /* check and disable procedures with INOUT / OUT parameters */
-      int inx = 1;
-      DO_SET (state_slot_t *, ssl, &qr->qr_parms)
-	{
-	  if (ssl->ssl_type == SSL_REF_PARAMETER_OUT ||
-	      ssl->ssl_type == SSL_REF_PARAMETER)
-	    {
-	      if (err)
-		*err = srv_make_new_error ("42000", "SQ205",
-		    "Procedure %s cannot be published for transactional replication "
-		    "(in publication %s) because it has out/inout parameter %s (parameter number %d). "
-		    "Publishing the calls to the procedure is disabled. "
-		    "Please remove the procedure from the transactional publication.",
-		    qr->qr_proc_name,
-		    qr->qr_proc_repl_acct,
-		    ssl->ssl_name ? ssl->ssl_name : "",
-		    inx
-		    );
-	      qr->qr_proc_repl_acct = NULL;
-	      return 0;
-	    }
-	  inx = inx + 1;
-	}
-      END_DO_SET ();
-    }
-  return 1;
-}
 
 static int
 sql_is_ddl (sql_tree_t * tree)
@@ -1466,7 +1442,8 @@ DBG_NAME(sql_compile_1) (DBG_PARAMS const char *string2, client_connection_t * c
 		}
 	      sql_pop_all_buffers ();
 	      SCS_STATE_POP;
-	      semaphore_leave (parse_sem);
+	      if (inside_sem)
+	        semaphore_leave (parse_sem);
 	      POP_CATCH;
 	      if (*err && strstr ((*(caddr_t**)err)[2], "RDFNI") )
 		{
@@ -1518,6 +1495,7 @@ DBG_NAME(sql_compile_1) (DBG_PARAMS const char *string2, client_connection_t * c
 	    {
 	      MP_DONE();
 	    }
+	  sc_free (&sc);
 	  SCS_STATE_POP;
 	  qr_free (qr);
 	  /*dk_free (string, -1);*/
@@ -1533,6 +1511,7 @@ DBG_NAME(sql_compile_1) (DBG_PARAMS const char *string2, client_connection_t * c
 	    {
 	      MP_DONE();
 	    }
+	  sc_free (&sc);
 	  SCS_STATE_POP;
 	  qr_free (qr);
 	  POP_CATCH;
@@ -1573,6 +1552,7 @@ DBG_NAME(sql_compile_1) (DBG_PARAMS const char *string2, client_connection_t * c
 	  {
 	    qr->qr_line_counts = hash_table_allocate (100);
 	    qr->qr_call_counts = id_str_hash_create (101);
+	    qr->qr_stats_mtx = mutex_allocate ();
 	  }
 #endif
       }
@@ -1650,11 +1630,6 @@ DBG_NAME(sql_compile_1) (DBG_PARAMS const char *string2, client_connection_t * c
 	      sch_proc_def (sc.sc_cc->cc_schema, qr->qr_proc_name);
       user_t * p_user = cli->cli_user;
 
-#ifdef REPLICATION_SUPPORT2
-      if (old_place)
-	qr->qr_proc_repl_acct = old_place->qr_proc_repl_acct;
-      qr_proc_repl_check_valid (qr, err);
-#endif
       /* Only DBA can create procedures with owner different than creator */
       if (p_user && !sec_user_has_group (0, p_user->usr_g_id))
 	{
