@@ -1,7 +1,7 @@
 /*
  *  sqlexp.c
  *
- *  $Id: sqlexp.c,v 1.16.2.21 2010/04/26 14:41:41 source Exp $
+ *  $Id: sqlexp.c,v 1.16.2.24 2011/07/19 23:38:34 source Exp $
  *
  *  Dynamic SQL Expression Generator
  *
@@ -236,11 +236,11 @@ sqlc_call_exp (sql_comp_t * sc, dk_set_t * code, state_slot_t * ret, ST * tree)
       return;
     }
   if ((sqlc_trans_funcs (sc, tree, ret)))
-    return;
-  if ((func_len > 10) && !stricmp (func + (func_len - 10), " (w/cache)") && (n_params > 0))
+    return;                                            /* 0123456789 */
+  if ((func_len > 9) && !stricmp (func + (func_len - 9), "__w_cache") && (n_params >= 1))
     {
-      state_slot_t *cache = ssl_new_inst_variable (sc->sc_cc, "cache", DV_ARRAY_OF_POINTER);
-      ((ptrlong *)(act_params[n_params - 1]))[0] = cache->ssl_index;
+      state_slot_t *aux_ssl = ssl_new_inst_variable (sc->sc_cc, "cache", DV_ARRAY_OF_POINTER);
+      ((ptrlong *)(act_params[n_params - 1]))[0] = aux_ssl->ssl_index;
     }
   if (ret_param)
     params = (state_slot_t **) t_alloc_box ((n_params + 1) * sizeof (caddr_t), DV_ARRAY_OF_POINTER);
@@ -375,6 +375,7 @@ sqlc_check_const_call (sql_comp_t * sc, ST * tree)
 	  dk_free_box (data);
 	  return ssl;
 	}
+      dk_free_box (data);
     }
   return NULL;
 }
@@ -481,7 +482,7 @@ sqlc_simple_case (sql_comp_t * sc, ST * tree, dk_set_t * code)
   int was_else = 0;
   jmp_label_t end = sqlc_new_label (sc);
   int n_exps = BOX_ELEMENTS (tree->_.comma_exp.exps);
-  state_slot_t *res = sqlc_new_temp (sc, "callret", DV_UNKNOWN);
+  state_slot_t *res = sqlc_new_temp (sc, "callretSimpleCASE", DV_UNKNOWN);
   state_slot_t *sel;
   int inx;
   ST **exps = tree->_.comma_exp.exps;
@@ -546,7 +547,7 @@ sqlc_searched_case (sql_comp_t * sc, ST * tree, dk_set_t * code)
   int was_else = 0;
   jmp_label_t end = sqlc_new_label (sc);
   int n_exps = BOX_ELEMENTS (tree->_.comma_exp.exps);
-  state_slot_t *res = sqlc_new_temp (sc, "callret", DV_UNKNOWN);
+  state_slot_t *res = sqlc_new_temp (sc, "callretSearchedCASE", DV_UNKNOWN);
   int inx;
   ST **exps = tree->_.comma_exp.exps;
   df_elt_t *dfe = sc->sc_so ? sqlo_df (sc->sc_so, tree) : NULL;
@@ -745,7 +746,10 @@ scalar_exp_generate (sql_comp_t * sc, ST * tree, dk_set_t * code)
 	state_slot_t *res = sqlc_check_const_call (sc, tree);
 	if (res)
 	  seg_return (res);
-	res = sqlc_new_temp (sc, "callret", DV_UNKNOWN);
+	if (NULL != tree->_.call.name)
+	  res = sqlc_new_temp (sc, tree->_.call.name, DV_UNKNOWN);
+	else
+	  res = sqlc_new_temp (sc, "callret", DV_UNKNOWN);
 	res->ssl_is_callret = 1;
 	tree = sqlo_udt_check_method_call (sc->sc_so, sc, tree);
 	sqlc_call_exp (sc, code, res, tree);
