@@ -1,7 +1,7 @@
 /*
  *  sqlfn.h
  *
- *  $Id: sqlfn.h,v 1.46.2.9 2010/01/31 20:43:26 source Exp $
+ *  $Id: sqlfn.h,v 1.46.2.15 2011/09/12 20:08:52 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -388,11 +388,12 @@ extern server_lock_t server_lock;
 
 void plh_free (placeholder_t * plh);
 
-caddr_t srv_make_new_error (const char *code, const char *virt_code, const char *msg,...)
+EXE_EXPORT (caddr_t, srv_make_new_error, (const char *code, const char *virt_code, const char *msg,...));
+#ifndef _USRDLL
 #ifdef __GNUC__
-__attribute__ ((format (printf, 3, 4)))
+caddr_t srv_make_new_error (const char *code, const char *virt_code, const char *msg,...) __attribute__ ((format (printf, 3, 4)));
+#endif
 #endif
-;
 void qi_enter (query_instance_t * qi);
 
 void qi_leave (query_instance_t * qi);
@@ -1003,7 +1004,7 @@ caddr_t  sf_make_new_log_name(dbe_storage_t * dbs);
 
 dk_set_t srv_get_logons (void);
 
-srv_stmt_t * cli_get_stmt_access (client_connection_t * cli, caddr_t id, int mode);
+srv_stmt_t * cli_get_stmt_access (client_connection_t * cli, caddr_t id, int mode, caddr_t * err_ret);
 caddr_t stmt_set_query (srv_stmt_t * stmt, client_connection_t * cli, caddr_t text,
 		stmt_options_t * opts);
 query_t * cli_cached_sql_compile (caddr_t query_text, client_connection_t *cli,
@@ -1072,23 +1073,23 @@ caddr_t bif_commit (caddr_t * qst, caddr_t * err_ret, state_slot_t ** args);
     } \
   QR_RESET_CODE  \
     { \
-      caddr_t err = NULL, err_2 = NULL; \
+      caddr_t _err_1 = NULL, _err_2 = NULL; \
       POP_QR_RESET; \
-      vdb_leave_1 (_qi2, &err); \
-      err_2 = subq_handle_reset (_qi2, reset_code); \
-      if (err && err_2) \
+      vdb_leave_1 (_qi2, &_err_1); \
+      _err_2 = subq_handle_reset (_qi2, reset_code); \
+      if (_err_1 && _err_2) \
        	{ \
-	  dk_free_tree (err); \
-          err = err_2; \
+	  dk_free_tree (_err_1); \
+          _err_1 = _err_2; \
 	} \
-      else if (err_2) \
-	err = err_2; \
-      if (err) \
+      else if (_err_2) \
+	_err_1 = _err_2; \
+      if (_err_1) \
 	{ \
 	  if (err_ret) \
-	    *err_ret = err; \
+	    *err_ret = _err_1; \
 	  else \
-	    sqlr_resignal (err); \
+	    sqlr_resignal (_err_1); \
 	} \
     } \
   END_QR_RESET; \
@@ -1101,7 +1102,8 @@ boxint safe_atoi (const char *data, caddr_t *err_ret);
 double safe_atof (const char *data, caddr_t *err_ret);
 caddr_t box_to_any (caddr_t data, caddr_t * err_ret);
 caddr_t box_to_any_1 (caddr_t data, caddr_t * err_ret, auto_pool_t *ap, int ser_flags);
-#define DKS_TO_OBY_KEY 2 /* flag to indicate that an rdf box with text should be stored with the text, not just id */
+#define DKS_TO_OBY_KEY 2 /*!< flag to indicate that an rdf box with text should be stored with the text, not just id */
+#define DKS_TO_HA_DISK_ROW 4 /*!< flag to indicate that the destination is a temp table with no sorting and box_to_any_1 serialization in a column */
 
 caddr_t box_to_shorten_any (caddr_t data, caddr_t * err_ret);
 char* __get_column_name (oid_t col_id, dbe_key_t *key);
@@ -1287,4 +1289,7 @@ void itc_assert_no_reg (it_cursor_t * itc);
 #define itc_assert_no_reg(itc)
 #endif
 
+caddr_t * itc_bm_array (it_cursor_t * itc, buffer_desc_t * buf);
+extern int32 log_proc_overwrite;
+
 #endif /* _SQLFN_H */
