@@ -1,7 +1,7 @@
 /*
  *  sqlo.h
  *
- *  $Id: sqlo.h,v 1.22.2.8 2010/02/01 12:53:46 source Exp $
+ *  $Id: sqlo.h,v 1.22.2.14 2011/10/13 11:05:09 source Exp $
  *
  *  sql opt graph
  *
@@ -281,6 +281,8 @@ struct df_elt_s
     struct {
       dbe_column_t *	col;
       op_virt_col_t *   vc;
+      float		card; /* if from rdf quad, card is given by p stats and is not the card of the dbe_column_t */
+      bitf_t		is_fixed:1; /* col eq to param or col imported from other dt, if all group cols fixed, gb can be dropped */
     } col;
     struct {
       int 	op;
@@ -425,6 +427,7 @@ struct sqlo_s
   char		so_dfe_unplace_pass; /* not generate errors in sqlo_dfe_unplace if dfe != sqlo_df ()*/
 #endif
   dk_set_t	so_in_list_nodes;
+  dk_set_t	so_all_list_nodes;
   dk_set_t *	so_inx_int_tried_ret; /* ref to where dfes tried with an inx int go so that the same inx int does not get tried in all permutations */
   uint32	so_last_sample_time; /* used for stopping compilation if longer is elapsed since last sample than the best plan's time */
   df_elt_t *	so_crossed_oby; /* If placing exp and there is an oby that is crossed, then set this to be the oby so that the exp can be added to its deps */
@@ -535,6 +538,7 @@ typedef struct index_choice_s
   char		ic_text_order;
   char		ic_geo_order;
   char		ic_o_string_range_lit; /* 1 if o is known to be a string 2 if literal strings */
+  dk_set_t	ic_inx_sample_cols;
 } index_choice_t;
 
 #define IC_OPT_ITERS 0 /* can change in or rdf inf iters into after test */
@@ -667,7 +671,7 @@ ptrlong sqlo_select_top_cnt (sqlo_t *so, ST *top_exp);
    ST_P (view, INTERSECT_ALL_ST))
 
 #define DFE_IS_PARAM(dfe) \
-	(dfe->dfe_type == DFE_CONST && (SYMBOLP (dfe->dfe_tree) || ST_P (dfe->dfe_tree, COL_DOTTED)))
+	(dfe->dfe_type == DFE_CONST && (SYMBOLP (dfe->dfe_tree) || ST_COLUMN (dfe->dfe_tree, COL_DOTTED)))
 
 #define DFE_IS_CONST(dfe) \
 	(dfe->dfe_type == DFE_CONST && !DFE_IS_PARAM (dfe))
@@ -741,6 +745,7 @@ void qn_ins_before (sql_comp_t * sc, data_source_t ** head, data_source_t * ins_
 #define NEXT_PAGE_COST 5
 #define INX_ROW_INS_COST 1 /* cost of itc_insert_dv into inx */
 #define HASH_ROW_INS_COST 1.6 /* cost of adding a row to hash */
+#define HASH_MEM_INS_COST 0.7
 #define HASH_LOOKUP_COST 0.9
 #define HASH_ROW_COST 0.3
 #define CV_INSTR_COST 0.1   /* avg cost of instruction in code_vec_run */
@@ -792,6 +797,11 @@ int dfe_is_eq_pred (df_elt_t * pred);
 float sqlo_index_path_cost (dk_set_t path, float * cost_ret, float * card_ret, char * sure_ret);
 data_source_t * sqlg_make_ts (sqlo_t * so, df_elt_t * tb_dfe);
 int dfe_is_o_ro2sq_range (df_elt_t * pred, df_elt_t * tb_dfe, df_elt_t ** o_col_dfe_ret, df_elt_t ** exp_dfe_ret, int * op_ret);
+void sqlg_in_iter_nodes (sqlo_t * so, data_source_t * ts, data_source_t ** head);
+int sqlo_has_col_ref (ST * tree);
+
+float dfe_exp_card (sqlo_t * so, df_elt_t * dfe);
+void sqlo_rdf_col_card (sqlo_t * so, df_elt_t * td_dfe, df_elt_t * dfe);
 
 
 #endif /* _SQLO_H */
