@@ -1,7 +1,7 @@
 /*
  *  sqloby.c
  *
- *  $Id: sqloby.c,v 1.7.2.3 2010/02/01 12:53:46 source Exp $
+ *  $Id: sqloby.c,v 1.7.2.6 2010/03/08 12:04:13 source Exp $
  *
  *  sql order and group compilation
  *
@@ -56,13 +56,13 @@ sqlo_oby_exp_cols (sqlo_t * so, ST * dt, ST** oby)
       if (INTEGERP (spec->_.o_spec.col))
 	{
 	  ptrlong nth = unbox ((box_t) spec->_.o_spec.col) - 1;
-	  if (nth >= (ptrlong) BOX_ELEMENTS (dt->_.select_stmt.selection))
+	  if (nth >= (ptrlong) BOX_ELEMENTS (dt->_.select_stmt.selection) || nth < 0)
 	    sqlc_error (so->so_sc->sc_cc, "37000", "index of column in order by out of range");
 	  spec->_.o_spec.col = (ST*) t_box_copy_tree (dt->_.select_stmt.selection[nth]);
 	  while (ST_P (spec->_.o_spec.col, BOP_AS))
 	    spec->_.o_spec.col = spec->_.o_spec.col->_.as_exp.left;
 	}
-      if (!ST_P (spec->_.o_spec.col, COL_DOTTED))
+      if (!ST_COLUMN (spec->_.o_spec.col, COL_DOTTED))
 	res = SPEC_MIXED;
     }
   END_DO_BOX;
@@ -174,7 +174,7 @@ sqlo_ot_oby_seq (sqlo_t * so, op_table_t * top_ot)
 
   DO_BOX (ST *, spec, inx, oby)
     {
-      if (!ST_P (spec->_.o_spec.col, COL_DOTTED))
+      if (!ST_COLUMN (spec->_.o_spec.col, COL_DOTTED))
 	return;
       col = spec->_.o_spec.col;
       col_ot = sqlo_cname_ot (so, col->_.col_ref.prefix);
@@ -366,7 +366,7 @@ sqlo_try_oby_order (sqlo_t * so, df_elt_t * tb_dfe)
 		tb_found = 1;
 	    }
 	}
-      /* not allow tables to be placed in index order if no tables were palced
+      /* not allow tables to be placed in index order if no tables were placed
 	 and the table does not have the first oby col */
       if (!tb_found && oby_nth > 0)
 	return 0;
@@ -816,7 +816,7 @@ sqlo_exp_cols_from_dt (sqlo_t * so, ST * tree, df_elt_t * dt_dfe, dk_set_t * ret
 {
   /* traverse exp and return all cols that exp depends on which are defined in dt_dfe */
   int inx;
-  if (ST_P (tree, COL_DOTTED))
+  if (ST_COLUMN (tree, COL_DOTTED))
     {
       df_elt_t * pt, *col_dfe = sqlo_df (so, tree);
       for (pt = dt_dfe->_.sub.last; pt; pt = pt->dfe_prev)
