@@ -1,7 +1,7 @@
 /*
  *  sqlpfn.h
  *
- *  $Id: sqlpfn.h,v 1.10.2.1 2010/10/18 11:37:58 source Exp $
+ *  $Id: sqlpfn.h,v 1.8.2.7 2011/08/26 17:57:06 source Exp $
  *
  *  SQL Parser Utility Functions
  *
@@ -61,7 +61,10 @@ ST ** asg_col_list (ST ** asg_list);
 
 ST ** sqlp_local_variable_decls (caddr_t * names, ST * dtp);
 
-caddr_t sqlp_box_id_upcase (const char * str);
+caddr_t DBG_NAME (sqlp_box_id_upcase) (DBG_PARAMS const char *str);
+#ifdef MALLOC_DEBUG
+#define sqlp_box_id_upcase(s) dbg_sqlp_box_id_upcase (__FILE__, __LINE__, s)
+#endif
 caddr_t t_sqlp_box_id_upcase (const char * str);
 caddr_t sqlp_box_upcase (const char * str);
 caddr_t t_sqlp_box_upcase (const char * str);
@@ -148,7 +151,9 @@ caddr_t * sqlp_string_col_list (caddr_t * lst);
 caddr_t sqlp_xml_col_name (ST * tree);
 extern int sqlp_xml_col_directive (char *id);
 long sqlp_xml_select_flags (char * mode, char * elt);
+extern void sqlp_tweak_selection_names (ST * tree);
 ptrlong sqlp_bunion_flag (ST * l, ST * r, long f);
+ST *sqlp_wpar_nonselect (ST *subq);
 ST * sqlp_inline_order_by (ST *tree, ST **oby);
 ST * sqlp_patch_call_if_special (ST * funcall_tree);
 ptrlong sqlp_cursor_name_to_type (caddr_t name);
@@ -241,5 +246,7 @@ ST * sqlp_infoschema_redirect (ST *texp);
 void sqlp_breakup (ST * sel);
 int sel_n_breakup (ST * sel);
 void sqlp_dt_header (ST * exp);
-
+caddr_t sqlp_col_num (caddr_t);
+int sqlp_is_num_lit (caddr_t x);
+caddr_t sqlp_minus (caddr_t n);
 #endif /* _SQLPFN_H */
