@@ -1,7 +1,7 @@
 /*
  *  update.c
  *
- *  $Id: update.c,v 1.11.2.8 2010/02/01 12:53:46 source Exp $
+ *  $Id: update.c,v 1.11.2.13 2011/10/19 15:18:21 source Exp $
  *
  *  UPDATE statements
  *
@@ -477,10 +477,13 @@ update_node_run_1 (update_node_t * upd, caddr_t * inst,
   int res;
   placeholder_t *pl = (placeholder_t *) qst_place_get (state, upd->upd_place);
   query_instance_t *qi = (query_instance_t *) QST_INSTANCE (state);
+  union {
+  void * dummy;
   dtp_t temp [2000];
+  } temp_un;
   LOCAL_RD (rd);
-  rd.rd_temp = temp;
-  rd.rd_temp_max = sizeof (temp);
+  rd.rd_temp = &(temp_un.temp[0]);
+  rd.rd_temp_max = sizeof (temp_un.temp);
   if (!pl)
     sqlr_new_error ("24000", "SR250", "Cursor not positioned on update. %s",
 		    upd->upd_place->ssl_name);
@@ -535,8 +538,8 @@ update_node_run_1 (update_node_t * upd, caddr_t * inst,
 	    new_rd.rd_non_comp_max = MAX_ROW_BYTES;
 	    new_rd.rd_key = cr_itc->itc_insert_key;
 	    upd_hi_pre (upd, qi);
-	    new_rd.rd_temp = temp;
-	    new_rd.rd_temp_max = sizeof (temp);
+	    new_rd.rd_temp = &(temp_un.temp[0]);
+	    new_rd.rd_temp_max = sizeof (temp_un.temp);
 	    update_quick (upd, state, cr_buf, &new_rd, &row_err);
 	    if (row_err)
 	      sqlr_resignal (row_err);
@@ -611,7 +614,7 @@ update_node_run_1 (update_node_t * upd, caddr_t * inst,
        because conversion may read blob by blob_to_string() and its itc will enter map.
     */
     ITC_LEAVE_MAPS (main_itc);
-#ifdef MTX_DEBUG
+#ifdef PAGE_DEBUG
     if (main_buf->bd_writer != THREAD_CURRENT_THREAD)
       GPF_T1 ("Must have write on buffer to check it");
 #endif
@@ -697,6 +700,7 @@ update_node_run_1 (update_node_t * upd, caddr_t * inst,
 		  goto next_key;
 		if (!key->key_distinct)
 		  {
+		    del_itc->itc_no_bitmap = 0; /* reset as prev ins may set it to true */
 		    res = itc_get_alt_key (del_itc, &del_buf, key, &rd);
 		    itc_delete_this (del_itc, &del_buf, res, NO_BLOBS);
 		  }
@@ -886,7 +890,7 @@ update_keyset_state_restore (update_node_t * upd, caddr_t * state, int * start)
    If the  upd_keyset is set and both inst and state are
    given, will remember the values in the ssls upd_place, upd_values,
    upd_trigger_args.  The placeholder is taken from the ssl and the ssl is set to
-   null withouot free. QST_GET_V () = NULL.
+   null without free. QST_GET_V () = NULL.
 
    If the node remembers states, do SRC_IN_STATE (upd, inst) = inst to mark
    this.  This means the will be continued later.  Means the update is done
@@ -1098,6 +1102,35 @@ itc_row_insert (it_cursor_t * itc, row_delta_t * rd, buffer_desc_t ** unq_buf,
   return DVC_LESS;
 }
 
+void
+row_insert_rd_len (row_delta_t * rd)
+{
+  dbe_key_t * key = rd->rd_key;
+  int inx = 0;
+  DO_ALL_CL (cl, key)
+    {
+      caddr_t val = rd->rd_values [inx];
+      switch (cl->cl_sqt.sqt_dtp)
+	{
+	  case DV_STRING:
+	  case DV_WIDE:
+	  case DV_ANY:
+	  case DV_OBJECT:
+	      rd->rd_non_comp_len += (box_length (val) - 1);
+	      break;
+	  case DV_BIN:
+	      rd->rd_non_comp_len += box_length (val);
+	      break;
+	  case DV_BLOB:
+	  case DV_BLOB_BIN:
+	  case DV_BLOB_WIDE:
+	      rd->rd_non_comp_len += DV_BLOB_LEN;
+	      break;
+	}
+      inx++;
+    }
+  END_DO_ALL_CL;
+}
 
 void
 row_insert_node_input (row_insert_node_t * ins, caddr_t * inst,
@@ -1118,6 +1151,9 @@ row_insert_node_input (row_insert_node_t * ins, caddr_t * inst,
   if (!key)
     sqlr_new_error ("42000", "RFW..", "Key id " BOXINT_FMT " undefined in row insert", unbox (row[0]));
   rd.rd_key = key;
+  rd.rd_non_comp_len = key->key_row_var_start[0];
+  rd.rd_non_comp_max = MAX_ROW_BYTES;
+  row_insert_rd_len (&rd);
   ITC_FAIL (it)
   {
     if (DVC_MATCH == itc_row_insert (it, &rd, &buf, 0, 0))
