@@ -1,7 +1,7 @@
 --
 --  users.sql
 --
---  $Id: users.sql,v 1.12.2.4 2010/02/01 02:16:19 source Exp $
+--  $Id: users.sql,v 1.12.2.12 2011/10/18 12:41:34 source Exp $
 --
 --  Unified user model
 --
@@ -190,6 +190,7 @@ nf:
 
 create procedure DB.DBA.SECURITY_CL_EXEC_AND_LOG (in txt varchar, in args any)
 {
+  set_user_id ('dba');
   cl_exec (txt, args);
   cl_exec ('log_text_array (?, ?)', vector (txt, args), 1);
 }
@@ -276,15 +277,15 @@ USER_CREATE (in _name varchar, in passwd varchar, in options any := NULL)
 		 _pwd_mode, _pwd_mode_data, _get_pwd, _login_qual,
 		 _u_home, _u_perms);
 
-  if (_sql_enable)
+  if (not _sql_enable) /* pure web accounts must be disabled for odbc/sql login */
     {
+      _disabled := 1;
+    }
       DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_set_user_struct (?,?,?,?,?,?,?)', vector (
 	  _name, passwd, _u_id, _prim_group_id, concat ('Q ', _login_qual), 0, _u_sys_name, _u_sys_pass ) );
       if (_disabled = 1)
         {
-          DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_user_enable (?, ?)', vector (
-            _name, 0 ) );
-        }
+      DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_user_enable (?, ?)', vector (_name, 0));
     }
 
   return _u_id;
@@ -308,9 +309,7 @@ USER_ROLE_CREATE (in _name varchar, in is_dav integer := 0)
   if (is_dav)
     _sql_enable := 0;
   insert into SYS_USERS (U_ID, U_NAME, U_GROUP, U_IS_ROLE, U_DAV_ENABLE, U_SQL_ENABLE) values (_g_id, _name, _g_id, 1, is_dav, _sql_enable);
-  if (_sql_enable)
-    DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_set_user_struct (?,?,?,?,?,?)',
-	  vector (_name, '', _g_id, _g_id, NULL, 1) );
+  DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_set_user_struct (?,?,?,?,?,?)', vector (_name, '', _g_id, _g_id, NULL, 1) );
   return _g_id;
 }
 ;
@@ -463,7 +462,7 @@ USER_GRANT_ROLE (in _name varchar, in _role varchar, in grant_opt integer := 0)
         {
 	  if (primary_group is null or inh[i] <> primary_group)
 	    {
-	      insert into SYS_ROLE_GRANTS (GI_SUPER, GI_SUB, GI_ADMIN, GI_DIRECT, GI_GRANT)
+	      insert soft SYS_ROLE_GRANTS (GI_SUPER, GI_SUB, GI_ADMIN, GI_DIRECT, GI_GRANT)
 		  values (_u_id, inh[i], grant_opt, 0, _g_id);
 	    }
           i := i + 1;
@@ -646,25 +645,15 @@ USER_SET_OPTION (in _name varchar, in opt varchar, in value any)
       U_ACCOUNT_DISABLED = _disabled
       where U_NAME = _name;
 
-  if (_sql_enable)
+  if (not _sql_enable) /* pure web accounts must be disabled for odbc/sql login */
     {
+      _disabled := 1;
+    }
       select pwd_magic_calc (U_NAME, U_PASSWORD, 1) into passwd from SYS_USERS where U_NAME = _name;
       DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_set_user_struct (?,?,?,?,?)',
-        vector (
-	  _name, passwd, _u_id, _u_group_id,
-	  case when _login_qual is not null then concat ('Q ', _login_qual) else NULL end ) );
-      DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_user_enable (?, ?)',
-        vector (_name, case when _disabled = 0 then 1 else 0 end) );
-    }
-  else
-    {
-      declare exit handler for sqlstate '28000'
-	{
-	  goto done;
-	};
-      DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_remove_user_struct(?)', vector (_name));
-      done:;
-    }
+      vector (_name, passwd, _u_id, _u_group_id,
+	case when length (_login_qual) then concat ('Q ', _login_qual) else NULL end));
+  DB.DBA.SECURITY_CL_EXEC_AND_LOG ('sec_user_enable (?, ?)', vector (_name, case when _disabled = 0 then 1 else 0 end));
 }
 ;
 
@@ -1054,7 +1043,7 @@ create procedure
 	if (fp is null)
 	  goto normal_auth;
 
-        ext_oid := cfg_item_value (virtuoso_ini_path(), 'Parameters', 'X509ExtensionOID');
+        ext_oid := virtuoso_ini_item_value ('Parameters', 'X509ExtensionOID');
 
         if (ext_oid is not null)
           new_user := get_certificate_info (7, null, null, null, ext_oid);
@@ -1081,9 +1070,11 @@ normal_auth:
     {
       rc := "DB"."DBA"."DBEV_LOGIN" (user_name, digest, session_random);
     }
-  else
+  else if (rc <= 0) -- only if not authenticated
     {
-      rc := DB.DBA.LDAP_LOGIN (user_name, digest, session_random);
+      rc := DB.DBA.FOAF_SSL_LOGIN (user_name, digest, session_random);
+      if (rc = 0)
+        rc := DB.DBA.LDAP_LOGIN (user_name, digest, session_random);
     }
   return rc;
 }
@@ -1316,6 +1307,90 @@ LDAP_VALIDATION_FAILURE:
 }
 ;
 
+-- FOAF+SSL login
+create table SYS_USER_WEBID (UW_U_NAME varchar, UW_WEBID varchar, primary key (UW_WEBID))
+alter index SYS_USER_WEBID on SYS_USER_WEBID partition cluster replicated
+create index SYS_USER_WEBID_NAME on SYS_USER_WEBID (UW_U_NAME) partition cluster replicated
+;
+
+create procedure
+DB.DBA.FOAF_SSL_LOGIN (inout user_name varchar, in digest varchar, in session_random varchar)
+{
+  declare stat, msg, meta, data, info, qr, hf, graph, gr, alts any;
+  declare agent varchar;
+  declare rc int;
+  rc := 0;
+  gr := null;
+
+  declare exit handler for sqlstate '*'
+    {
+      rollback work;
+      goto err_ret;
+    }
+  ;
+
+  if (client_attr ('client_ssl') = 0)
+    return 0;
+
+  info := get_certificate_info (9);
+  agent := get_certificate_info (7, null, null, null, '2.5.29.17');
+
+  if (not isarray (info) or agent is null)
+    return 0;
+  alts := regexp_replace (agent, ',[ ]*', ',', 1, null);
+  alts := split_and_decode (alts, 0, '\0\0,:');
+  if (alts is null)
+    return 0;
+  agent := get_keyword ('URI', alts);
+  if (agent is null)
+    return 0;
+
+  hf := rfc1808_parse_uri (agent);
+  hf[5] := '';
+  gr := uuid ();
+  graph := WS.WS.VFS_URI_COMPOSE (hf);
+  qr := sprintf ('sparql load <%S> into graph <%S>', graph, gr);
+  stat := '00000';
+  exec (qr, stat, msg);
+  commit work;
+  qr := sprintf (
+        'sparql define input:storage "" '||
+	' prefix cert: <http://www.w3.org/ns/auth/cert#> '||
+	' prefix rsa: <http://www.w3.org/ns/auth/rsa#> ' ||
+  	' select (str (bif:coalesce (?exp_val, ?exp))) (str (bif:coalesce (?mod_val, ?mod))) '||
+	' from <%S> '||
+  	' where { '||
+	' 	  ?id cert:identity <%S> ; rsa:public_exponent ?exp ; rsa:modulus ?mod . ' ||
+	' 	  optional { ?exp cert:decimal ?exp_val . ?mod cert:hex ?mod_val . } '||
+	'       } ',
+	gr, agent);
+  stat := '00000';
+  exec (qr, stat, msg, vector (), 0, meta, data);
+  if (stat = '00000' and length (data))
+    {
+      foreach (any _row in data) do
+	{
+	  if (_row[0] = cast (info[1] as varchar) and
+	      lower (regexp_replace (_row[1], '[^A-Z0-9a-f]', '', 1, null)) = bin2hex (info[2]))
+    {
+      declare uname varchar;
+      uname := (select UW_U_NAME from SYS_USER_WEBID where UW_WEBID = agent);
+      if (length (uname))
+	{
+	  user_name := uname;
+          rc := 1;
+	}
+    }
+	}
+    }
+  err_ret:
+  if (gr is not null)
+    exec (sprintf ('sparql clear graph <%S>', gr), stat, msg);
+  commit work;
+  return rc;
+}
+;
+
 create procedure
 USERS_GET_DEF_QUAL (in dta varchar)
 {
