@@ -1,7 +1,7 @@
 /*
  *  xmlparser.h
  *
- *  $Id: xmlparser.h,v 1.9.2.2 2011/03/08 11:52:56 source Exp $
+ *  $Id: xmlparser.h,v 1.9.2.3 2011/10/28 21:46:53 source Exp $
  *
  *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
  *  project.
@@ -401,7 +401,7 @@ typedef struct rdf1808_split_s {
 
 extern void rfc1808_parse_uri (const char *iri, rdf1808_split_t *split_ret);
 extern void rfc1808_parse_wide_uri (const wchar_t *iri, rdf1808_split_t *split_ret);
-extern caddr_t rfc1808_expand_uri (/*query_instance_t *qi,*/ ccaddr_t base_uri, ccaddr_t rel_uri,
+extern caddr_t rfc1808_expand_uri (ccaddr_t base_uri, ccaddr_t rel_uri,
   ccaddr_t output_cs_name, int do_resolve_like_http_get,
   ccaddr_t base_string_cs_name, /* Encoding used for base_uri IFF it is a narrow string, neither DV_UNAME nor WIDE */
   ccaddr_t rel_string_cs_name, /* Encoding used for rel_uri IFF it is a narrow string, neither DV_UNAME nor WIDE */
